/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Dimf = require('./types/Dimf')
const InfoProx = require('./types/InfoProx')
const NatObj = require('./types/NatObj')
const StyleMarqueSegment = require('./types/StyleMarqueSegment')
const StyleEncadrement = require('./types/StyleEncadrement')
const StyleFleche = require('./types/StyleFleche')
const StyleMarqueAngle = require('./types/StyleMarqueAngle')
const StyleRemplissage = require('./types/StyleRemplissage')
const StyleTrait = require('./types/StyleTrait')
const constantes = require('./kernel/constantes')
const levels = require('./kernel/levels')
const constructionsBase = require('./kernel/constructionsBase')
const constructions = require('./kernel/constructions')
const NatCal = require('./types/NatCalAdd')
const Color = require('./types/Color')
const MotifPoint = require('./types/MotifPoint')
const kernel = require('./kernel/kernel')
const kernelAdd = require('./kernel/kernelAdd')
const OutilAnnuler = require('./outils/OutilAnnuler')
const OutilCapt = require('./outils/OutilCapt')
const OutilCaptNom = require('./outils/OutilCaptNom')
const OutilCarre = require('./outils/OutilCarre')
const OutilCentreGrav = require('./outils/OutilCentreGrav')
const OutilCentreCercle = require('./outils/OutilCentreCercle')
const OutilMilieu = require('./outils/OutilMilieu')
const OutilCerOA = require('./outils/OutilCerOA')
const OutilCerOR = require('./outils/OutilCerOR')
const OutilCerOAB = require('./outils/OutilCerOAB')
const OutilArcPetit = require('./outils/OutilArcPetit')
const OutilArcGrand = require('./outils/OutilArcGrand')
const OutilArcDirect = require('./outils/OutilArcDirect')
const OutilArcIndirect = require('./outils/OutilArcIndirect')
const OutilArcPetitParAng = require('./outils/OutilArcPetitParAng')
const OutilArcGrandParAng = require('./outils/OutilArcGrandParAng')
const OutilArcDirectParAng = require('./outils/OutilArcDirectParAng')
const OutilArcIndirectParAng = require('./outils/OutilArcIndirectParAng')
const OutilDemiDt = require('./outils/OutilDemiDt')
const DataInputStream = require('./entreesSorties/DataInputStream')
const ButtonTool = require('./interface/ButtonTool')
const ButtonToolAdd = require('./interface/ButtonToolAdd')
const ButtonStyleButton = require('./interface/ButtonStyleButton')
const MarqueSegmentButton = require('./interface/MarqueSegmentButton')
const StyleFlecheButton = require('./interface/StyleFlecheButton')
const MarqueAngleButton = require('./interface/MarqueAngleButton')
const ColorChoicePanel = require('./interface/ColorChoicePanel')
const StyleRemplissageButton = require('./interface/StyleRemplissageButton')
const StopButton = require('./interface/StopButton')
const ExpandableBar = require('./interface/ExpandableBar')
const LineStyleButton = require('./interface/LineStyleButton')
const OneColorButton = require('./interface/OneColorButton')
const Slider = require('./interface/Slider')
const ToolbarArrow = require('./interface/ToolbarArrow')
const Gestionnaire = require('./Gestionnaire')
const CPointBase = require('./objets/CPointBase')
const OutilPointageCapture = require('./pointage/OutilPointageCapture')
const OutilPointageClic = require('./pointage/OutilPointageClic')
const OutilPointageCre = require('./pointage/OutilPointageCre')
const OutilPointageObjetClignotant = require('./pointage/OutilPointageObjetClignotant')
const OutilPointageInt = require('./pointage/OutilPointageInt')
const OutilPointageRapporteur = require('./pointage/OutilPointageRapporteur')
const OutilPointageCaptureNom = require('./pointage/OutilPointageCaptureNom')
const OutilPointageTranslation = require('./pointage/OutilPointageTranslation')
const OutilPointageExecMac = require('./pointage/OutilPointageExecMac')
const OutilPointageClicOuPt = require('./pointage/OutilPointageClicOuPt')
const OutilAdd = require('./outils/OutilAdd')
const OutilPtLib = require('./outils/OutilPtLib')
const OutilMarquerPt = require('./outils/OutilMarquerPt')
const OutilDemarquerPt = require('./outils/OutilDemarquerPt')
const OutilPunaiser = require('./outils/OutilPunaiser')
const OutilDepunaiser = require('./outils/OutilDepunaiser')
const OutilDtAB = require('./outils/OutilDtAB')
const OutilDtParCoef = require('./outils/OutilDtParCoef')
const OutilDtBis = require('./outils/OutilDtBis')
const OutilDtHor = require('./outils/OutilDtHor')
const OutilDtMed = require('./outils/OutilDtMed')
const OutilDtPar = require('./outils/OutilDtPar')
const OutilDtPer = require('./outils/OutilDtPer')
const OutilDtVer = require('./outils/OutilDtVer')
const OutilDtParEq = require('./outils/OutilDtParEq')
const OutilMarqueAng = require('./outils/OutilMarqueAng')
const OutilMarqueAngOr = require('./outils/OutilMarqueAngOr')
const OutilMarqueAngOrSD = require('./outils/OutilMarqueAngOrSD')
const OutilMarqueAngOrSI = require('./outils/OutilMarqueAngOrSI')
const OutilMarqueSeg = require('./outils/OutilMarqueSeg')
const OutilRefaire = require('./outils/OutilRefaire')
const OutilRot = require('./outils/OutilRot')
const OutilRapporteur = require('./outils/OutilRapporteur')
const OutilArcParRapporteur = require('./outils/OutilArcParRapporteur')
const OutilArcGrandParRapporteur = require('./outils/OutilArcGrandParRapporteur')
const OutilSymAxiale = require('./outils/OutilSymAxiale')
const OutilSymCentrale = require('./outils/OutilSymCentrale')
const OutilTrans = require('./outils/OutilTrans')
const OutilTransParCoord = require('./outils/OutilTransParCoord')
const OutilHom = require('./outils/OutilHom')
const OutilSim = require('./outils/OutilSim')
const OutilImageInv = require('./outils/OutilImageInv')
const OutilSup = require('./outils/OutilSup')
const OutilCurseur = require('./outils/OutilCurseur')
const OutilTangente = require('./outils/OutilTangente')
const OutilCalcul = require('./outils/OutilCalcul')
const OutilVariable = require('./outils/OutilVariable')
const OutilSolutionEq = require('./outils/OutilSolutionEq')
const OutilDerivee = require('./outils/OutilDerivee')
const OutilDeriveePart = require('./outils/OutilDeriveePart')
const OutilMatrice = require('./outils/OutilMatrice')
const OutilCalculMat = require('./outils/OutilCalculMat')
const OutilMatriceAleat = require('./outils/OutilMatriceAleat')
const OutilMatriceParForm = require('./outils/OutilMatriceParForm')
const OutilDet = require('./outils/OutilDet')
const OutilRenommerCalcul = require('./outils/OutilRenommerCalcul')
const OutilSuiteRec = require('./outils/OutilSuiteRec')
const OutilSuiteRec2 = require('./outils/OutilSuiteRec2')
const OutilSuiteRec3 = require('./outils/OutilSuiteRec3')
const OutilSuiteRecComplexe = require('./outils/OutilSuiteRecComplexe')
const OutilSuiteRecComplexe2 = require('./outils/OutilSuiteRecComplexe2')
const OutilSuiteRecComplexe3 = require('./outils/OutilSuiteRecComplexe3')
const OutilMax = require('./outils/OutilMax')
const OutilMin = require('./outils/OutilMin')
const OutilRepere = require('./outils/OutilRepere')
const OutilCalculComp = require('./outils/OutilCalculComp')
const OutilSeg = require('./outils/OutilSeg')
const OutilSegmentParLong = require('./outils/OutilSegmentParLong')
const OutilTriangleEq = require('./outils/OutilTriangleEq')
const OutilVect = require('./outils/OutilVect')
const OutilParallelog = require('./outils/OutilParallelog')
const OutilPolygone = require('./outils/OutilPolygone')
const OutilPolygoneReg = require('./outils/OutilPolygoneReg')
const OutilLigneBrisee = require('./outils/OutilLigneBrisee')
const OutilProj = require('./outils/OutilProj')
const OutilModifObjGraph = require('./outils/OutilModifObjGraph')
const OutilZoomPlus = require('./outils/OutilZoomPlus')
const OutilZoomMoins = require('./outils/OutilZoomMoins')
const OutilTaillePlus = require('./outils/OutilTaillePlus')
const OutilTailleMoins = require('./outils/OutilTailleMoins')
const OutilModeTrace = require('./outils/OutilModeTrace')
const OutilModePointsAuto = require('./outils/OutilModePointsAuto')
const OutilRecalculer = require('./outils/OutilRecalculer')
const OutilGomme = require('./outils/OutilGomme')
const OutilRideau = require('./outils/OutilRideau')
const OutilTranslationFigure = require('./outils/OutilTranslationFigure')
const OutilExecutionMacro = require('./outils/OutilExecutionMacro')
const OutilNew = require('./outils/OutilNew')
const OutilModifObjNum = require('./outils/OutilModifObjNum')
const OutilPalette = require('./outils/OutilPalette')
const OutilNommer = require('./outils/OutilNommer')
const OutilInt = require('./outils/OutilInt')
const OutilPtParCoord = require('./outils/OutilPtParCoord')
const OutilPtParAff = require('./outils/OutilPtParAff')
const OutilPtLie = require('./outils/OutilPtLie')
const OutilPtBaseEnt = require('./outils/OutilPtBaseEnt')
const OutilPtInterieur = require('./outils/OutilPtInterieur')
const OutilPtParAbs = require('./outils/OutilPtParAbs')
const OutilBarycentre = require('./outils/OutilBarycentre')
const OutilPtParMultVec = require('./outils/OutilPtParMultVec')
const OutilPtParSommeVec = require('./outils/OutilPtParSommeVec')
const OutilMesLong = require('./outils/OutilMesLong')
const OutilMesLongOr = require('./outils/OutilMesLongOr')
const OutilMesAngNor = require('./outils/OutilMesAngNor')
const OutilMesAngOr = require('./outils/OutilMesAngOr')
const OutilMesAbs = require('./outils/OutilMesAbs')
const OutilMesAbsRep = require('./outils/OutilMesAbsRep')
const OutilMesOrdRep = require('./outils/OutilMesOrdRep')
const OutilMesAffRep = require('./outils/OutilMesAffRep')
const OutilMesCoefDir = require('./outils/OutilMesCoeffDir')
const OutilMesLongLigne = require('./outils/OutilMesLongLigne')
const OutilMesAire = require('./outils/OutilMesAire')
const OutilMesProSca = require('./outils/OutilMesProSca')
const OutilFonc = require('./outils/OutilFonc')
const OutilFoncComp = require('./outils/OutilFoncComp')
const OutilFonc2Var = require('./outils/OutilFonc2Var')
const OutilFonc3Var = require('./outils/OutilFonc3Var')
const OutilFoncComp2Var = require('./outils/OutilFoncComp2Var')
const OutilFoncComp3Var = require('./outils/OutilFoncComp3Var')
const OutilPartieReelle = require('./outils/OutilPartieReelle')
const OutilPartieImaginaire = require('./outils/OutilPartieImaginaire')
const OutilModule = require('./outils/OutilModule')
const OutilArgument = require('./outils/OutilArgument')
const OutilTestExistence = require('./outils/OutilTestExistence')
const OutilTestEq = require('./outils/OutilTestEq')
const OutilTestFact = require('./outils/OutilTestFact')
const OutilTestEqNatOp = require('./outils/OutilTestEqNatOp')
const OutilTestDepVar = require('./outils/OutilTestDepVar')
const OutilSomInd = require('./outils/OutilSomInd')
const OutilProdInd = require('./outils/OutilProdInd')
const OutilInteg = require('./outils/OutilInteg')
const OutilSurface = require('./outils/OutilSurface')
const OutilSurfaceLieuDroite = require('./outils/OutilSurfaceLieuDroite')
const OutilSurface2Lieux = require('./outils/OutilSurface2Lieux')
const OutilSurfaceLieu2Pts = require('./outils/OutilSurfaceLieu2Pts')
const OutilDemiPlan = require('./outils/OutilDemiPlan')
const OutilCouronne = require('./outils/OutilCouronne')
const OutilLieuParPtLie = require('./outils/OutilLieuParPtLie')
const OutilLieuDiscretParPtLie = require('./outils/OutilLieuDiscretParPtLie')
const OutilLieuParVariable = require('./outils/OutilLieuParVariable')
const OutilLieuDiscretParVariable = require('./outils/OutilLieuDiscretParVariable')
const OutilLieuObjetParPtLie = require('./outils/OutilLieuObjetParPtLie')
const OutilLieuObjetParVariable = require('./outils/OutilLieuObjetParVariable')
const OutilCourbeFonc = require('./outils/OutilCourbeFonc')
const OutilCourbeFoncCr = require('./outils/OutilCourbeFoncCr')
const OutilCourbeAvecTan = require('./outils/OutilCourbeAvecTan')
const OutilCourbePoly = require('./outils/OutilCourbePoly')
const OutilGrapheSuiteRec = require('./outils/OutilGrapheSuiteRec')
const OutilGrapheSuiteRecComp = require('./outils/OutilGrapheSuiteRecComp')
const OutilAffichageValeur = require('./outils/OutilAffichageValeur')
const OutilAffichageValeurLiePt = require('./outils/OutilAffichageValeurLiePt')
const OutilAffichageEq = require('./outils/OutilAffichageEq')
const OutilAffichageEqLie = require('./outils/OutilAffichageEqLie')
const OutilAffichageCoord = require('./outils/OutilAffichageCoord')
const OutilAffichageCoordLie = require('./outils/OutilAffichageCoordLie')
const OutilLatex = require('./outils/OutilLatex')
const OutilLatexLiePt = require('./outils/OutilLatexLiePt')
const OutilCommentaire = require('./outils/OutilCommentaire')
const OutilCommentaireLiePt = require('./outils/OutilCommentaireLiePt')
const OutilImageLibre = require('./outils/OutilImageLibre')
const OutilImageLiee = require('./outils/OutilImageLiee')
const OutilEditeurFormule = require('./outils/OutilEditeurFormule')
const OutilCreationLiaison = require('./outils/OutilCreationLiaison')
const OutilCreationLiaisonAff = require('./outils/OutilCreationLiaisonAff')
const OutilSuppressionLiaison = require('./outils/OutilSuppressionLiaison')
const OutilSuppressionLiaisonAff = require('./outils/OutilSuppressionLiaisonAff')
const OutilObjetDuplique = require('./outils/OutilObjetDuplique')
const OutilObjetClone = require('./outils/OutilObjetClone')
const OutilReclassDebObjGra = require('./outils/OutilReclassDebObjGra')
const OutilReclassFinObjGra = require('./outils/OutilReclassFinObjGra')
const OutilCopierStyle = require('./outils/OutilCopierStyle')
const OutilAbsOrRep = require('./outils/OutilAbsOrRep')
const OutilOrdOrRep = require('./outils/OutilOrdOrRep')
const OutilUnitexRep = require('./outils/OutilUnitexRep')
const OutilUniteyRep = require('./outils/OutilUniteyRep')
const OutilOptionsFig = require('./outils/OutilOptionsFig')
const OutilCodeBase64 = require('./outils/OutilCodeBase64')
const OutilCodeTikz = require('./outils/OutilCodeTikz')
const OutilExport = require('./outils/OutilExportFig')
const OutilSavePNG = require('./outils/OutilSavePNG')
const OutilSavePNGWithUnity = require('./outils/OutilSavePNGWithUnity')
const OutilSaveJPG = require('./outils/OutilSaveJPG')
const OutilSaveSVG = require('./outils/OutilSaveSVG')
const OutilExportHTML = require('./outils/OutilExportHTML')
const OutilCopy = require('./outilsWithElectron/OutilCopy')
const OutilCopyWithUnity = require('./outilsWithElectron/OutilCopyWithUnity')
const OutilAnimation = require('./outils/OutilAnimation')
const OutilOpen = require('./outils/OutilOpen')
const OutilSave = require('./outils/OutilSave')
const OutilGraduationAxes = require('./outils/OutilGraduationAxes')
const OutilProtocole = require('./outils/OutilProtocole')
const OutilHelp = require('./outils/OutilHelp')
const OutilToggleToolsAdd = require('./outils/OutilToggleToolsAdd')
const OutilLongUnit = require('./outils/OutilLongUnit')
const OutilMacApp = require('./outils/OutilMacApp')
const OutilMacDisp = require('./outils/OutilMacDisp')
const OutilMacAppParAut = require('./outils/OutilMacAppParAut')
const OutilMacAnim = require('./outils/OutilMacAnim')
const OutilMacDispParAut = require('./outils/OutilMacDispParAut')
const OutilMacAnimTr = require('./outils/OutilMacAnimTr')
const OutilMacTraceAuto = require('./outils/OutilMacTraceAuto')
const OutilMacTraceAutoVa = require('./outils/OutilMacTraceAutoVa')
const OutilMacAffValVar = require('./outils/OutilMacAffValVar')
const OutilMacClign = require('./outils/OutilMacClign')
const OutilMacIncVar = require('./outils/OutilMacIncVar')
const OutilMacDecVar = require('./outils/OutilMacDecVar')
const OutilMacModifVar = require('./outils/OutilMacModifVar')
const OutilMacAffPtLie = require('./outils/OutilMacAffPtLie')
const OutilMacReaff = require('./outils/OutilMacReaff')
const OutilMacActTr = require('./outils/OutilMacActTr')
const OutilMacDesactTr = require('./outils/OutilMacDesactTr')
const OutilMacPause = require('./outils/OutilMacPause')
const OutilMacSuiteMac = require('./outils/OutilMacSuiteMac')
const OutilMacJouantSon = require('./outils/OutilMacJouantSon')
const OutilMacBoucAnim = require('./outils/OutilMacBoucAnim')
const OutilMacBoucTr = require('./outils/OutilMacBoucleTr')
const OutilMacConsRec = require('./outils/OutilMacConsRec')
const OutilMacConsIter = require('./outils/OutilMacConsIter')
const OutilAddObjMac = require('./outils/OutilAddObjMac')
const OutilSupObjMac = require('./outils/OutilSupObjMac')
const OutilFusionImpConst = require('./outils/OutilFusionImpConst')
const OutilModifConst = require('./outils/OutilModifConst')
const OutilSupConst = require('./outils/OutilSupConst')
const OutilImpConstFig = require('./outils/OutilImpConstFig')
const OutilSaveConst = require('./outils/OutilSaveConst')
const OutilAddConst = require('./outils/OutilAddConst')
const OutilGestionConst = require('./outils/OutilGestionConst')
const OutilChoixSrcNumConst = require('./outils/OutilChoixSrcNumConst')
const OutilChoixSrcGraphConst = require('./outils/OutilChoixSrcGraphConst')
const OutilChoixFinGraphConst = require('./outils/OutilChoixFinGraphConst')
const OutilChoixFinNumConst = require('./outils/OutilChoixFinNumConst')
const OutilFinirConst = require('./outils/OutilFinirConst')
const OutilReInitConst = require('./outils/OutilReinitConst')
const OutilCreationConst = require('./outils/OutilCreationConst')
const CAffLiePt = require('./objets/CAffLiePt')
const CCommentaire = require('./objets/CCommentaire')
const CListeObjets = require('./objets/CListeObjets')
const CMathGraphDoc = require('./objets/CMathGraphDoc')
const NameEditor = require('./interface/NameEditor')
const CMacroAnimationFigure = require('./objetsAdd/CMacroAnimationFigure')
const InfoAnim = require('./types/InfoAnim')
const $ = require('jquery')
const DataOutputStream = require('./entreesSorties/DataOutputStream')
const AvertDlg = require('./dialogs/AvertDlg')
const textLoadMtgApp = require('./kernel/textLoadMtgApp')
const MtgAppLecteur = require('./MtgAppLecteur')
const CSousListeObjets = require('./objets/CSousListeObjets')
const CValeurAngle = require('./objets/CValeurAngle')

module.exports = MtgApp

/**
 * Classe de l'application mathGraph32 permettant de créer ou modifier une figure
 * @constructor
 * @param {SVGElement} svg Le svg dans lequel l'application travaille (il doit avoir un id)
 * @param {MtgOptions} mtgOptions Les informations sur l'initialisation de l'application
 */
function MtgApp (svg, mtgOptions) {
  if (typeof mtgOptions !== 'object') mtgOptions = {}
  /**
   * Le svg contenant mathgraph
   * @type {SVGElement}
   */
  this.svg = svg
  /**
   * id du svg
   * @type {string}
   */
  this.id = this.svg.id

  // init des propriétés déduites de mtgOptions
  /**
   * True pour un affichage plus adapté aux dys
   * @type {boolean}
   */
  this.dys = Boolean(mtgOptions.dys)
  /**
   * true si on est dans electron
   * @type {boolean}
   */
  this.electron = Boolean(mtgOptions.electron)
  if (mtgOptions.bplayer) {
    /**
     * Player éventuel
     * @type {MtgAppLecteur}
     */
    this.player = new MtgAppLecteur(false) // false pour ne pas recharger les langues
  }
  /**
   * Mode aperçu dans Labomep
   * @type {boolean}
   */
  this.preview = Boolean(mtgOptions.preview)
  /**
   * Callback pour envoyer la figure
   * @type {function}
   */
  this.functionOnSave = mtgOptions.functionOnSave

  // init language
  const language = mtgOptions.language || ''
  if (language === '') {
    /**
     * La langue choisie (par défaut suivant les préférences du navigateur), parmi fr, en, es
     * @type {string}
     */
    this.language = kernel.userLan()
  } else if (['fr', 'en', 'es'].includes(language)) {
    this.language = language
  } else {
    console.error(new Error(`${language} n’est pas une langue gérée`))
    this.language = 'en'
  }

  // init des textes d'après la langue
  textLoadMtgApp.initTextes(this.language, () => {
    // this est toujours l'instance en cours de construction grace à la fct fléclée de la ligne précédente
    let ba, inps, i, j, c, ind, tab
    /**
     * Url absolue du dossier des images (avec slash de fin)
     * @type {string}
     */
    this.imagePath = 'https://www.mathgraph32.org/images/'
    /**
     * null par défaut (pas de cadre pour visualiser un cadre de dimensions données par l'utilisateur)
     * On pourra demander un tel cadre via les options de la figure
     * @type {*}
     */
    this.cadre = null
    /**
     * Pointe sur le svg contenant les icônes des outils supplémentaires.
     * null quand les outils supplémentaires ne sont pas disponibles,
     * @type {SVGElement}
     */
    this.svgToolsAdd = null
    /**
     * Préférences d'animation
     * @type {InfoAnim}
     */
    this.pref_Anim = new InfoAnim(100, 20, true, false)
    /**
     * Taille de la police des noms d'objets
     * @type {number}
     */
    this.pref_TaillePoliceNom = this.dys ? 22 : 16
    // On charge les 4 figures donnant les niveaux de fonctionnement prédéfinis
    this.loadLevels()

    // On définit le niveau de fonctionnement du logiciel
    // y'a des options en global qui écrasent ce qu'on nous a passé concernant local & localPath
    if (typeof window.mtgOptions === 'object' && window.mtgOptions.localPath) {
      /**
       * Fonctionnement en local
       * @type {boolean}
       */
      this.local = true
      /**
       * Si local, chemin local vers les images (avec slash de fin)
       * @type {*|string}
       */
      this.localPath = window.mtgOptions.localPath
    } else if (mtgOptions.local !== undefined) {
      if (typeof mtgOptions.local === 'string') {
        this.local = true
        this.localPath = mtgOptions.local
      } else if (mtgOptions.local) {
        this.local = true
        this.localPath = './'
      } else {
        this.local = false // pas besoin de localPath dans ce cas
      }
    } else {
      this.local = false
    }
    // on s'assure que localPath fini par un slash s'il existe
    if (this.localPath && this.localPath.substr(-1) !== '/') this.localPath += '/'
    /**
     * coefficient multiplicateur pour la taille des images exportées
     * @type {number}
     */
    this.pref_coefMult = 2
    /**
     * Affiche les mesures si true
     * @type {boolean}
     */
    this.pref_DispMeas = mtgOptions.displayMeasures === undefined ? true : mtgOptions.displayMeasures
    /**
     * Si true des points sont créés automatiquement au clic avec certains outils (dys impose false)
     * @type {boolean}
     */
    this.pref_PointsAuto = this.dys ? false : Boolean(mtgOptions.pointsAuto)
    /**
     * ???
     * @type {boolean|*}
     */
    this.codeBase64 = mtgOptions.codeBase64 === undefined ? true : mtgOptions.codeBase64
    // @todo toujours un booléen ? Si oui il faudrait plutôt this.codeBase64 = typeof mtgOptions.codeBase64 === 'boolean' ? mtgOptions.codeBase64 : true
    // @todo idem pour les propriétés qui suivent
    if (mtgOptions.open !== undefined) {
      /**
       * Si true on affiche le bouton ouvrir
       * @type {boolean}
       */
      this.open = mtgOptions.newFig
    } else {
      this.open = true
    }
    if (mtgOptions.save !== undefined) {
      /**
       * Si true on affiche le bouton sauvegarder
       * @type {boolean}
       */
      this.save = mtgOptions.save
    } else {
      this.save = true
    }
    // if (mtgOptions.newFig !== undefined) this.newFig = mtgOptions.newFig; else this.newFig = true; // Modification version 6.3.5
    // @todo mettre plutôt this.onlyPoints = Boolean(mtgOptions.onlyPoints) ?
    if (mtgOptions.onlyPoints !== undefined) {
      /**
       * ???
       * @type {boolean}
       */
      this.onlyPoints = mtgOptions.onlyPoints
    } else {
      this.onlyPoints = false
    }
    // if (mtgOptions.options !== undefined) this.options = mtgOptions.options; else this.options = true; // Modification version 6.3.5
    /**
     * Facteur de zoom (mtgOptions.zoomFactor ou 1 par défaut)
     * @type {number}
     */
    this.zoomFactor = typeof mtgOptions.zoomFactor === 'number' ? mtgOptions.zoomFactor : 1
    if (mtgOptions.stylePointCroix) {
      /**
       * Style de point (cf MotifPoint)
       * @type {number}
       */
      this.stylePoint = MotifPoint.grandMult
    }
    // if (mtgOptions.edition !== undefined) this.edition = mtgOptions.edition; else this.edition = false
    /**
     * Flag pour savoir si on est en mode édition (sinon player only)
     * @type {boolean}
     */
    this.edition = Boolean(mtgOptions.edition)
    if (this.dys) {
      constantes.arrowIconWidth *= 1.5
      constantes.svgPanelWidth += constantes.arrowIconWidth / 4
    }
    if (mtgOptions.level !== undefined) {
      var lev = mtgOptions.level
      if (typeof lev === 'number') {
        if ((lev >= 0) && (lev <= 3)) {
          /**
           * Index du level courant (utilisé dans getResult)
           * @type {number}
           */
          this.levelIndex = lev
          /**
           * Les 4 figures du level courant
           */
          this.level = this.levels[lev]
        } else {
          this.level = this.levels[3]
        }
      } else {
        if (typeof lev === 'string') {
          var niv = parseInt(lev) // Nécessaire pour la version electron où le niveau est sauvé comme une chaîne de caractères
          if (niv >= 0 && niv < 4) {
            this.levelIndex = niv // Utilisé dans getResult
            this.level = this.levels[niv]
          } else {
            try {
              this.level = this.loadDoc('', lev)
            } catch (e) {
              this.level = this.levels[3]
            }
          }
        }
      }
    } else {
      this.level = this.levels[3]
    }

    /**
     * Si true, la figure s'affiche automatiquement dès qu'elle est prête (sinon faudra appeler calculateAndDisplay)
     * @type {boolean}
     */
    this.displayOnLoad = (mtgOptions.displayOnLoad === undefined) ? true : mtgOptions.displayOnLoad
    const zf = this.zoomFactor

    const self = this
    // Initialisation des données de la barre de droite
    /**
     * Couleur active
     * @type {Color}
     */
    this.couleurActive = Color.black
    /**
     * Largeur de trait
     * @type {number}
     */
    this.thickness = this.dys ? 2 : 1
    if (!this.stylePoint) this.stylePoint = this.dys ? MotifPoint.grandRond : MotifPoint.petitRond
    /**
     * @type {number}
     */
    this.styleMarqueSegment = StyleMarqueSegment.marqueSimple
    /**
     * @type {number}
     */
    this.styleFleche = StyleFleche.FlecheLonguePleine
    /**
     * @type {number}
     */
    this.styleMarqueAngle = StyleMarqueAngle.marqueSimple
    /**
     * @type {number}
     */
    this.styleRemplissage = StyleRemplissage.quadrillagePleinTransp
    /**
     * Pointeur sur le commentaire d'indication, null au départ
     * @type {null}
     */
    this.comm = null
    //
    const w = parseFloat(svg.getAttribute('width'))
    const h = parseInt(svg.getAttribute('height'))
    /**
     * Dimensions de la fenêtre
     * @type {Dimf}
     */
    this.dimf = new Dimf(w - constantes.svgPanelWidth * zf - constantes.rightPanelWidth * zf, h - constantes.topIconSize * zf)

    // Ajout version mtgApp. Etait auparavant dans CEditeurFormule
    const par = svg.parentNode // Le div parent
    // $(par).css('font-weight', 'normal').css('font-family', '"Times New Roman", Times, serif;'); // Pour que les textes ne prennent pas par défaut le style du parent du div qui peut être bold
    // $(par).css('font-weight', 'normal').css('font-family', 'serif'); // Inutile fait en css par la class  svgMtg
    if (par.style.position === '') par.style.position = 'relative'
    var divSvgAux = kernel.ce('div', {
      id: 'divsvgaux',
      position: 'absolute',
      style: 'position:absolute;left:0px;top:0px;width:' + w + 'px;height:' + h + 'px;pointer-events:none'
    })
    par.appendChild(divSvgAux)
    // On crée un svg caché qui sera chargé de contenir tous les affichages de textes affichés provisoirement
    // pour connaître leur taille (nécessaire pour compatibilité avec les onglets de spip avec un chargement en display none)
    // Version 6.4 : On donne à svgAux la classe svgMtg pour que les affichages de texte utilisent une police serif interne au navigateur
    var svgAux = kernel.cens('svg', {
      class: 'svgMtg',
      id: 'mtgSvgAux',
      width: w,
      height: h,
      visibility: 'hidden'
    })
    divSvgAux.appendChild(svgAux)
    /**
     * div centré sur l'écran pour les boîtes de dialogue
     * @type {HTMLDivElement}
     */
    this.divDlg = kernel.ce('div', {
      id: 'divMtgDlg',
      style: 'position:absolute;halign:center;margin-left:auto;margin-right:auto;top:0px;width:' + w + 'px;height:450px;pointer-events:none'
    })
    par.appendChild(this.divDlg)
    //
    this.creeDefs()
    /**
     * Document contenant les macros constructions de base utilisées
     * @type {CMathGraphDoc}
     */
    this.docCons = new CMathGraphDoc('', true, true)
    ba = kernel.base64Decode(constructionsBase)
    inps = new DataInputStream(ba)
    this.docCons.read(inps)
    /**
     * Document pour contenir les constructions avancées comme les courbes avec crochet
     * @type {CMathGraphDoc}
     */
    this.docConsAv = new CMathGraphDoc('', true, true)
    ba = kernel.base64Decode(constructions.constSup)
    inps = new DataInputStream(ba)
    this.docConsAv.read(inps)
    /**
     * ??
     * @type {Gestionnaire}
     */
    this.gestionnaire = new Gestionnaire(this)
    /**
     * tableau contenant les id des boîtes de dialogues ouvertes
     * @type {string[]}
     */
    this.dlg = []
    /**
     * Barre d'outils
     * @type {SVGElement}
     */
    this.toolBar = kernel.cens('svg', {
      id: this.id + 'toolBarPanel',
      x: String(constantes.svgPanelWidth * zf),
      y: 0,
      width: w,
      height: constantes.toolbarHeight * zf
    })
    this.toolBar.style.cursor = 'default'
    svg.appendChild(this.toolBar)
    /**
     * svg de la figure
     * @type {SVGElement}
     */
    // Version 6.4 : On donne à svgAux la classe svgMtg pour que les affichages de texte utilisent une police serif interne au navigateur
    this.svgFigure = kernel.cens('svg', {
      class: 'svgMtg',
      id: this.id + 'figurePanel',
      'shape-rendering': 'geometricPrecision',
      x: String(constantes.svgPanelWidth * zf),
      y: String(constantes.toolbarHeight * zf),
      width: String(w - constantes.svgPanelWidth * zf - constantes.rightPanelWidth * zf),
      height: String(h - constantes.topIconSize * zf)
    })
    // Ajout version 6.5.2 : On a besoin de connaître le zoomFactor de l'application quand on positionne
    // les éditeurs de formule et les dialogues associés aux variables
    this.svgFigure.zoomFactor = this.zoomFactor
    this.svgFigure.style.pointerEvents = 'none'
    // Je n'ai pas réussi à faire répondre le svgFigure aux événements souris.
    // On crée donc un rectangle à l'intérieur pour répondre aux événements souris de la figure
    var svgRect = kernel.cens('rect', {
      id: 'svgrect',
      x: String(constantes.svgPanelWidth * zf),
      y: String(constantes.toolbarHeight * zf),
      width: String(w - constantes.svgPanelWidth * zf - constantes.rightPanelWidth * zf),
      height: String(h - constantes.topIconSize * zf),
      stroke: 'none',
      fill: 'none',
      // "pointer-events" : "all",
      visibility: 'hidden'
    })
    $(svgRect).css('pointer-events', 'all')
    svg.appendChild(svgRect)
    // On donne la possibilité de faire un drag and drop d'une figure mtg32
    svgRect.addEventListener('drop', function (ev) {
      self.onDropFile(ev)
    })
    svgRect.addEventListener('dragover', function (ev) {
      ev.stopPropagation()
      kernel.preventDefault(ev)
      ev.dataTransfer.dropEffect = 'copy'
    })
    svg.appendChild(this.svgFigure)
    /**
     * Panneau ?
     * @type {SVGElement}
     */
    this.svgPanel = kernel.cens('svg', {
      id: 'mtgleftpanel',
      x: 0,
      y: 0,
      width: w,
      height: h
      // "pointer-events" : "none",
    })
    this.svgPanel.style.PointerEvents = 'none'
    this.svgPanel.style.cursor = 'default'
    svg.appendChild(this.svgPanel)
    /**
     * Panneau de droite avec les styles ?
     * @type {SVGElement}
     */
    this.rightPanel = kernel.cens('svg', {
      // id : this.id + "rightPanel",
      id: 'mtgrightpanel',
      x: String(w - constantes.rightPanelWidth * zf),
      // y : 0, // On laisse un peu de plave pour les tips
      y: String(constantes.toolbarHeight * zf + 26),
      width: String(constantes.rightPanelWidth * zf),
      height: String(h - constantes.topIconSize * zf)
    })
    this.rightPanel.style.cursor = 'default'
    svg.appendChild(this.rightPanel)
    // Préparation des composants de la barre d'outil de droite
    // Un g qui va contenir les boutons de choix de couleur prédéfinie
    var yc = constantes.toolbarHeight * zf // Cette variable contiendra successivement les ordonnées des composants de la barre de droite
    /**
     * Style de ligne
     * @type {SVGGElement}
     */
    this.gLineStyle = kernel.cens('g', {
      transform: 'translate(0,' + yc + ')'
    })
    /**
     * boutons de choix de style de ligne
     * @type {Array}
     */
    this.lineStyleButtons = []
    this.rightPanel.appendChild(this.gLineStyle)
    const tabls = constantes.tableStyleTrait
    for (i = 0; i < tabls.length; i++) {
      c = tabls[i]
      this.lineStyleButtons.push(new LineStyleButton(this, c[0], c[1], i, yc))
    }
    /**
     * boutons de choix de style de point
     * @type {Array}
     */
    this.buttonStyleButtons = []
    /**
     * Panneau des styles de point
     * @type {SVGGElement}
     */
    this.pointStylePanel = kernel.cens('g', {
      transform: 'translate(' + constantes.lineStyleWidth * zf + ',' + yc + ')'
    })
    ind = 0
    tab = [[MotifPoint.croix, 'Croix'], [MotifPoint.multi, 'Mult'], [MotifPoint.losange, 'Losange'],
      [MotifPoint.petitRond, 'PetitRond'], [MotifPoint.rond, 'Rond'], [MotifPoint.pave, 'Pave'],
      [MotifPoint.grandRond, 'GrandRond'], [MotifPoint.grandMult, 'GrandMult']]
    for (i = 0; i < 2; i++) {
      for (j = 0; j < 4; j++) {
        c = tab[ind]
        this.buttonStyleButtons.push(new ButtonStyleButton(this, c[0], 'stylePoint' + c[1], j, i, yc))
        ind++
      }
    }
    new ButtonStyleButton(this, MotifPoint.pixel, 'stylePointPixel', 4, 0, yc)
    this.rightPanel.appendChild(this.pointStylePanel)
    this.selectButton(this.getStylePoint(), this.buttonStyleButtons)
    yc += constantes.lineStyleButtonHeight * zf * tabls.length
    // Fin de la création des boutons de choix de style de point

    /**
     * slider de choix d'épaisseur de trait
     * @type {Slider}
     */
    this.thicknessSlider = new Slider(this, constantes.rightPanelWidth, constantes.sliderHeight, 1, 12, this.dys ? 2 : 1, 2, 1,
      'ChoixEpaisseur', 'translate(0,' + yc + ')', '', yc, function (newval) {
        self.setThickness(newval)
      })
    this.rightPanel.appendChild(this.thicknessSlider.container)
    yc += constantes.sliderHeight * zf
    // Fin de l'ajout du slider de choix d'épaisseur de trait

    /**
     * boutons de choix de style de marque de segment
     * @type {Array}
     */
    this.marqueSegmentButtons = []
    /**
     * Marque de segment
     * @type {SVGGElement}
     */
    this.marqueSegmentPanel = kernel.cens('g', {
      transform: 'translate(0,' + yc + ')'
    })
    ind = 0
    tab = [[StyleMarqueSegment.marqueSimple, '1'], [StyleMarqueSegment.marqueTriple, '3'],
      [StyleMarqueSegment.marqueDouble, '2'], [StyleMarqueSegment.marqueCroix, '4']]
    for (i = 0; i < 2; i++) {
      for (j = 0; j < 2; j++) {
        c = tab[ind]
        this.marqueSegmentButtons.push(new MarqueSegmentButton(this, c[0], 'styleMarqueSegment' + c[1], j, i, yc))
        ind++
      }
    }
    this.rightPanel.appendChild(this.marqueSegmentPanel)
    this.selectButton(this.styleMarqueSegment, this.marqueSegmentButtons)
    // Fin de la création des boutons de choix de style de marque de segment

    /**
     * boutons de choix de style de flèche
     * @type {Array}
     */
    this.styleFlecheButtons = []
    /**
     * Bouton courant du style de flèche ?
     * @type {SVGGElement}
     */
    this.styleFlechePanel = kernel.cens('g', {
      transform: 'translate(' + String(3 * constantes.buttonMarqueWidth * zf) + ',' + yc + ')'
    })
    ind = 0
    tab = [[StyleFleche.FlecheLonguePleine, '1'], [StyleFleche.FlecheLongueSimple, '3'],
      [StyleFleche.FlecheCourtePleine, '2'], [StyleFleche.FlecheCourteSimple, '4']]
    for (i = 0; i < 2; i++) {
      for (j = 0; j < 2; j++) {
        c = tab[ind]
        this.styleFlecheButtons.push(new StyleFlecheButton(this, c[0], 'styleFleche' + c[1], j, i, yc))
        ind++
      }
    }
    this.rightPanel.appendChild(this.styleFlechePanel)
    this.selectButton(this.styleFleche, this.styleFlecheButtons)
    yc += constantes.buttonMarqueWidth * zf * 2
    // Fin de la création des boutons de choix de style de flèche

    /**
     * boutons de choix de style de marque d'angle
     * @type {Array}
     */
    this.marqueAngleButtons = []
    /**
     * Bouton courant du style de marque d'angle ?
     * @type {SVGGElement}
     */
    this.marqueAnglePanel = kernel.cens('g', {
      transform: 'translate(0,' + yc + ')'
    })
    ind = 0
    tab = [[StyleMarqueAngle.marqueSimple, '1'], [StyleMarqueAngle.marquePleine, '6'],
      [StyleMarqueAngle.marqueSimple1Trait, '2'], [StyleMarqueAngle.marquePleine1Trait, '7'],
      [StyleMarqueAngle.marqueSimple2Traits, '3'], [StyleMarqueAngle.marquePleine2Traits, '8'],
      [StyleMarqueAngle.marqueSimple3Traits, '4'], [StyleMarqueAngle.marquePleine3Traits, '9'],
      [StyleMarqueAngle.marqueSimpleCroix, '5'], [StyleMarqueAngle.marquePleineCroix, '10']]
    for (i = 0; i < 5; i++) {
      for (j = 0; j < 2; j++) {
        c = tab[ind]
        this.marqueAngleButtons.push(new MarqueAngleButton(this, c[0], 'styleMarqueAngle' + c[1], j, i, yc))
        ind++
      }
    }
    this.rightPanel.appendChild(this.marqueAnglePanel)
    this.selectButton(this.styleMarqueAngle, this.marqueAngleButtons)
    yc += constantes.buttonMarqueWidth * zf * 2
    // Fin de la création des boutons de choix de style de marque d'angle

    /**
     * boutons de choix de type de remplissage
     * @type {Array}
     */
    this.styleRemplissageButtons = []
    /**
     * Bouton courant du type de remplissage ?
     * @type {SVGGElement}
     */
    this.styleRemplissagePanel = kernel.cens('g', {
      transform: 'translate(0,' + yc + ')'
    })
    ind = 0
    tab = [[StyleRemplissage.quadrillagePleinTransp, 'Transp'],
      [StyleRemplissage.quadrillage45Haut, '45Haut'],
      [StyleRemplissage.quadrillagePlein, 'Plein'],
      [StyleRemplissage.quadrillageHorizontal, 'Hor'],
      [StyleRemplissage.quadrillage45Bas, '45Bas'],
      [StyleRemplissage.quadrillageVertical, 'Ver']]
    for (i = 0; i < 3; i++) {
      for (j = 0; j < 2; j++) {
        c = tab[ind]
        this.styleRemplissageButtons.push(new StyleRemplissageButton(this, c[0], 'styleHachure' + c[1], j, i, yc))
        ind++
      }
    }
    this.rightPanel.appendChild(this.styleRemplissagePanel)
    this.selectButton(this.styleRemplissage, this.styleRemplissageButtons)
    yc += constantes.buttonFillHeight * zf * 2

    // Chargement du document de la figure
    // this.doc = new CMathGraphDoc(this.id + "figurePanel") // Modifié version 6.3.0
    /**
     * La figure
     * @type {CMathGraphDoc}
     */
    this.doc = new CMathGraphDoc(this.id) // Modifié version 6.2.4
    // Ligne suivante nécessaire pour que les macros d'activation et désactivation du mode trace activent ou non
    // l'icône correspondante
    this.doc.app = this
    const datafig = mtgOptions.fig
    if (typeof datafig === 'string') {
      if (datafig === '') {
        // on ne précise ni figure ni préférences, figure vide par défaut
        this.initAvecLongueurUnite()
        this.doc.listePr.uniteAngle = kernel.uniteAngle.degre
      } else {
        if (datafig.indexOf('TWF0aEdy') === 0) {
          // Ça ressemble à une figure en base 64 valide, on la charge
          // this.doc = this.loadDoc(this.id + "figurePanel", datafig); // Modifié version 6.3.0
          this.doc = this.loadDoc(this.id, datafig)
        } else {
          // Sous electron quand on double clique sur un fichier pour l'ouvrir dataFig est une chaîne contenant un flux binaire
          var newdoc = this.getDocFromString(datafig)
          if (newdoc === null) {
            new AvertDlg(this, 'errInvFile')
            this.initAvecLongueurUnite()
            this.doc.listePr.uniteAngle = kernel.uniteAngle.degre
            if (this.electron) resetDocument() // eslint-disable-line no-undef
          } else {
            this.doc = newdoc
            // Pour qu'on puisse sauver la figure ouverte par un doule clic
            if (this.electron) setNewPath('') // eslint-disable-line no-undef
          }
        }
        // Ligne suivante nécessaire pour que les macros d'activation et désactivation du mode trace activent ou non
        // l'icône correspondante
        this.doc.app = this
      }
    } else {
      /*
      this.doc.gTraces = kernel.cens("g",{
        id : this.idDoc+"Traces"
      })
      this.svgFigure.appendChild(this.doc.gTraces)
      */
      var datarep = datafig.datarep
      var uniteAngle = datafig.unity === 'rad' ? kernel.uniteAngle.radian : kernel.uniteAngle.degre
      switch (datafig.type) {
        case 'simple' :
          this.initSansLongueurUnite()
          break
        case 'orthonormal' :
          this.initAvecRepereOrthonormal(uniteAngle, datarep.quadhor, datarep.quadver, datarep.grid,
            datarep.withvect, datarep.grad)
          /**
           * Préférence du type de figure au démarrage (frameGrid|frameDotted|unity)
           * @type {string}
           */
          this.pref_StartFig = datarep.quadhor ? 'frameGrid' : 'frameDotted'
          break
        case 'orthogonal' :
          this.initAvecRepereOrthonormal(uniteAngle, datarep.quadhor, datarep.quadver, datarep.grid,
            datarep.withvect, datarep.grad)
          break
        case 'unity' :
        default :
          this.initAvecLongueurUnite(uniteAngle)
          this.pref_StartFig = 'unity'
      }
    }
    const doc = this.doc
    doc.dimf = this.dimf
    doc.app = this
    /**
     * Raccourci pour la liste de tous les objets (idem this.doc.listePr)
     * @type {CListeObjets}
     */
    this.listePr = doc.listePr
    /**
     * Style de ligne
     * @type {number}
     */
    this.lineStyle = StyleTrait.styleTraitContinu
    this.selectButton(this.lineStyle, this.lineStyleButtons)
    /**
     *
     * @type {SVGGElement}
     */
    this.colorPanel = kernel.cens('g', {
      transform: 'translate(0,' + yc + ')'
    })
    var colorPanelHeight = (4 * constantes.colorButtonHeight) * zf + 2
    this.rightPanel.appendChild(this.colorPanel)
    this.colorPanel.appendChild(kernel.cens('rect', {
      x: 0,
      y: 0,
      stroke: constantes.buttonBackGroundColor,
      width: constantes.rightPanelWidth * zf,
      height: colorPanelHeight,
      fill: constantes.buttonFrameColor
    }))
    // Ajout des 16 boutons de choix de couleur
    tab = [[0, 0, 0], [0, 0, 255], [0, 255, 0], [255, 0, 0], [0, 255, 255], [255, 0, 255], [255, 255, 0], [127, 127, 127],
      [0, 0, 127], [0, 127, 0], [127, 0, 0], [0, 127, 127], [127, 0, 127], [127, 127, 0], [102, 102, 102], [51, 51, 51]]
    ind = 0
    for (i = 0; i < 4; i++) {
      for (j = 0; j < 4; j++) {
        c = tab[ind]
        new OneColorButton(this, c[0], c[1], c[2], i, j, yc)
        ind++
      }
    }
    //
    yc += colorPanelHeight
    // Création des deux ellipses : Une pour lancer un color picker et l'autre pour montrer l'effet de transparence
    /** @type {ColorChoicePanel} */
    this.colorChoicePanel = new ColorChoicePanel(this, yc)
    this.rightPanel.appendChild(this.colorChoicePanel.container)
    yc += constantes.colorChoicePanelHeight * zf
    /** @type {Slider} */
    this.opacitySlider = new Slider(this, constantes.rightPanelWidth, constantes.sliderHeight, 5, 100, 20, 4, 5,
      'ChoixOpacite', 'translate(0,' + yc + ')', '%', yc, function (newval) {
        self.setOpacity(newval)
      })
    this.rightPanel.appendChild(this.opacitySlider.container)
    yc += constantes.sliderHeight * zf
    /**
     * Panneau du bouton stop
     * @type {SVGGElement}
     */
    this.stopPanel = kernel.cens('g', {
      transform: 'translate(0,' + yc + ')'
    })
    this.rightPanel.appendChild(this.stopPanel)
    /** @type {StopButton} */
    this.stopButton = new StopButton(this, 'stop', yc)
    this.stopPanel.appendChild(this.stopButton.container)
    this.showStopButton(false)

    // Création outils de pointage
    this.creeOutilsPointage()
    /**
     * nombre d'objets de la figure au chargement
     * @type {number}
     */
    this.nbObj = this.listePr.longueur()
    /**
     * True si on est en mode bilan (à priori dans LaboMep)
     * @type {boolean}
     */
    this.modeBilan = Boolean(mtgOptions.resultatContenu) // True si on est en
    /// //////////////////////////////////////////////////////////////////////////////////////////////////
    // On regarde si c'est un exercice de construction et si on n'est pas en mode édition
    // Si oui this.listePourConst pointera sur une liste contenant les objets pour l'on peut utilier
    // quand l'utilisateur construit de nouveaux objets.
    // S'il s'agit d'un exercice de construction, this.estExercice est true et this.nbObjInit contient
    // le nombre initial d'objets
    /// //////////////////////////////////////////////////////////////////////////////////////////////////
    if (typeof datafig === 'string') {
      /**
       * ???
       * @type {*}
       */
      this.macroPourConst = this.getMacroPourConst()
      if (this.isExercise() && !this.edition) {
        /**
         * True si on est un exercice de construction (mais pas en édition)
         * @type {boolean}
         */
        this.estExercice = true
        // this.nbObjInit = mtgOptions.hasOwnProperty('resultatContenu') ? mtgOptions.resultatContenu.nbObjInit : this.listePr.longueur()
        // this.nbObjInit = mtgOptions.resultatContenu !== undefined ? mtgOptions.resultatContenu.nbObjInit : this.listePr.longueur()
        /**
         * Nb d'objets dans le résultat (passé par mtgOptions.resultatContenu) ou la figure
         * @type {number}
         */
        this.nbObjInit = (mtgOptions.resultatContenu && mtgOptions.resultatContenu.nbObjInit) || this.nbObj
        /**
         * ???
         * @type {CListeObjets}
         */
        this.listePourConst = this.listePourConstruction(this.nbObjInit)
        /**
         * Liste des index des objets à construire (dans la macro) ?
         * @type {number[]}
         */
        this.arrayObjAConst = this.arrayObjAConstruire()
        this.pref_PointsAuto = false
        this.pref_DispMeas = false
      } else {
        // pas un exo de construction (ou alors on l'édite)
        this.listePourConst = this.listePr
        this.estExercice = false
      }
    } else {
      // pas de figure passée
      this.macroPourConst = null
      this.listePourConst = this.listePr
      this.estExercice = false
    }
    if (mtgOptions.newFig !== undefined) {
      /**
       * Si true on affichera le bouton "nouvelle figure"
       * @type {boolean}
       */
      this.newFig = mtgOptions.newFig
    } else {
      this.newFig = this.electron || !this.estExercice
    }
    if (mtgOptions.options !== undefined) {
      /**
       * Si true on affichera le bouton options
       * @type {boolean}
       */
      this.options = mtgOptions.options
    } else {
      this.options = this.electron || !this.estExercice
    }
    if (this.functionOnSave) {
      // on attend un résultat, faut le bouton pour l'envoyer
      /**
       * Si true on affichera le bouton de sauvegarde
       * @type {boolean}
       */
      this.save = true
    } else {
      // Si on est en mode aperçu sous LaboMep et que ce n'est pas un exercice de construction, pas de bouton de sauvegarde
      if (!this.estExercice && this.preview) this.save = false
      // FIXME sinon save reste undefined ?
    }

    // Création des outils de création et manipulation d'objets
    this.creeOutils()
    // this.opacity = doc.opacity
    // On crée les barres de menu déroulables horizontales
    this.creeExpandableBars()
    //
    const li = new CListeObjets()
    li.ajoutePointsPunaises(doc.listePr)
    /**
     * ?
     * @type {CListeObjets}
     */
    this.listeARecalculer = new CListeObjets()
    /**
     * ?
     * @type {CListeObjets}
     */
    this.listeObjetsVisuels = new CListeObjets()
    this.listeObjetsVisuels.associeA(doc)
    /**
     * ?
     * @type {null|?}
     */
    this.elementCapture = null // Spécifique version mtgApp
    /**
     * Objets pour gérer le clignotement utilisé par les outils de création
     * @type {CListeObjets}
     */
    this.listeClignotante = new CListeObjets()
    /**
     * ?
     * @type {boolean}
     */
    this.clignotementPair = true
    /**
     * Point qui servira à suivre les mouvements de la souris
     * @type {CPointBase}
     */
    this.mousePoint = new CPointBase(this.listePr, null, false, Color.black, true, 0, 0, true, '', 13, MotifPoint.petitRond, false, true, 0, 0)
    //
    // Spécial JavaScript. Chaque document aura un membre pointCapture
    /**
     * Les objets qu'on ne peut pas désigner
     * @type {CListeObjets}
     */
    this.listeExclusion = li
    /**
     * ?
     * @type {InfoProx}
     */
    this.infoProx = new InfoProx()
    // Un affichage de texte pour afficher la nature des objets à désigner quand la souris bouge
    this.creeCommentaireDesignation()

    // Modifié version 6.3.0
    /*
    this.commentaireTip = new CCommentaire(li, null, false, Color.black, 0, 0, 0, 6, true, null,
      this.dys ? 16 : 14, StyleEncadrement.Effet3D,
      true, Color.yellow, CAffLiePt.alignHorRight, CAffLiePt.alignVerTop, "", new CValeurAngle(this.listePr, 0))
      */
    /**
     * Un affichage de texte pour afficher une sorte de tipTool quand la souris survole un bouton
     * @type {CCommentaire}
     */
    this.commentaireTip = new CCommentaire(this.listePr, null, false, Color.black, 0, 0, 0, 6, true, null,
      this.dys ? 16 : 14, StyleEncadrement.Effet3D,
      true, Color.yellow, CAffLiePt.alignHorRight, CAffLiePt.alignVerTop, '', new CValeurAngle(this.listePr, 0))

    // Contrairement à la version Java il faut afficher même vide
    this.commentaireTip.positionne()
    this.commentaireTip.affiche(this.svgPanel, false, doc.couleurFond, doc.opacity)
    //

    // On mémorise les gestionnaires de souris et de touch pour pouvoir éventuellement les détruire
    this.onmdown = function (evt) {
      if (self.hasBarExpanded()) {
        var eb = self.getExpandedBar()
        if (eb !== null) {
          var point = kernelAdd.mousePosition(svg, evt, self.zoomFactor)
          // On regarde si on a cliqué dans le figure et en dehors de la barre déroulée
          // Si oui on replie la barre
          // Ici nécesaire de mettre un setTimeOut car sinon si on clique sur un bouton le temps qu'il
          // réponde la barre sera déjà repliée
          if (point.x >= constantes.svgPanelWidth * zf && !((point.x <= eb.right) && (point.y >= eb.top) && (point.y <= eb.bottom))) {
            setTimeout(function () {
              self.unFoldExpandableBars()
            }, 500)
          }
        }
      }
      // On ne traite les événements souris que si une boîte de dialogue n'est pas ouverte
      if (self.dlg.length === 0 && self.doc.isActive) self.outilPointageActif.mousedown(evt)
    }
    this.onmmove = function (evt) {
      // On ne traite les événements souris que si une boîte de dialogue n'est pas ouverte
      self.doc.hasMouse = true
      if (self.dlg.length === 0 && self.doc.isActive) {
        self.outilPointageActif.mousemove(evt)
        document.body.style.cursor = self.outilActif.cursor
      }
    }
    this.onmup = function (evt) {
      // On ne traite les événements souris que si uen boîte de dialogue n'est pas ouverte
      if (self.dlg.length !== 0 || !self.doc.isActive) return
      self.releaseSliders() // Pour que on capture un slider à droite il soit relaché quand on cliuque en dehors
      self.outilPointageActif.mouseup(evt)
    }
    this.ontstart = function (evt) {
      if (self.hasBarExpanded()) {
        var eb = self.getExpandedBar()
        if (eb !== null) {
          var point = kernelAdd.touchPosition(svg, evt, self.zoomFactor)
          // On regarde si on a cliqué dans le figure et en dehors de la barre déroulée
          // Si oui on replie la barre
          // Ici nécesaire de mettre un setTimeOut car sinon si on clique sur un bouton le temps qu'il
          // réponde la barre sera déjà repliée
          if (point.x >= constantes.svgPanelWidth * zf && !((point.x <= eb.right) && (point.y >= eb.top) && (point.y <= eb.bottom))) {
            setTimeout(function () {
              self.unFoldExpandableBars()
            }, 500)
          }
        }
      }
      // On ne traite les événements souris que si une boîte de dialogue n'est pas ouverte
      if (self.dlg.length === 0 && self.doc.isActive) self.outilPointageActif.touchstart(evt)
    }
    this.ontmove = function (evt) {
      // On ne traite les événements souris que si uen boîte de dialogue n'est pas ouverte
      if (self.dlg.length !== 0 || !self.doc.isActive) return
      self.outilPointageActif.touchmove(evt)
      document.body.style.cursor = self.outilActif.cursor
      if (self.outilActif.isWorking()) {
        // Pour éviter que quand on capture un élément on fasse dérouler la figure
        kernel.preventDefault(evt)
        evt.stopPropagation()
      }
    }
    this.ontend = function (evt) {
      // On ne traite les événements souris que si uen boîte de dialogue n'est pas ouverte
      if (self.dlg.length === 0 && self.doc.isActive) self.outilPointageActif.touchend(evt)
    }
    this.ontcancel = function (evt) {
      // On ne traite les événements souris que si uen boîte de dialogue n'est pas ouverte
      if (self.dlg.length === 0 && self.doc.isActive) self.outilPointageActif.touchcancel(evt)
    }
    // svg = pergola.doc
    // C'est le rectangle inclus dans le svg parent qui répond aux événements souris
    svgRect.addEventListener('mousedown', this.onmdown, false)
    svgRect.addEventListener('mousemove', this.onmmove, false)
    svgRect.addEventListener('mouseup', this.onmup, false)
    svgRect.addEventListener('touchstart', this.ontstart, false)
    svgRect.addEventListener('touchmove', this.ontmove, false)
    svgRect.addEventListener('touchend', this.ontend, false)
    svgRect.addEventListener('touchcancel', this.ontcancel, false)

    this.gestionnaire.initialise()
    /**
     * Éditeur de formule pour entrer les noms de points et droites à la volée
     * @type {NameEditor}
     */
    this.nameEditor = new NameEditor(this)
    /**
     * macro d'animation de point lié qui servira pour les animations directes
     * @type {CMacroAnimationFigure}
     */
    this.macroAnimation = new CMacroAnimationFigure(this)

    this.prepareTracesEtImageFond()
    // Pour chaque variable associée à un affichage de valeur avec boutons plus et moins on crée un div
    this.listePr.creePaneVariables()
    // Au démarrage l'outil actif est l'outil de capture
    /**
     * Outil courant
     * @type {OutilCapt}
     */
    this.outilActif = this.outilCapt
    /**
     *
     * @type {OutilCapt}
     */
    this.outilActifPrec = this.outilCapt
    this.activeOutilDem()
    // On met à jour le slider d'opacité pour tenir compte de celle du document en cours
    this.updateOpacitySlider()

    // Sur un exercice de construction, en mode élève ou en mode aperçu,
    // on recalcule aléatoirement la figure
    let brandom = Boolean(mtgOptions.brandom)
    if (this.estExercice && (this.functionOnSave || this.preview)) brandom = true
    if (this.displayOnLoad) this.calculateAndDisplay(brandom)
    // La figure est maintenant prête pour être modifiée (utile pour les exercices de construction
    if (typeof mtgOptions.callBackAfterReady === 'function') {
      // tant pis pour l'appelant si sa callback plante et qu'il ne le gère pas, on continue quand même
      try {
        mtgOptions.callBackAfterReady(this)
      } catch (error) {
        console.error(error)
      }
    }
  }) // initTextes
} // MtgApp

/**
 * Fonction sélectionnant l'outil actif au démarrage : L'outil d'exécution de macro si une macro est visible
 * sinon l'outil de capture
 */
MtgApp.prototype.activeOutilDem = function () {
  if (this.listePr.nombreObjetsParNatureVisibles(NatObj.NMacro) > 0) {
    this.outilActif.deselect()
    this.outilActif = this.outilExecutionMacro
    this.outilActif.select()
  } else this.activeOutilCapt()
  var doc = this.doc
  if (doc.modeTraceActive) this.buttonModeTrace.activate(true)
  this.buttonModePointsAuto.activate(this.pref_PointsAuto)
}

MtgApp.prototype.prepareTracesEtImageFond = function () {
  var doc = this.doc

  this.svgFigure.appendChild(kernel.cens('rect', {
    id: 'coulFond',
    width: '100%',
    height: '100%',
    fill: doc.couleurFond.rgb()
  }))

  doc.gTraces = kernel.cens('g', {
    id: doc.idDoc + 'Traces'
  })
  this.svgFigure.appendChild(doc.gTraces)
  if (doc.imageFond !== null) {
    var g = kernel.cens('image', {
      x: '0',
      y: '0',
      width: doc.widthImageFond,
      height: doc.heightImageFond,
      'pointer-events': 'none' // Ajout version 6.3.4
    })
    g.setAttributeNS('http://www.w3.org/1999/xlink', 'href', 'data:image/png;base64,' + kernel.base64Encode(doc.imageFond, true))
    this.gImageFond = g
    this.svgFigure.insertBefore(g, doc.gTraces)
    if (!doc.imageFondVisible) $(g).css('visibility', 'hidden')
  }
}
/**
 * Fonction renvoyant un CmathGraphDoc d'id id et représenté par la chaîne Base64 base64
 * @param id
 * @param base64
 * @returns {CMathGraphDoc}
 */
MtgApp.prototype.loadDoc = function (id, base64) {
  var doc = new CMathGraphDoc(id)
  var ba = kernel.base64Decode(base64)
  var inps = new DataInputStream(ba)
  doc.read(inps)
  return doc
}

MtgApp.prototype.getBase64Code = function () {
  return this.doc.getBase64Code()
}

/**
 * Fonction chargeant dans this.levels[] les 4 documents servant à filtrer les outils suivant le niveau d'utilisation
 * demandé
 */
MtgApp.prototype.loadLevels = function () {
  this.levels = []
  for (var i = 0; i < 4; i++) {
    this.levels[i] = this.loadDoc('', levels[i])
  }
}

/**
 * Fonction donnant à this.doc une image de fond contenu dans file et appelant callBackOK en cas de succès
 * @param file
 * @param callBackOnOK
 */
MtgApp.prototype.setImageFond = function (file, callBackOnOK = null) {
  const self = this
  let imgType = file.name.split('.')
  imgType = imgType[imgType.length - 1].toLowerCase() // On utilise toLowerCase() pour éviter les extensions en majuscules
  const allowedTypes = ['png', 'jpg', 'jpeg', 'gif']
  const ind = allowedTypes.indexOf(imgType)
  if (ind !== -1) {
    const natImage = DataOutputStream.getNatImage(imgType)
    const reader = new FileReader() // eslint-disable-line no-undef
    reader.readAsDataURL(file)
    reader.addEventListener('load', function () {
      const database64 = reader.result
      const tab = database64.split(',')
      const image = kernel.ce('img', {
        src: database64
      })
      image.onload = function () {
        var doc = self.doc
        doc.widthImageFond = image.width
        doc.heightImageFond = image.height
        doc.natImageFond = natImage
        doc.imageFond = kernel.base64Decode(tab[tab.length - 1], true)
        var g = kernel.cens('image', {
          x: '0',
          y: '0',
          width: image.width,
          height: image.height
        })
        g.setAttributeNS('http://www.w3.org/1999/xlink', 'href', database64)
        self.svgFigure.insertBefore(g, doc.gTraces)
        self.gImageFond = g
        doc.imageFondVisible = true
        if (callBackOnOK !== null) callBackOnOK()
      }
    })
  } else new AvertDlg(this, 'FichierErr')
}

MtgApp.prototype.deleteImageFond = function () {
  this.svgFigure.removeChild(this.gImageFond)
  this.doc.natImageFond = DataOutputStream.nat_noImage
  this.doc.imageFond = null
}
/**
 * Fonction recalculant la figure et l'affichant.
 * @param {boolean} brandom  Si true, les calculs aléatoires par la fonction rand()
 * sont réactualisés.
 * @returns {undefined}
 */
MtgApp.prototype.calculateAndDisplay = function (brandom = false) {
  this.calculate(brandom)
  this.display()
}

MtgApp.prototype.calculate = function (brandom = false) {
  var j
  var doc = this.doc
  var liste = this.listePr
  var svg = this.svgFigure
  var dimf = this.dimf
  var macroDem = liste.macroDemarrage
  if (macroDem !== null) {
    // Ajout version 5.1
    macroDem.setMacroLanceuse(null)
    //
    if (macroDem.className === 'CMacroSuiteMacros') {
      for (j = 0; j < macroDem.listeAssociee.longueur(); j++) {
        macroDem.listeAssociee.get(j).execute(svg, dimf, doc.couleurFond, doc.opacity, false)
      }
    } else {
      // Version 4.9.7 : Ajout d'un dernier paramètre pour que la macro ne crée pas d'élements graphiques
      liste.macroDemarrage.execute(svg, dimf, doc.couleurFond, doc.opacity, false)
    }
  }
  liste.positionne(brandom, dimf)
}

MtgApp.prototype.display = function () {
  this.listePr.setReady4MathJax()
  // Modification version 6.4.1
  // MathJax.Hub.Queue([this.callBack, this.svgFigure, this.doc])
  const self = this
  MathJax.Hub.Queue(function () {
    self.callBack()
  })
}

/**
 * Fonction appelée lors d'un reclassement d'objets.
 * Elle détruit toutes les implémentations graphiques d'objets pour les reconstruire dans le nouvel ordre des objets.
 */
MtgApp.prototype.reCreateDisplay = function () {
  const list = this.listePr
  const svg = this.svgFigure
  this.removeSurfacePatterns()
  list.removegElements(svg)
  list.setReady4MathJax()
  const self = this
  /* Modifié version 6.4.1
  MathJax.Hub.Queue([this.callBack, svg, this.doc])
  */
  MathJax.Hub.Queue(function () {
    self.callBack()
  })
}
/**
 * Fonction de callback appelée lorsque MathJax a traité toues les formules
 * en LaTeX et que la figure est prête pour affichage.
 * @param {CMathGraphDoc} doc  le document à afficher.
 * @param {svg} svg  Le svg contenant la figure.
 * @param {Dimf} dimf  Les dimensions du svg contenant la figure.
 * @param {boolean} bDispOnLoad  true si on doit afficher la figure après calculs.
 * @returns {undefined}
 */
/* Modifé version 6.4.1
MtgApp.prototype.callBack = function (svg, doc) {
  var liste = doc.listePr
  liste.afficheTout(0, svg, true, doc.couleurFond, doc.opacity)
}
*/
MtgApp.prototype.callBack = function () {
  const doc = this.doc
  const svg = this.svgFigure
  const liste = doc.listePr
  // Dernier paramètre immediat à true rajouté pour version 6.6.4
  liste.afficheTout(0, svg, true, doc.couleurFond, doc.opacity, true)
}

MtgApp.prototype.creeCommentaireDesignation = function () {
  /** @type {CCommentaire} */
  this.commentaireDesignation = new CCommentaire(this.listePr, null, false, Color.red, 0, 0, 0, 6, true, null,
    this.dys ? 18 : 13, StyleEncadrement.Sans, false, Color.white, CAffLiePt.alignHorRight, CAffLiePt.alignVerTop, '',
    new CValeurAngle(this.listePr, 0))
  // Contrairement à la version Java il faut afficher même vide
  this.commentaireDesignation.positionne()
  this.commentaireDesignation.affiche(this.svgFigure, false, this.doc.couleurFond, this.doc.opacity)
}

MtgApp.prototype.actionClignotement = function () {
  if (this.outilActif.timer === null) return
  this.clignotementPair = !this.clignotementPair
  this.listeClignotante.montreTout(this.clignotementPair)
  this.listeClignotante.update(this.svgFigure, this.doc.couleurFond, this.doc.opacity, true)
}
/**
 * Fonction retirant tous
 * les éléments graphiques du svg de la figure.
 * @returns {undefined}
 */
MtgApp.prototype.retireTout = function () {
  // Modification version 6.3.5 : On met d'abord la variable hasgElement de tous les objets graphiques à false car
  // une fonction de callback peut encore être appelée demandant un update des éléments graphiques
  var list = this.listePr
  for (var i = 0; i < list.longueur(); i++) {
    var el = list.get(i)
    if (el.estDeNature(NatObj.NTtObj)) {
      el.hasgElement = false
    }
  }
  // On retire d'abord tous les enfants du svg
  var svg = this.svgFigure
  var self = this
  // Version 6.4.0 : Pas d'utilisationd e la pile MathJax car alors on retire tout y compris le svg de traces.
  /*
  MathJax.Hub.Queue(function () {
    while (svg.childNodes.length !== 0) svg.removeChild(svg.childNodes[0])
  })
  */
  while (svg.childNodes.length !== 0) svg.removeChild(svg.childNodes[0])
  self.removeSurfacePatterns()
}

MtgApp.prototype.cacheDesignation = function () {
  if (this.commentaireDesignation.masque) return
  this.commentaireDesignation.cache()
  $(this.commentaireDesignation.g).attr('visibility', 'hidden')
}

MtgApp.prototype.setDesignation = function (x, y, ch) {
  var com = this.commentaireDesignation
  var svg = this.svgFigure
  if (com.masque) com.montre()
  com.donneChaine(kernel.getStr(ch))
  com.placeNom(x, y)
  com.positionne()
  // Pour que le commentaire de désigantion soit bien visible on le met toujours en fin des objets du svg
  try {
    svg.removeChild(com.g)
  } catch (e) {
  }
  svg.appendChild(com.g)
  this.commentaireDesignation.update(svg)
}

/**
 * Fonction mettant dans this.commentaireTip le texte contenu dans src.tip
 * src peut être un bouton de la barre de gauche, du haut ou un composant de la barre d'outils de droite
 * Pour les composants de la barre d'outil de droite ils doivent posséder un this.y qui
 * indique la ligne d'affichage du tip.
 * @param src
 */
// Version 6.6.1 : Ajout d'un try-catch suite à rapport BusNag
MtgApp.prototype.setTip = function (src) {
  var x; var y
  var zf = this.zoomFactor
  try {
    if (this.commentaireTip.masque) this.commentaireTip.montre()
    this.commentaireTip.donneChaine(src.tip)
    var c = src.container
    var tf = c.getAttribute('transform')
    var a = tf.indexOf('(')
    var b = tf.indexOf(',')
    if (b === -1) b = tf.indexOf(' ')
    var nov = b === -1
    if (nov) b = tf.length
    var tx = parseInt(tf.substring(a + 1))
    var ty = nov ? 0 : parseInt(tf.substring(b + 1))
    this.commentaireTip.alignementHorizontal = CAffLiePt.alignHorLeft
    // Sous explorer on peut avoir un attribut du type translate() avec un seul argument
    switch (src.target) {
      case 'left' :
        x = tx + constantes.leftIconSize * zf + 5
        y = ty + constantes.leftIconSize * zf / 2
        break
      case 'top' :
        x = tx + constantes.topIconSize * zf
        y = ty + constantes.topIconSize * zf
        break
      case 'right' :
        this.commentaireTip.alignementHorizontal = CAffLiePt.alignHorRight
        x = parseInt(this.rightPanel.getAttribute('x')) - 10
        y = src.y + constantes.toolbarHeight * zf
        break
      case 'float' : // Outils de la palette d'outils supplémentaire
        this.commentaireTip.alignementHorizontal = CAffLiePt.alignHorRight
        var xg = parseInt($('#svgToolsAdd').attr('x'))
        x = xg + tx
        y = ty + constantes.topIconSize * zf
    }
    this.commentaireTip.placeNom(x, y)
    this.commentaireTip.positionne()
    this.commentaireTip.update(this.svgPanel)
    src.tipDisplayed = true
  } catch (e) {
  }
}

MtgApp.prototype.cacheTip = function (btn) {
  this.annuleTipsButtons()
  var c = this.commentaireTip
  if (arguments.length > 0) {
    btn.tipDisplayed = false
    if (c.chaineAffichage !== btn.tip) return
  }
  if (c.masque) return
  this.commentaireTip.cache()
  $(this.commentaireTip.g).attr('visibility', 'hidden')
}

MtgApp.prototype.getCouleur = function () {
  return this.couleurActive
}

MtgApp.prototype.getStylePoint = function () {
  return this.stylePoint
}

MtgApp.prototype.getTaillePoliceNom = function () {
  return this.pref_TaillePoliceNom
}

MtgApp.prototype.getStyleMarqueSegment = function () {
  return this.styleMarqueSegment
}

MtgApp.prototype.getStyleMarqueAngle = function () {
  return this.styleMarqueAngle
}

MtgApp.prototype.getStyleFleche = function () {
  return this.styleFleche
}

MtgApp.prototype.getStyleRemplissage = function () {
  return this.styleRemplissage
}

// A revoir après création de l'interface
MtgApp.prototype.getThickness = function () {
  return this.thickness
}

MtgApp.prototype.setThickness = function (val) {
  this.thickness = val
}

MtgApp.prototype.selectThickness = function (val) {
  this.thickness = val
  this.thicknessSlider.update(val)
}

MtgApp.prototype.setOpacity = function (val) {
  var v = val / 100
  // this.opacity = v; // Supprimé version 6.2.0
  this.doc.opacity = v
  this.listePr.setOpacity(v)
  this.colorChoicePanel.update()
}

MtgApp.prototype.getStyleTrait = function () {
  return new StyleTrait(this.listePr, this.lineStyle, this.getThickness())
}

MtgApp.prototype.selectButton = function (style, tabButtons) {
  for (var i = 0; i < tabButtons.length; i++) {
    var btn = tabButtons[i]
    var isActivated = btn.isActivated
    var isTarget = (btn.style === style)
    if (isTarget) {
      if (!isActivated) btn.activate(true)
    } else if (isActivated) btn.activate(false)
  }
}

/**
 *
 * Fonction sélectionnant dans la palette de style de trait le style style
 * @param {StyleTrait} style
 */
MtgApp.prototype.selectLineStyle = function (style) {
  this.lineStyle = style.style
  this.selectButton(style.style, this.lineStyleButtons)
}

MtgApp.prototype.selectFillStyle = function (style) {
  this.styleRemplissage = style
  this.selectButton(style, this.styleRemplissageButtons)
}

MtgApp.prototype.selectColor = function (color) {
  this.couleurActive = color
  this.colorChoicePanel.update()
}

MtgApp.prototype.supprimeObjetsVisuels = function () {
  if (this.listeObjetsVisuels.longueur() !== 0) {
    this.listeObjetsVisuels.montreTout(false)
    this.listeObjetsVisuels.update(this.svgFigure, this.doc.couleurFond, this.doc.opacity, true)
    this.listeObjetsVisuels.removegElements(this.svgFigure)
    this.listeObjetsVisuels.retireTout()
  }
}

MtgApp.prototype.updateObjetsVisuels = function () {
  if (this.listeObjetsVisuels.longueur() !== 0) {
    this.listeObjetsVisuels.positionne(false, this.dimf)
    this.listeObjetsVisuels.update(this.svgFigure, this.doc.couleurFond, this.doc.opacity, true)
  }
}

MtgApp.prototype.ajouteObjetVisuel = function (el) {
  el.positionne(false, this.dimf)
  // if (el.estDeNature(NatObj.NTtObj)) el.creeAffichage(this.svgFigure, true, this.doc.couleurFond, this.doc.opacity)
  this.listeObjetsVisuels.add(el)
}

// Ajout version 6.4.4
/**
 * Fonction ajoutant les éléments graphiques de la liste d'objets visuels depuis l'indice inddeb
 * @param {int} inddeb
 */
MtgApp.prototype.afficheObjetVisuels = function (inddeb) {
  const doc = this.doc
  this.listeObjetsVisuels.afficheTout(inddeb, this.svgFigure, true, doc.couleurFond, doc.opacity) // Ajout version 6.4.4
}

MtgApp.prototype.ajoutePoint = function (x, y) {
  var pt = new CPointBase(this.listePourConst, null, false, this.getCouleur(), false, 0, 0, false,
    '', this.getTaillePoliceNom(), this.getStylePoint(), false, true, x, y, false)
  // this.ajouteElementSansEditionNom(pt); // A revoir mtgApp
  pt.positionne(false, this.dimf)
  pt.creeAffichage(this.svgFigure, false, this.doc.couleurFond, this.doc.opacity)
  this.ajouteElement(pt)
  return pt
}

MtgApp.prototype.genereNomPourPoint = function () {
  return this.listePr.genereNomPourPoint()
}

MtgApp.prototype.ajouteElement = function (pt, bEditionNom) {
  var bEdit = (arguments.length === 2) ? bEditionNom : true
  pt.positionne(false, this.dimf)
  if (pt.getNatureCalcul() === NatCal.NVariable) {
    this.listePr.insert(1, pt)
    if (this.estExercice) this.listePourConst.insert(1, pt)
    // Important version 4.8.0 La variable étant rajoutée en début de liste
    // tous les indices doivent être recalculés
    this.listePr.updateIndexes()
  } else {
    this.listePr.add(pt)
    pt.id = this.listePr.id + '#' + pt.index
    if (this.estExercice) this.listePourConst.add(pt)
  }
  if (bEdit && pt.estDeNature(NatObj.NObjNommable) && (pt.className !== 'CDroiteClone') &&
    (pt.className !== 'CPointClone') && pt.existe && !pt.masque) {
    // Si on est en train d'éditer le nom d'un point ou si on est en mode exercice on n'édite pas le nom d'une droite
    var el = this.nameEditor.eltAssocie
    // if (pt.estDeNature(NatObj.NDroite) && ((el !== null) || this.estExercice)) return; // Modifié version 6.3.0
    if (pt.estDeNature(NatObj.NDroite) && (el !== null)) return // Modifié version 6.3.0
    this.nameEditor.associeA(pt)
    this.nameEditor.setPosition()
  }
  pt.metAJour() // A revoir mtgApp
}

/**
 * Fonction insérant dans la liste des objets créés l'objet elAdd à l'indice index
 * @param elAdd
 * @param index
 */
MtgApp.prototype.insereElement = function (elAdd, index) {
  this.listePr.insereElement(elAdd, index)
  if (this.estExercice) this.listePourConst = this.listePourConstruction()
  this.listePr.updateIndexes()
}

MtgApp.prototype.detruitDernierElement = function () {
  this.listePr.detruitDernierObjet()
  if (this.estExercice) this.listePourConst.detruitDernierObjet()
}

/**
 * Fonction détruisant les n derniers éléments de la liste principale
 * @param n Le nombre d'éléments à détruire
 */
MtgApp.prototype.detruitDerniersElements = function (n) {
  for (var i = 0; i < n; i++) {
    this.detruitDernierElement()
    if (this.estExercice) this.listePourConst.detruitDernierObjet()
  }
}

MtgApp.prototype.detruitElement = function (el) {
  this.listePr.detruitObjet(el)
  if (this.estExercice) this.listePourConst = this.listePourConstruction()
  this.listePr.updateIndexes()
}

/**
 * Fonction créant les outils de pointage
 * @constructor
 */
MtgApp.prototype.creeOutilsPointage = function () {
  /** @type {OutilPointageCapture} */
  this.outilPointageCapture = new OutilPointageCapture(this)
  /** @type {OutilPointageClic} */
  this.outilPointageClic = new OutilPointageClic(this)
  /** @type {OutilPointageCre} */
  this.outilPointageCre = new OutilPointageCre(this)
  /** @type {OutilPointageObjetClignotant} */
  this.outilPointageObjetClignotant = new OutilPointageObjetClignotant(this)
  /** @type {OutilPointageInt} */
  this.outilPointageInt = new OutilPointageInt(this)
  /** @type {OutilPointageRapporteur} */
  this.outilPointageRapporteur = new OutilPointageRapporteur(this)
  /** @type {OutilPointageCaptureNom} */
  this.outilPointageCaptureNom = new OutilPointageCaptureNom(this)
  /** @type {OutilPointageTranslation} */
  this.outilPointageTranslation = new OutilPointageTranslation(this)
  /** @type {OutilPointageExecMac} */
  this.outilPointageExecMac = new OutilPointageExecMac(this)
  /** @type {OutilPointageClicOuPt} */
  this.outilPointageClicOuPt = new OutilPointageClicOuPt(this)
}

MtgApp.prototype.creeOutils = function () {
  // On crée un tableau formé de tous les boutons associés à des outils
  this.buttons = [] // Chaque constructeur de bouton l'ajoutera au tableau
  /** @type {OutilCapt} */
  this.outilCapt = new OutilCapt(this)
  /** @type {ButtonTool} */
  this.buttonCapt = new ButtonTool(this, 'Capt', 'left', true)
  /** @type {OutilPtLib} */
  this.outilPtLib = new OutilPtLib(this)
  /** @type {ButtonTool} */
  this.buttonPtLib = new ButtonTool(this, 'PtLib', 'left')
  /** @type {OutilMilieu} */
  this.outilMilieu = new OutilMilieu(this)
  /** @type {ButtonTool} */
  this.buttonMilieu = new ButtonTool(this, 'Milieu', 'left')
  /** @type {OutilCentreGrav} */
  this.outilCentreGrav = new OutilCentreGrav(this)
  /** @type {ButtonTool} */
  this.buttonCentreGrav = new ButtonTool(this, 'CentreGrav', 'left')
  /** @type {OutilMarquerPt} */
  this.outilMarquerPt = new OutilMarquerPt(this)
  /** @type {ButtonTool} */
  this.buttonMarquerPt = new ButtonTool(this, 'MarquerPt', 'left')
  /** @type {OutilDemarquerPt} */
  this.outilDemarquerPt = new OutilDemarquerPt(this)
  /** @type {ButtonTool} */
  this.buttonDemarquerPt = new ButtonTool(this, 'DemarquerPt', 'left')
  /** @type {OutilPunaiser} */
  this.outilPunaiser = new OutilPunaiser(this)
  /** @type {ButtonTool} */
  this.buttonPunaiser = new ButtonTool(this, 'Punaiser', 'left')
  /** @type {OutilDepunaiser} */
  this.outilDepunaiser = new OutilDepunaiser(this)
  /** @type {ButtonTool} */
  this.buttonDepunaiser = new ButtonTool(this, 'Depunaiser', 'left')
  /** @type {OutilDtAB} */
  this.outilDtAB = new OutilDtAB(this)
  /** @type {ButtonTool} */
  this.buttonDtAB = new ButtonTool(this, 'DtAB', 'left')
  /** @type {OutilDtPar} */
  this.outilDtPar = new OutilDtPar(this)
  /** @type {ButtonTool} */
  this.buttonDtPar = new ButtonTool(this, 'DtPar', 'left')
  /** @type {OutilDtPer} */
  this.outilDtPer = new OutilDtPer(this)
  /** @type {ButtonTool} */
  this.buttonDtPer = new ButtonTool(this, 'DtPer', 'left')
  /** @type {OutilDtBis} */
  this.outilDtBis = new OutilDtBis(this)
  /** @type {ButtonTool} */
  this.buttonDtBis = new ButtonTool(this, 'DtBis', 'left')
  /** @type {OutilDtMed} */
  this.outilDtMed = new OutilDtMed(this)
  /** @type {ButtonTool} */
  this.buttonDtMed = new ButtonTool(this, 'DtMed', 'left')
  /** @type {OutilDtHor} */
  this.outilDtHor = new OutilDtHor(this)
  /** @type {ButtonTool} */
  this.buttonDtHor = new ButtonTool(this, 'DtHor', 'left')
  /** @type {OutilDtVer} */
  this.outilDtVer = new OutilDtVer(this)
  /** @type {ButtonTool} */
  this.buttonDtVer = new ButtonTool(this, 'DtVer', 'left')
  /** @type {OutilDtParCoef} */
  this.outilDtParCoef = new OutilDtParCoef(this)
  /** @type {ButtonTool} */
  this.buttonDtParCoef = new ButtonTool(this, 'DtParCoef', 'left')
  /** @type {OutilDtParEq} */
  this.outilDtParEq = new OutilDtParEq(this)
  /** @type {ButtonTool} */
  this.buttonDtParEq = new ButtonTool(this, 'DtParEq', 'left')
  /** @type {OutilDemiDt} */
  this.outilDemiDt = new OutilDemiDt(this)
  /** @type {ButtonTool} */
  this.buttonDemiDt = new ButtonTool(this, 'DemiDt', 'left')
  /** @type {OutilSeg} */
  this.outilSeg = new OutilSeg(this)
  /** @type {ButtonTool} */
  this.buttonSeg = new ButtonTool(this, 'Seg', 'left')
  /** @type {OutilSegmentParLong} */
  this.outilSegmentParLong = new OutilSegmentParLong(this)
  /** @type {ButtonTool} */
  this.buttonSegmentParLong = new ButtonTool(this, 'SegmentParLong', 'left')
  /** @type {OutilVect} */
  this.outilVect = new OutilVect(this)
  /** @type {ButtonTool} */
  this.buttonVect = new ButtonTool(this, 'Vect', 'left')
  /** @type {OutilCerOA} */
  this.outilCerOA = new OutilCerOA(this)
  /** @type {ButtonTool} */
  this.buttonCerOA = new ButtonTool(this, 'CerOA', 'left')
  /** @type {OutilCerOR} */
  this.outilCerOR = new OutilCerOR(this)
  /** @type {ButtonTool} */
  this.buttonCerOR = new ButtonTool(this, 'CerOR', 'left')
  /** @type {OutilCerOAB} */
  this.outilCerOAB = new OutilCerOAB(this)
  /** @type {ButtonTool} */
  this.buttonCerOAB = new ButtonTool(this, 'CerOAB', 'left')
  /** @type {OutilArcPetit} */
  this.outilArcPetit = new OutilArcPetit(this)
  /** @type {ButtonTool} */
  this.buttonArcPetit = new ButtonTool(this, 'ArcPetit', 'left')
  /** @type {OutilArcGrand} */
  this.outilArcGrand = new OutilArcGrand(this)
  /** @type {ButtonTool} */
  this.buttonArcGrand = new ButtonTool(this, 'ArcGrand', 'left')
  /** @type {OutilArcDirect} */
  this.outilArcDirect = new OutilArcDirect(this)
  /** @type {ButtonTool} */
  this.buttonArcDirect = new ButtonTool(this, 'ArcDirect', 'left')
  /** @type {OutilArcIndirect} */
  this.outilArcIndirect = new OutilArcIndirect(this)
  /** @type {ButtonTool} */
  this.buttonArcIndirect = new ButtonTool(this, 'ArcIndirect', 'left')
  /** @type {OutilArcPetitParAng} */
  this.outilArcPetitParAng = new OutilArcPetitParAng(this)
  /** @type {ButtonTool} */
  this.buttonArcPetitParAng = new ButtonTool(this, 'ArcPetitParAng', 'left')
  /** @type {OutilArcGrandParAng} */
  this.outilArcGrandParAng = new OutilArcGrandParAng(this)
  /** @type {ButtonTool} */
  this.buttonArcGrandParAng = new ButtonTool(this, 'ArcGrandParAng', 'left')
  /** @type {OutilArcDirectParAng} */
  this.outilArcDirectParAng = new OutilArcDirectParAng(this)
  /** @type {ButtonTool} */
  this.buttonArcDirectParAng = new ButtonTool(this, 'ArcDirectParAng', 'left')
  /** @type {OutilArcIndirectParAng} */
  this.outilArcIndirectParAng = new OutilArcIndirectParAng(this)
  /** @type {ButtonTool} */
  this.buttonArcIndirectParAng = new ButtonTool(this, 'ArcIndirectParAng', 'left')
  /** @type {OutilMarqueSeg} */
  this.outilMarqueSeg = new OutilMarqueSeg(this)
  /** @type {ButtonTool} */
  this.buttonMarqueSeg = new ButtonTool(this, 'MarqueSeg', 'left')
  /** @type {OutilMarqueAng} */
  this.outilMarqueAng = new OutilMarqueAng(this)
  /** @type {ButtonTool} */
  this.buttonMarqueAng = new ButtonTool(this, 'MarqueAng', 'left')
  /** @type {OutilMarqueAngOr} */
  this.outilMarqueAngOr = new OutilMarqueAngOr(this)
  /** @type {ButtonTool} */
  this.buttonMarqueAngOr = new ButtonTool(this, 'MarqueAngOr', 'left')
  /** @type {OutilMarqueAngOrSD} */
  this.outilMarqueAngOrSD = new OutilMarqueAngOrSD(this)
  /** @type {ButtonTool} */
  this.buttonMarqueAngOrSD = new ButtonTool(this, 'MarqueAngOrSD', 'left')
  /** @type {OutilMarqueAngOrSI} */
  this.outilMarqueAngOrSI = new OutilMarqueAngOrSI(this)
  /** @type {ButtonTool} */
  this.buttonMarqueAngOrSI = new ButtonTool(this, 'MarqueAngOrSI', 'left')
  /** @type {OutilCarre} */
  this.outilCarre = new OutilCarre(this)
  /** @type {ButtonTool} */
  this.buttonCarre = new ButtonTool(this, 'Carre', 'left')
  /** @type {OutilTriangleEq} */
  this.outilTriangleEq = new OutilTriangleEq(this)
  /** @type {ButtonTool} */
  this.buttonTriangleEq = new ButtonTool(this, 'TriangleEq', 'left')
  /** @type {OutilParallelog} */
  this.outilParallelog = new OutilParallelog(this)
  /** @type {ButtonTool} */
  this.buttonParallelog = new ButtonTool(this, 'Parallelog', 'left')
  /** @type {OutilPolygone} */
  this.outilPolygone = new OutilPolygone(this)
  /** @type {ButtonTool} */
  this.buttonPolygone = new ButtonTool(this, 'Polygone', 'left')
  /** @type {OutilPolygoneReg} */
  this.outilPolygoneReg = new OutilPolygoneReg(this)
  /** @type {ButtonTool} */
  this.buttonPolygoneReg = new ButtonTool(this, 'PolygoneReg', 'left')
  /** @type {OutilLigneBrisee} */
  this.outilLigneBrisee = new OutilLigneBrisee(this)
  /** @type {ButtonTool} */
  this.buttonLigneBrisee = new ButtonTool(this, 'LigneBrisee', 'left')
  /** @type {OutilRot} */
  this.outilRot = new OutilRot(this)
  /** @type {ButtonTool} */
  this.buttonRot = new ButtonTool(this, 'Rot', 'left')
  /** @type {OutilRapporteur} */
  this.outilRapporteur = new OutilRapporteur(this)
  /** @type {ButtonTool} */
  this.buttonRapporteur = new ButtonTool(this, 'Rapporteur', 'left')
  /** @type {OutilArcParRapporteur} */
  this.outilArcParRapporteur = new OutilArcParRapporteur(this)
  /** @type {ButtonTool} */
  this.buttonArcParRapporteur = new ButtonTool(this, 'ArcParRapporteur', 'left')
  /** @type {OutilArcGrandParRapporteur} */
  this.outilArcGrandParRapporteur = new OutilArcGrandParRapporteur(this)
  /** @type {ButtonTool} */
  this.buttonArcGrandParRapporteur = new ButtonTool(this, 'ArcGrandParRapporteur', 'left')
  /** @type {OutilSymAxiale} */
  this.outilSymAxiale = new OutilSymAxiale(this)
  /** @type {ButtonTool} */
  this.buttonSymAxiale = new ButtonTool(this, 'SymAxiale', 'left')
  /** @type {OutilSymCentrale} */
  this.outilSymCentrale = new OutilSymCentrale(this)
  /** @type {ButtonTool} */
  this.buttonSymCentrale = new ButtonTool(this, 'SymCentrale', 'left')
  /** @type {OutilTrans} */
  this.outilTrans = new OutilTrans(this)
  /** @type {ButtonTool} */
  this.buttonTrans = new ButtonTool(this, 'Trans', 'left')
  /** @type {OutilTransParCoord} */
  this.outilTransParCoord = new OutilTransParCoord(this)
  /** @type {ButtonTool} */
  this.buttonTransParCoord = new ButtonTool(this, 'TransParCoord', 'left')
  /** @type {OutilHom} */
  this.outilHom = new OutilHom(this)
  /** @type {ButtonTool} */
  this.buttonHom = new ButtonTool(this, 'Hom', 'left')
  /** @type {OutilSim} */
  this.outilSim = new OutilSim(this)
  /** @type {ButtonTool} */
  this.buttonSim = new ButtonTool(this, 'Sim', 'left')
  /** @type {OutilCalcul} */
  this.outilCalcul = new OutilCalcul(this)
  /** @type {ButtonTool} */
  this.buttonCalcul = new ButtonTool(this, 'Calcul', 'left')
  /** @type {OutilCurseur} */
  this.outilCurseur = new OutilCurseur(this)
  /** @type {ButtonTool} */
  this.buttonCurseur = new ButtonTool(this, 'Curseur', 'left')
  /** @type {OutilTangente} */
  this.outilTangente = new OutilTangente(this)
  /** @type {ButtonTool} */
  this.buttonTangente = new ButtonTool(this, 'Tangente', 'left')

  /** @type {OutilVariable} */
  this.outilVariable = new OutilVariable(this)
  /** @type {ButtonTool} */
  this.buttonVariable = new ButtonTool(this, 'Variable', 'left')
  /** @type {OutilSolutionEq} */
  this.outilSolutionEq = new OutilSolutionEq(this)
  /** @type {ButtonTool} */
  this.buttonSolutionEq = new ButtonTool(this, 'SolutionEq', 'left')
  /** @type {OutilDerivee} */
  this.outilDerivee = new OutilDerivee(this)
  /** @type {ButtonTool} */
  this.buttonDerivee = new ButtonTool(this, 'Derivee', 'left')
  /** @type {OutilSuiteRec} */
  this.outilSuiteRec = new OutilSuiteRec(this)
  /** @type {ButtonTool} */
  this.buttonSuiteRec = new ButtonTool(this, 'SuiteRec', 'left')
  /** @type {OutilSuiteRec} */
  this.outilSuiteRec2 = new OutilSuiteRec2(this)
  /** @type {OutilSuiteRec} */
  this.outilSuiteRec3 = new OutilSuiteRec3(this)
  /** @type {OutilSuiteRecComplexe} */
  this.outilSuiteRecComplexe = new OutilSuiteRecComplexe(this)
  /** @type {ButtonTool} */
  this.buttonSuiteRecComplexe = new ButtonTool(this, 'SuiteRecComplexe', 'left')
  /** @type {OutilSuiteRecComplexe} */
  this.outilSuiteRecComplexe2 = new OutilSuiteRecComplexe2(this)
  /** @type {OutilSuiteRecComplexe} */
  this.outilSuiteRecComplexe3 = new OutilSuiteRecComplexe3(this)

  /** @type {OutilMax} */
  this.outilMax = new OutilMax(this)
  /** @type {ButtonTool} */
  this.buttonMax = new ButtonTool(this, 'Max', 'left')
  /** @type {OutilMin} */
  this.outilMin = new OutilMin(this)
  /** @type {ButtonTool} */
  this.buttonMin = new ButtonTool(this, 'Min', 'left')
  /** @type {OutilRepere} */
  this.outilRepere = new OutilRepere(this)
  /** @type {ButtonTool} */
  this.buttonRepere = new ButtonTool(this, 'Repere', 'left')
  /** @type {OutilCalculComp} */
  this.outilCalculComp = new OutilCalculComp(this)
  /** @type {ButtonTool} */
  this.buttonCalculComp = new ButtonTool(this, 'CalculComp', 'left')
  /** @type {OutilFonc} */
  this.outilFonc = new OutilFonc(this)
  /** @type {ButtonTool} */
  this.buttonFonc = new ButtonTool(this, 'Fonc', 'left')
  /** @type {OutilFoncComp} */
  this.outilFoncComp = new OutilFoncComp(this)
  /** @type {ButtonTool} */
  this.buttonFoncComp = new ButtonTool(this, 'FoncComp', 'left')
  /** @type {OutilSurface} */
  this.outilSurface = new OutilSurface(this)
  /** @type {ButtonTool} */
  this.buttonSurface = new ButtonTool(this, 'Surface', 'left')
  /** @type {OutilSurfaceLieuDroite} */
  this.outilSurfaceLieuDroite = new OutilSurfaceLieuDroite(this)
  /** @type {ButtonTool} */
  this.buttonSurfaceLieuDroite = new ButtonTool(this, 'SurfaceLieuDroite', 'left')
  /** @type {OutilSurface} */
  this.outilSurface2Lieux = new OutilSurface2Lieux(this)
  /** @type {ButtonTool} */
  this.buttonSurface2Lieux = new ButtonTool(this, 'Surface2Lieux', 'left')
  /** @type {OutilSurfaceLieu2Pts} */
  this.outilSurfaceLieu2Pts = new OutilSurfaceLieu2Pts(this)
  /** @type {ButtonTool} */
  this.buttonSurfaceLieu2Pts = new ButtonTool(this, 'SurfaceLieu2Pts', 'left')
  /** @type {OutilDemiPlan} */
  this.outilDemiPlan = new OutilDemiPlan(this)
  /** @type {OutilCouronne} */
  this.outilCouronne = new OutilCouronne(this)
  /** @type {ButtonTool} */
  this.buttonCouronne = new ButtonTool(this, 'Couronne', 'left')
  /** @type {ButtonTool} */
  this.buttonDemiPlan = new ButtonTool(this, 'DemiPlan', 'left')
  /** @type {OutilLieuParPtLie} */
  this.outilLieuParPtLie = new OutilLieuParPtLie(this)
  /** @type {ButtonTool} */
  this.buttonLieuParPtLie = new ButtonTool(this, 'LieuParPtLie', 'left')
  /** @type {OutilLieuDiscretParPtLie} */
  this.outilLieuDiscretParPtLie = new OutilLieuDiscretParPtLie(this)
  /** @type {ButtonTool} */
  this.buttonLieuDiscretParPtLie = new ButtonTool(this, 'LieuDiscretParPtLie', 'left')
  /** @type {OutilLieuParVariable} */
  this.outilLieuParVariable = new OutilLieuParVariable(this)
  /** @type {ButtonTool} */
  this.buttonLieuParVariable = new ButtonTool(this, 'LieuParVariable', 'left')
  /** @type {OutilLieuDiscretParVariable} */
  this.outilLieuDiscretParVariable = new OutilLieuDiscretParVariable(this)
  /** @type {ButtonTool} */
  this.buttonLieuDiscretParVariable = new ButtonTool(this, 'LieuDiscretParVariable', 'left')
  /** @type {OutilLieuObjetParPtLie} */
  this.outilLieuObjetParPtLie = new OutilLieuObjetParPtLie(this)
  /** @type {ButtonTool} */
  this.buttonLieuObjetParPtLie = new ButtonTool(this, 'LieuObjetParPtLie', 'left')
  /** @type {OutilLieuObjetParVariable} */
  this.outilLieuObjetParVariable = new OutilLieuObjetParVariable(this)
  /** @type {ButtonTool} */
  this.buttonLieuObjetParVariable = new ButtonTool(this, 'LieuObjetParVariable', 'left')
  /** @type {OutilCourbeFonc} */
  this.outilCourbeFonc = new OutilCourbeFonc(this)
  /** @type {ButtonTool} */
  this.buttonCourbeFonc = new ButtonTool(this, 'CourbeFonc', 'left')
  /** @type {OutilCourbeFoncCr} */
  this.outilCourbeFoncCr = new OutilCourbeFoncCr(this)
  /** @type {ButtonTool} */
  this.buttonCourbeFoncCr = new ButtonTool(this, 'CourbeFoncCr', 'left')
  /** @type {OutilCourbeAvecTan} */
  this.outilCourbeAvecTan = new OutilCourbeAvecTan(this)
  /** @type {ButtonTool} */
  this.buttonCourbeAvecTan = new ButtonTool(this, 'CourbeAvecTan', 'left')
  /** @type {OutilCourbePoly} */
  this.outilCourbePoly = new OutilCourbePoly(this)
  /** @type {ButtonTool} */
  this.buttonCourbePoly = new ButtonTool(this, 'CourbePoly', 'left')
  /** @type {OutilGrapheSuiteRec} */
  this.outilGrapheSuiteRec = new OutilGrapheSuiteRec(this)
  /** @type {ButtonTool} */
  this.buttonGrapheSuiteRec = new ButtonTool(this, 'GrapheSuiteRec', 'left')
  /** @type {OutilGrapheSuiteRecComp} */
  this.outilGrapheSuiteRecComp = new OutilGrapheSuiteRecComp(this)
  /** @type {ButtonTool} */
  this.buttonGrapheSuiteRecComp = new ButtonTool(this, 'GrapheSuiteRecComp', 'left')

  /** @type {OutilInt} */
  this.outilInt = new OutilInt(this)
  /** @type {ButtonTool} */
  this.buttonInt = new ButtonTool(this, 'Int', 'left')
  /** @type {OutilProj} */
  this.outilProj = new OutilProj(this)
  /** @type {ButtonTool} */
  this.buttonProj = new ButtonTool(this, 'Proj', 'left')
  /** @type {OutilPtParCoord} */
  this.outilPtParCoord = new OutilPtParCoord(this)
  /** @type {ButtonTool} */
  this.buttonPtParCoord = new ButtonTool(this, 'PtParCoord', 'left')
  /** @type {OutilPtParAff} */
  this.outilPtParAff = new OutilPtParAff(this)
  /** @type {ButtonTool} */
  this.buttonPtParAff = new ButtonTool(this, 'PtParAff', 'left')
  /** @type {OutilPtLie} */
  this.outilPtLie = new OutilPtLie(this)
  /** @type {ButtonTool} */
  this.buttonPtLie = new ButtonTool(this, 'PtLie', 'left')
  /** @type {OutilPtBaseEnt} */
  this.outilPtBaseEnt = new OutilPtBaseEnt(this)
  /** @type {ButtonTool} */
  this.buttonPtBaseEnt = new ButtonTool(this, 'PtBaseEnt', 'left')
  /** @type {OutilPtInterieur} */
  this.outilPtInterieur = new OutilPtInterieur(this)
  /** @type {ButtonTool} */
  this.buttonPtInterieur = new ButtonTool(this, 'PtInterieur', 'left')
  /** @type {OutilPtParAbs} */
  this.outilPtParAbs = new OutilPtParAbs(this)
  /** @type {ButtonTool} */
  this.buttonPtParAbs = new ButtonTool(this, 'PtParAbs', 'left')
  /** @type {OutilBarycentre} */
  this.outilBarycentre = new OutilBarycentre(this)
  /** @type {ButtonTool} */
  this.buttonBarycentre = new ButtonTool(this, 'Barycentre', 'left')
  /** @type {OutilPtParMultVec} */
  this.outilPtParMultVec = new OutilPtParMultVec(this)
  /** @type {ButtonTool} */
  this.buttonPtParMultVec = new ButtonTool(this, 'PtParMultVec', 'left')
  /** @type {OutilPtParSommeVec} */
  this.outilPtParSommeVec = new OutilPtParSommeVec(this)
  /** @type {ButtonTool} */
  this.buttonPtParSommeVec = new ButtonTool(this, 'PtParSommeVec', 'left')
  /** @type {OutilMesLong} */
  this.outilMesLong = new OutilMesLong(this)
  /** @type {ButtonTool} */
  this.buttonMesLong = new ButtonTool(this, 'MesLong', 'left')
  /** @type {OutilMesLongOr} */
  this.outilMesLongOr = new OutilMesLongOr(this)
  /** @type {ButtonTool} */
  this.buttonMesLongOr = new ButtonTool(this, 'MesLongOr', 'left')
  /** @type {OutilMesAngNor} */
  this.outilMesAngNor = new OutilMesAngNor(this)
  /** @type {ButtonTool} */
  this.buttonMesAngNor = new ButtonTool(this, 'MesAngNor', 'left')
  /** @type {OutilMesAngOr} */
  this.outilMesAngOr = new OutilMesAngOr(this)
  /** @type {ButtonTool} */
  this.buttonMesAngOr = new ButtonTool(this, 'MesAngOr', 'left')
  /** @type {OutilMesAbs} */
  this.outilMesAbs = new OutilMesAbs(this)
  /** @type {ButtonTool} */
  this.buttonMesAbs = new ButtonTool(this, 'MesAbs', 'left')
  /** @type {OutilMesAbsRep} */
  this.outilMesAbsRep = new OutilMesAbsRep(this)
  /** @type {ButtonTool} */
  this.buttonMesAbsRep = new ButtonTool(this, 'MesAbsRep', 'left')
  /** @type {OutilMesOrdRep} */
  this.outilMesOrdRep = new OutilMesOrdRep(this)
  /** @type {ButtonTool} */
  this.buttonMesOrdRep = new ButtonTool(this, 'MesOrdRep', 'left')
  /** @type {OutilMesAffRep} */
  this.outilMesAffRep = new OutilMesAffRep(this)
  /** @type {ButtonTool} */
  this.buttonMesAffRep = new ButtonTool(this, 'MesAffRep', 'left')
  /** @type {OutilMesCoefDir} */
  this.outilMesCoefDir = new OutilMesCoefDir(this)
  /** @type {ButtonTool} */
  this.buttonMesCoefDir = new ButtonTool(this, 'MesCoefDir', 'left')
  /** @type {OutilMesLongLigne} */
  this.outilMesLongLigne = new OutilMesLongLigne(this)
  /** @type {ButtonTool} */
  this.buttonMesLongLigne = new ButtonTool(this, 'MesLongLigne', 'left')
  /** @type {OutilMesAire} */
  this.outilMesAire = new OutilMesAire(this)
  /** @type {ButtonTool} */
  this.buttonMesAire = new ButtonTool(this, 'MesAire', 'left')
  /** @type {OutilMesProSca} */
  this.outilMesProSca = new OutilMesProSca(this)
  /** @type {ButtonTool} */
  this.buttonMesProSca = new ButtonTool(this, 'MesProSca', 'left')
  /** @type {OutilAffichageValeur} */
  this.outilAffichageValeur = new OutilAffichageValeur(this)
  /** @type {ButtonTool} */
  this.buttonAffichageValeur = new ButtonTool(this, 'AffichageValeur', 'left')
  /** @type {OutilAffichageEq} */
  this.outilAffichageEq = new OutilAffichageEq(this)
  /** @type {ButtonTool} */
  this.buttonAffichageEq = new ButtonTool(this, 'AffichageEq', 'left')
  /** @type {OutilAffichageEqLie} */
  this.outilAffichageEqLie = new OutilAffichageEqLie(this)
  /** @type {ButtonTool} */
  this.buttonAffichageEqLie = new ButtonTool(this, 'AffichageEqLie', 'left')
  /** @type {OutilAffichageCoord} */
  this.outilAffichageCoord = new OutilAffichageCoord(this)
  /** @type {ButtonTool} */
  this.buttonAffichageCoord = new ButtonTool(this, 'AffichageCoord', 'left')
  /** @type {OutilAffichageCoordLie} */
  this.outilAffichageCoordLie = new OutilAffichageCoordLie(this)
  /** @type {ButtonTool} */
  this.buttonAffichageCoordLie = new ButtonTool(this, 'AffichageCoordLie', 'left')
  /** @type {OutilAffichageValeurLiePt} */
  this.outilAffichageValeurLiePt = new OutilAffichageValeurLiePt(this)
  /** @type {ButtonTool} */
  this.buttonAffichageValeurLiePt = new ButtonTool(this, 'AffichageValeurLiePt', 'left')
  /** @type {OutilLatex} */
  this.outilLatex = new OutilLatex(this)
  /** @type {ButtonTool} */
  this.buttonLatex = new ButtonTool(this, 'Latex', 'left')
  /** @type {OutilLatexLiePt} */
  this.outilLatexLiePt = new OutilLatexLiePt(this)
  /** @type {ButtonTool} */
  this.buttonLatexLiePt = new ButtonTool(this, 'LatexLiePt', 'left')
  /** @type {OutilCommentaire} */
  this.outilCommentaire = new OutilCommentaire(this)
  /** @type {ButtonTool} */
  this.buttonCommentaire = new ButtonTool(this, 'Commentaire', 'left')
  /** @type {OutilCommentaireLiePt} */
  this.outilCommentaireLiePt = new OutilCommentaireLiePt(this)
  /** @type {ButtonTool} */
  this.buttonCommentaireLiePt = new ButtonTool(this, 'CommentaireLiePt', 'left')
  /** @type {OutilImageLibre} */
  this.outilImageLibre = new OutilImageLibre(this)
  /** @type {ButtonTool} */
  this.buttonImageLibre = new ButtonTool(this, 'ImageLibre', 'left')
  /** @type {OutilImageLiee} */
  this.outilImageLiee = new OutilImageLiee(this)
  /** @type {ButtonTool} */
  this.buttonImageLiee = new ButtonTool(this, 'ImageLiee', 'left')
  /** @type {OutilEditeurFormule} */
  this.outilEditeurFormule = new OutilEditeurFormule(this)
  /** @type {ButtonTool} */
  this.buttonEditeurFormule = new ButtonTool(this, 'EditeurFormule', 'left')
  /** @type {OutilCreationLiaison} */
  this.outilCreationLiaison = new OutilCreationLiaison(this)
  /** @type {ButtonTool} */
  this.buttonCreationLiaison = new ButtonTool(this, 'CreationLiaison', 'left')
  /** @type {OutilCreationLiaisonAff} */
  this.outilCreationLiaisonAff = new OutilCreationLiaisonAff(this)
  /** @type {ButtonTool} */
  this.buttonCreationLiaisonAff = new ButtonTool(this, 'CreationLiaisonAff', 'left')
  /** @type {OutilSuppressionLiaison} */
  this.outilSuppressionLiaison = new OutilSuppressionLiaison(this)
  /** @type {ButtonTool} */
  this.buttonSuppressionLiaison = new ButtonTool(this, 'SuppressionLiaison', 'left')
  /** @type {OutilSuppressionLiaisonAff} */
  this.outilSuppressionLiaisonAff = new OutilSuppressionLiaisonAff(this)
  /** @type {ButtonTool} */
  this.buttonSuppressionLiaisonAff = new ButtonTool(this, 'SuppressionLiaisonAff', 'left')
  /** @type {OutilAnimation} */
  this.outilAnimation = new OutilAnimation(this)
  /** @type {ButtonTool} */
  this.buttonAnimation = new ButtonTool(this, 'Animation', 'left')
  /** @type {OutilObjetDuplique} */
  this.outilObjetDuplique = new OutilObjetDuplique(this)
  /** @type {OutilObjetClone} */
  this.outilObjetClone = new OutilObjetClone(this)
  /** @type {OutilAbsOrRep} */
  this.outilAbsOrRep = new OutilAbsOrRep(this)
  /** @type {OutilOrdOrRep} */
  this.outilOrdOrRep = new OutilOrdOrRep(this)
  /** @type {OutilUnitexRep} */
  this.outilUnitexRep = new OutilUnitexRep(this)
  /** @type {OutilUniteyRep} */
  this.outilUniteyRep = new OutilUniteyRep(this)

  /// //////////////////////////////////////////////////////////////////////////////////////////////////////////
  //                                      Outils de la barre horizontale                                     //
  /// //////////////////////////////////////////////////////////////////////////////////////////////////////////
  /** @type {OutilNew} */
  this.outilNew = new OutilNew(this)
  /** @type {OutilOpen} */
  this.outilOpen = new OutilOpen(this)
  /** @type {OutilSave} */
  this.outilSave = new OutilSave(this)
  /** @type {OutilAnnuler} */
  this.outilAnnuler = new OutilAnnuler(this)
  /** @type {OutilRefaire} */
  this.outilRefaire = new OutilRefaire(this)
  /** @type {OutilSup} */
  this.outilSup = new OutilSup(this)
  /** @type {OutilModifObjGraph} */
  this.outilModifObjGraph = new OutilModifObjGraph(this)
  /** @type {OutilZoomPlus} */
  this.outilZoomPlus = new OutilZoomPlus(this)
  /** @type {OutilZoomMoins} */
  this.outilZoomMoins = new OutilZoomMoins(this)
  /** @type {OutilModifObjNum} */
  this.outilModifObjNum = new OutilModifObjNum(this)
  /** @type {OutilNommer} */
  this.outilNommer = new OutilNommer(this)
  /** @type {OutilCaptNom} */
  this.outilCaptNom = new OutilCaptNom(this)
  /** @type {OutilPalette} */
  this.outilPalette = new OutilPalette(this)
  /** @type {OutilGomme} */
  this.outilGomme = new OutilGomme(this)
  /** @type {OutilRideau} */
  this.outilRideau = new OutilRideau(this)
  /** @type {OutilTranslationFigure} */
  this.outilTranslationFigure = new OutilTranslationFigure(this)
  /** @type {OutilModeTrace} */
  this.outilModeTrace = new OutilModeTrace(this)
  /** @type {OutilModePointsAuto} */
  this.outilModePointsAuto = new OutilModePointsAuto(this)
  /** @type {OutilRecalculer} */
  this.outilRecalculer = new OutilRecalculer(this)
  /** @type {OutilExecutionMacro} */
  this.outilExecutionMacro = new OutilExecutionMacro(this)
  if (!this.estExercice) {
    /** @type {OutilHelp} */
    this.outilHelp = new OutilHelp(this)
  }
  /** @type {OutilToggleToolsAdd} */
  this.outilToggleToolsAdd = new OutilToggleToolsAdd(this)
  if (!this.estExercice) {
    /** @type {OutilCodeBase64} */
    this.outilCodeBase64 = new OutilCodeBase64(this)
    /** @type {OutilCodeTikz} */
    this.outilCodeTikz = new OutilCodeTikz(this)
    /** @type {OutilExport} */
    this.outilExport = new OutilExport(this)
    /** @type {OutilSavePNG} */
    this.outilSavePNG = new OutilSavePNG(this) // Sera utilisé par l'outil d'exportation
    /** @type {OutilSavePNGWithUnity} */
    this.outilSavePNGWithUnity = new OutilSavePNGWithUnity(this)
    /** @type {OutilSaveJPG} */
    this.outilSaveJPG = new OutilSaveJPG(this) // Sera utilisé par l'outil d'exportation
    /** @type {OutilSaveSVG} */
    this.outilSaveSVG = new OutilSaveSVG(this) // Sera utilisé par l'outil d'exportation
    /** @type {OutilExportHTML} */
    this.outilExportHTML = new OutilExportHTML(this) // Sera utilisé par l'outil d'exportation

    if (this.electron) {
      /** @type {OutilCopy} */
      this.outilCopy = new OutilCopy(this)
      /** @type {OutilCopyWithUnity} */
      this.outilCopyWithUnity = new OutilCopyWithUnity(this)
    }
    // Les outils suivants appartiennent à la barre d'outils supplémentaire obtenue en cliquant sur le bouton +/-
    // leurs icônes ne sont créées que quand on clique sur ce bouton
    /** @type {OutilReclassDebObjGra} */
    this.outilReclassDebObjGra = new OutilReclassDebObjGra(this)
    /** @type {OutilReclassFinObjGra} */
    this.outilReclassFinObjGra = new OutilReclassFinObjGra(this)
    /** @type {OutilCopierStyle} */
    this.outilCopierStyle = new OutilCopierStyle(this)
    /** @type {OutilTaillePlus} */
    this.outilTaillePlus = new OutilTaillePlus(this)
    /** @type {OutilTailleMoins} */
    this.outilTailleMoins = new OutilTailleMoins(this)
    /** @type {OutilHelp} */
    this.outilHelp = new OutilHelp(this)
    /** @type {OutilToggleToolsAdd} */
    this.outilToggleToolsAdd = new OutilToggleToolsAdd(this)

    // Création des 4 listes servant à créer une nouvelle construction
    /** @type {CSousListeObjets} */
    this.listeSrcNG = new CSousListeObjets(this.listePr)
    /** @type {CSousListeObjets} */
    this.listeSrcG = new CSousListeObjets(this.listePr)
    /** @type {CSousListeObjets} */
    this.listeFinNG = new CSousListeObjets(this.listePr)
    /** @type {CSousListeObjets} */
    this.listeFinG = new CSousListeObjets(this.listePr)
  }
  /** @type {OutilProtocole} */
  this.outilProtocole = new OutilProtocole(this)
  if (this.options) {
    /** @type {OutilOptionsFig} */
    this.outilOptionsFig = new OutilOptionsFig(this)
  }

  /// /////////////////////////////////////////////////////////////////////////////////////////////
  // On met à jour les icônes de la barre horizontale
  this.updateToolbar()
  /// /////////////////////////////////////////////////////////////////////////////////////////////
  // Fin de la création des outils de la barre horizontale
  /// ////////////////////////////////////////////////////////////////////////////////////////////
  // Outils supplémentaires disponibles quand on clique sur un bouton + en fin de barre déroulante
  //
  /**
   * Outil pour créer le centre d'un cercle
   * @type {OutilCentreCercle}
   */
  this.outilCentreCercle = new OutilCentreCercle(this)
  /** @type {OutilAdd} */
  this.outilAddPoint = new OutilAdd(this, 'AddPoint', ['CentreCercle'])
  /** @type {ButtonToolAdd} */
  this.buttonAddPoint = new ButtonToolAdd(this, false, this.outilAddPoint, 'Add')
  /**
   * Outil pour créer l'image d'un point par une inversion
   * @type {OutilImageInv}
   */
  this.outilImageInv = new OutilImageInv(this)
  /** @type {OutilAdd} */
  this.outilAddTransf = new OutilAdd(this, 'AddTransf', ['ImageInv'])
  /** @type {ButtonToolAdd} */
  this.buttonAddTransf = new ButtonToolAdd(this, false, this.outilAddTransf, 'Add')
  /**
   * Outil pour créer des objets numériques supplémentaires
   * @type {OutilFonc2Var}
   */
  this.outilFonc2Var = new OutilFonc2Var(this)
  /** @type {OutilFonc} */
  this.outilFonc3Var = new OutilFonc3Var(this)
  /** @type {OutilFoncComp} */
  this.outilFoncComp2Var = new OutilFoncComp2Var(this)
  /** @type {OutilFoncComp} */
  this.outilFoncComp3Var = new OutilFoncComp3Var(this)
  /** @type {OutilPartieReelle} */
  this.outilPartieReelle = new OutilPartieReelle(this)
  /** @type {OutilPartieImaginaire} */
  this.outilPartieImaginaire = new OutilPartieImaginaire(this)
  /** @type {OutilModule} */
  this.outilModule = new OutilModule(this)
  /** @type {OutilArgument} */
  this.outilArgument = new OutilArgument(this)
  /** @type {OutilTestExistence} */
  this.outilTestExistence = new OutilTestExistence(this)
  /** @type {OutilTestEq} */
  this.outilTestEq = new OutilTestEq(this)
  /** @type {OutilTestFact} */
  this.outilTestFact = new OutilTestFact(this)
  /** @type {OutilTestEqNatOp} */
  this.outilTestEqNatOp = new OutilTestEqNatOp(this)
  /** @type {OutilTestDepVar} */
  this.outilTestDepVar = new OutilTestDepVar(this)
  /** @type {OutilSomInd} */
  this.outilSomInd = new OutilSomInd(this)
  /** @type {OutilProdInd} */
  this.outilProdInd = new OutilProdInd(this)
  /** @type {OutilInteg} */
  this.outilInteg = new OutilInteg(this)
  /** @type {OutilDeriveePart} */
  this.outilDeriveePart = new OutilDeriveePart(this)
  /** @type {OutilMatrice} */
  this.outilMatrice = new OutilMatrice(this)
  /** @type {OutilCalculMat} */
  this.outilCalculMat = new OutilCalculMat(this)
  /** @type {OutilMatriceAleat} */
  this.outilMatriceAleat = new OutilMatriceAleat(this)
  /** @type {OutilMatriceParForm} */
  this.outilMatriceParForm = new OutilMatriceParForm(this)
  /** @type {OutilDet} */
  this.outilDet = new OutilDet(this)
  /** @type {OutilRenommerCalcul} */
  this.outilRenommerCalcul = new OutilRenommerCalcul(this)
  /** @type {OutilGraduationAxes} */
  this.outilGraduationAxes = new OutilGraduationAxes(this)
  /** @type {OutilLongUnit} */
  this.outilLongUnit = new OutilLongUnit(this)
  /// ///////////////////////////////////////////////////////////////////////////////////////
  //                                Outils pour créer des macros
  /// ///////////////////////////////////////////////////////////////////////////////////////
  /** @type {OutilMacApp} */
  this.outilMacApp = new OutilMacApp(this)
  /** @type {OutilMacDisp} */
  this.outilMacDisp = new OutilMacDisp(this)
  /** @type {OutilMacAppParAut} */
  this.outilMacAppParAut = new OutilMacAppParAut(this)
  /** @type {OutilMacDispParAut} */
  this.outilMacDispParAut = new OutilMacDispParAut(this)
  /** @type {OutilMacAnim} */
  this.outilMacAnim = new OutilMacAnim(this)
  /** @type {OutilMacAnimTr} */
  this.outilMacAnimTr = new OutilMacAnimTr(this)
  /** @type {OutilMacTraceAuto} */
  this.outilMacTraceAuto = new OutilMacTraceAuto(this)
  /** @type {OutilMacTraceAutoVa} */
  this.outilMacTraceAutoVa = new OutilMacTraceAutoVa(this)
  /** @type {OutilMacAffValVar} */
  this.outilMacAffValVar = new OutilMacAffValVar(this)
  /** @type {OutilMacClign} */
  this.outilMacClign = new OutilMacClign(this)
  /** @type {OutilMacIncVar} */
  this.outilMacIncVar = new OutilMacIncVar(this)
  /** @type {OutilMacDecVar} */
  this.outilMacDecVar = new OutilMacDecVar(this)
  /** @type {OutilMacModifVar} */
  this.outilMacModifVar = new OutilMacModifVar(this)
  /** @type {OutilMacAffPtLie} */
  this.outilMacAffPtLie = new OutilMacAffPtLie(this)
  /** @type {OutilMacReaff} */
  this.outilMacReaff = new OutilMacReaff(this)
  /** @type {OutilMacActTr} */
  this.outilMacActTr = new OutilMacActTr(this)
  /** @type {OutilMacDesactTr} */
  this.outilMacDesactTr = new OutilMacDesactTr(this)
  /** @type {OutilMacPause} */
  this.outilMacPause = new OutilMacPause(this)
  /** @type {OutilMacSuiteMac} */
  this.outilMacSuiteMac = new OutilMacSuiteMac(this)
  /** @type {OutilMacJouantSon} */
  this.outilMacJouantSon = new OutilMacJouantSon(this)
  /** @type {OutilMacBoucAnim} */
  this.outilMacBoucAnim = new OutilMacBoucAnim(this)
  /** @type {OutilMacBoucTr} */
  this.outilMacBoucTr = new OutilMacBoucTr(this)
  /** @type {OutilMacConsRec} */
  this.outilMacConsRec = new OutilMacConsRec(this)
  /** @type {OutilMacConsIter} */
  this.outilMacConsIter = new OutilMacConsIter(this)
  /** @type {OutilAddObjMac} */
  this.outilAddObjMac = new OutilAddObjMac(this)
  /** @type {OutilSupObjMac} */
  this.outilSupObjMac = new OutilSupObjMac(this)
  /** @type {OutilFusionImpConst} */
  this.outilFusionImpConst = new OutilFusionImpConst(this)
  /** @type {OutilModifConst} */
  this.outilModifConst = new OutilModifConst(this)
  /** @type {OutilSupConst} */
  this.outilSupConst = new OutilSupConst(this)
  /** @type {OutilImpConstFig} */
  this.outilImpConstFig = new OutilImpConstFig(this)
  /** @type {OutilSaveConst} */
  this.outilSaveConst = new OutilSaveConst(this)
  /** @type {OutilAddConst} */
  this.outilAddConst = new OutilAddConst(this)
  /** @type {OutilGestionConst} */
  this.outilGestionConst = new OutilGestionConst(this)
  /** @type {OutilChoixSrcNumConst} */
  this.outilChoixSrcNumConst = new OutilChoixSrcNumConst(this)
  /** @type {OutilChoixSrcGraphConst} */
  this.outilChoixSrcGraphConst = new OutilChoixSrcGraphConst(this)
  /** @type {OutilChoixFinGraphConst} */
  this.outilChoixFinGraphConst = new OutilChoixFinGraphConst(this)
  /** @type {OutilChoixFinNumConst} */
  this.outilChoixFinNumConst = new OutilChoixFinNumConst(this)
  /** @type {OutilFinirConst} */
  this.outilFinirConst = new OutilFinirConst(this)
  /** @type {OutilReInitConst} */
  this.outilReInitConst = new OutilReInitConst(this)
  /** @type {OutilCreationConst} */
  this.outilCreationConst = new OutilCreationConst(this)
  /**
   * Cet outil ne sera pas associé à un bouton mais utilisé dans le dialogue d'exportation
   * @type {OutilAdd}
   */
  this.outilAddExport = new OutilAdd(this, '', (this.electron ? ['Copy', 'CopyWithUnity'] : []).concat(['SavePNG', 'SaveJPG', 'SaveSVG', 'SavePNGWithUnity', 'CodeTikz', 'CodeBase64', 'ExportHTML']))
  /**
   * Outil utilisé dans l'outil de gestion des constructions
   * @type {OutilAdd}
   */
  this.outilAddGestionConst = new OutilAdd(this, '', ['AddConst', 'ImpConstFig', 'ModifConst', 'SupConst', 'SaveConst', 'FusionImpConst'])
  /**
   * Outil utilisé dans l'outil de gestion des constructions
   * @type {OutilAdd}
   */
  this.outilAddCreationConst = new OutilAdd(this, '', ['ChoixSrcNumConst', 'ChoixSrcGraphConst', 'ChoixFinGraphConst',
    'ChoixFinNumConst', 'FinirConst', 'ReInitConst'])

  /// //////////////////////////////////////////////////////////////////////////////////////////////
  /** @type {OutilAdd} */
  this.outilAddCalcul = new OutilAdd(this, 'AddCalcul', ['RenommerCalcul', 'GraduationAxes', 'TestExistence', 'Fonc2Var',
    'Fonc3Var', 'FoncComp2Var', 'FoncComp3Var', 'SuiteRec2', 'SuiteRec3', 'SuiteRecComplexe2', 'SuiteRecComplexe3',
    'PartieReelle', 'PartieImaginaire', 'Module', 'Argument', 'DeriveePart', 'Matrice', 'CalculMat', 'MatriceAleat',
    'MatriceParForm', 'Det', 'Integ', 'TestEq', 'TestFact', 'TestEqNatOp', 'TestDepVar', 'SomInd', 'ProdInd',
    'AbsOrRep', 'OrdOrRep', 'UnitexRep', 'UniteyRep'])
  /** @type {ButtonToolAdd} */
  this.buttonAddCalcul = new ButtonToolAdd(this, true, this.outilAddCalcul, 'Add', 'ChoixOutil', '')
  // Outil pour créer des macros
  /** @type {OutilAdd} */
  this.outilAddDisp = new OutilAdd(this, 'AddDisp', ['MacApp', 'MacAppParAut', 'MacDisp', 'MacDispParAut', 'MacAnim', 'MacAnimTr', 'MacTraceAuto',
    'MacTraceAutoVa', 'MacAffValVar', 'MacClign', 'MacIncVar', 'MacDecVar', 'MacModifVar', 'MacAffPtLie', 'MacReaff',
    'MacActTr', 'MacDesactTr', 'MacPause', 'MacSuiteMac', 'MacJouantSon', 'MacBoucAnim', 'MacBoucTr', 'MacConsRec', 'MacConsIter'])
  /** @type {ButtonToolAdd} */
  this.buttonAddDisp = new ButtonToolAdd(this, true, this.outilAddDisp, 'AddMac', 'NewMac', 'MacroAdd')
  // Outil pour créer des objets divers
  /** @type {OutilAdd} */
  this.outilAddDivers = new OutilAdd(this, 'AddDivers', ['LongUnit', 'ObjetDuplique', 'ObjetClone', 'AddObjMac',
    'SupObjMac'])
  /** @type {ButtonToolAdd} */
  this.buttonAddDivers = new ButtonToolAdd(this, false, this.outilAddDivers, 'Add')
}

MtgApp.prototype.creeExpandableBars = function () {
  this.expandableBars = []
  /** @type {ExpandableBar} */
  this.barPoints = new ExpandableBar(this, 'Points', ['PtLib', 'Int', 'PtLie', 'Milieu', 'PtParCoord', 'PtBaseEnt',
    'Proj', 'PtInterieur', 'CentreGrav', 'PtParAbs', 'PtParMultVec', 'PtParSommeVec', 'Barycentre', 'PtParAff', 'CreationLiaison', 'SuppressionLiaison',
    'Punaiser', 'Depunaiser', 'MarquerPt', 'DemarquerPt', 'Animation', 'AddPoint'])
  this.expandableBars.push(this.barPoints)
  new ToolbarArrow(this, this.barPoints)
  /** @type {ExpandableBar} */
  this.barDroites = new ExpandableBar(this, 'Droites', ['DtAB', 'DtPar', 'DtPer', 'DtBis', 'DtMed', 'DtHor', 'DtVer', 'DtParCoef',
    'DtParEq', 'Tangente'])
  this.expandableBars.push(this.barDroites)
  new ToolbarArrow(this, this.barDroites)
  /** @type {ExpandableBar} */
  this.barSegments = new ExpandableBar(this, 'Segments', ['Seg', 'DemiDt', 'Vect', 'SegmentParLong'])
  this.expandableBars.push(this.barSegments)
  new ToolbarArrow(this, this.barSegments)
  /** @type {ExpandableBar} */
  this.barCercles = new ExpandableBar(this, 'Cercles', ['CerOA', 'CerOR', 'CerOAB', 'ArcPetit', 'ArcGrand', 'ArcParRapporteur',
    'ArcGrandParRapporteur', 'ArcDirect', 'ArcIndirect', 'ArcPetitParAng', 'ArcGrandParAng', 'ArcDirectParAng', 'ArcIndirectParAng'])
  this.expandableBars.push(this.barCercles)
  new ToolbarArrow(this, this.barCercles)
  /** @type {ExpandableBar} */
  this.barPolys = new ExpandableBar(this, 'Polys', ['Polygone', 'LigneBrisee', 'Carre', 'TriangleEq', 'Parallelog', 'PolygoneReg'])
  this.expandableBars.push(this.barPolys)
  new ToolbarArrow(this, this.barPolys)
  /** @type {ExpandableBar} */
  this.barMarques = new ExpandableBar(this, 'Marques', ['MarqueSeg', 'MarqueAng', 'MarqueAngOr', 'MarqueAngOrSD', 'MarqueAngOrSI'])
  this.expandableBars.push(this.barMarques)
  new ToolbarArrow(this, this.barMarques)
  /** @type {ExpandableBar} */
  this.barLieux = new ExpandableBar(this, 'Lieux', ['LieuParPtLie', 'LieuParVariable', 'LieuDiscretParPtLie', 'LieuDiscretParVariable',
    'CourbeFonc', 'CourbeFoncCr', 'CourbeAvecTan', 'CourbePoly', 'GrapheSuiteRec', 'GrapheSuiteRecComp', 'LieuObjetParPtLie', 'LieuObjetParVariable'])
  this.expandableBars.push(this.barLieux)
  new ToolbarArrow(this, this.barLieux)
  /** @type {ExpandableBar} */
  this.barTransf = new ExpandableBar(this, 'Transf', ['Trans', 'SymAxiale', 'SymCentrale', 'Rot', 'Hom', 'Sim', 'Rapporteur',
    'TransParCoord', 'AddTransf'])
  this.expandableBars.push(this.barTransf)
  new ToolbarArrow(this, this.barTransf)
  /** @type {ExpandableBar} */
  this.barMes = new ExpandableBar(this, 'Mes', ['MesLong', 'MesLongOr', 'MesAngNor', 'MesAbsRep', 'MesOrdRep', 'MesAngOr', 'MesAbs',
    'MesCoefDir', 'MesLongLigne', 'MesAire', 'MesProSca', 'MesAffRep'])
  this.expandableBars.push(this.barMes)
  new ToolbarArrow(this, this.barMes)
  /** @type {ExpandableBar} */
  this.barDisp = new ExpandableBar(this, 'Disp', ['AffichageValeur', 'AffichageValeurLiePt', 'Commentaire',
    'CommentaireLiePt', 'Latex', 'LatexLiePt', 'ImageLibre', 'ImageLiee', 'AffichageEq', 'AffichageEqLie', 'AffichageCoord',
    'AffichageCoordLie', 'EditeurFormule', 'CreationLiaisonAff', 'SuppressionLiaisonAff', 'AddDisp'])
  this.expandableBars.push(this.barDisp)
  new ToolbarArrow(this, this.barDisp)
  /** @type {ExpandableBar} */
  this.barCalculs = new ExpandableBar(this, 'Calculs', ['Curseur', 'Calcul', 'Fonc', 'Variable', 'Repere', 'SolutionEq',
    'Max', 'Min', 'Derivee', 'SuiteRec', 'CalculComp', 'FoncComp', 'SuiteRecComplexe', 'AddCalcul'])
  this.expandableBars.push(this.barCalculs)
  new ToolbarArrow(this, this.barCalculs)
  /** @type {ExpandableBar} */
  this.barSurfaces = new ExpandableBar(this, 'Surfaces', ['Surface', 'SurfaceLieuDroite', 'Surface2Lieux', 'SurfaceLieu2Pts', 'Couronne', 'DemiPlan'])
  this.expandableBars.push(this.barSurfaces)
  new ToolbarArrow(this, this.barSurfaces)
  /** @type {ExpandableBar} */
  this.barDivers = new ExpandableBar(this, 'Divers', ['AddDivers'])
  this.expandableBars.push(this.barDivers)
  new ToolbarArrow(this, this.barDivers)

  this.updateToolsToolBar()
}
