/*
 * Created by yvesb on 12/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const NatCal = require('../types/NatCal')
const PaneNombreDecimales = require('./PaneNombreDecimales')
const PaneAlignement = require('./PaneAlignement')
const PaneTaillePolice = require('./PaneTaillePolice')
const PaneModeAffichage = require('./PaneModeAffichage')
const MtgInputWithCharSpe = require('./MtgInputWithCharSpe')
const CValeurAffichee = require('../objets/CValeurAffichee')
const CAffLiePt = require('../objets/CAffLiePt')
const StyleEncadrement = require('../types/StyleEncadrement')
const AvertDlg = require('../dialogs/AvertDlg')
module.exports = AffichageValeurDlg

/**
 * Dialogue servant à créer ou modifier un affichage de valeur
 * @param app L'application propriétaire
 * @param valAff L'affichage de valeur à modifier
 * @param modification true si on modifie un affichage de valeur déjà créé
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annulation
 * @constructor
 */
function AffichageValeurDlg (app, valAff, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'affvaldlg', callBackOK, callBackCancel)
  this.valAff = valAff
  var list = app.listePr
  var estFinal = valAff.estElementFinal
  this.modification = modification
  var self = this
  var indmax = modification ? list.indexOf(valAff) - 1 : list.longueur() - 1
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  // La première ligne du tableau contient un tableau formé de trois colonnes : Le label, l'éditeur de nom de la valeur
  // et en troisième colonne un tableau de choix de nombre de décimales
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  var trHaut = kernel.ce('tr')
  tabHaut.appendChild(trHaut)
  var td = kernel.ce('td')
  trHaut.appendChild(td)
  if (!(this.modification && valAff.valeurAssociee.estElementIntermediaire())) {
    var caption = kernel.ce('caption', {
      class: 'mtgcaption'
    })
    $(caption).html(kernel.getStr('Valeur'))
    td.appendChild(caption)
    // En haut et à gauche un tableau contenant le label, l'éditeur de nom et dessous le bouton Valeurs
    var tabHautGauche = kernel.ce('table')
    td.appendChild(tabHautGauche)
    tr = kernel.ce('tr')
    tabHautGauche.appendChild(tr)
    this.inf = list.listeParNatCal(app, NatCal.NTteValROuC, indmax)
    this.select = kernel.ce('select', {
      size: 8, // Le nombre de lignes visibles par défaut
      style: 'width:180px'
    })
    this.select.onchange = function () {
      self.updatePreview()
    }
    td.appendChild(this.select)
    if (estFinal) $(this.select).attr('disabled', true)
    // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
    for (var i = 0; i < this.inf.noms.length; i++) {
      var option = kernel.ce('Option', {
        class: 'mtgOption'
      })
      if (modification ? (this.inf.pointeurs[i] === valAff.valeurAssociee) : i === 0) option.setAttribute('selected', 'selected')
      $(option).html(this.inf.noms[i])
      this.select.appendChild(option)
    }
  }

  td = kernel.ce('td')
  trHaut.appendChild(td)
  this.paneNbDec = new PaneNombreDecimales(valAff.nombreDecimales, function () { self.updatePreview() })
  td.appendChild(this.paneNbDec.container)

  // Un tableau central pour contenir les deux éditeurs de pré et post chaines
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabCentre = kernel.ce('table')
  tr.appendChild(tabCentre)
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tabCentre.appendChild(td)
  var tabCentreGauche = kernel.ce('table')
  td.appendChild(tabCentreGauche)
  tr = kernel.ce('tr')
  tabCentreGauche.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  td.appendChild(label)
  $(label).html(kernel.getStr('EnTete'))
  td = kernel.ce('td')
  tr.appendChild(td)
  /*
  var input = kernel.ce("input", {
    type : "text",
    id : "entete",
  });
  */
  var input = new MtgInputWithCharSpe(app, { id: 'entete' }, ['grec', 'math', 'arrows'], false, this)
  td.appendChild(input)
  input.onkeyup = function () {
    self.updatePreview()
  }

  tr = kernel.ce('tr')
  tabCentreGauche.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  td.appendChild(label)
  $(label).html(kernel.getStr('SuiviDe'))
  td = kernel.ce('td')
  tr.appendChild(td)
  /*
  input = kernel.ce("input", {
    type : "text",
    id : "suivide",
  });
  */
  input = new MtgInputWithCharSpe(app, { id: 'suivide' }, ['grec', 'math', 'arrows'], false, this)
  td.appendChild(input)
  input.onkeyup = function () {
    self.updatePreview()
  }

  // Au centre à droite un svg d'aperçu
  td = kernel.ce('td')
  tabCentre.appendChild(td)
  var tabCentreDroit = kernel.ce('table')
  td.appendChild(tabCentreDroit)
  caption = kernel.ce('caption', {
    class: 'mtgcaption'
  })
  $(caption).html(kernel.getStr('Apercu'))
  tabCentreDroit.appendChild(caption)
  td = kernel.ce('td')
  $(td).css('border', '1px solid lightgray')
  tabCentreDroit.appendChild(td)
  this.svg = kernel.cens('svg', {
    width: '200px',
    height: '80px'
  })
  td.appendChild(this.svg)
  /* Ajout version 6.4.1 pour un rendu correct de l'aperçu à cause d'un bug MathJax */
  const rootFontFamily = getComputedStyle(document.body).fontFamily
  const containerFontFamily = getComputedStyle(this.svg.parentNode).fontFamily
  if (rootFontFamily && rootFontFamily !== containerFontFamily) {
    this.svg.parentNode.style.fontFamily = rootFontFamily
  }
  // Dans la troisième ligne du tableau principal, un tableau de trois colonnes contenant un panneau de choix de
  // mode d'affichage, un panneau de choix d'alignement et un manneua de choix de taille de police
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  this.paneAlign = new PaneAlignement(app, valAff.alignementHorizontal, valAff.alignementVertical)
  var tableBas = kernel.ce('table')
  tr.appendChild(tableBas)
  td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  td.appendChild(this.paneAlign.container)
  td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  this.paneTaillePolice = new PaneTaillePolice(valAff.taillePolice, function () { self.updatePreview() })
  tableBas.appendChild(this.paneTaillePolice.container)
  td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  // Pas d'édition de l'angle d'affichage si c'est un objet final de macro.
  this.paneModeAffichage = new PaneModeAffichage(this, false, valAff.effacementFond, valAff.encadrement,
    valAff.couleurFond, estFinal ? null : valAff.angText.chaineInfo(), indmax, function () { self.updatePreview() })
  td.appendChild(this.paneModeAffichage.container)
  if (modification) {
    $(this.inputVal).val(valAff.valeurAssociee.getNom())
    $('#entete').val(valAff.enTete)
    $('#suivide').val(valAff.postChaine)
  }

  this.create('AffVal', 580)
}

AffichageValeurDlg.prototype = new MtgDlg()

AffichageValeurDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  this.paneModeAffichage.init()
  this.updatePreview()
}

AffichageValeurDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  var app = this.app
  var val = this.valAff
  var pma = this.paneModeAffichage
  if (!pma.validate()) {
    new AvertDlg(app, 'Incorrect')
    return
  } // Faute dans l'angle de rotation du texte
  val.encadrement = pma.getStyleEnc()
  val.effacementFond = pma.getEffFond()
  val.couleurFond = pma.getColor()
  val.nombreDecimales = this.paneNbDec.getDigits()
  // Si l'alignement vertical ou horizontal est changé on annule tout décalage manuel précédent
  var newHorAlign = this.paneAlign.getHorAlign()
  var newVerAlign = this.paneAlign.getVerAlign()
  if ((val.alignementHorizontal !== newHorAlign) || (val.alignementVertical !== newVerAlign)) {
    val.decX = 0
    val.decY = 0
  }
  if (!val.estElementFinal && !(this.modification && val.valeurAssociee.estElementIntermediaire())) {
    val.valeurAssociee = this.inf.pointeurs[this.select.selectedIndex]
    val.angText = pma.getAng()
  }
  //
  val.alignementHorizontal = newHorAlign
  val.alignementVertical = newVerAlign
  val.taillePolice = this.paneTaillePolice.getTaille()
  val.enTete = $('#entete').val()
  val.postChaine = $('#suivide').val()
  val.positionne(false, app.dimf)
  /* Supprimé version 6.4.1
  if (val.existe) {
    val.setReady4MathJax()
    if (this.modification) {
      // Il faut récréer l'affichage complètement
      MathJax.Hub.Queue(function () {
        val.reCreateDisplay(app.svgFigure)
      })
    }
  } else val.removegElement(app.svgFigure)
  */
  // Finalement il faut mrecréer l'affichage si, apr exemple, on a choisi d'eefacer le fond alors qu'il ne l'était pas
  val.setReady4MathJax()
  if (this.modification && this.app.dlg[0] !== 'ProtocoleDlg') {
    MathJax.Hub.Queue(function () {
      val.reCreateDisplay(app.svgFigure)
    })
  }
  if (this.callBackOK !== null) this.callBackOK()
  if (this.modification) app.gestionnaire.enregistreFigureEnCours('ModifObj')
  this.destroy()
}

AffichageValeurDlg.prototype.updatePreview = function () {
  var app = this.app
  var list = app.listePr
  var valAff = this.valAff.valeurAssociee
  var pointeur = (this.modification && valAff.estElementIntermediaire()) ? valAff : this.inf.pointeurs[this.select.selectedIndex]

  var pma = this.paneModeAffichage
  var affv = new CValeurAffichee(list, null, false, this.valAff.couleur, 0, 0, 0, 0, false, null,
    this.paneTaillePolice.getTaille(), pma.getStyleEnc(), pma.getEffFond(),
    pma.getColor(), CAffLiePt.alignHorLeft, CAffLiePt.alignVerTop,
    pointeur, $('#entete').val(), $('#suivide').val(), this.paneNbDec.getDigits())
  affv.positionneFull(false, app.dimf)
  if (affv.existe) {
    affv.setReady4MathJax()
    var self = this
    MathJax.Hub.Queue(function () {
      while (self.svg.childNodes.length !== 0) self.svg.removeChild(self.svg.childNodes[0])
      var g = affv.createg()
      if (affv.encadrement !== StyleEncadrement.Sans) $(g).attr('transform', 'translate(2,2)')
      self.svg.appendChild(g)
    })
  } else while (this.svg.childNodes.length !== 0) this.svg.removeChild(this.svg.childNodes[0])
}
