/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('jquery')

module.exports = ChoixFinNumDlg

/**
 * Dialogue de choix des objets finaux numériques pour une construction
 * @param app mtgApp propriétaire
 * @constructor
 */
function ChoixFinNumDlg (app) {
  MtgDlg.call(this, app, 'ChoixFinNumDlg')
  this.inf = app.listePr.listObjFinDisp(app)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('FinDlg1'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  this.select = kernel.ce('select', {
    size: 12, // Le nombre de lignes visibles par défaut
    multiple: 'multiple', // Sélection multiple possible pour cette liste
    style: 'width:300px'
  })
  tr.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (app.listeFinNG.contains(this.inf.pointeurs[i])) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    option.onmousedown = function (ev) {
      var sel = $(this)
      var issel = sel.prop('selected')
      sel.prop('selected', !issel)
      kernel.preventDefault(ev)
    }
    this.select.appendChild(option)
  }
  this.create('ChoixFinNumConst', 400)
}

ChoixFinNumDlg.prototype = new MtgDlg()

ChoixFinNumDlg.prototype.OK = function () {
  var listNG = this.app.listeFinNG
  listNG.retireTout()
  var options = this.select.options
  for (var i = 0; i < options.length; i++) {
    if ($(options[i]).prop('selected')) listNG.add(this.inf.pointeurs[i])
  }
  this.destroy()
  this.app.activeOutilCapt()
}
