/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const MtgInput = require('./MtgInput')
const EditeurConst = require('./EditeurConst')
const $ = require('jquery')

module.exports = CodeTikzDlg

/**
 * Dialogue de choix d'angle de rotation
 * @constructor
 * @param app La mtgApp propriétaire
 * @param {function} callBack Fonction de callBack à appler après validation
 */
function CodeTikzDlg (app, callBack) {
  var tr, label, div
  MtgDlg.call(this, app, 'tikzDlg')
  this.callBack = callBack
  var list = app.listePr
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  var rad1 = kernel.ce('input', {
    type: 'radio',
    id: 'rad1',
    name: 'rad'
  })
  div.appendChild(rad1)
  rad1.onclick = function () {
    $('#div2').css('display', 'none')
    $('#div1').css('display', 'block')
  }
  label = kernel.ce('label', {
    for: 'rad1'
  })
  $(label).html(kernel.getStr('TikzDlg1'))
  div.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div', {
    id: 'div1'
  })
  tr.appendChild(div)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('TikzDlg3'))
  div.appendChild(label)
  var input1 = new MtgInput({
    id: 'input1',
    size: 4
  })
  div.appendChild(input1)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  var rad2 = kernel.ce('input', {
    type: 'radio',
    id: 'rad2',
    name: 'rad'
  })
  div.appendChild(rad2)
  rad2.onclick = function () {
    $('#div1').css('display', 'none')
    $('#div2').css('display', 'block')
  }
  label = kernel.ce('label', {
    for: 'rad2'
  })
  $(label).html(kernel.getStr('TikzDlg2'))
  div.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div', {
    id: 'div2'
  })
  tr.appendChild(div)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('TikzDlg4'))
  div.appendChild(label)
  var input2 = new MtgInput({
    id: 'input2',
    size: 4
  })
  div.appendChild(input2)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  var cb = kernel.ce('input', {
    type: 'checkbox',
    id: 'cb'
  })
  div.appendChild(cb)
  label = kernel.ce('label', {
    for: 'cb'
  })
  $(label).html(kernel.getStr('TikzDlg5'))
  div.appendChild(label)
  if (list.pointeurLongueurUnite === null) {
    $(rad1).attr('disabled', true)
    $('#div1').css('display', 'none')
    $('#rad2').attr('checked', 'checked')
    $(input2).val('12')
  } else {
    $('#rad1').attr('checked', 'checked')
    $(input1).val('1')
    $('#div2').css('display', 'none')
  }

  this.editeur1 = new EditeurConst(app, input1, 0.1, 10, false)
  this.editeur2 = new EditeurConst(app, input2, 1, 40, false)
  // Création de la boîte de dialogue par jqueryui
  this.create('CodeTikz', 550)
}

CodeTikzDlg.prototype = new MtgDlg()

CodeTikzDlg.prototype.OK = function () {
  var app = this.app
  var list = app.listePr
  var bu = {}
  var bcheck1 = $('#rad1').prop('checked')
  if (bcheck1 ? this.editeur1.validate() : this.editeur2.validate()) {
    if (bcheck1) {
      bu.useFigUnity = true
      bu.infoLength = parseFloat($('#input1').val())
    } else {
      bu.useFigUnity = false
      bu.infoLength = parseFloat($('#input2').val())
    }
    // On considère qu'un figure normal a une largeur de 12 cm. Sinon on applique un coefficient
    // pour l'épaisseur des traits
    var widthcm = bu.useFigUnity ? (app.dimf.x / list.pointeurLongueurUnite.rendLongueur() * bu.infoLength)
      : bu.infoLength
    var coef = widthcm / 12
    if (coef > 1) coef = 1
    var code = this.app.doc.getTikz(app, coef, bu, $('#cb').prop('checked'))
    this.destroy()
    this.callBack(code)
  }
}
