/*
 * Created by yvesb on 23/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const CValeur = require('../objets/CValeur')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const MtgInput = require('./MtgInput')
module.exports = CoefBarycentreDlg

/**
 * Dialogue appelé pour donner à un point pondéré d'un barycentre son coefficient
 * @constructor
 * @param app La MtgApp proppriétaire
 * @param bar Le barycentre propriétaire du noeud qui contient le point
 * @param noeud Le point du barycentre dont on modifie le coefficient
 * @param modificationtrue si on modifie un point pondéré déjà créé
 * @param callBackOK null ou fonction à appeler apès OK
 * @param callBackCancel null ou fonction à appeler après Cancel
 */
function CoefBarycentreDlg (app, bar, noeud, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'coefDirDlg', callBackOK, callBackCancel)
  this.modification = modification
  this.bar = bar
  this.noeud = noeud
  const self = this
  this.modification = modification
  const list = app.listePr
  // Un CValeur pour stocker la valeur choisie avant vérification
  this.valeur = new CValeur(list, 1)
  const tab = kernel.ce('table')
  this.appendChild(tab)
  const tr = kernel.ce('tr')
  tab.appendChild(tr)
  let td = kernel.ce('td')
  tr.appendChild(td)
  const label = kernel.ce('label')
  $(label).html(kernel.getStr('Coef'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  const input = new MtgInput({
    id: 'mtginput'
  })
  td.appendChild(input)
  input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  $(input).val(noeud.poids.calcul.chaineCalculSansPar())
  input.select()
  const indmax = modification ? list.indexOf(bar) - 1 : list.longueur() - 1
  const paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, input, true, indmax)
  tab.appendChild(paneBoutonsValFonc)
  if (modification) $(input).val(noeud.poids.calcul.chaineCalculSansPar())
  this.editor = new EditeurvaleurReelle(app, input, indmax, this.valeur, null)
  this.create('CoefBar', 450)
}

CoefBarycentreDlg.prototype = new MtgDlg()

CoefBarycentreDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

CoefBarycentreDlg.prototype.OK = function () {
  var sel = $('#mtginput')
  // if (this.app.lastDlgId() !== this.id) return;
  if (this.editor.validate(sel.val())) {
    this.noeud.poids = this.valeur
    if (this.callBackOK !== null) this.callBackOK()
    this.destroy()
  } else sel.focus()
}
