/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const constantes = require('../kernel/constantes')
const Color = require('../types/Color')
const $ = require('../global/jQuery-ui')
require('../../outilsExternes/spectrum/spectrum.js')
require('../../outilsExternes/spectrum/spectrum.css')

module.exports = CouleurFondDlg

/**
 * Dialogue de choix de la couleur de fond de la figure
 * @param app L'application propriétaire
 * @param callBackOK Fonction de callBack à appeler une fois la couleur choisie
 * @constructor
 */
function CouleurFondDlg (app, callBackOK) {
  MtgDlg.call(this, app, 'CoulFondDlg', callBackOK)
  var self = this
  var doc = app.doc
  var div = kernel.ce('div')
  this.appendChild(div)
  this.input = kernel.ce('input', {
    type: 'text',
    id: 'colorPicker'
  })
  div.appendChild(this.input)
  $('#colorPicker').spectrum({
    color: doc.couleurFond.rgb(),
    cancelText: kernel.getStr('Cancel'),
    chooseText: kernel.getStr('OK'),
    flat: true,
    showInput: true,
    showButtons: false,
    showPalette: true,
    preferredFormat: 'hex',
    palette: constantes.colorPalette,
    clickoutFiresChange: false,
    change: function (color) {
      self.color = new Color(Math.round(color._r), Math.round(color._g), Math.round(color._b))
    }
  })
  $('#sp-cancel').css('visibility', 'hidden')
  this.create('CouFond', 500)
}

CouleurFondDlg.prototype = new MtgDlg()

CouleurFondDlg.prototype.OK = function () {
  var doc = this.app.doc
  var color = $('#colorPicker').spectrum('get')
  var coul = new Color(Math.round(color._r), Math.round(color._g), Math.round(color._b))
  doc.couleurFond = coul
  $('#coulFond').attr('fill', coul.rgb())
  this.destroy()
  this.callBackOK()
}
