/*
 * Created by yvesb on 07/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const CValeur = require('../objets/CValeur')
const NatCal = require('../types/NatCal')
const PaneListeReperes = require('./PaneListeReperes')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
const EditeurValeurReelle = require('./EditeurValeurReelle')
const AvertDlg = require('./AvertDlg')
const MtgInput = require('./MtgInput')
module.exports = CourbeFoncDlg

function CourbeFoncDlg (app, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'CourbeDlg', callBackOK, callBackCancel)
  var self = this
  var list = app.listePr
  this.inf = list.listeParNatCal(app, NatCal.NFoncR1Var, -1)
  // 2 CValeur pour stocker les valeurs pour a et b.
  this.valeura = new CValeur(list)
  this.valeurb = new CValeur(list)

  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  // Un premier tableau en haut pour contenir la liste des fonctions et la liste des repères
  var tabListes = kernel.ce('table')
  $(tabListes).css('margin', ' 0 auto')
  tr.appendChild(tabListes)
  var td = kernel.ce('td')
  tabListes.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:220px'
  })
  $(caption).html(kernel.getStr('InsDynDlg1'))
  td.appendChild(caption)
  this.select = kernel.ce('SELECT', {
    size: 5, // Le nombre de lignes visibles par défaut
    style: 'width:220px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.select.appendChild(option)
  }
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tabListes.appendChild(td)
  this.paneListeRep = new PaneListeReperes(app, list.longueur() - 1)
  td.appendChild(this.paneListeRep.getTab())
  // Un tableau pour contenir les boutons radio de choix d'intervalle pour la courbe
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabRadio = kernel.ce('table')
  tr.appendChild(tabRadio)
  tr = kernel.ce('tr')
  tabRadio.appendChild(tr)
  var label = kernel.ce('label', {
    for: 'radio1'
  })
  $(label).html(kernel.getStr('courbeDlg1'))
  tr.appendChild(label)
  // Tableau formé des fonctions a appeler quand on coche une des quatre cases
  // Pour j variant de 1 à 4 : Courbe sur R, courbe sur [a;b], courbe sur [a; + infini[, courbe sur ]- infini; a]
  var tabFonc = ['hideInputab', 'showInputab', 'showInputa', 'showInputa']
  for (let j = 1; j < 5; j++) {
    var radio = kernel.ce('input', {
      type: 'radio',
      id: 'radio' + j,
      name: 'interv'
    })
    tr.appendChild(radio)
    if (j === 1) radio.setAttribute('checked', 'checked')
    radio.onchange = function () {
      self[tabFonc[j - 1]]()
    }
    label = kernel.ce('label', {
      for: 'radio' + j
    })
    $(label).html(kernel.getStr('courbeDlg' + String(j + 1)))
    tr.appendChild(label)
  }

  var trValeurs = kernel.ce('tr')
  tabPrincipal.appendChild(trValeurs)
  var paneValeurs = kernel.ce('table', {
    id: 'paneVal'
  })
  trValeurs.appendChild(paneValeurs)
  td = kernel.ce('td')
  paneValeurs.appendChild(td)
  var paneValeursGauche = kernel.ce('table', {
    id: 'paneVala'
  })
  td.appendChild(paneValeursGauche)
  tr = kernel.ce('tr')
  paneValeursGauche.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label', {
    id: 'labela'
  })
  $(label).html(kernel.getStr('Valeura') + ' :')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  // Editeur pour la valeur de a
  this.inputa = new MtgInput()
  $(this.inputa).attr('size', 15)
  td.appendChild(this.inputa)
  var paneBoutonsValFonca = new PaneBoutonsValFonc(this.app, this, this.inputa, true, -1)
  paneValeursGauche.appendChild(paneBoutonsValFonca)

  td = kernel.ce('td')
  paneValeurs.appendChild(td)
  var paneValeursDroit = kernel.ce('table', {
    id: 'paneValb'
  })
  td.appendChild(paneValeursDroit)
  tr = kernel.ce('tr')
  paneValeursDroit.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label', {
    id: 'labelb'
  })
  $(label).html(kernel.getStr('Valeurb') + ' :')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  // Editeur pour la valeur de b
  this.inputb = new MtgInput()
  $(this.inputb).attr('size', 15)
  td.appendChild(this.inputb)
  var paneBoutonsValFoncb = new PaneBoutonsValFonc(this.app, this, this.inputb, true, -1)
  paneValeursDroit.appendChild(paneBoutonsValFoncb)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneInfo = kernel.ce('table')
  tr.appendChild(paneInfo)
  tr = kernel.ce('tr')
  paneInfo.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  $(td).css('vertical-align', 'top')
  label = kernel.ce('label')
  $(label).html(kernel.getStr('InfoFonc'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var textarea = kernel.ce('textarea', {
    id: 'info',
    cols: 35,
    rows: 2,
    disabled: 'true'
  })
  tr.appendChild(textarea)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  // Un tableau contenant l'éditeur du nombre de points
  var paneInput = kernel.ce('table')
  tr.appendChild(paneInput)
  tr = kernel.ce('tr')
  paneInput.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('courbeDlg10'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var inputnb = new MtgInput({
    id: 'inputnb',
    size: 3
  })
  $(inputnb).val('500')
  td.appendChild(inputnb)

  // En bas un tableau pour les trois checkbox
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneBas = kernel.ce('table', {
    cellspacing: 4
  })
  tr.appendChild(paneBas)
  tr = kernel.ce('tr')
  paneBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  var div = kernel.ce('div')
  td.appendChild(div)
  var cb1 = kernel.ce('input', {
    id: 'cb1',
    type: 'checkbox'
  })
  $(cb1).attr('checked', 'checked')
  div.appendChild(cb1)
  label = kernel.ce('label', {
    for: 'cb1'
  })
  $(label).html(kernel.getStr('courbeDlg7'))
  div.appendChild(label)

  td = kernel.ce('td')
  tr.appendChild(td)
  div = kernel.ce('div')
  td.appendChild(div)
  var cb2 = kernel.ce('input', {
    id: 'cb2',
    type: 'checkbox'
  })
  $(cb2).attr('checked', 'checked')
  div.appendChild(cb2)
  label = kernel.ce('label', {
    for: 'cb2'
  })
  $(label).html(kernel.getStr('courbeDlg8'))
  div.appendChild(label)

  tr = kernel.ce('tr')
  paneBas.appendChild(tr)

  td = kernel.ce('td')
  tr.appendChild(td)
  div = kernel.ce('div')
  td.appendChild(div)
  var cb3 = kernel.ce('input', {
    id: 'cb3',
    type: 'checkbox'
  })
  $(cb3).attr('checked', 'checked')
  div.appendChild(cb3)
  label = kernel.ce('label', {
    for: 'cb3'
  })
  $(label).html(kernel.getStr('courbeDlg9'))
  div.appendChild(label)
  var indf = list.longueur() - 1
  this.editeura = new EditeurValeurReelle(app, this.inputa, indf, this.valeura, null)
  this.editeurb = new EditeurValeurReelle(app, this.inputb, indf, this.valeurb, null)
  this.hideInputab()
  this.onSelectChange()
  this.create('CourbeFonc', 650)
}

CourbeFoncDlg.prototype = new MtgDlg()

CourbeFoncDlg.prototype.showInputab = function () {
  $('#paneVala').css('visibility', 'visible')
  $('#paneValb').css('visibility', 'visible')
}

CourbeFoncDlg.prototype.showInputa = function () {
  $('#paneVala').css('visibility', 'visible')
  $('#paneValb').css('visibility', 'hidden')
}
CourbeFoncDlg.prototype.hideInputab = function () {
  $('#paneVala').css('visibility', 'hidden')
  $('#paneValb').css('visibility', 'hidden')
}

CourbeFoncDlg.prototype.onSelectChange = function () {
  $('#info').val(this.inf.pointeurs[this.select.selectedIndex].infoHist())
}

CourbeFoncDlg.prototype.OK = function () {
  var courbeSurR = $('#radio1').prop('checked')
  var courbeSurab = $('#radio2').prop('checked')
  var courbeSuraInf = $('#radio3').prop('checked')
  var courbeSurInfa = $('#radio4').prop('checked')
  var editeuraVis = courbeSurab || courbeSuraInf || courbeSurInfa

  if ((editeuraVis ? this.editeura.validate() : true) && (courbeSurab ? this.editeurb.validate() : true)) {
    var app = this.app
    var nbPts = parseInt($('#inputnb').val())
    if (nbPts < 4 || nbPts > 5000 || isNaN(nbPts)) {
      new AvertDlg(app, 'Incorrect', function () {
        $('#inputnb').focus()
      })
    } else {
      var list = app.listePr
      var nomx = list.genereNomPourCalcul('x', true)
      var nomy = list.genereNomPourCalcul('y', true)
      var nompt = list.genereNomPourPointOuDroite('x', true)
      var ptLieCache = $('#cb1').prop('checked')
      var ptGenCache = $('#cb2').prop('checked')
      var gestionAuto = $('#cb3').prop('checked')
      var rep = this.paneListeRep.getSelectedRep()
      var fonc = this.inf.pointeurs[this.select.selectedIndex]
      var ind = list.longueur() // Pour svoir à partir de quel indice créer les g elements
      if (courbeSurR) { app.listePr.ajouteCourbeSurR(app, rep, fonc, nompt, nomx, nomy, nbPts, ptLieCache, ptGenCache, gestionAuto) } else
      if (courbeSurab) { app.listePr.ajouteCourbeSurab(app, rep, fonc, nompt, nomx, nomy, nbPts, ptLieCache, ptGenCache, gestionAuto, this.valeura, this.valeurb) } else
      if (courbeSuraInf) { app.listePr.ajouteCourbeSuraInf(app, rep, fonc, nompt, nomx, nomy, nbPts, ptLieCache, ptGenCache, gestionAuto, this.valeura) } else { app.listePr.ajouteCourbeSurInfa(app, rep, fonc, nompt, nomx, nomy, nbPts, ptLieCache, ptGenCache, gestionAuto, this.valeura) }
      list.afficheTout(ind, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
      app.gestionnaire.enregistreFigureEnCours('CourbeFonc')
      this.destroy()
      app.activeOutilCapt()
    }
  }
}
