/*
 * Created by yvesb on 14/05/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const NatCal = require('../types/NatCalAdd')
const PaneModeAffichage = require('./PaneModeAffichage')
const PaneTaillePolice = require('./PaneTaillePolice')
const MtgInputWithCharSpe = require('./MtgInputWithCharSpe')
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
module.exports = EditeurFormuleDlg

/**
 *
 * @param app
 * @param editeur L'éditeur de formule à modifier
 * @param modification true si on modifie un éditeur déjà présent dans la figure
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annulation
 * @constructor
 */
function EditeurFormuleDlg (app, editeur, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'DeriveeDlg', callBackOK, callBackCancel)
  this.editeur = editeur
  this.modification = modification
  var self = this
  var list = app.listePr
  var indmax = modification ? list.indexOf(editeur) - 1 : -1
  this.inf = list.listeParNatCal(app, NatCal.NCalculOuFonctionParFormule, indmax)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:180px'
  })
  $(caption).html(kernel.getStr('EditForDlg1'))
  td.appendChild(caption)
  this.selectCalc = kernel.ce('SELECT', {
    size: 6, // Le nombre de lignes visibles par défaut
    style: 'width:180px'
  })
  this.selectCalc.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.selectCalc)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.inf.pointeurs[i] === editeur.calculAssocie) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.selectCalc.appendChild(option)
  }
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:60px'
  })
  $(caption).html(kernel.getStr('Colonnes'))
  td.appendChild(caption)
  this.selectNbcol = kernel.ce('SELECT', {
    size: 6, // Le nombre de lignes visibles par défaut
    style: 'width:60px'
  })
  for (i = 2; i <= 50; i++) {
    option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (editeur.nbcol === i) option.setAttribute('selected', 'selected')
    $(option).html(i)
    this.selectNbcol.appendChild(option)
  }
  td.appendChild(this.selectNbcol)
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  this.paneModeAffichage = new PaneModeAffichage(this, false, editeur.effacementFond,
    editeur.encadrement, editeur.couleurFond)
  td.appendChild(this.paneModeAffichage.container)
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  this.paneTaillePolice = new PaneTaillePolice(editeur.taillePolice)
  td.appendChild(this.paneTaillePolice.container)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabEntete = kernel.ce('table')
  tr.appendChild(tabEntete)
  td = kernel.ce('td')
  tabEntete.appendChild(td)
  var tabEnTeteGauche = kernel.ce('table')
  td.appendChild(tabEnTeteGauche)
  tr = kernel.ce('tr')
  tabEnTeteGauche.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  td.appendChild(label)
  $(label).html(kernel.getStr('EnTete'))
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = new MtgInputWithCharSpe(app, { id: 'entete' }, ['grec', 'math', 'arrows'], false, this)
  td.appendChild(input)
  tr = kernel.ce('tr')
  tabEnTeteGauche.appendChild(tr)
  tr.appendChild(kernel.ce('td'))
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('EditForDlg2'))
  td.appendChild(label)
  td = kernel.ce('td')
  tabEntete.appendChild(td)
  var div = kernel.ce('div')
  td.appendChild(div)
  // CheckBox pour le signes de multiplication implicites
  var cb1 = kernel.ce('input', {
    id: 'cb1',
    type: 'checkbox'
  })
  div.appendChild(cb1)
  label = kernel.ce('label', {
    for: 'cb1'
  })
  $(label).html(kernel.getStr('EditForDlg3'))
  div.appendChild(label)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabCentre = kernel.ce('table')
  tr.appendChild(tabCentre)

  // CheckBox pour l'affichage de la formule en LaTeX
  tr = kernel.ce('tr')
  tabCentre.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  var cb2 = kernel.ce('input', {
    id: 'cb2',
    type: 'checkbox'
  })
  div.appendChild(cb2)
  // Suivant que ce chehckBox est checked ou non on montre ou pas les autres checkBoxes
  cb2.onclick = function () {
    $('#tabComp').css('visibility', ($('#cb2').prop('checked')) ? 'visible' : 'hidden')
  }
  label = kernel.ce('label', {
    for: 'cb2'
  })
  $(label).html(kernel.getStr('EditForDlg4'))
  div.appendChild(label)
  tr = kernel.ce('tr')
  tabCentre.appendChild(tr)
  var tabComp = kernel.ce('table', {
    id: 'tabComp'
  })
  tr.appendChild(tabComp)
  tr = kernel.ce('tr')
  tabComp.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('EditForDlg5'))
  div.appendChild(label)
  var inputprecode = kernel.ce('input', {
    id: 'precode',
    type: 'text'
  })
  div.appendChild(inputprecode)
  // Un checkBox pour l'affichage initial de la formule
  tr = kernel.ce('tr')
  tabComp.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  var cb3 = kernel.ce('input', {
    id: 'cb3',
    type: 'checkbox'
  })
  div.appendChild(cb3)
  label = kernel.ce('label', {
    for: 'cb3'
  })
  $(label).html(kernel.getStr('EditForDlg6'))
  div.appendChild(label)
  // Un checkBox pour l'affichage en temps réel de la formule
  tr = kernel.ce('tr')
  tabComp.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  var cb4 = kernel.ce('input', {
    id: 'cb4',
    type: 'checkbox'
  })
  div.appendChild(cb4)
  label = kernel.ce('label', {
    for: 'cb4'
  })
  $(label).html(kernel.getStr('EditForDlg7'))
  div.appendChild(label)

  // En bas du tableau un champ d'info sur la fonction sélectionnée
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas = kernel.ce('table')
  tr.appendChild(tabBas)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('ChoixValeurDlg3'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  var inputinf = kernel.ce('textarea', {
    id: 'mtginfo',
    disabled: 'true',
    cols: 60,
    rows: 3
  })
  tr.appendChild(inputinf)

  $('#entete').val(editeur.enTete)
  $('#precode').val(editeur.preCodeLaTeX)
  if (editeur.variables1Car) $('#cb1').attr('checked', 'checked')
  if (editeur.affichageFormule) $('#cb2').attr('checked', 'checked')
  if (editeur.affLaTeXPremierAffichage) $('#cb3').attr('checked', 'checked')
  if (editeur.affichageTempsReel) $('#cb4').attr('checked', 'checked')
  if (!editeur.affichageFormule) $('#tabComp').css('visibility', 'hidden')

  this.onSelectChange()
  this.create('EditeurFormule', 650)
}

EditeurFormuleDlg.prototype = new MtgDlg()

EditeurFormuleDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  this.paneModeAffichage.init()
}

EditeurFormuleDlg.prototype.onSelectChange = function () {
  var ind, el
  ind = this.selectCalc.selectedIndex
  if (ind === -1) return
  el = this.inf.pointeurs[ind]
  $('#mtginfo').text(el.infoHist())
}

EditeurFormuleDlg.prototype.OK = function () {
  var newgLatex
  var app = this.app
  var self = this
  var edit = this.editeur
  var svg = app.svgFigure
  var doc = app.doc
  var couleurFond = doc.couleurFond
  var opacity = doc.opacity
  // edit.removegElement(svg);
  edit.deleteComponent()
  if (this.modification) {
    var oldg = edit.g
    if (edit.affichageFormule) var oldglatex = edit.latex.g
  }
  var affichageFormuleInit = edit.affichageFormule
  edit.calculAssocie = this.inf.pointeurs[this.selectCalc.selectedIndex]
  edit.taillePolice = this.paneTaillePolice.getTaille()
  edit.nbcol = this.selectNbcol.selectedIndex + 2
  edit.enTete = $('#entete').val()
  edit.preCodeLaTeX = $('#precode').val()
  edit.variables1Car = $('#cb1').prop('checked')
  edit.affichageFormule = $('#cb2').prop('checked')
  edit.affLaTeXPremierAffichage = $('#cb3').prop('checked')
  edit.affichageTempsReel = $('#cb4').prop('checked')
  var pma = this.paneModeAffichage
  edit.encadrement = pma.getStyleEnc()
  edit.effacementFond = pma.getEffFond()
  edit.couleurFond = pma.getColor()
  edit.creeEditeur(svg)
  edit.positionne(false, app.dimf)
  edit.setReady4MathJax()
  MathJax.Hub.Queue(function () {
    if (self.modification) {
      var newg = edit.createg()
      edit.g = newg
      svg.replaceChild(newg, oldg)
      if (edit.affichageFormule) {
        if (affichageFormuleInit) {
          newgLatex = edit.latex.createg()
          edit.latex.g = newgLatex
          edit.glatex = newgLatex
          if (oldglatex !== null) svg.replaceChild(newgLatex, oldglatex); else svg.appendChild(newgLatex)
        } else {
          // Modification version 6.3.3
          // self.editeur.creeAffichage(app.svgFigure, false, couleurFond, opacity);
          newgLatex = edit.latex.createg()
          edit.latex.g = newgLatex
          edit.glatex = newgLatex
          svg.appendChild(newgLatex)
        }
      } else {
        if (affichageFormuleInit) {
          svg.removeChild(edit.latex.g)
          edit.latex = null
        }
      }
    } else self.editeur.creeAffichage(app.svgFigure, false, couleurFond, opacity)
    if (self.callBackOK) self.callBackOK()
    if (self.modification) app.gestionnaire.enregistreFigureEnCours('ModifObj')
    self.destroy()
  })
}
