/*
 * Created by yvesb on 30/12/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Pointeur = require('../types/Pointeur')
// const kernelAdd = require('../kernel/kernelAdd')
const $ = require('jquery')
require('jquery-textrange')
const CalcC = require('../kernel/CalcC')
const AvertDlg = require('./AvertDlg')
module.exports = EditeurValeurComplexe

/**
 * Objet associé à un input d'une boîte de dialogue et chargé de regarder si
 * la syntaxe du contenu de l'éditeur est correcte.
 * @param app L'application mtgApp propriétaire
 * @param input L'éditeur associé
 * @param indfin L'indice de fin de l'analyse de syntaxe dans la liste des objets créés dans l'appli
 * @param valueLe CValeurComp auquel sera affecté la formule entrée.
 * @param variable null pour un calcul, variable formelle pour une fonction
 * @constructor
 */
function EditeurValeurComplexe (app, input, indfin, value, variable) {
  this.app = app
  this.input = input
  this.indfin = indfin
  this.value = value
  this.variable = variable
}

EditeurValeurComplexe.prototype.validate = function () {
  var app = this.app
  var list = app.listePr
  var listePourConst = app.listePourConst
  var inderr = new Pointeur(0)
  var self = this
  var ch = $(this.input).val()
  if (CalcC.verifieSyntaxeComplexe(listePourConst, ch, inderr, this.indfin, this.variable)) {
    // var calc = new CalcC.ccbComp(ch, list, 0, ch.length - 1, this.variable, listePourConst) // Corrigé version 6.4.1
    var calc = CalcC.ccbComp(ch, list, 0, ch.length - 1, this.variable, listePourConst)
    this.value.donneCalcul(calc)
    try {
      list.initialiseNombreIterations()
      // Si on n'édite pas une formule de fonction on calcule le résultat et on le met dans this.res
      // pour pouvoir rajouter une condition sur le résultat
      if (this.variable === null) calc.resultatComplexe(false, this.value.valeurComplexe)
      return true
    } catch (e) {
      /* Modifié version 6.0
      inderr.setValue(ch.length);
      $(this.input).textrange('setcursor',ch.length);
      new AvertDlg(app, "ErrCalculNonExist", function() {
        self.input.focus();
        // Test ligne suivante des fois que j'aie oublié de déclarer un éditeur comme MtgInput
        if (self.input.marquePourErreur) self.input.marquePourErreur();
      });
      return false;
      */
      new AvertDlg(app, 'ErrCalculNonExist')
      return true // Version 6.0 : On accepte de créer un calcul qui n'existe pas
    }
  } else {
    var col = inderr.getValue()
    $(this.input).textrange('setcursor', col)
    new AvertDlg(app, 'ErrSyntaxe', function () {
      self.input.focus()
      // Test ligne suivante des fois que j'aie oublié de déclarer un éditeur comme MtgInput
      if (self.input.marquePourErreur) self.input.marquePourErreur()
    })
    return false
  }
}
