/*
 * Created by yvesb on 24/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const PaneListeReperes = require('./PaneListeReperes')
const AvertDlg = require('./AvertDlg')
const Pointeur = require('../types/Pointeur')
const MtgInput = require('../dialogs/MtgInput')
module.exports = EqDroiteDlg

/**
 * Dialogue de choix d'angle de rotation
 * @constructor
 * @param app La mtgApp propriétaire
 * @param dte La droite dont le coefficient directeur doit être changé
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annumlation
 */
function EqDroiteDlg (app, dte, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'eqDroiteDlg', callBackOK, callBackCancel)
  var self = this
  this.dte = dte
  this.modification = modification
  var list = app.listePr
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('EqDroite') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.input = new MtgInput({
    id: 'mtginput',
    size: 24
  })
  td.appendChild(this.input)
  this.input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var indmax = modification ? list.indexOf(dte) - 1 : list.longueur() - 1
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.input, true, indmax)
  tab.appendChild(paneBoutonsValFonc)
  tr = kernel.ce('tr')
  tab.appendChild(tr)
  this.paneListeRep = new PaneListeReperes(app, indmax)
  tr.appendChild(this.paneListeRep.getTab())

  if (modification) $(this.input).val(dte.equation.chaineCalculSansPar(['x', 'y']))
  this.create('DtParEq', 500)
}

EqDroiteDlg.prototype = new MtgDlg()

EqDroiteDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

EqDroiteDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  var inderr = new Pointeur(0)
  // Si l'équation est correcte, this.verifieFormules se charge de modifier la droite
  var valid = this.dte.verifieFormules($(this.input).val(), inderr)
  if (valid) {
    this.input.demarquePourErreur()
    this.dte.rep = this.paneListeRep.getSelectedRep()
    if (this.callBackOK !== null) this.callBackOK()
    if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
    this.destroy()
  } else {
    var col = inderr.getValue()
    $(this.input).textrange('setcursor', col)
    // $(this.input).focus();
    var self = this
    new AvertDlg(this.app, 'ErrSyntaxe', function () {
      self.input.focus()
      self.input.marquePourErreur()
    })
  }
}
