/*
 * Created by yvesb on 06/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const EditeurValeurComplexe = require('./EditeurValeurComplexe')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const CValeur = require('../objets/CValeur')
const CValeurComp = require('../objets/CValeurComp')
const AvertDlg = require('./AvertDlg')
const NatCal = require('../types/NatCal')
const PaneListeReperes = require('./PaneListeReperes')
const MtgInput = require('./MtgInput')
module.exports = FonctionDlg

/**
 * Dialogue de création ou modification d'une fonction réelle ou  complexe
 * @constructor
 * @param app La mtgApp propriétaire
 * @param fonc La fonction dont la formule doit être modifiée
 * @param modification {boolean} : true si le dialogue est ouvert pour modifier un calcul déjà existant
 * @param bReel {boolean} : true pour une foonction réelle et false pour un fonction complexe
 */
function FonctionDlg (app, fonc, modification, bReel, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'fonctiondlg', callBackOK, callBackCancel)
  var self = this
  this.fonc = fonc
  this.modification = modification
  this.bReel = bReel
  var list = app.listePr
  // La formule obtenue pour le calcul sera renvoyée dans un CValeur
  this.valeur = bReel ? new CValeur(list) : new CValeurComp(list)
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var tab1 = kernel.ce('table')
  td.appendChild(tab1)
  tr = kernel.ce('tr')
  tab1.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NomFonction') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputName = new MtgInput()
  td.appendChild(this.inputName)

  // Des éditeurs pour le nom de la ou les variables formelles
  var nomvar = fonc.variableFormelle()
  this.nbVar = nomvar.length
  var intitule = []
  if (nomvar.length === 1) intitule.push(kernel.getStr('Variable') + ' : ')
  else {
    for (var i = 0; i < nomvar.length; i++) {
      intitule.push(kernel.getStr('Variable') + ' ' + String(i + 1) + ' : ')
    }
  }
  for (i = 0; i < nomvar.length; i++) {
    tr = kernel.ce('tr')
    tab1.appendChild(tr)
    td = kernel.ce('td')
    tr.appendChild(td)
    label = kernel.ce('label')
    $(label).html(intitule[i])
    td.appendChild(label)
    td = kernel.ce('td')
    tr.appendChild(td)
    this['inputNomVar' + i] = new MtgInput()
    $(this['inputNomVar' + i]).val(nomvar[i])
    // if (modification) $(this["inputNomVar" + i]).attr("disabled", true);
    td.appendChild(this['inputNomVar' + i])
    this['editeurNomVar' + i] = new EditeurNomCalcul(app, false, this['inputNomVar' + i])
  }
  //
  tr = kernel.ce('tr')
  tab1.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Formule') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputFormula = new MtgInput({
    size: 50
  })
  td.appendChild(this.inputFormula)
  this.inputFormula.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  this.inputName.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      if ($(self.inputFormula).val() === '') self.inputFormula.focus()
      else {
        $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
        self.OK()
      }
    }
  }
  if (modification) {
    $(this.inputName).val(fonc.nomCalcul)
    this.inputName.setAttribute('disabled', true)
    $(this.inputFormula).val(fonc.chaineCalcul)
  }
  var indmax = modification ? app.listePr.indexOf(fonc) - 1 : list.longueur() - 1
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.inputFormula, bReel, indmax)
  tab1.appendChild(paneBoutonsValFonc)
  this.editeurName = new EditeurNomCalcul(app, !modification, this.inputName)
  // Si on est en train de créer une fonction et qu'il existe un repère dans la figure on rajoute une
  // checkbox proposant de tracer la courbe et un éditeur pour entre le nombre de points de la courbe
  if (!modification && fonc.estDeNatureCalcul(NatCal.NFonction)) {
    var rep = list.premierRepVis()
    if (rep !== null) {
      tr = kernel.ce('tr')
      tab.appendChild(tr)
      var tab2 = kernel.ce('table')
      tr.appendChild(tab2)
      tr = kernel.ce('tr')
      $(tr).css('vertical-align', 'middle')
      tab2.appendChild(tr)
      td = kernel.ce('td')
      tr.appendChild(td)
      label = kernel.ce('label', {
        for: 'cbc'
      })
      $(label).html(kernel.getStr('TracerCourbe') + ' : ')
      td.appendChild(label)
      td = kernel.ce('td')
      tr.appendChild(td)
      this.cbCourbe = kernel.ce('input', {
        id: 'cbc',
        type: 'checkBox'
      })
      this.cbCourbe.setAttribute('checked', 'checked')
      this.cbCourbe.onclick = function () {
        var b = $(self.cbCourbe).prop('checked')
        $(self.tabListeRep).css('visibility', b ? 'visible' : 'hidden')
        $(self.inputNbPts).css('visibility', b ? 'visible' : 'hidden')
        $(self.labelNbPts).css('visibility', b ? 'visible' : 'hidden')
      }
      td.appendChild(this.cbCourbe)
      td = kernel.ce('td')
      $(td).css('vertical-align', 'middle')
      tr.appendChild(td)
      this.paneListeRep = new PaneListeReperes(app, indmax)
      this.tabListeRep = this.paneListeRep.getTab()
      td.appendChild(this.tabListeRep)
      tr = kernel.ce('tr')
      tab.appendChild(tr)
      this.labelNbPts = kernel.ce('label')
      $(this.labelNbPts).html(kernel.getStr('nbPtsCourbe') + ' : ')
      tr.appendChild(this.labelNbPts)
      this.inputNbPts = kernel.ce('input', {
        type: 'text',
        size: 4
      })
      $(this.inputNbPts).val('500')
      tr.appendChild(this.inputNbPts)
    }
  }
  this.create(this.bReel ? 'Fonction' : 'FoncComp', 680)
}

FonctionDlg.prototype = new MtgDlg()

FonctionDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $(self.modification ? self.inputFormula : self.inputName).focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

FonctionDlg.prototype.OK = function () {
  var app = this.app
  var list = app.listePr
  var self = this
  // if (app.lastDlgId() !== this.id) return;
  // On regarde si l'entrée est correcte sur le plan syntaxique
  if (!this.modification) {
    if (!this.editeurName.validate()) return
  }
  for (var i = 0; i < this.nbVar; i++) {
    if (!this['editeurNomVar' + i].validate()) return
  }
  var ar = []
  for (var j = 0; j < this.nbVar; j++) ar.push($(this['inputNomVar' + j]).val())
  // Si les noms pour le calcul et les variables sont corrects on passe à la formule
  var ind = this.modification ? list.indexOf(this.fonc) - 1 : list.longueur() - 1
  var editeurFormule = this.bReel ? new EditeurvaleurReelle(app, this.inputFormula, ind, this.valeur, ar)
    : new EditeurValeurComplexe(app, this.inputFormula, ind, this.valeur, ar)
  if (editeurFormule.validate($(this.inputFormula).val())) {
    if (!self.modification) this.fonc.nomCalcul = $(this.inputName).val()
    this.fonc.chaineVariable = (this.nbVar === 1) ? ar[0] : ar
    // Si on propose de tracer la courbe, on vérifie que le nombre de points demandés est correct
    if (!this.modification) {
      if ($(self.cbCourbe).prop('checked')) {
        var nb = parseInt($(this.inputNbPts).val())
        if (nb < 4 || nb > 5000 || isNaN(nb)) {
          new AvertDlg(this.app, 'nbPtsCourbeMsg', function () {
            self.inputNbPts.focus()
          })
          return
        }
      }
    }
    this.fonc.calcul = this.valeur.calcul
    this.fonc.chaineCalcul = this.fonc.calcul.chaineCalculSansPar(this.fonc.variableFormelle())
    if (!this.modification) {
      ind = list.longueur()
      app.ajouteElement(this.fonc)
      if ($(this.cbCourbe).prop('checked')) {
        list.ajouteCourbeSurR(app, this.paneListeRep.getSelectedRep(), this.fonc, list.genereNomPourPointOuDroite('x'),
          list.genereNomPourCalcul('x', true), list.genereNomPourCalcul('y', true), nb, true, true, true)
        list.afficheTout(ind, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
      }
    }
    app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj'
      : this.bReel ? 'Fonction' : 'FoncComp')
    this.destroy()
    if (this.callBackOK !== null) this.callBackOK()
  } else this.inputFormula.focus()
}
