/*
 * Created by yvesb on 14/12/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const CValeur = require('../objets/CValeur')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const MtgInput = require('./MtgInput')
module.exports = HomothetieDlg

/**
 * Dialogue de choix de rapport d'homothétie
 * @constructor
 * @param app La mtgApp propriétaire
 * @param hom L'homothétie dont le rapport doit être changé
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annumlation
 */
function HomothetieDlg (app, hom, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'homothetieDlg', callBackOK, callBackCancel)
  var self = this
  this.hom = hom
  this.modification = modification
  // Un CValeur pour stocker la valeur choisie avant vérification
  var list = app.listePr
  this.valeurRap = new CValeur(list, 1)
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Rapport') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = new MtgInput({
    id: 'mtginput'
  })
  td.appendChild(input)
  input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var indmax = modification ? list.indexOf(hom) - 1 : list.longueur() - 1
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, input, true, indmax)
  tab.appendChild(paneBoutonsValFonc)
  this.editor = new EditeurvaleurReelle(app, input, indmax, this.valeurRap, null)
  if (modification) $(input).val(hom.rapport.calcul.chaineCalculSansPar())
  this.create('RapportHom', 400)
}

HomothetieDlg.prototype = new MtgDlg()

HomothetieDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

HomothetieDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  // On regarde si l'entrée est correcte sur le plan syntaxique
  var valid = this.editor.validate($('#mtginput').val())
  // Modifié version 6.4.8. On eccepte une homothétie de rapport nul
  if (valid) {
    this.hom.rapport = this.valeurRap
    this.destroy()
    if (this.callBackOK !== null) this.callBackOK()
    // if (this.modification) this.app.gestionnaire.enregistreFigureEnCours("ModifObj");
  }
}
