/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const PaneListeReperes = require('./PaneListeReperes')
const MtgInput = require('./MtgInput')
const EditeurNomCalcul = require('./EditeurNomCalcul')
module.exports = InfoRepDlg

/**
 * Dialogue Permettant de créer ou modifier un calcul prenant comme valeur l'abscisse à l'origine,
 * l'ordonnée à l'origine, l'unité en x ou l'unité en y d'un repère
 * @constructor
 * @param app La mtgApp propriétaire
 * @param calc L'objet à modifier du type CAbscisseOrigineRep, COrdonneeOrigineRep, CUnitexRep ou CUniteyRep
 * @param {int} info  Vatt 0 pour une abscisse à l'origine, 1 pour une ordonnée à l'origine,
 * 2 pour une unité en x et 2 pour une unité en y.
 * La seule chose qu'on peut changer dans cette boîte de dialogue si modification est true est le choix du repère
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annumlation
 */
function InfoRepDlg (app, calc, info, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'coefDirDlg', callBackOK, callBackCancel)
  this.calc = calc
  var self = this
  var ar = ['AbsOrRep', 'OrdOrRep', 'UnitexRep', 'UniteyRep']
  this.title = ar[info]
  this.modification = modification
  var list = app.listePr
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NomCalcul') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputName = new MtgInput()
  td.appendChild(this.inputName)
  this.inputName.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var indmax = modification ? list.indexOf(calc) - 1 : list.longueur() - 1
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  tr.appendChild(kernel.ce('td'))
  td = kernel.ce('td')
  tr.appendChild(td)
  this.paneListeRep = new PaneListeReperes(app, indmax)
  td.appendChild(this.paneListeRep.getTab())
  this.editeurName = new EditeurNomCalcul(app, !modification, this.inputName)
  if (modification) {
    $(this.inputName).val(calc.nomCalcul)
    $(this.inputName).attr('disabled', true)
  }
  this.create(this.title, 450)
}

InfoRepDlg.prototype = new MtgDlg()

InfoRepDlg.prototype.OK = function () {
  var calc = this.calc
  if (this.editeurName.validate()) {
    calc.rep = this.paneListeRep.getSelectedRep()
    if (!this.modification) {
      calc.nomCalcul = $(this.inputName).val()
      this.app.ajouteElement(calc)
    }
    this.app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : this.title)
    this.destroy()
  }
}
