/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const CValeur = require('../objets/CValeur')
const NatCal = require('../types/NatCal')
const CIntegrale = require('../objets/CIntegrale')
const AvertDlg = require('../dialogs/AvertDlg')
const MtgInput = require('./MtgInput')

module.exports = IntegraleDlg

/**
 * Dialogue de création ou modification d'une solution approchée d'équation
 * @constructor
 * @param app La mtgApp propriétaire
 * @param integ L'intégrale à modifier
 * @param modification {boolean} : true si le dialogue est ouvert pour modifier un calcul déjà existant
 * @param callBackOK Fonction éventuelle à appeler après validation
 * @param callBackCancel Fonction éventuelle à appeler après annulation
 */
function IntegraleDlg (app, integ, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'Intdlg', callBackOK, callBackCancel)
  this.integ = integ
  this.modification = modification
  var self = this
  var list = app.listePr
  var indmax = modification ? list.indexOf(integ) - 1 : list.longueur() - 1
  this.inf = list.listeParNatCal(app, NatCal.NFoncR1Var, indmax)
  // 3 CValeur pour stocker les valeurs a, b (bornes d'intégration) et n (nombre d'intervalles).
  this.valeura = new CValeur(list)
  this.valeurb = new CValeur(list)
  this.valeurn = new CValeur(list)

  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('IntegDlg1'))
  tr.appendChild(label)
  // Une ligne d'édition du nom du calcul
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var div = kernel.ce('div')
  tr.appendChild(div)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('NomCalcul') + ' : ')
  div.appendChild(label)
  // Editeur pour le nom du calcul
  this.inputName = new MtgInput({
    size: 12
  })
  div.appendChild(this.inputName)
  /// ////////////////////////////////////////////////////////////////////
  // Au centre un tableau contenant deux colonnes : A gauche la liste  //
  // pour le choix de la fonction et à droite un tableau de trois      //
  // pour les éditeurs de a, b et n                                    //
  /// ////////////////////////////////////////////////////////////////////
  tr = kernel.ce('table')
  tabPrincipal.appendChild(tr)
  var tabCentre = kernel.ce('table')
  tr.appendChild(tabCentre)
  var td = kernel.ce('td')
  tabCentre.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:150px'
  })
  $(caption).html(kernel.getStr('IntegDlg2'))
  td.appendChild(caption)
  this.select = kernel.ce('select', {
    size: 10, // Le nombre de lignes visibles par défaut
    style: 'width:150px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.inf.pointeurs[i] === integ.fonctionAssociee) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.select.appendChild(option)
  }
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tabCentre.appendChild(td)
  var tabCentreDroit = kernel.ce('table')
  td.appendChild(tabCentreDroit)
  tr = kernel.ce('tr')
  tabCentreDroit.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('SolEqDlg5'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  // Editeur pour la valeur de a
  this.inputa = new MtgInput({
    size: 24
  })
  td.appendChild(this.inputa)
  var paneBoutonsValFonca = new PaneBoutonsValFonc(this.app, this, this.inputa, true, indmax)
  tabCentreDroit.appendChild(paneBoutonsValFonca)

  tr = kernel.ce('tr')
  tabCentreDroit.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('SolEqDlg6'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  // Editeur pour la valeur de b
  this.inputb = new MtgInput({
    size: 24
  })
  td.appendChild(this.inputb)
  var paneBoutonsValFoncb = new PaneBoutonsValFonc(this.app, this, this.inputb, true, indmax)
  tabCentreDroit.appendChild(paneBoutonsValFoncb)

  tr = kernel.ce('tr')
  tabCentreDroit.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('IntegDlg3'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  // Editeur pour la valeur de b
  this.inputn = new MtgInput({
    size: 24
  })
  td.appendChild(this.inputn)
  var paneBoutonsValFoncn = new PaneBoutonsValFonc(this.app, this, this.inputn, true, indmax)
  tabCentreDroit.appendChild(paneBoutonsValFoncn)
  // En bas du tableau un champ d'info sur la fonction sélectionnée
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas = kernel.ce('table')
  tr.appendChild(tabBas)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('InfoFonc'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  var inputinf = kernel.ce('textarea', {
    id: 'mtginfo',
    disabled: 'true',
    cols: 40,
    rows: 3
  })
  tr.appendChild(inputinf)

  if (modification) {
    $(this.inputName).val(integ.nomCalcul)
    this.inputName.setAttribute('disabled', true)
    $(this.inputa).val(integ.a.chaineInfo())
    $(this.inputb).val(integ.b.chaineInfo())
    $(this.inputn).val(integ.n.chaineInfo())
  }
  this.editeurName = new EditeurNomCalcul(app, !modification, this.inputName)
  this.editeura = new EditeurvaleurReelle(app, this.inputa, indmax, this.valeura, null)
  this.editeurb = new EditeurvaleurReelle(app, this.inputb, indmax, this.valeurb, null)
  this.editeurn = new EditeurvaleurReelle(app, this.inputn, indmax, this.valeurn, null)
  this.onSelectChange()
  this.create('Integ', 550)
}

IntegraleDlg.prototype = new MtgDlg()

IntegraleDlg.prototype.onSelectChange = function () {
  var ind, el
  ind = this.select.selectedIndex
  if (ind === -1) return
  el = this.inf.pointeurs[ind]
  $('#mtginfo').text(el.infoHist())
}

IntegraleDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  if (!this.modification) {
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $(self.inputName).focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    }, 100)
  } else {
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $(self.select).focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    }, 100)
  }
}

IntegraleDlg.prototype.OK = function () {
  // On regarde si l'entrée est correcte sur le plan syntaxique
  if (!this.modification) {
    if (!this.editeurName.validate()) return
  }
  if (this.editeura.validate($(this.inputa).val()) && this.editeurb.validate($(this.inputb).val()) &&
    this.editeurn.validate($(this.inputn).val())) {
    var valn = this.valeurn.valeur
    if (valn > 0 && valn <= CIntegrale.valeurMaxiDen) {
      this.integ.nomCalcul = $(this.inputName).val()
      this.integ.fonctionAssociee = this.inf.pointeurs[this.select.selectedIndex]
      this.integ.a = this.valeura
      this.integ.b = this.valeurb
      this.integ.n = this.valeurn
      if (!this.modification) this.app.ajouteElement(this.integ)
      this.app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : 'Integ')
      this.destroy()
      if (this.callBackOK !== null) this.callBackOK()
    } else {
      this.inputn.marquePourErreur()
      new AvertDlg(this.app, 'Incorrect')
    }
  }
}
