/*
 * Created by yvesb on 06/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const CValeur = require('../objets/CValeur')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const AvertDlg = require('./AvertDlg')
const PaneAlignement = require('./PaneAlignement')
const PaneTaillePolice = require('./PaneTaillePolice')
const PaneModeAffichage = require('./PaneModeAffichage')
const MtgInput = require('./MtgInput')
module.exports = LieuObjetDlg

/**
 * Dialogue de choix d'angle de rotation
 * @constructor
 * @param app La mtgApp propriétaire
 * @param lieu Le lieu dont le nombre d'objets doit être changé
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annumlation
 */
function LieuObjetDlg (app, lieu, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'LieuObjetDlg', callBackOK, callBackCancel)
  var self = this
  this.lieu = lieu
  this.modification = modification
  this.estElementFinal = this.lieu.estElementFinal
  var list = app.listePr
  // Un CValeur pour stocker la valeur choisie avant vérification
  this.valeur = new CValeur(list, this.lieu.nombreTraces.valeur)
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  var tr2 = kernel.ce('tr')
  tabHaut.appendChild(tr2)
  var td = kernel.ce('td')
  tr2.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('lieuObjetDlg1'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr2.appendChild(td)
  var input = new MtgInput({
    id: 'mtginput'
  })
  $(input).val(this.valeur.valeur)
  td.appendChild(input)
  input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  this.input = input
  var indmax = modification ? list.indexOf(lieu) - 1 : list.longueur() - 1
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, input, true, indmax)
  tabHaut.appendChild(paneBoutonsValFonc)
  //
  this.editor = new EditeurvaleurReelle(app, input, indmax, this.valeur, null)
  if (modification) $(input).val(lieu.nombreTraces.calcul.chaineCalculSansPar())
  if (this.estElementFinal) {
    $(input).attr('disabled', true)
    $(paneBoutonsValFonc).css('visibility', 'hidden')
  }
  // Si on modifie un lieu d'objets d'ffichage (graduationde repère par exemple) on donne la possibilité de
  // changer certaines caractéristiques de l'affichage
  this.hasLowerPane = modification && lieu.estCapturableSouris()
  if (this.hasLowerPane) {
    var el = lieu.elementAssocie // C'est forcément un affichage de texte
    this.paneAlign = new PaneAlignement(app, el.alignementHorizontal, el.alignementVertical)
    tr = kernel.ce('tr')
    tab.appendChild(tr)
    var tableBas = kernel.ce('table')
    tr.appendChild(tableBas)
    td = kernel.ce('td', {
      valign: 'top'
    })
    tableBas.appendChild(td)
    td.appendChild(this.paneAlign.container)
    td = kernel.ce('td', {
      valign: 'top'
    })
    tableBas.appendChild(td)
    this.paneTaillePolice = new PaneTaillePolice(el.taillePolice)
    tableBas.appendChild(this.paneTaillePolice.container)
    td = kernel.ce('td', {
      valign: 'top'
    })
    tableBas.appendChild(td)
    this.paneModeAffichage = new PaneModeAffichage(this, false, el.effacementFond, el.encadrement, el.couleurFond)
    td.appendChild(this.paneModeAffichage.container)
  }
  this.create('LieuObj', 500)
}

LieuObjetDlg.prototype = new MtgDlg()

LieuObjetDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  // if (this.hasLowerPane) this.paneModeAffichage.init(); Doit être appelé avant qu'on donne le focus
  // et l'appel du focus soit être décalé via un setTimeout
  if (this.hasLowerPane) this.paneModeAffichage.init()
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    if (self.modification) $('#mtginput').blur() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    else $('#mtginput').focus()
  }, 100)
}

LieuObjetDlg.prototype.OK = function () {
  if (this.estElementFinal || this.editor.validate($('#mtginput').val())) {
    var self = this
    var va = this.valeur.valeur
    if ((va < 2) || (va > 100000) || (va !== Math.floor(va))) { new AvertDlg(this.app, 'Incorrect', function () { $('#mtginput').focus(); self.input.marquePourErreur() }) } else {
      if (!this.estElementFinal) this.lieu.nombreTraces = this.valeur
      // Si on modifie un lieu d'objets d'affichage (graduationde repère par exemple) on donne la possibilité de
      // changer certaines caractéristiques de l'affichage
      if (this.hasLowerPane) {
        var el = this.lieu.elementAssocie // C'est un affichage
        var elh = this.paneAlign.getHorAlign()
        var elv = this.paneAlign.getVerAlign()
        var taille = this.paneTaillePolice.getTaille()
        var effFond = this.paneModeAffichage.getEffFond()
        var encadrement = this.paneModeAffichage.getStyleEnc()
        var coulFond = this.paneModeAffichage.getColor()
        // Si l'alignement a changé il faut recréer le lieu d'objet entièrement
        var modif = (elh !== el.alignementHorizontal) || (elv !== el.alignementVertical) || (taille !== el.taillePolice) ||
          (effFond !== el.effacementFond) || (encadrement !== el.encadrement) || (coulFond !== el.couleurFond)
        el.alignementHorizontal = elh
        el.alignementVertical = elv
        el.taillePolice = taille
        el.effacementFond = effFond
        el.encadrement = encadrement
        el.couleurFond = coulFond
        if (modif) this.lieu.metAJour()
      }
      if (this.callBackOK !== null) this.callBackOK()
      if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
      this.destroy()
    }
  }
}
