/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const $ = require('jquery')
const kernel = require('../kernel/kernel')
const ListeMacros = require('./ListeMacros')
const NatCal = require('../types/NatCal')

module.exports = ListeObjNumPrConst

/**
 * Objet représentant une liste d'objets numériques utilisés dans la boîte de dialogue de choix d'objets
 * numériques sourves pour une construction.
 * Herite de certaines propriétés de ListeMacros
 * @constructor
 * @param app L'application propriétaire
 * @param bListeGauche true si c'est la liste de gauche de la boîte de dialogue qui présente les objets disponibles
 * sinon il s'agit de la liste de droite qui représente les objets numériques sélectionnés
 * @param listeSourcesCalculsChoisis Pour la liste de gauche pointe sur la liste de droite
 * @param callBackOnDblClick Fonction de callBack à appeler si on double clique sur un élément, seulement pour la liste de gauche
 */
function ListeObjNumPrConst (app, bListeGauche, listeSourcesCalculsChoisis, callBackOnDblClick = null) {
  this.app = app
  this.callBackOnDblClick = callBackOnDblClick

  this.select = kernel.ce('select', {
    size: 12, // Le nombre de lignes visibles par défaut
    style: 'width:220px'
  })
  if (bListeGauche) this.metAJourListe(listeSourcesCalculsChoisis)
  else {
    var list = app.listeSrcNG
    this.int = [] // Contiendra la liste des noms des éléments de la liste
    this.pointeurs = [] // Contiendra des pointeurs sur les macros correspondantes
    var prem = true
    for (var i = 0; i < list.longueur(); i++) {
      var el = list.get(i)
      var nom = el.getNom()
      this.int.push(nom)
      this.pointeurs.push(el)
      var option = kernel.ce('option', {
        class: 'mtgOption'
      })
      $(option).html(nom)
      this.select.appendChild(option)
      if (prem) {
        $(option).attr('selected', 'selected')
        prem = false
      }
    }
  }
}

ListeObjNumPrConst.prototype = new ListeMacros()

/**
 * metAJour la liste d'éléments disponibles comme éléments sources numériques pour une construction
 * sachant que listeSourcesCalculsChoisis contient la liste d'éléments déjà choisis.
 * @param listeSourcesCalculsChoisis
 */
ListeObjNumPrConst.prototype.metAJourListe = function (listeSourcesCalculsChoisis) {
  var i; var j; var el
  var app = this.app
  var list = app.listePr
  var prem = true
  this.retAll() // Appelé depuis ListeMacros
  for (i = 0; i < list.longueur(); i++) {
    var elb = list.get(i)
    if ((elb.estDeNatureCalcul(NatCal.NObjCalcPourSourcesProto)) && !elb.estElementIntermediaire() &&
        (listeSourcesCalculsChoisis.pointeurs.indexOf(elb) === -1)) {
      var valide = true
      for (j = 0; (j < listeSourcesCalculsChoisis.pointeurs.length) && valide; j++) {
        el = listeSourcesCalculsChoisis.pointeurs[j]
        valide = !el.depDe(elb) && !elb.depDe(el)
      }
      for (j = 0; (j < app.listeSrcG.longueur()) && valide; j++) {
        el = app.listeSrcG.get(j)
        valide = !el.depDe(elb) && !elb.depDe(el)
      }
      if (valide) {
        var nom = elb.getNom()
        this.int.push(nom)
        this.pointeurs.push(elb)
        var option = kernel.ce('option', {
          class: 'mtgOption'
        })
        $(option).html(nom)
        this.select.appendChild(option)
        if (this.callBackOnDblClick) option.ondblclick = this.callBackOnDblClick
        if (prem) {
          $(option).attr('selected', 'selected')
          prem = false
        }
      }
    }
  }
}

/**
 * Fonction insérant dans la liste devant l'élément sélectionné l'objet el
 * @param el
 */
ListeObjNumPrConst.prototype.insert = function (el) {
  var sel = this.select
  var ind = sel.selectedIndex
  if (sel.selectedIndex === -1) this.add(el)
  else {
    var nom = el.getNom()
    this.int.splice(ind, 0, nom)
    this.pointeurs.splice(ind, 0, el)
    var option = kernel.ce('option', {
      class: 'mtgOption'
    })
    $(option).html(nom)
    $(sel.options[ind]).before(option)
  }
}

/**
 * Fonction insérant au début de la liste la macro mac
 * @param el
 */
ListeObjNumPrConst.prototype.add = function (el) {
  var nom = el.getNom()
  this.int.push(nom)
  this.pointeurs.push(el)
  var option = kernel.ce('option', {
    class: 'mtgOption'
  })
  $(option).html(nom)
  this.select.appendChild(option)
  this.select.selectedIndex = this.select.length - 1
}
