/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MacroDlg = require('./MacroDlg')
const CSousListeObjets = require('../objets/CSousListeObjets')
const $ = require('jquery')

module.exports = MacroAppDispParAutDlg

/**
 * Dialogue de création d'une macro d'apparition ou de disparition d'objets à partir des objets d'une autre macro
 * @constructor
 * @param app La mtgApp propriétaire
 * @param bApparition true pour une macro d'apparition, false pour une macro de disparition
 * mac La macro d'apprition ou de disparition passée en paramètre
 * param callBackOK Fonction de callBack à appler àprès appui sur OK
 * param callBackOK Fonction de callBack à appler àprès appui sur Cancel
 */
function MacroAppDispParAutDlg (app, bApparition, mac, callBackOK, callBackCancel) {
  MacroDlg.call(this, app, mac, false, callBackOK, callBackCancel)
  this.bApparition = bApparition
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  // Ajout de l'éditeur d'intitulé //////////////////////////////
  this.addEditInt(tabPrincipal)
  /// /////////////////////////////////////////////////////////////
  // Ajout du pane d'information sur la macro //////////////////////
  this.addPaneInfo(tabPrincipal)
  /// ///////////////////////////////////////////////////////////////
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('MacAppDispDlg1'))
  tr.appendChild(label)
  this.inf = app.listePr.listeMacrosAvecListeModif(false)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  this.select = kernel.ce('select', {
    size: 8, // Le nombre de lignes visibles par défaut
    style: 'width:300px'
  })
  var self = this
  this.select.onchange = function () {
    self.onSelectChange()
  }

  tr.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    option.ondblclick = function () { self.OK() }
    this.select.appendChild(option)
  }

  /// ///////////////////////////////////////////////////////////////
  // En bas, ajout d'un tableau de trois colonnes contenant un
  // panneau de choix de mode d'alignement et un panneau de choix
  // de taille de police
  /// ///////////////////////////////////////////////////////////////
  this.addPaneBas(tabPrincipal)
  this.onSelectChange()

  this.create(bApparition ? 'MacAppParAut' : 'MacDispParAut', 500)
}

MacroAppDispParAutDlg.prototype = new MacroDlg()

MacroAppDispParAutDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  var app = this.app
  if ($('#inputint').val() !== '') {
    var selectedMac = this.inf.pointeurs[this.select.selectedIndex]
    var list = selectedMac.listeAssociee
    // Il ne faut pas affecter à la macro la même liste que celle choisie car sinon si on ajoute ou retire
    // des objets à la permière ils seront aussi rajoutés ou supprimés de la nouvelle.
    var newlist = new CSousListeObjets()
    for (var i = 0; i < list.longueur(); i++) newlist.add(list.get(i))
    this.mac.listeAssociee = newlist
    app.ajouteElement(this.mac)
    this.saveBasic()
    this.mac.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
    this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
    this.destroy()
    if (this.callBackOK !== null) this.callBackOK()
  } else this.avertIncorrect()
}

MacroAppDispParAutDlg.prototype.onSelectChange = function () {
  $('#ta').val(this.inf.pointeurs[this.select.selectedIndex].infoHist())
}
