/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MacroDlg = require('./MacroDlg')
const ListeMacros = require('./ListeMacros')
const $ = require('jquery')
const AvertDlg = require('./AvertDlg')

module.exports = MacSuiteMacDlg

/**
 * Dialogue de création ou modification d'une macro d'exécution d'une suite de macros
 * @constructor
 * @param app La mtgApp propriétaire
 * @param mac La macro qui doit être modifiée
 * @param modification {boolean} : true si le dialogue est ouvert pour modifier une macro déjà existante
 */
function MacSuiteMacDlg (app, mac, modification, callBackOK, callBackCancel) {
  MacroDlg.call(this, app, mac, modification, callBackOK, callBackCancel)
  var i; var btn
  var self = this
  var list = app.listePr
  var indmax = modification ? list.indexOf(mac) - 1 : list.longueur() - 1
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 2
  })
  this.appendChild(tabPrincipal)
  // Ajout de l'éditeur d'intitulé /////////////////////////////////
  this.addEditInt(tabPrincipal)
  /// ///////////////////////////////////////////////////////////////
  // Ajout du pane d'information sur la macro //////////////////////
  this.addPaneInfo(tabPrincipal)
  /// ///////////////////////////////////////////////////////////////
  // Ajout du pane pour présenter les deux listes de macros ////////
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabCentre = kernel.ce('table')
  tr.appendChild(tabCentre)
  tr = kernel.ce('tr')
  tabCentre.appendChild(tr)
  let td = kernel.ce('td')
  tr.appendChild(td)
  var tabCentreGauche = kernel.ce('table')
  td.appendChild(tabCentreGauche)
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  var tabCentreDroit = kernel.ce('table')
  td.appendChild(tabCentreDroit)
  tr = kernel.ce('tr')
  tabCentreGauche.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:220px'
  })
  $(caption).html(kernel.getStr('MacDispo'))
  td.appendChild(caption)
  this.listMacG = new ListeMacros(app, true, indmax, function () {
    self.addFin()
  }, false)
  td.appendChild(this.listMacG.select)
  var btns = ['Ins', 'AddDeb', 'AddFin']
  for (i = 0; i < 3; i++) {
    tr = kernel.ce('tr')
    tabCentreGauche.appendChild(tr)
    btn = kernel.ce('button', {
      tabindex: -1,
      style: 'width:220px;font-size:13px'
    })
    btn.owner = this
    btn.onclick = self['onBtn' + btns[i]]
    $(btn).html(kernel.getStr(btns[i]))
    tr.appendChild(btn)
  }
  tr = kernel.ce('tr')
  tabCentreDroit.appendChild(tr)
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:220px'
  })
  $(caption).html(kernel.getStr('MacAExec'))
  td.appendChild(caption)
  // Un tableau pour contenir la liste de droite et les flèches haut et bas
  var tabListe = kernel.ce('table')
  td.appendChild(tabListe)
  var trtab = kernel.ce('tr')
  tabListe.appendChild(trtab)
  var tdtab = kernel.ce('td')
  trtab.appendChild(tdtab)
  this.listMacD = new ListeMacros(app, false, indmax, null, false)
  if (modification) this.listMacD.addMacrosOf(mac)
  tdtab.appendChild(this.listMacD.select)
  btns = ['Ret', 'RetAll']
  for (i = 0; i < 2; i++) {
    tr = kernel.ce('tr')
    tabCentreDroit.appendChild(tr)
    btn = kernel.ce('button', {
      tabindex: -1,
      style: 'width:220px;font-size:13px'
    })
    btn.owner = this
    btn.onclick = self['onBtn' + btns[i]]
    $(btn).html(kernel.getStr(btns[i]))
    tr.appendChild(btn)
  }

  // Une dernière colonne pour contenir les flèches de déplacement
  tdtab = kernel.ce('td')
  $(tdtab).css('vertical-align', 'center')
  trtab.appendChild(tdtab)
  var tabIcones = kernel.ce('table')
  tdtab.appendChild(tabIcones)
  var ar = ['up', 'down']
  var af = [this.up, this.down]
  for (i = 0; i < 2; i++) {
    tr = kernel.ce('tr')
    tabIcones.appendChild(tr)
    td = kernel.ce('td')
    tr.appendChild(td)
    td.owner = this
    $(td).css('padding', '1px')
    $(td).html(app.local
      ? "<img src='" + app.localPath + 'images/' + ar[i] + ".png' width='22px' height='22px' />"
      : "<img src='" + app.imagePath + ar[i] + ".png' width='22px' height='22px' crossorigin='anonymous' />"
    )
    td.onclick = af[i]
  }

  /// ///////////////////////////////////////////////////////////////
  this.addPaneBas(tabPrincipal)
  this.create('MacSuiteMac', 520)
}

MacSuiteMacDlg.prototype = new MacroDlg()

MacSuiteMacDlg.prototype.OK = function () {
  var mac = this.mac
  var ch = $('#inputint').val()
  if (ch !== '') {
    var list = this.listMacD.getList()
    if (list.longueur() !== 0) {
      this.saveBasic()
      mac.listeAssociee = list
      if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
      this.destroy()
      if (this.callBackOK !== null) this.callBackOK()
    } else new AvertDlg(this.app, 'ErrNoMac')
  } else this.avertIncorrect()
}

MacSuiteMacDlg.prototype.up = function () {
  this.owner.listMacD.up()
}

MacSuiteMacDlg.prototype.down = function () {
  this.owner.listMacD.down()
}

MacSuiteMacDlg.prototype.onBtnIns = function () {
  var owner = this.owner
  owner.listMacD.insert(owner.listMacG.getSelectedItem())
}

MacSuiteMacDlg.prototype.onBtnAddDeb = function () {
  var owner = this.owner
  owner.listMacD.insertDeb(owner.listMacG.getSelectedItem())
}

MacSuiteMacDlg.prototype.onBtnAddFin = function () {
  this.owner.addFin()
}

MacSuiteMacDlg.prototype.addFin = function () {
  this.listMacD.add(this.listMacG.getSelectedItem())
}

MacSuiteMacDlg.prototype.onBtnRet = function () {
  var owner = this.owner
  owner.listMacD.ret()
}

MacSuiteMacDlg.prototype.onBtnRetAll = function () {
  var owner = this.owner
  owner.listMacD.retAll()
}
