/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MacroDlg = require('./MacroDlg')
const PaneFreq = require('./PaneFreq')
const PaneChoixVar = require('./PaneChoixVar')
const $ = require('jquery')

module.exports = MacTraceAutoVaDlg

/**
 * Dialogue de création ou modification d'une macro de trace automatique d'objets par variable
 * @constructor
 * @param app La mtgApp propriétaire
 * @param mac La macro qui doit être modifiée
 * @param modification {boolean} : true si le dialogue est ouvert pour modifier une macro déjà existante
 */
function MacTraceAutoVaDlg (app, mac, modification, callBackOK, callBackCancel) {
  MacroDlg.call(this, app, mac, modification, callBackOK, callBackCancel)
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 2
  })
  this.appendChild(tabPrincipal)
  // Ajout de l'éditeur d'intitulé /////////////////////////////////
  this.addEditInt(tabPrincipal)
  /// ///////////////////////////////////////////////////////////////
  // Ajout du pane d'information sur la macro //////////////////////
  this.addPaneInfo(tabPrincipal)
  /// ///////////////////////////////////////////////////////////////////////////////////////////////////
  // Au centre un tableau avec à gauche le choix de la variable et à droite le choix de la fréquence  //
  var tabCentre = kernel.ce('tab', {
    cellspacing: 2
  })
  tabPrincipal.appendChild(tabCentre)
  var tr = kernel.ce('tr')
  tabCentre.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  // Ajout du pane de choix de la variable qui un tab
  this.paneChoixVar = new PaneChoixVar(app, false)
  td.appendChild(this.paneChoixVar.container)
  // Ajout du panneau de choix de fréquence qui est un td /////////
  this.paneFreq = new PaneFreq(mac.frequenceAnimationMacro, true)
  tr.appendChild(this.paneFreq.container)
  /// ////////////////////////////////////////////////////////////////////////////////////////////////////
  this.addPaneBas(tabPrincipal)
  this.create('MacTraceAutoVa', 500)
}

MacTraceAutoVaDlg.prototype = new MacroDlg()

MacTraceAutoVaDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  var mac = this.mac
  var ch = $('#inputint').val()
  if (ch !== '') {
    mac.intitule = ch
    mac.frequenceAnimationMacro = this.paneFreq.getFreq()
    mac.variableAssociee = this.paneChoixVar.getVar()
    if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
    this.destroy()
    if (this.callBackOK !== null) this.callBackOK()
  } else this.avertIncorrect()
}
