/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const MtgInput = require('./MtgInput')
const AvertDlg = require('./AvertDlg')
const $ = require('./../global/jQuery-ui')
module.exports = ModifConstDlg

/**
 * Boîte de dialogue servant à modifier un prototype de la figure
 * @param app L'application MtgApp propriétaire
 * @param proto Le prototype à modifier
 * @param callBackOK Fonction de callBack à appeler si l'utilisateur valide par OK
 * @param callBackCancel Fonction de callBack à appeler si l'utilisateur annule
 * @constructor
 */
function ModifConstDlg (app, proto, callBackOK, callBackCancel = null) {
  MtgDlg.call(this, app, 'ModifConstDlg', callBackOK, callBackCancel)
  this.proto = proto
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var div = kernel.ce('div')
  tr.appendChild(div)
  var label = kernel.ce('label')
  div.appendChild(label)
  $(label).html(kernel.getStr('NomConst'))
  this.inputName = new MtgInput({
    id: 'nom',
    size: 36
  })
  $(this.inputName).val(proto.nom)
  div.appendChild(this.inputName)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Info'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var inputinf = kernel.ce('textarea', {
    id: 'comm', // Ne pas utiliser "ta" comme id car déjà utilisé dans le dialogue appelant
    cols: 70,
    rows: 12
  })
  tr.appendChild(inputinf)
  $(inputinf).val(proto.commentaire)
  this.create(kernel.getStr('infConst'), 600)
}

ModifConstDlg.prototype = new MtgDlg()

ModifConstDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  const self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $(self.inputName).focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

ModifConstDlg.prototype.OK = function () {
  var nom = $('#nom').val()
  if (nom !== '') {
    this.proto.nom = nom
    this.proto.commentaire = $('#comm').val()
    this.destroy()
    this.callBackOK()
  } else {
    var self = this
    new AvertDlg(this.app, 'Incorrect', function () {
      self.inputName.marquePourErreur()
      self.inputName.focus()
    })
  }
}
