/*
 * Created by yvesb on 22/06/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const CoefBarycentreDlg = require('./CoefBarycentreDlg')
const $ = require('./../global/jQuery-ui')
module.exports = ModificationBarycentreDlg

/**
 * Boîte de dialogue servant à modifier les coefficients de barycentre bar
 * @param app L'application MtgApp propriétaire
 * @param bar Le barycentre à modifier
 * @param callBackOK Fonction de callBack à appeler si l'utilisateur valide par OK
 * @param callBackCancel Fonction de callback appelée si l'utilisateur abandonne la modification
 * @constructor
 */
function ModificationBarycentreDlg (app, bar, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'ModificationBarDlg', callBackOK, callBackCancel)
  this.bar = bar
  var self = this
  var list = this.app.listePr
  // On crée un clone du barycentre. Au cas où l'utilisateur abandonnerait la boîte de dialogue par cancel,
  // on réaffectera à bar les coefficients de barClone
  this.barClone = bar.getClone(list, list)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var td = kernel.ce('td')
  tabPrincipal.appendChild(td)
  $(td).css('vertical-align', 'top')
  this.select = kernel.ce('select', {
    size: 6, // Le nombre de lignes visilbles par défaut
    style: 'width:250px'
  })
  td.appendChild(this.select)
  this.initList()

  td = kernel.ce('td')
  tabPrincipal.appendChild(td)
  $(td).css('vertical-align', 'middle')
  var btnModifier = kernel.ce('button', {
    tabindex: -1
  })
  td.appendChild(btnModifier)
  this.btnModifier = btnModifier
  btnModifier.onclick = function () { self.onBtnModifier() }
  $(btnModifier).html(kernel.getStr('Modifier'))
  // Création de la boîte de dialogue par jqueryui
  this.create('ModifBar', 450)
}
ModificationBarycentreDlg.prototype = new MtgDlg()

ModificationBarycentreDlg.prototype.initList = function () {
  var i, option, self, bar, noeud
  self = this
  for (i = this.select.options.length - 1; i >= 0; i--) this.select.remove(i)
  bar = this.bar
  for (i = 0; i < bar.col.length; i++) {
    option = kernel.ce('Option', {
      class: 'mtgOption',
      id: 'opt' + i // Pour mouvoir modifier directement un membre de la liste
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    noeud = bar.col[i]
    $(option).html(noeud.pointAssocie.nom + ' : ' + noeud.poids.chaineInfo())
    option.ondblclick = function () { self.onBtnModifier() }
    this.select.appendChild(option)
  }
}

ModificationBarycentreDlg.prototype.onBtnModifier = function () {
  var ind = this.select.selectedIndex
  var self = this
  new CoefBarycentreDlg(this.app, this.bar, this.bar.col[ind], true,
    function () { // Appelé si l'utilisateur a validé le nouveau coefficient.
      var noeud = self.bar.col[ind]
      $('#opt' + ind).html(noeud.pointAssocie.nom + ' : ' + noeud.poids.chaineInfo())
    },
    function () {
    }
  )
}

ModificationBarycentreDlg.prototype.onCancel = function () {
  // On remet les coefficients du barycentre à leur état initial
  var col = this.bar.col
  var colClone = this.barClone.col
  for (var i = 0; i < col.length; i++) {
    col[i].poids = colClone[i].poids
  }
  this.destroy()
}

ModificationBarycentreDlg.prototype.OK = function () {
  this.destroy()
  this.callBackOK()
}
