/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const CValeur = require('../objets/CValeur')
const EditeurValeurReelle = require('./EditeurValeurReelle')
const MtgInput = require('./MtgInput')

module.exports = NewRepDlg

/**
 * Dialogue de création d'une nouvelle figure munie d'un repère
 * @param app Application propriétaire
 * @param uniteAng L'unité d'angle d la figure (degré ou radian)
 * @param callBackOK Fonctio de callBack à appeler après avoir validé
 * @constructor
 */
function NewRepDlg (app, uniteAng, callBackOK) {
  MtgDlg.call(this, app, 'NewRepDlg', callBackOK)
  this.uniteAng = uniteAng
  // On crée 4 CValeur pour abscisses et ordonnées à l'origine et unités sur les axes
  var list = app.listePr
  this.absor = new CValeur(list, 0)
  this.ordor = new CValeur(list, 0)
  this.unitex = new CValeur(list, 1)
  this.unitey = new CValeur(list, 1)
  var tabPrincipal = kernel.ce('table', { cellspacing: 5 })
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tab2 = kernel.ce('table')
  tr.appendChild(tab2)
  tr = kernel.ce('tr')
  tab2.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  // Le bouton radio de choix de repère orthonormal
  var input = kernel.ce('input', {
    id: 'rdorthon',
    type: 'radio',
    name: 'mtgtyperep'
  })
  input.setAttribute('checked', 'checked') // Repère orthonormal coché par défaut
  td.appendChild(input)
  td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label', {
    for: 'rdorthon'
  })
  $(label).html(kernel.getStr('NewRepDlg1'))
  td.appendChild(label)

  td = kernel.ce('td')
  tr.appendChild(td)
  // Le bouton radio de choix de repère orthogonal
  input = kernel.ce('input', {
    id: 'rdorthog',
    type: 'radio',
    name: 'mtgtyperep'
  })
  td.appendChild(input)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdorthog'
  })
  $(label).html(kernel.getStr('NewRepDlg2'))
  td.appendChild(label)

  td = kernel.ce('td')
  tr.appendChild(td)
  // Le bouton radio de choix de repère oblique quelconque
  input = kernel.ce('input', {
    id: 'rdobli',
    type: 'radio',
    name: 'mtgtyperep'
  })
  td.appendChild(input)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdobli'
  })
  $(label).html(kernel.getStr('NewRepDlg3'))
  td.appendChild(label)

  td = kernel.ce('td')
  tr.appendChild(td)
  // Le bouton radio de choix de repère oblique normé
  input = kernel.ce('input', {
    id: 'rdobinor',
    type: 'radio',
    name: 'mtgtyperep'
  })
  td.appendChild(input)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdobinor'
  })
  $(label).html(kernel.getStr('NewRepDlg4'))
  td.appendChild(label)

  // 3 checkbox radio de choix de type de quadrillage
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tab3 = kernel.ce('table')
  tr.appendChild(tab3)
  td = kernel.ce('td')
  tab3.appendChild(td)
  // Le bouton radio de choix de quadrillage vertical
  input = kernel.ce('input', {
    id: 'rdquadver',
    type: 'checkBox'
  })
  input.setAttribute('checked', 'checked')
  td.appendChild(input)
  td = kernel.ce('td')
  tab3.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdquadver'
  })
  $(label).html(kernel.getStr('NewRepDlg5'))
  td.appendChild(label)

  // quadrillé
  td = kernel.ce('td')
  tab3.appendChild(td)
  // Le bouton radio de choix de quadrillage horizontal
  input = kernel.ce('input', {
    id: 'rdquadhor',
    type: 'checkBox'
  })
  input.setAttribute('checked', 'checked')
  td.appendChild(input)
  td = kernel.ce('td')
  tab3.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdquadhor'
  })
  $(label).html(kernel.getStr('NewRepDlg6'))
  td.appendChild(label)

  // Le bouton radio de choix de quadrillage pointille
  td = kernel.ce('td')
  tab3.appendChild(td)
  // Le bouton radio de choix de repère orthonormal
  input = kernel.ce('input', {
    id: 'rdquadpoint',
    type: 'checkBox'
  })
  td.appendChild(input)
  td = kernel.ce('td')
  tab3.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdquadpoint'
  })
  $(label).html(kernel.getStr('NewRepDlg7'))
  td.appendChild(label)

  // Quatre éditeurs de formule pour l'abscisse à l'origine, l'ordonnée à l'origine et les unités
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tab5 = kernel.ce('table')
  tr.appendChild(tab5)
  tr = kernel.ce('tr')
  tab5.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('OrAbs'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputabsor = new MtgInput({
    id: 'inputabsor'
  })
  td.appendChild(this.inputabsor)
  $(this.inputabsor).val(0)

  tr = kernel.ce('tr')
  tab5.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('OrOrd'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputordor = new MtgInput({
    id: 'inputordor'
  })
  td.appendChild(this.inputordor)
  $(this.inputordor).val(0)

  tr = kernel.ce('tr')
  tab5.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Unitex'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputunitex = new MtgInput({
    id: 'inputunitex'
  })
  td.appendChild(this.inputunitex)
  $(this.inputunitex).val(1)

  tr = kernel.ce('tr')
  tab5.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Unitey'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputunitey = new MtgInput({
    id: 'inputunitey'
  })
  td.appendChild(this.inputunitey)
  $(this.inputunitey).val(1)

  // Boutons de choix de type de graduation
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tab4 = kernel.ce('table')
  tr.appendChild(tab4)
  td = kernel.ce('td')
  tab4.appendChild(td)
  // Le bouton radio de choix de repère sans graduation
  input = kernel.ce('input', {
    id: 'rdsansgr',
    type: 'radio',
    name: 'mtggrad'
  })
  td.appendChild(input)
  td = kernel.ce('td')
  tab4.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdsansgr'
  })
  $(label).html(kernel.getStr('NewRepDlg12'))
  td.appendChild(label)

  td = kernel.ce('td')
  tab4.appendChild(td)
  // Le bouton radio de choix de graduation simple
  input = kernel.ce('input', {
    id: 'rdgrsimple',
    type: 'radio',
    name: 'mtggrad'
  })
  td.appendChild(input)
  input.setAttribute('checked', 'checked') // Gradution simple par défaut
  td = kernel.ce('td')
  tab4.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdgrsimple'
  })
  $(label).html(kernel.getStr('NewRepDlg13'))
  td.appendChild(label)

  kernel.ce('td')
  tab4.appendChild(td)
  // Le bouton radio de choix de graduation pour trigo
  input = kernel.ce('input', {
    id: 'rdgrtrig',
    type: 'radio',
    name: 'mtggrad'
  })
  td.appendChild(input)
  td = kernel.ce('td')
  tab4.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdgrtrig'
  })
  $(label).html(kernel.getStr('NewRepDlg14'))
  td.appendChild(label)

  // On rajoute un checkBox
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  input = kernel.ce('input', {
    id: 'cbavecvect',
    type: 'checkBox'
  })
  tr.appendChild(input)
  label = kernel.ce('label', {
    for: 'cbavecvect'
  })
  $(label).html(kernel.getStr('NewRepDlg15'))
  tr.appendChild(label)
  // Pour les qautre lignes suivantes 1 comme avant dernier paramètre car une fois
  // La liste effacée, on rajoute la constante pi et ces 4 valeurs ne peuve,t dépendre que de pi
  this.editororx = new EditeurValeurReelle(app, this.inputabsor, 1, this.absor, null)
  this.editorory = new EditeurValeurReelle(app, this.inputordor, 1, this.ordor, null)
  this.editorux = new EditeurValeurReelle(app, this.inputunitex, 1, this.unitex, null)
  this.editoruy = new EditeurValeurReelle(app, this.inputunitey, 1, this.unitey, null)
  // Création de la boîte de dialogue par jqueryui
  this.create('NewRepDlg', 600)
}

NewRepDlg.prototype = new MtgDlg()

NewRepDlg.prototype.OK = function () {
  var app = this.app
  if (this.editororx.validate($('#inputabsor').val())) {
    if (this.editorory.validate($('#inputordor').val())) {
      if (this.editorux.validate($('#inputunitex').val())) {
        if (this.editoruy.validate($('#inputunitey').val())) {
          app.listePr.deleteComponents()
          app.listePr.retireTout()
          app.retireTout()
          app.resetDoc()
          // app.listePr vient de changer
          var list = app.listePr
          app.prepareTracesEtImageFond()
          list.uniteAngle = this.uniteAng
          list.ajouteConstantePi()
          app.creeCommentaireDesignation()
          var quadver = $('#rdquadver').prop('checked')
          var quadhor = $('#rdquadhor').prop('checked')
          var quadpoint = $('#rdquadpoint').prop('checked')
          var avecvect = $('#cbavecvect').prop('checked')
          var avecgrad = !$('#rdsansgr').prop('checked')
          if ($('#rdorthon').prop('checked')) {
            list.ajouteRepereOrthonormal(app.dimf, quadhor, quadver, quadpoint, 'O', 'I', 'J', avecvect ? 'i' : '',
              avecvect ? 'j' : '', this.absor, this.ordor, this.unitex, this.unitey, avecgrad)
          } else {
            if ($('#rdorthog').prop('checked')) {
              list.ajouteRepereOrthogonal(app.dimf, quadhor, quadver, quadpoint, 'O', 'I', 'J', avecvect ? 'i' : '',
                avecvect ? 'j' : '', this.absor, this.ordor, this.unitex, this.unitey, avecgrad)
            } else {
              if ($('#rdobli').prop('checked')) {
                list.ajouteRepereOblique(app.dimf, quadhor, quadver, quadpoint, 'O', 'I', 'J', avecvect ? 'i' : '',
                  avecvect ? 'j' : '', this.absor, this.ordor, this.unitex, this.unitey, avecgrad)
              } else {
                list.ajouteRepereObliqueNorme(app.dimf, quadhor, quadver, quadpoint, 'O', 'I', 'J',
                  avecvect ? 'i' : '', avecvect ? 'j' : '', this.absor, this.ordor, this.unitex, this.unitey, avecgrad)
              }
            }
          }
          if ($('#rdgrsimple').prop('checked')) app.creeGrad('GraduationAxesRepere')
          else if ($('#rdgrtrig').prop('checked')) {
            app.creeGrad('GraduationReperePourTrigo')
            list.uniteAngle = kernel.uniteAngle.radian
          }
          app.updateToolsToolBar()
          app.calculateAndDisplay(false)
          if (app.cadre !== null) app.createCadre(app.widthCadre, app.heightCadre)
          app.gestionnaire.enregistreFigureEnCours('NewFig')
          // On met à jour les icônes de la barre horizontale
          app.updateToolbar()
          this.callBackOK()
          this.destroy()
        } else this.inputunitey.focus()
      } else this.inputunitex.focus()
    } else this.inputordor.focus()
  } else {
    this.inputabsor.focus()
  }
}
