/*
 * Created by yvesb on 04/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const PaneListeReperes = require('./PaneListeReperes')
const MtgInput = require('../dialogs/MtgInput')
module.exports = NomMesureDlg

/**
 * Fonction appelée pour valider le nom d'une mesure. Si bListeRep est true, alors la boîte de dialogue contient
 * une liste de repères parmis lesquels l'utilisateur doit chosiir, ind étabt le dernier indice permis pour les repères
 * @constructor
 * @param app L'application propriétaire
 * @param title Le titre de la boîte de dialogue
 * @param callBackOK Fonction appelée si l'utilisateur clique sur OK :
 * Si bListeRep est absent ou false : un seul paramètre pour la fonction qui est la chaîne entrée dans l'éditeur
 * Sinon un deuxième paramèter qui contient le repère choisi.
 * @param callBackCancel Fonction à appeler si l'utilisateur annule.
 * @param bListeRep {boolean} : true si l'utilisateur doit choisir un repère dans une liste
 */
function NomMesureDlg (app, title, callBackOK, callBackCancel, bListeRep = false) {
  MtgDlg.call(this, app, 'nomMesuredlg', callBackOK, callBackCancel)
  this.bListeRep = bListeRep
  var self = this
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NomMesure') + ' : ')
  td.appendChild(label)
  this.inputName = new MtgInput()
  td.appendChild(this.inputName)

  if (bListeRep) {
    tr = kernel.ce('tr')
    tab.appendChild(tr)
    this.paneListeRep = new PaneListeReperes(app, app.listePr.longueur() - 1)
    tr.appendChild(this.paneListeRep.getTab())
  }

  this.inputName.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }

  this.editeurName = new EditeurNomCalcul(app, true, this.inputName)
  this.create(title, 400)
}

NomMesureDlg.prototype = new MtgDlg()

NomMesureDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $(self.inputName).focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

NomMesureDlg.prototype.onCancel = function () {
  this.destroy()
  this.callBackCancel()
}

NomMesureDlg.prototype.OK = function () {
  // Remplacé par la ligne suivante. Si un message d'avertissement est en cours, on ne tient pas compte de OKs
  // if (this.app.lastDlgId() !== this.id) return;
  // On regarde si l'entrée est correcte sur le plan syntaxique
  var st = $(this.inputName).val()
  if (this.editeurName.validate()) {
    this.destroy()
    if (this.bListeRep) this.callBackOK(st, this.paneListeRep.getSelectedRep())
    else this.callBackOK(st)
  }
}
