/*
 * Created by yvesb on 16/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const kernelAdd = require('../kernel/kernelAdd')
const MtgDlg = require('./MtgDlg')
const OptionsAnimDlg = require('./OptionsAnimDlg')
const ImageFondDlg = require('./ImageFondDlg')
const ConfirmDlg = require('./ConfirmDlg')
const ChoixOutilsDlg = require('./ChoixOutilsDlg')
const $ = require('./../global/jQuery-ui')
const CMathGraphDoc = require('../objetsAdd/CMathGraphDocAdd')
const MtgInput = require('./MtgInput')
const EditeurConst = require('./EditeurConst')
const AboutDlg = require('./AboutDlg')
const TextFigDlg = require('./TextFigDlg')
const ChoixLangDlg = require('./ChoixLangDlg')
const CoulFondDlg = require('./CouleurFondDlg')
const ChoixEltsFigesDlg = require('./ChoixEltsFigesDlg')
const ChoixMacDemDlg = require('./ChoixMacDemDlg')
module.exports = OptionsFigDlg

function OptionsFigDlg (app, callBackOK) {
  MtgDlg.call(this, app, 'OptionsFigDlg', callBackOK)
  let cb
  var self = this
  /// ////////////////////////////////////////////////////////////////////////////////////////////////
  // On crée un nouveau document pour stocker les choix d'outils au cas où l'utilateur sorte de la
  // boîte de dialogue sans valider
  this.doc = new CMathGraphDoc('doc')
  this.doc.setIdMenusFromDoc(app.doc)
  /// ////////////////////////////////////////////////////////////////////////////////////////////////

  var list = app.listePr
  // On cherche d'abord si on utilise ou non un niveau prédéfini
  for (var niv = 0; niv < 4; niv++) {
    if (app.level === app.levels[niv]) break
  }
  // Test suivant car on peut avoir chargé un niveau personnalisé
  if (niv < 4) this.niv = niv; else this.niv = 3
  // this.niv vaut 4 si la figure utilise pas une figure spécifique pour le niveau d'utilisation
  // Dans ce cas on ne donne pas la possibilité de changer de niveau d'utilisation
  //
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabUnite = kernel.ce('table')
  tr.appendChild(tabUnite)
  tr = kernel.ce('tr')
  tabUnite.appendChild(tr)
  var div = kernel.ce('div')
  tr.appendChild(div)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('unAng'))
  div.appendChild(label)
  var radioDegre = kernel.ce('input', {
    type: 'radio',
    id: 'degre',
    name: 'unite'
  })
  div.appendChild(radioDegre)
  if (list.uniteAngle === kernel.uniteAngle.degre) $('#degre').attr('checked', 'checked')
  label = kernel.ce('label', {
    for: 'degre'
  })
  $(label).html(kernel.getStr('Degre'))
  div.appendChild(label)
  label = kernel.ce('label', {
    for: 'radian'
  })
  var radioRadian = kernel.ce('input', {
    type: 'radio',
    id: 'radian',
    name: 'unite',
    style: 'margin-left:15px'
  })
  div.appendChild(radioRadian)
  if (list.uniteAngle === kernel.uniteAngle.radian) $('#radian').attr('checked', 'checked')
  $(label).html(kernel.getStr('Radian'))
  div.appendChild(label)
  /// ///////////////////////////////////////////////////////////////////////////////////////
  // Quatre boutons radio pour le choix du niveau d'id niv0 à niv3                        //
  /// ///////////////////////////////////////////////////////////////////////////////////////
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabNiveau = kernel.ce('table')
  tr.appendChild(tabNiveau)
  for (niv = 0; niv < 4; niv++) {
    if (niv % 2 === 0) {
      tr = kernel.ce('tr')
      tabNiveau.appendChild(tr)
    }
    var td = kernel.ce('td')
    tr.appendChild(td)
    var rad = kernel.ce('input', {
      type: 'radio',
      id: 'niv' + niv,
      name: 'niv'
    })
    td.appendChild(rad)
    if (this.niv === niv) $(rad).attr('checked', 'checked')
    label = kernel.ce('label', {
      for: 'niv' + niv
    })
    $(label).html(kernel.getStr('niv' + niv))
    td.appendChild(label)
  }
  if (app.electron) {
    // Pour la version electron on propose de redémarrer avec le niveau choisi
    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    div = kernel.ce('div')
    tr.appendChild(div)
    cb = kernel.ce('input', {
      type: 'checkbox',
      id: 'cbstartlevel'
    })
    div.appendChild(cb)
    label = kernel.ce('label', {
      for: 'cbstartlevel'
    })
    $(label).html(kernel.getStr('NivDem'))
    div.appendChild(label)
    // Pour la version electron on propose de redémarrer en mode dys
    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    div = kernel.ce('div')
    tr.appendChild(div)
    cb = kernel.ce('input', {
      type: 'checkbox',
      id: 'cbdys'
    })
    div.appendChild(cb)
    label = kernel.ce('label', {
      for: 'cbdys'
    })
    $(label).html(kernel.getStr('DysDem'))
    div.appendChild(label)
    if (app.dys) $(cb).attr('checked', 'checked')
    // Pour la version electron on peut choisir la figure de démarrage. 3 choix possibles
    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    var tabStart = kernel.ce('table')
    tr.appendChild(tabStart)
    tr = kernel.ce('tr')
    tabStart.appendChild(tr)
    td = kernel.ce('td')
    tr.appendChild(td)
    label = kernel.ce('label')
    $(label).html(kernel.getStr('start'))
    td.appendChild(label)
    var choix = ['frameGrid', 'frameDotted', 'unity']
    for (var i = 0; i < 3; i++) {
      td = kernel.ce('td')
      tr.appendChild(td)
      var rd = kernel.ce('input', {
        type: 'radio',
        id: 'rdst' + i,
        name: 'start'
      })
      if (app.pref_StartFig === choix[i]) $(rd).attr('checked', 'checked')
      td.appendChild(rd)
      label = kernel.ce('label')
      $(label).html(kernel.getStr('start' + i))
      td.appendChild(label)
    }
  }
  /// /////////////////////////////////////////////////////////////////////////////////////////////
  // Une checkBox pour personnaliser les outils disponibles. Si un choix a déjà été fait        //
  // cette case est cochée et le bouton de choix des outils est visible.                        //
  // Sinon le bouton de choix des outils n'apparait que si on coche la case                     //
  // Toute action sur la checkBox réinitialise les outils à aucun outil interdit                //
  /// /////////////////////////////////////////////////////////////////////////////////////////////
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  cb = kernel.ce('input', {
    type: 'checkbox',
    id: 'cbtools'
  })
  if (app.doc.hasOwnTools()) $(cb).attr('checked', 'checked')
  div.appendChild(cb)
  cb.onclick = function () {
    var checked = $('#cbtools').prop('checked')
    $('#div').css('display', checked ? 'inline' : 'none')
    if (!checked) self.doc.setIdMenus(app, true, [], 3)
  }
  label = kernel.ce('label', {
    for: 'cbtools'
  })
  $(label).html(kernel.getStr('OutilsPer'))
  div.appendChild(label)
  /// /////////////////////////////////////////////////////////////////////////////////////////////
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div', {
    id: 'div'
  })
  tr.appendChild(div)
  // var btnTools = kernelAdd.getButtonArrow(app, "tools");
  var btnTools = kernel.ce('button', {
    style: 'margin-left:20px;'
  })
  $(btnTools).html(kernel.getStr('ChoixOutils'))
  btnTools.onclick = function () {
    new ChoixOutilsDlg(self.app, self.doc, self.getSelectedNiv())
  }
  div.appendChild(btnTools)
  $(div).css('display', app.doc.hasOwnTools() ? 'inline' : 'none')

  /// ////////////////////////////////////////////////////////////////////////////////////////////////
  // Une checkBox pour afficher ou non directement les mesures de longueur et d'angle sur la figure
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  cb = kernel.ce('input', {
    type: 'checkbox',
    id: 'cbmes'
  })
  if (app.pref_DispMeas) $(cb).attr('checked', 'checked')
  div.appendChild(cb)
  label = kernel.ce('label', {
    for: 'cbmes'
  })
  $(label).html(kernel.getStr('DispMeas'))
  div.appendChild(label)
  /// /////////////////////////////////////////////////////////////////////////////////////////////
  // Une checkBox pour donner la possibilité d'afficher un cadre grisé de dimensions données
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  cb = kernel.ce('input', {
    type: 'checkbox',
    id: 'cbcadre'
  })
  div.appendChild(cb)
  cb.onclick = function () {
    self.onClickDim()
  }
  if (app.cadre !== null) $('#cbcadre').attr('checked', 'checked')
  label = kernel.ce('label', {
    for: 'cbcadre'
  })
  $(label).html(kernel.getStr('AffCad'))
  div.appendChild(label)
  /// /////////////////////////////////////////////////////////////////////////////////////////////
  // Une checkBox pour montrer ou cacher l'image de fond s'il y en a une
  if (app.doc.imageFond !== null) {
    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    div = kernel.ce('div')
    tr.appendChild(div)
    cb = kernel.ce('input', {
      type: 'checkbox',
      id: 'cbimfvis'
    })
    div.appendChild(cb)
    if (app.doc.imageFondVisible) $(cb).attr('checked', true)
    cb.onclick = function () {
      var b = $('#cbimfvis').prop('checked')
      $(self.app.gImageFond).css('visibility', b ? 'visible' : 'hidden')
      self.app.doc.imageFondVisible = b
    }
    label = kernel.ce('label', {
      for: 'cbimfvis'
    })
    $(label).html(kernel.getStr('ImFondVis'))
    div.appendChild(label)
  }

  // Dessous deux champs d'édition pour largeur et hauteur du cadre désiré
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div', {
    id: 'divdim'
  })
  tr.appendChild(div)
  label = kernel.ce('label', {
    style: 'margin-left: 20px;'
  })
  $(label).html(kernel.getStr('Larg') + ' : ')
  div.appendChild(label)
  var w = Math.floor(app.dimf.x)
  var inputw = new MtgInput({
    id: 'inpw',
    size: 4
  })
  div.appendChild(inputw)
  if (app.cadre !== null) $(inputw).val(app.widthCadre)
  label = kernel.ce('label')
  $(label).css('font-size', '13px')
  $(label).html('(10 ' + kernel.getStr('a') + ' ' + w + ')')
  div.appendChild(label)

  label = kernel.ce('label', {
    style: 'margin-left: 20px;'
  })
  $(label).html(kernel.getStr('Haut') + ' : ')
  div.appendChild(label)
  var h = Math.floor(app.dimf.y)
  var inputh = new MtgInput({
    id: 'inph',
    size: 4
  })
  div.appendChild(inputh)
  if (app.cadre !== null) $(inputh).val(app.heightCadre)
  label = kernel.ce('label')
  $(label).css('font-size', '13px')
  $(label).html('(10 ' + kernel.getStr('a') + ' ' + h + ')')
  div.appendChild(label)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('coefMult'))
  div.appendChild(label)
  var inputCoef = new MtgInput({
    id: 'inpcoef',
    size: 4
  })
  div.appendChild(inputCoef)
  $('#inpcoef').val(app.pref_coefMult)

  this.editw = new EditeurConst(app, inputw, 10, w)
  this.edith = new EditeurConst(app, inputh, 10, h)
  if (app.cadre === null) this.onClickDim()

  this.editcoef = new EditeurConst(app, inputCoef, 0.25, 4, false)

  /// /////////////////////////////////////////////////////////////////////////////////////////////
  // En bas on rajoute des items avec des flèches pour ouvrir d'autres boîtes de dialogue
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas = kernel.ce('table')
  tr.appendChild(tabBas)

  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  var btn = kernelAdd.getButtonArrow(app, 'figes')
  btn.onclick = function () {
    new ChoixEltsFigesDlg(self.app)
  }
  tr.appendChild(btn)
  label = kernel.ce('label', {
    class: 'labelarrow',
    for: 'figes'// Pour qu'on puisse activer l'outil en cliquant sur le label
  })
  $(label).html(kernel.getStr('EltsFiges'))
  tr.appendChild(label)

  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  btn = kernelAdd.getButtonArrow(app, 'anim')
  btn.onclick = function () {
    new OptionsAnimDlg(self.app, null)
  }
  tr.appendChild(btn)
  label = kernel.ce('label', {
    class: 'labelarrow',
    for: 'anim'// Pour qu'on puisse activer l'outil en cliquant sur le label
  })
  $(label).html(kernel.getStr('OptionsAnim'))
  tr.appendChild(label)

  if (app.doc.imageFond === null) {
    tr = kernel.ce('tr')
    tabBas.appendChild(tr)
    btn = kernelAdd.getButtonArrow(app, 'imFond')
    btn.onclick = function () {
      new ImageFondDlg(self.app, function () {
        self.destroy()
      })
    }
    tr.appendChild(btn)
    label = kernel.ce('label', {
      class: 'labelarrow',
      for: 'imFond'// Pour qu'on puisse activer l'outil en cliquant sur le label
    })
    $(label).html(kernel.getStr('ChoixImFond'))
    tr.appendChild(label)
  } else {
    tr = kernel.ce('tr')
    tabBas.appendChild(tr)
    btn = kernelAdd.getButtonArrow(app, 'imFond')
    btn.onclick = function () {
      new ConfirmDlg(self.app, 'AvertNonAnnul', function () {
        self.app.deleteImageFond()
        self.destroy()
      })
    }
    tr.appendChild(btn)
    label = kernel.ce('label', {
      class: 'labelarrow',
      for: 'imFond'// Pour qu'on puisse activer l'outil en cliquant sur le label
    })
    $(label).html(kernel.getStr('DelImFond'))
    tr.appendChild(label)
  }

  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  btn = kernelAdd.getButtonArrow(app, 'couFond')
  btn.onclick = function () {
    new CoulFondDlg(self.app, function () {
      self.destroy()
    })
  }
  tr.appendChild(btn)
  label = kernel.ce('label', {
    class: 'labelarrow',
    for: 'couFond'// Pour qu'on puisse activer l'outil en cliquant sur le label
  })
  $(label).html(kernel.getStr('CouFond'))
  tr.appendChild(label)

  if (app.electron) {
    tr = kernel.ce('tr')
    tabBas.appendChild(tr)
    btn = kernelAdd.getButtonArrow(app, 'choixL')
    btn.onclick = function () {
      new ChoixLangDlg(self.app)
    }
    tr.appendChild(btn)
    label = kernel.ce('label', {
      class: 'labelarrow',
      for: 'choixL'// Pour qu'on puisse activer l'outil en cliquant sur le label
    })
    $(label).html(kernel.getStr('ChoixLang'))
    tr.appendChild(label)
  }

  if (list.listeMacPourDem().noms.length > 1) {
    tr = kernel.ce('tr')
    tabBas.appendChild(tr)
    btn = kernelAdd.getButtonArrow(app, 'macdem')
    btn.onclick = function () {
      new ChoixMacDemDlg(self.app)
    }
    tr.appendChild(btn)
    label = kernel.ce('label', {
      class: 'labelarrow',
      for: 'macdem'// Pour qu'on puisse activer l'outil en cliquant sur le label
    })
    $(label).html(kernel.getStr('ChoixMacDem'))
    tr.appendChild(label)
  }

  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  btn = kernelAdd.getButtonArrow(app, 'TextFig')
  btn.onclick = function () {
    new TextFigDlg(self.app)
  }
  tr.appendChild(btn)
  label = kernel.ce('label', {
    class: 'labelarrow',
    for: 'TextFig'// Pour qu'on puisse activer l'outil en cliquant sur le label
  })
  $(label).html(kernel.getStr('TextFig'))
  tr.appendChild(label)

  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  btn = kernelAdd.getButtonArrow(app, 'about')
  btn.onclick = function () {
    new AboutDlg(self.app)
  }
  tr.appendChild(btn)
  label = kernel.ce('label', {
    class: 'labelarrow',
    for: 'about'// Pour qu'on puisse activer l'outil en cliquant sur le label
  })
  $(label).html(kernel.getStr('About'))
  tr.appendChild(label)

  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  btn = kernelAdd.getButtonArrow(app, 'Web')
  btn.onclick = function () {
    window.open('http://www.mathgraph32.org')
  }
  tr.appendChild(btn)
  label = kernel.ce('label', {
    class: 'labelarrow',
    for: 'Web'// Pour qu'on puisse activer l'outil en cliquant sur le label
  })
  $(label).html(kernel.getStr('Web'))
  tr.appendChild(label)

  // Création de la boîte de dialogue par jqueryui
  this.create('OptionsFig', 700)
}

OptionsFigDlg.prototype = new MtgDlg()

OptionsFigDlg.prototype.onClickDim = function () {
  var app = this.app
  $('#divdim').css('display', $('#cbcadre').prop('checked') ? 'inline' : 'none')
  $('#inpw').val(Math.floor(app.dimf.x) - 10)
  $('#inph').val(Math.floor(app.dimf.y) - 10)
}

OptionsFigDlg.prototype.OK = function () {
  var app = this.app
  var list = app.listePr
  var sel = $('#cbcadre')
  if ((sel.prop('checked') ? (this.editw.validate() && this.edith.validate()) : true) && this.editcoef.validate()) {
    list.uniteAngle = $('#radian').prop('checked') ? kernel.uniteAngle.radian : kernel.uniteAngle.degre
    // On regarde le niveau choisi est le même que le niveau précédent
    var niv = this.getSelectedNiv()
    app.levelIndex = niv // Utilisé dans getResult()
    app.level = app.levels[niv]
    // if ($("#cbtools").prop("checked")) { // Supprimé car quand on décoche la case outils personnailisés il faut revenir à tous les outils
    app.doc.setIdMenusFromDoc(this.doc)
    // On met à jour les icônes de la barre horizontale
    app.updateToolbar()
    // }
    // Pour la version electron si la case est cochéee on appelle setLevel de la page index.html de façon que
    // le niveau soit mémorisé par l'application
    // et idem pour le redémarrage en mode dys
    // Idem pour l'affichage des mesures sur la figure
    if (app.electron) {
      var choix = ['frameGrid', 'frameDotted', 'unity']
      if ($('#cbstartlevel').prop('checked')) setLevel(niv) // eslint-disable-line no-undef
      // setLevel est défini dans index.html
      setDysMode($('#cbdys').prop('checked')) // eslint-disable-line no-undef
      // setDysMode est défini dans index.html
      setDisplayMeasures($('#cbmes').prop('checked')) // eslint-disable-line no-undef
      // setDisplayMeasures est défini dans index.html
      for (var i = 0; i < 3; i++) if ($('#rdst' + i).prop('checked')) break
      setStartFig(choix[i]) // eslint-disable-line no-undef
      // setStratFifg est défini dans index.html
    }
    if (app.cadre !== null) app.deleteCadre()
    if (sel.prop('checked')) {
      var wr = parseInt($('#inpw').val())
      var hr = parseInt($('#inph').val())
      app.createCadre(wr, hr)
    }
    app.pref_coefMult = parseFloat($('#inpcoef').val())
    app.pref_DispMeas = $('#cbmes').prop('checked')
    app.updateToolsToolBar()
    app.activeOutilCapt()
    this.callBackOK()
    this.destroy()
  }
}

OptionsFigDlg.prototype.getSelectedNiv = function () {
  for (var niv = 0; niv < 4; niv++) {
    if ($('#niv' + niv).prop('checked')) return niv
  }
}
