/*
 * Created by yvesb on 14/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const kernelAdd = require('../kernel/kernelAdd')
const MtgDlg = require('./MtgDlg')
const $ = require('jquery')
module.exports = OutilAddDlg

/**
 * Boîte de dialogue qui apparaît quand on clique sur une icône + située à droite de certaines
 * barres d'outils déroulables et présentant des outils sous forme d'items sur lesquels il faut cliquer
 * @constructor
 * @param app L'application propriétaire
 * @param outilAdd du type OutilAdd
 * @param {string} title Le titre de la boîte de dialogue
 */
function OutilAddDlg (app, outilAdd, title) {
  MtgDlg.call(this, app, 'OutilAddDlg')
  var self = this
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var arr = outilAdd.toolArray
  for (let i = 0; i < arr.length; i++) { // Ici un let est indispensable
    var tool = app['outil' + arr[i]]
    if (tool.activationValide() && app.doc.toolDispo(tool.toolIndex) && app.level.toolDispo(tool.toolIndex)) {
      var tr = kernel.ce('tr')
      tab.appendChild(tr)
      var btn = kernelAdd.getButtonArrow(app, 'file' + i)
      btn.tool = app['outil' + arr[i]]
      btn.onclick = function (ev) {
        self.destroy()
        self.stopEvent(ev)
        // this.tool.select();
        app.selectTool(arr[i])
      }
      tr.appendChild(btn)
      var label = kernel.ce('label', {
        class: 'labelarrow',
        for: 'file' + i // Pour quon puisse activer l'outil en cliquant sur le label
      })
      $(label).html(kernel.getStr(arr[i]))
      tr.appendChild(label)
    }
  }
  // Attention : Pour cette boîte de dialogue ne pas appeler create
  var buttons = {}
  buttons[kernel.getStr('Fermer')] = function (ev) {
    self.destroy()
    self.stopEvent(ev)
    self.app.activeOutilCapt()
  }

  $('#' + self.id).dialog({
    modal: true,
    title: kernel.getStr(title),
    buttons: buttons,
    close: function (ev) {
      self.stopEvent(ev)
      self.destroy()
    },
    width: 550,
    closeOnEscape: false,
    // On centre la boîte de dialogue sur le div parent de l'application
    position: { my: 'center', at: 'center', of: '#divMtgDlg' }
  })
}

OutilAddDlg.prototype = new MtgDlg()
