/*
 * Created by yvesb on 08/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

// const $ = require('./jQuery-ui')
const $ = require('jquery')
const kernel = require('../kernel/kernel')
module.exports = PaneAlignement

/**
 * Classe représentant un tableau de deux lignes contenant sur une première ligne les 3 choix possibles d'alignemnt
 * horizontal (0 à 2) et sur une deuxième ligne les 3 choix possibles d'alignement vertical (3 à 5)
 * this.container contient le tableau proprement dit
 * @constructor
 * @param app L'application propriétaire
 * @param alignHor L'alignement horizontal coché à la création
 * @param alignVer L'alignement vertical coché à la création
 */
function PaneAlignement (app, alignHor, alignVer) {
  var tab, caption, tr, td, input, i, ar, tabver, trver
  tab = kernel.ce('table')
  caption = kernel.ce('caption', {
    class: 'mtgcaption'
  })
  $(caption).html(kernel.getStr('Align') + ' :')
  tab.appendChild(caption)
  $(tab).css('border', '1px solid lightgray')
  tr = kernel.ce('tr')
  tab.appendChild(tr)
  ar = ['AlignementHorGauche', 'AlignementHorCentre', 'AlignementHorDroit']
  for (i = 0; i < 3; i++) {
    td = kernel.ce('td')
    tr.appendChild(td)
    tabver = kernel.ce('table')
    td.appendChild(tabver)
    trver = kernel.ce('tr')
    tabver.appendChild(trver)
    $(trver).html(app.local
      ? "<img src='" + app.localPath + 'images/' + ar[i] + ".gif' />"
      : "<img src='" + app.imagePath + ar[i] + ".gif' crossorigin='anonymous' />"
    )
    trver = kernel.ce('tr')
    tabver.appendChild(trver)
    input = kernel.ce('input', {
      id: 'radiohor' + i,
      type: 'radio',
      name: 'grouphor'
    })
    tabver.appendChild(input)
    // Les alignements verticaux vont de 0 à 2
    if (i === alignHor) input.setAttribute('checked', true)
  }
  tr = kernel.ce('tr')
  tab.appendChild(tr)
  ar = ['AlignementVertHaut', 'AlignementVertCentre', 'AlignementVertBas']
  for (i = 0; i < 3; i++) {
    td = kernel.ce('td')
    tr.appendChild(td)
    tabver = kernel.ce('table')
    td.appendChild(tabver)
    trver = kernel.ce('tr')
    tabver.appendChild(trver)
    $(trver).html(app.local
      ? "<img src='" + app.localPath + 'images/' + ar[i] + ".gif' />"
      : "<img src='" + app.imagePath + ar[i] + ".gif' crossorigin='anonymous' />"
    )
    trver = kernel.ce('tr')
    tabver.appendChild(trver)
    input = kernel.ce('input', {
      id: 'radiover' + String(i),
      type: 'radio',
      name: 'groupver'
    })
    tabver.appendChild(input)
    // Les alignements verticaux vont de 0 à 2
    if (i === alignVer) input.setAttribute('checked', true)
  }
  this.container = tab
}

/**
 * Fonction renvoyant un entier correspondant à l'alignement horizontal choisi
 * @returns {number}
 */
PaneAlignement.prototype.getHorAlign = function () {
  for (var i = 0; i < 3; i++) {
    if ($('#' + 'radiohor' + i).prop('checked')) return i
  }
}

/**
 * Fonction renvoyant un entier correspondant à l'alignement vertical choisi
 * @returns {number}
 */
PaneAlignement.prototype.getVerAlign = function () {
  for (var i = 0; i < 3; i++) {
    if ($('#' + 'radiover' + i).prop('checked')) return i
  }
}
