/*
 * Created by yvesb on 10/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
// const $ = require('./jQuery-ui')
const $ = require('jquery')
const kernel = require('../kernel/kernel')
const constantes = require('../kernel/constantes')
const ColorButton = require('../interface/ColorButton')
const MtgInput = require('./MtgInput') // Ajout version 6.0
const EditeurValeurReelle = require('./EditeurValeurReelle')
const CValeurAngle = require('../objets/CValeurAngle')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
module.exports = PaneModeAffichage

/**
 * Classe représentant un tableau de deux colonnes permettant de choisir si un affichage doit :
 * Effacer ou non le fond qu'il recouvre
 * Avoir un encadrement et si oui un encadrement simple ou 3D
 * Si on a choisi un effacement du fond, un bouton de choix de couleur permet de faire apparaître. un colorPicker
 * pour le choix de la couleur de fond de l'affichage.
 * this.container contient le tableau proprement dit
 * @param dlg Le dialogue contenant le panneau
 * @param bPourMacro true si c'est dans une boîte de dialogue pour créer ou modifier une macro, false sinon
 *                     Si true, pas de style d'encadrement proposé
 * @param effacementFond true si la case encadrement du fond doit être cochée
 * @param styleEncadrement Entier de 0 à 2 fournissant le styme d'encadrement (voir StyleEncadrement)
 * @param color La couleur initiale pour le boton de choix de couleur de fond
 * @param indMax indice Maxi que l'angle de rotation du texte peut utiliser
 * @param {boolean} chaineValInit  null si on ne doit pas éditer un angle d'afficahe, sinon la chaîne du calcul de l'angle
 * @param callBack Fonction (éventuelle) à appeler en cas de changement de style d'encadrement
 * @constructor
 */
function PaneModeAffichage (dlg, bPourMacro, effacementFond, styleEncadrement, color, chaineValInit = null, indMax = -1, callBack = null) {
  this.callBack = callBack
  this.chaineValInit = chaineValInit
  var app = dlg.app
  var listePr = app.listePr
  this.valeurAngle = new CValeurAngle(listePr, 0)
  var self = this
  var tabPrincipal = kernel.ce('table') // Le tableau principal qui sera this.container
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tab = kernel.ce('table', {
    cellspacing: 2
  })
  tr.appendChild(tab)
  this.container = tabPrincipal
  var caption = kernel.ce('caption', {
    class: 'mtgcaption'
  })
  $(caption).html(kernel.getStr('ModeAff') + ' :')
  tab.appendChild(caption)
  $(tab).css('border', '1px solid lightgray')
  var td = kernel.ce('td') // La colonne de gauche qui contiendra un tableau de 4 lignes
  tab.appendChild(td)
  var tabgauche = kernel.ce('table')
  td.appendChild(tabgauche)
  tr = kernel.ce('tr')
  tabgauche.appendChild(tr)
  var cbEffacementFond = kernel.ce('input', {
    type: 'checkbox',
    id: 'cbEffFond'
  })
  tr.appendChild(cbEffacementFond)
  if (effacementFond) cbEffacementFond.setAttribute('checked', 'checked')
  cbEffacementFond.onclick = function () {
    self.oncheckBoxChange()
  }
  var label = kernel.ce('label', {
    for: 'cbEffFond'
  })
  $(label).html(kernel.getStr('EffFond'))
  tr.appendChild(label)
  if (!bPourMacro) {
    // On ajoute maintenant trois ligne contenant trois boutons radios pour le style d'encadrement suivis du labe
    // Leurs id seront radio0, radio1 et radio2
    var ar = ['EncSans', 'EncSimple', 'Enc3D']
    for (var i = 0; i < 3; i++) {
      tr = kernel.ce('tr')
      tabgauche.appendChild(tr)
      // Radio Button "Sans encadrement"
      var radio = kernel.ce('input', {
        type: 'radio',
        id: 'panemoderd' + i,
        name: 'encad'
      })
      radio.onchange = function () {
        if (self.callBack !== null) self.callBack()
      }
      if (i === styleEncadrement) radio.setAttribute('checked', 'checked')
      tr.appendChild(radio)
      label = kernel.ce('label', {
        for: 'panemoderd' + i
      })
      $(label).html(kernel.getStr(ar[i]))
      tr.appendChild(label)
    }
  }
  // Dans la colonne de droite un tableau de deux lignes contenant le titre et au-dessous un bouton de choix de couleur
  td = kernel.ce('td') // La colonne de droite
  tab.appendChild(td)
  var tabdroit = kernel.ce('table', {
    id: 'panemodetabdroit'
  })
  $('#paneModetabdroit').css('visibility', effacementFond ? 'visible' : 'hidden')
  td.appendChild(tabdroit)
  tr = kernel.ce('tr')
  tabdroit.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('CoulFond'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabdroit.appendChild(tr)
  // Un svg pour contenir le bouton de choix de couleur
  var w = Math.floor(constantes.lineStyleWidth)
  var h = Math.floor(constantes.lineStyleWidth / 2)
  var svg = kernel.cens('svg', {
    width: w,
    height: h
  })
  svg.appendChild(kernel.cens('rect', {
    x: 0,
    y: 0,
    width: w,
    height: h,
    fill: constantes.buttonFrameColor
  }))
  tr.appendChild(svg)
  // Création du bouton de choix de couleur
  this.colorButton = new ColorButton(dlg, color, this.callBack)
  svg.appendChild(this.colorButton.container)
  /// ////////////////////////////////////////////////////////////////////////
  // Version 6.0 : On ajoute une éditeur pour le choix de l'angle d'affichage
  if (chaineValInit !== null) {
    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    label = kernel.ce('label')
    $(label).html(kernel.getStr('Angle') + ' (' + ((listePr.uniteAngle === kernel.uniteAngle.radian) ? 'rad' : 'deg') + ')' + ' : ')
    tr.appendChild(label)
    this.input = new MtgInput({
      id: 'inputAngle'
    })
    tr.appendChild(this.input)
    this.editeur = new EditeurValeurReelle(dlg.app, this.input, indMax, this.valeurAngle, null)
    $(this.input).val(chaineValInit)
    // Version 6.2.2 : On ajoute des boutons sous l'éditeur d'angle
    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    var paneBoutonsValFonc = new PaneBoutonsValFonc(app, dlg, this.input, true, indMax)
    tr.appendChild(paneBoutonsValFonc)
  }
}

PaneModeAffichage.prototype.oncheckBoxChange = function () {
  if (this.callBack) this.callBack()
  $('#panemodetabdroit').css('visibility', $('#cbEffFond').prop('checked') ? 'visible' : 'hidden')
}

/**
 * Fonction qui doit être appelée par la boîte de dialogue appelante pour que le bouton de choix de couleur
 * et le label au-dessys soient cachés à l'ouverture de la boîte de dialogue
 */
PaneModeAffichage.prototype.init = function () {
  this.oncheckBoxChange()
}

PaneModeAffichage.prototype.getEffFond = function () {
  return $('#cbEffFond').prop('checked')
}

PaneModeAffichage.prototype.getStyleEnc = function () {
  for (var i = 0; i < 3; i++) {
    if ($('#panemoderd' + i).prop('checked')) return i
  }
}

PaneModeAffichage.prototype.getColor = function () {
  return this.colorButton.color
}

PaneModeAffichage.prototype.validate = function () {
  if (this.chaineValInit !== null) return this.editeur.validate()
  else return true
}

PaneModeAffichage.prototype.getAng = function () {
  return this.valeurAngle
}
