/*
 * Created by yvesb on 31/12/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurComplexe = require('./EditeurValeurComplexe')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const PaneListeReperes = require('./PaneListeReperes')
const MtgInput = require('./MtgInput')

module.exports = PtParAffDlg

/**
 * Dialogue de choix d'angle de rotation
 * @constructor
 * @param app La mtgApp propriétaire
 * @param rot La rotation dont l'angle doit être changé
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 */
function PtParAffDlg (app, point, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'ptParAffDlg', callBackOK, callBackCancel)
  var self = this
  this.point = point
  this.modification = modification
  var list = app.listePr
  var tabprincipal = kernel.ce('table')
  this.appendChild(tabprincipal)
  var trprincipal = kernel.ce('tr')
  tabprincipal.appendChild(trprincipal)
  var td = kernel.ce('td')
  trprincipal.appendChild(td)

  var tab = kernel.ce('table')
  // this.appendChild(tab);
  td.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Affixe') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var inputAff = new MtgInput({
    id: 'mtginputAff'
  })
  td.appendChild(inputAff)
  inputAff.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var indmax = modification ? list.indexOf(point) - 1 : list.longueur() - 1
  var paneBoutonsValFoncAff = new PaneBoutonsValFonc(this.app, this, inputAff, false, indmax) // false car on est en mode complexe
  tab.appendChild(paneBoutonsValFoncAff)
  this.editorAff = new EditeurvaleurComplexe(app, inputAff, indmax, point.affixe, null)

  if (modification) $(inputAff).val(point.affixe.calcul.chaineCalculSansPar())
  // On rajoute à droite un panneau pour le choix du repère
  td = kernel.ce('td')
  trprincipal.appendChild(td)
  this.paneListeReperes = new PaneListeReperes(this.app, indmax)
  td.appendChild(this.paneListeReperes.getTab())
  if (modification) this.paneListeReperes.selectRep(point.rep)
  this.create('PtParAff', 550)
}

PtParAffDlg.prototype = new MtgDlg()

PtParAffDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginputAff').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

PtParAffDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  // On regarde si l'entrée est correcte sur le plan syntaxique
  var sel = $('#mtginputAff')
  if (this.editorAff.validate(sel.val())) {
    this.point.rep = this.paneListeReperes.getSelectedRep()
    if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
    if (this.callBackOK !== null) this.callBackOK()
    this.destroy()
  } else sel.focus()
}
