/*
 * Created by yvesb on 17/12/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const CValeur = require('../objets/CValeur')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const AvertDlg = require('./AvertDlg')
const MtgInput = require('./MtgInput')

module.exports = RayonDlg

/**
 * Dialogue de choix d'angle de rotation
 * @constructor
 * @param app La mtgApp propriétaire
 * @param cer Le cercle dont lle rayon doit être changé
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annumlation
 */
function RayonDlg (app, cer, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'rayonDlg', callBackOK, callBackCancel)
  var self = this
  this.cer = cer
  this.modification = modification
  var list = app.listePr
  // Un CValeur pour stocker la valeur choisie avant vérification
  this.valeurRay = new CValeur(list, 1)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Rayon') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = new MtgInput({
    id: 'mtginput'
  })
  td.appendChild(input)
  input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var indmax = modification ? list.indexOf(cer) - 1 : list.longueur() - 1
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, input, true, indmax)
  tabHaut.appendChild(paneBoutonsValFonc)
  // On rajoute un checkBox pour savoir si le rayon est en pixels ou non
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var inp = kernel.ce('input', {
    id: 'mtgcb',
    type: 'checkBox'
  })
  // inp.setAttribute("checked",modification ? cer.inPixels : false);
  if (modification && cer.inPixels) inp.setAttribute('checked', 'checked')
  // Ajout version 6.4.9
  if (list.pointeurLongueurUnite === null) inp.setAttribute('disabled', true)
  tr.appendChild(inp)
  label = kernel.ce('label', {
    for: 'mtgcb'
  })
  $(label).html(kernel.getStr('RayPix'))
  tr.appendChild(label)
  //
  this.editor = new EditeurvaleurReelle(app, input, indmax, this.valeurRay, null)
  if (modification) $(input).val(cer.r.calcul.chaineCalculSansPar())
  this.create('RayCer', 400)
}

RayonDlg.prototype = new MtgDlg()

RayonDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

RayonDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  var valid = this.editor.validate($('#mtginput').val())
  if (valid) {
    if (this.valeurRay.valeur <= 0) { new AvertDlg(this.app, 'avRay', function () { $('#mtginput').focus() }) } else {
      this.cer.inPixels = $('#mtgcb').prop('checked')
      this.cer.r = this.valeurRay
      if (this.callBackOK !== null) this.callBackOK()
      if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
      this.destroy()
    }
  }
}
