/*
 * Created by yvesb on 02/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const CValeur = require('../objets/CValeur')
const CValeurAngle = require('../objets/CValeurAngle')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const MtgInput = require('./MtgInput')
const ChoixAngleUsuelDlg = require('./ChoixAngleUsuelDlg')

module.exports = SimilitudeDlg

/**
 * Dialogue de choix d'angle de rotation
 * @constructor
 * @param app La mtgApp propriétaire
 * @param sim La similitude à changer
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 */
function SimilitudeDlg (app, sim, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'simDlg', callBackOK, callBackCancel)
  var self = this
  this.sim = sim
  this.modification = modification
  var list = app.listePr
  // Un CValeur pour stocker la valeur de l'angle avant vérification
  this.valeur = new CValeurAngle(list, 0)
  // Un CValeur pour stocker la valeur du rapport avant vérification
  this.valeurRap = new CValeur(list, 1)
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Angle') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var inputAngle = new MtgInput({
    id: 'mtginputAng',
    list: 'mtgAngleList'
  })
  td.appendChild(inputAngle)
  td = kernel.ce('td')
  tr.appendChild(td)
  $(td).css('padding', '1px')
  $(td).html(app.local
    ? "<img src='" + app.localPath + "images/dropdown.png' width='22px' height='22px' />"
    : "<img src='" + app.imagePath + "dropdown.png' width='22px' height='22px' crossorigin='anonymous' />"
  )
  td.onclick = function () {
    new ChoixAngleUsuelDlg(app, inputAngle, list.uniteAngle)
  }
  var indmax = modification ? app.listePr.indexOf(sim) - 1 : list.longueur() - 1
  var paneBoutonsValFoncAng = new PaneBoutonsValFonc(this.app, this, inputAngle, true, indmax)
  tab.appendChild(paneBoutonsValFoncAng)
  this.editorAngle = new EditeurvaleurReelle(app, inputAngle, indmax, this.valeur, null)

  tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Rapport') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var inputRapport = new MtgInput({
    id: 'mtginputRap'
  })
  td.appendChild(inputRapport)
  inputRapport.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var paneBoutonsValFoncRap = new PaneBoutonsValFonc(this.app, this, inputRapport, true, indmax)
  tab.appendChild(paneBoutonsValFoncRap)
  this.editorRapport = new EditeurvaleurReelle(app, inputRapport, indmax, this.valeurRap, null)
  if (modification) {
    $(inputRapport).val(sim.rapport.calcul.chaineCalculSansPar())
    $(inputAngle).val(sim.angle.calcul.chaineCalculSansPar())
  }
  this.create('Sim', 400)
}

SimilitudeDlg.prototype = new MtgDlg()

SimilitudeDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginputAng').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

SimilitudeDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  // On regarde si l'entrée est correcte sur le plan syntaxique
  var selang = $('#mtginputAng')
  var selrap = $('#mtginputRap')
  if (this.editorAngle.validate(selang.val())) {
    if (this.editorRapport.validate(selrap.val())) {
      this.sim.angle = this.valeur
      this.sim.rapport = this.valeurRap
      if (this.callBackOK !== null) this.callBackOK()
      // this.app.outilActif.actionApresDlg();
      this.destroy()
    } else selrap.focus()
  } else selang.focus()
}
