/*
 * Created by yvesb on 15/04/2017.
 */
/*
 * Created by yvesb on 01/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const CValeur = require('../objets/CValeur')
const CValeurComp = require('../objets/CValeurComp')
const NatCal = require('../types/NatCal')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const EditeurValeurReelle = require('./EditeurValeurReelle')
const EditeurValeurComplexe = require('./EditeurValeurComplexe')
const AvertDlg = require('./AvertDlg')
const MtgInput = require('./MtgInput')
module.exports = SuiteRecDlg

/**
 * Boîte de dialogue de création ou modification d'une suite de la formue u(n+1) = f(u(n)) réelle ou complexe
 * @constructor
 * @param app  L'application propriétaire
 * @param suite  La suité récurrente réelle ou complexe
 * @param {int} type
 *    1 pour suite u(n+1)=f[u(n)]
 *    2 pour une suite u(n+1)=f[n,u(n)]
 *    3 pour une suite u(n+2)=f[n,u(n+1),u(n)]
 * @param {boolean} bReel  true si on édite une suite réelle, false pour une suite complexe
 * @param {boolean} modification true si on modifie une suite déjà existante
 * @param callBackOK  Eventuelle fonction de callBack à appeler après appui sur Cancel
 * @param callBackCancel  Eventuelle fonction de callBack à appeler après appui sur Cancel
 */
function SuiteRecDlg (app, suite, type, bReel, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'DeriveeDlg', callBackOK, callBackCancel)
  var nat
  this.suite = suite
  this.type = type
  this.modification = modification
  var self = this
  var list = app.listePr
  // 2 CValeur pour stocker les valeurs pour les valeurs pour le premier terme et le nombre de termes.
  this.valeurPrem = bReel ? new CValeur(list) : new CValeurComp(list)
  if (type === 3) this.valeurSec = bReel ? new CValeur(list) : new CValeurComp(list)
  this.valeurNb = new CValeur(list)

  var indmax = modification ? list.indexOf(suite) - 1 : list.longueur() - 1

  switch (type) {
    case 1 :
      nat = bReel ? NatCal.NFoncR1Var : NatCal.NFonctionComplexe
      break
    case 2 :
      nat = bReel ? NatCal.NFonction2Var : NatCal.NFonctionComplexe2Var
      break
    case 3 :
      nat = bReel ? NatCal.NFonction3Var : NatCal.NFonctionComplexe3Var
      break
  }
  this.inf = list.listeParNatCal(app, nat, indmax)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneHaut = kernel.ce('table')
  tr.appendChild(paneHaut)
  tr = kernel.ce('tr')
  $(tr).css('vertical-align', 'top')
  paneHaut.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('suiteRecDlg1'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputName = new MtgInput({
    size: 4,
    style: 'margin-right:20px'
  })
  td.appendChild(this.inputName)
  td = kernel.ce('td')
  tr.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:180px'
  })
  $(caption).html(kernel.getStr('suiteRecDlg2'))
  td.appendChild(caption)
  this.select = kernel.ce('select', {
    size: 6, // Le nombre de lignes visibles par défaut
    style: 'width:180px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.inf.pointeurs[i] === suite.fonctionAssociee) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.select.appendChild(option)
  }
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneMilieu = kernel.ce('table')
  tr.appendChild(paneMilieu)
  tr = kernel.ce('tr')
  paneMilieu.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('InfoFonc'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  paneMilieu.appendChild(tr)
  var inputinf = kernel.ce('textarea', {
    id: 'info',
    disabled: 'true',
    cols: 50,
    rows: 2
  })
  tr.appendChild(inputinf)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneBas = kernel.ce('table')
  tr.appendChild(paneBas)
  tr = kernel.ce('tr')
  paneBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('PremTerm'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputPrem = new MtgInput({
    size: 30
  })
  td.appendChild(this.inputPrem)
  var paneBoutonsValFoncPrem = new PaneBoutonsValFonc(this.app, this, this.inputPrem, bReel, indmax)
  paneBas.appendChild(paneBoutonsValFoncPrem)
  if (type === 3) {
    tr = kernel.ce('tr')
    paneBas.appendChild(tr)
    td = kernel.ce('td')
    tr.appendChild(td)
    label = kernel.ce('label')
    $(label).html(kernel.getStr('chinfo186'))
    td.appendChild(label)
    td = kernel.ce('td')
    tr.appendChild(td)
    this.inputSec = new MtgInput({
      size: 30
    })
    td.appendChild(this.inputSec)
    var paneBoutonsValFoncSec = new PaneBoutonsValFonc(this.app, this, this.inputSec, bReel, indmax)
    paneBas.appendChild(paneBoutonsValFoncSec)
  }

  tr = kernel.ce('tr')
  paneBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('NbTerm'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputNbTerm = new MtgInput({
    size: 30
  })
  td.appendChild(this.inputNbTerm)
  var paneBoutonsValFoncNbTerm = new PaneBoutonsValFonc(this.app, this, this.inputNbTerm, true, indmax)
  paneBas.appendChild(paneBoutonsValFoncNbTerm)
  if (modification) {
    $(this.inputName).val(this.suite.nomCalcul)
    $(this.inputName).attr('disabled', true)
    $(this.inputPrem).val(suite.premierTerme.calcul.chaineCalculSansPar())
    if (type === 3) $(this.inputSec).val(suite.deuxiemeTerme.calcul.chaineCalculSansPar())
    $(this.inputNbTerm).val(suite.nombreTermes.calcul.chaineCalculSansPar())
  }

  this.editeurName = new EditeurNomCalcul(app, !this.modification, this.inputName)
  this.editeurPrem = bReel ? new EditeurValeurReelle(app, this.inputPrem, indmax, this.valeurPrem, null)
    : new EditeurValeurComplexe(app, this.inputPrem, indmax, this.valeurPrem, null)
  if (type === 3) {
    this.editeurSec = bReel ? new EditeurValeurReelle(app, this.inputSec, indmax, this.valeurSec, null)
      : new EditeurValeurComplexe(app, this.inputSec, indmax, this.valeurSec, null)
  }
  this.editeurNbTerm = new EditeurValeurReelle(app, this.inputNbTerm, indmax, this.valeurNb, null)
  this.onSelectChange()
  this.info = 'SuiteRec' + ((type === 1) ? '' : type)
  this.create(this.info, 500)
}

SuiteRecDlg.prototype = new MtgDlg()

SuiteRecDlg.prototype.onSelectChange = function () {
  $('#info').val(this.inf.pointeurs[this.select.selectedIndex].infoHist())
}

SuiteRecDlg.prototype.OK = function () {
  if (this.editeurName.validate() && this.editeurPrem.validate() &&
    ((this.type === 3) ? this.editeurSec.validate() : true) && this.editeurNbTerm.validate()) {
    if (!this.modification) this.suite.nomCalcul = $(this.inputName).val()
    var self = this
    var nb = this.valeurNb.valeur
    if ((nb <= 0) || (nb > 100000) || nb !== Math.floor(nb)) {
      new AvertDlg(this.app, 'Incorrect', function () {
        $(self.inputNbTerm).focus()
        self.inputNbTerm.marquePourErreur()
      })
    } else {
      this.suite.fonctionAssociee = this.inf.pointeurs[this.select.selectedIndex]
      this.suite.premierTerme = this.valeurPrem
      if (this.type === 3) this.suite.deuxiemeTerme = this.valeurSec
      this.suite.nombreTermes = this.valeurNb
      var app = this.app
      if (!this.modification) app.ajouteElement(this.suite)
      app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : this.info)
      this.destroy()
      if (this.callBackOK !== null) this.callBackOK()
    }
  }
}
