/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const MtgInput = require('./MtgInput')
const NatCal = require('../types/NatCalAdd')
const Nat = require('../types/Nat')
module.exports = TestDepVarDlg

function TestDepVarDlg (app, test, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'DeriveeDlg', callBackOK, callBackCancel)
  this.test = test
  this.modification = modification
  var self = this
  var list = app.listePr
  var indmax = modification ? list.indexOf(test) - 1 : -1
  this.inf = list.listeParNatCal(app, NatCal.NTteFoncRouC, indmax)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NomCalcul') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = new MtgInput({
    id: 'mtginput'
  })
  input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  td.appendChild(input)
  if (modification) {
    $(input).val(test.nomCalcul)
    $(input).attr('disabled', true)
  }
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('suiteRecDlg2') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.select = kernel.ce('SELECT', {
    size: 8, // Le nombre de lignes visibles par défaut
    style: 'width:180px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.inf.pointeurs[i] === test.fonctionAssociee) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.select.appendChild(option)
  }
  // Un ligne pour choisir par rapport à quelle variable on teste la dépendance
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabMilieu = kernel.ce('table')
  tr.appendChild(tabMilieu)
  tr = kernel.ce('tr')
  tabMilieu.appendChild(tr)
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('DepRap'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.select2 = kernel.ce('select', {
    size: 3// Le nombre de lignes visibles par défaut
    // style : "width:180px",
  })
  td.appendChild(this.select2)

  // En bas du tableau un champ d'info sur la fonction sélectionnée
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas = kernel.ce('table')
  tr.appendChild(tabBas)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('InfoFonc'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  var inputinf = kernel.ce('textarea', {
    id: 'mtginfo',
    disabled: 'true',
    cols: 40,
    rows: 4
  })
  tr.appendChild(inputinf)
  this.editeurName = new EditeurNomCalcul(app, !this.modification, input)
  for (var j = 0; j < test.nbVar; j++) {
    if (j === test.indiceVariableDerivation) $('#option' + j).attr('selected', 'selected')
  }
  this.onSelectChange(true)
  this.create('TestDepVar', 500)
}

TestDepVarDlg.prototype = new MtgDlg()

TestDepVarDlg.prototype.onSelectChange = function (binit) {
  var ind, fonc, option
  ind = this.select.selectedIndex
  fonc = this.inf.pointeurs[ind]
  $('#mtginfo').val(fonc.infoHist())
  $(this.select2).empty()
  if (fonc.estDeNatureCalcul(Nat.or(NatCal.NFonctionReelle1Variable, NatCal.NFonctionComplexe))) {
    option = kernel.ce('Option', {
      class: 'mtgOption',
      id: 'option0'
    })
    $(option).html(fonc.chaineVariable)
    $(option).attr('selected', 'selected')
    this.select2.appendChild(option)
  } else {
    for (var i = 0; i < fonc.nbVar; i++) {
      option = kernel.ce('Option', {
        class: 'mtgOption',
        id: 'option' + i
      })
      $(option).html(fonc.getVariable()[i])
      var indsel = binit ? this.test.indiceVariable : 0
      if (i === indsel) $(option).attr('selected', 'selected')
      this.select2.appendChild(option)
    }
  }
}

TestDepVarDlg.prototype.OK = function () {
  var app = this.app
  // if (app.lastDlgId() !== this.id) return;
  if (this.editeurName.validate()) {
    if (!this.modification) this.test.nomCalcul = $('#mtginput').val()
    this.test.fonctionAssociee = this.inf.pointeurs[this.select.selectedIndex]
    this.test.indiceVariable = this.select2.selectedIndex
    if (!this.modification) app.ajouteElement(this.test)
    app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : 'TestDepVar')
    this.destroy()
    if (this.callBackOK !== null) this.callBackOK()
  }
}

TestDepVarDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  if (!this.modification) {
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    }, 100)
  } else {
    var self = this
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $(self.select2).focus() // Pour que le focus soit donné à la liste de choix de vriable
    }, 100)
  }
}
