/*
 * Created by yvesb on 23/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const NatCal = require('../types/NatCal')
const MtgDlg = require('./MtgDlg')
//
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const MtgInput = require('./MtgInput')
module.exports = TestExistenceDlg

/**
 * Boîte de dialogue servant à créer un test d'existence d'une valeur
 * @param app L'application associée
 * @param obj Le complexe associé
 * @param modification true si on modifie un objet déjà créé
 * @param callBackOK function appelée si l'utilisateur valide ou null
 * @param callBackCancel Fonction appelée si l'utilisateur ferme la boîte de dialogue ou null
 * @constructor
 */
function TestExistenceDlg (app, obj, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'TestExistence', callBackOK, callBackCancel)
  this.obj = obj
  this.modification = modification
  var self = this
  var list = app.listePr
  var indmax = modification ? list.indexOf(obj) - 1 : -1
  this.inf = list.listeParNatCal(app, NatCal.NTteValROuC, indmax)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NomCalcul') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = new MtgInput({
    id: 'mtginput'
  })
  td.appendChild(input)
  if (modification) {
    $(input).val(obj.nomCalcul)
    $(input).attr('disabled', true)
  }

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabCentre = kernel.ce('table')
  tr.appendChild(tabCentre)
  td = kernel.ce('td')
  tabCentre.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgCaption',
    style: 'width:180px'
  })
  $(caption).html(kernel.getStr('TestExistence') + ' ' + kernel.getStr('de') + ' :')
  td.appendChild(caption)

  this.select = kernel.ce('select', {
    size: 8, // Le nombre de lignes visibles par défaut
    style: 'width:180px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.inf.pointeurs[i] === obj.valeurAssociee) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.select.appendChild(option)
  }
  // En bas du tableau un champ d'info sur le complexe
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas = kernel.ce('table')
  tr.appendChild(tabBas)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('ChoixValeurDlg3'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  var inputinf = kernel.ce('textarea', {
    id: 'mtginfo',
    disabled: 'true',
    cols: 40,
    rows: 4
  })
  tr.appendChild(inputinf)
  this.editeurName = new EditeurNomCalcul(app, !this.modification, input)
  this.onSelectChange()
  this.create('TestExistence', 450)
}

TestExistenceDlg.prototype = new MtgDlg()

TestExistenceDlg.prototype.onSelectChange = function () {
  var ind, el
  ind = this.select.selectedIndex
  if (ind === -1) return
  el = this.inf.pointeurs[ind]
  $('#mtginfo').text(el.infoHist())
}

TestExistenceDlg.prototype.OK = function () {
  var app = this.app
  // if (app.lastDlgId() !== this.id) return;
  // On regarde si l'entrée est correcte sur le plan syntaxique
  if (this.editeurName.validate()) {
    if (!this.modification) this.obj.nomCalcul = $('#mtginput').val()
    this.obj.valeurAssociee = this.inf.pointeurs[this.select.selectedIndex]
    if (!this.modification) app.ajouteElement(this.obj)
    app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : 'TestExistence')
    this.destroy()
    if (this.callBackOK !== null) this.callBackOK()
  }
}

TestExistenceDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  if (!this.modification) {
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    }, 100)
  } else {
    var self = this
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $(self.select).focus() // Pour que le focus soit donné à la liste de gauche
    }, 100)
  }
}
