/*
 * Created by yvesb on 14/02/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const $ = require('./../global/jQuery-ui')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const CValeur = require('../objets/CValeur')
const AvertDlg = require('./AvertDlg')
const MtgInput = require('./MtgInput')

module.exports = VariableDlg

/**
 * Dialogue de création ou modification d'une variable
 * @constructor
 * @param app La mtgApp propriétaire
 * @param va La variable à modifier
 * @param callBackOK : Fonction de callBack à appeler aorès clic sur OK
 * @param callBackCancel : Fonction de callBack à appeler aorès clic sur Cancel
 * @param modification {boolean} : true si le dialogue est ouvert pour modifier une variable déjà existante
 */
function VariableDlg (app, va, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'calculdlg', callBackOK, callBackCancel)
  this.va = va
  this.modification = modification
  var list = app.listePr
  var self = this
  // Les formules obtenues pour les valeur mini, maxi, le pas et la valeur actuelle seront renvoyées dans un CValeur
  this.valMin = new CValeur(list)
  this.valMax = new CValeur(list)
  this.valPas = new CValeur(list)
  this.valAct = new CValeur(list)

  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NomVar') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  /*
  this.inputName = kernel.ce("input", {
    type: "text",
  });
  */
  this.inputName = new MtgInput()
  td.appendChild(this.inputName)

  tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  label = kernel.ce('label')
  $(label).html(kernel.getStr('ValMin'))
  td.appendChild(label)
  tr.appendChild(td)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputValMin = new MtgInput()
  td.appendChild(this.inputValMin)
  this.editeurValMin = new EditeurvaleurReelle(app, this.inputValMin, 0, this.valMin, null)

  tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  label = kernel.ce('label')
  $(label).html(kernel.getStr('ValMax'))
  td.appendChild(label)
  tr.appendChild(td)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputValMax = new MtgInput()
  td.appendChild(this.inputValMax)
  this.editeurValMax = new EditeurvaleurReelle(app, this.inputValMax, 0, this.valMax, null)

  tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  label = kernel.ce('label')
  $(label).html(kernel.getStr('ValPas'))
  td.appendChild(label)
  tr.appendChild(td)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputValPas = new MtgInput()
  td.appendChild(this.inputValPas)
  this.editeurValPas = new EditeurvaleurReelle(app, this.inputValPas, 0, this.valPas, null)

  tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  label = kernel.ce('label')
  $(label).html(kernel.getStr('ValAct'))
  td.appendChild(label)
  tr.appendChild(td)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputValAct = new MtgInput()
  td.appendChild(this.inputValAct)
  this.inputValAct.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }

  this.editeurValAct = new EditeurvaleurReelle(app, this.inputValAct, 0, this.valAct, null)

  tr = kernel.ce('tr')
  tab.appendChild(tr)
  label = kernel.ce('label', {
    for: 'cbdlg'
  })
  $(label).html(kernel.getStr('DlgAss'))
  tr.appendChild(label)
  this.cbDlgAss = kernel.ce('input', {
    id: 'cbdlg',
    type: 'checkbox'
  })
  label.appendChild(this.cbDlgAss)

  if (modification) {
    $(this.inputName).val(va.nomCalcul)
    $(this.inputName).attr('disabled', true)
    $(this.inputValMin).val(va.chaineValeurMini)
    $(this.inputValMax).val(va.chaineValeurMaxi)
    $(this.inputValPas).val(va.chaineValeurPas)
    $(this.inputValAct).val(va.valeurActuelle)
    if (va.dialogueAssocie) $('#cbdlg').attr('checked', 'checked')
  }

  this.editeurName = new EditeurNomCalcul(app, !modification, this.inputName)
  this.create('Variable', 500)
}

VariableDlg.prototype = new MtgDlg()

VariableDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    if (self.modification) {
      self.inputValAct.focus()
      self.inputValAct.select()
    } else self.inputName.focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

VariableDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  // var time = +new Date();
  // if (time - this.timeStart < MtgDlg.delay) return; // Moins d'une seconde dans doute mauvais endroit de clic sur un bouton
  var app = this.app
  var self = this
  // On regarde si l'entrée est correcte sur le plan syntaxique
  if (this.editeurName.validate()) {
    if (this.editeurValMin.validate($(this.inputValMin).val())) {
      if (this.editeurValMax.validate($(this.inputValMax).val())) {
        if (this.editeurValPas.validate($(this.inputValPas).val())) {
          if (this.editeurValAct.validate($(this.inputValAct).val())) {
            if (!this.modification) this.va.nomCalcul = $(this.inputName).val()
            var va = this.va
            var min = this.valMin.rendValeur()
            var max = this.valMax.rendValeur()
            var pas = this.valPas.rendValeur()
            var act = this.valAct.rendValeur()
            // On vérifie la validité des valeurs entrées
            if (min >= max) {
              new AvertDlg(app, 'vardlg1', function () { self.inputValMax.focus() })
              return
            }
            if (((pas > max - min) || (pas <= 0))) {
              new AvertDlg(app, 'vardlg2', function () { self.inputValPas.focus() })
              return
            }
            if ((act < min) || (act > max)) {
              new AvertDlg(app, 'vardlg3', function () { self.inputValAct.focus() })
              return
            }
            va.chaineValeurMini = $(this.inputValMin).val()
            va.valeurMini = min
            va.chaineValeurMaxi = $(this.inputValMax).val()
            va.valeurMaxi = max
            va.chaineValeurPas = $(this.inputValPas).val()
            va.valeurPas = pas
            va.valeurActuelle = act
            if (!this.modification) this.app.ajouteElement(va)
            // On regarde s'il fauut créer au supprimer le div associé à la variable
            var dlgass = va.dialogueAssocie
            var b = $('#cbdlg').prop('checked')
            if (dlgass !== b) {
              app.removePaneVariables()
              va.dialogueAssocie = b
              app.listePr.creePaneVariables()
            }
            this.app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : 'Variable')
            this.destroy()
            if (this.callBackOK !== null) this.callBackOK()
          } else this.inputValAct.focus()
        } else this.inputValPas.focus()
      } else this.inputValMax.focus()
    } else this.inputValMin.focus()
  }
}
