/*
 * Created by yvesb on 05/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const constantes = require('../kernel/constantes')
// const $ = require('jquery')
const $ = require('../global/jQuery-ui')

module.exports = Button

function Button (app, fileName, tip, w, h, bframe = true, bNoZoomFactor = false) {
  if (arguments.length === 0) return
  this.app = app
  this.fileName = fileName
  var zf = bNoZoomFactor ? 1 : app.zoomFactor
  this.w = w * zf
  this.h = h * zf
  this.tip = kernel.getStr(tip) || ''
  this.bframe = bframe
  this.clicks = 0
  this.isArrow = this.fileName.indexOf('arrow') === 0
}
/**
 * Foonction donnant au bouton l'apparence activé ou non suibant la valeur de bActivate
 * @param bActivate
 */
Button.prototype.activate = function (bActivate) {
  $(this.mask).attr('fill-opacity', '0')
  $(this.frameRect).attr('fill', (bActivate) ? 'url(#buttonGradAct)'
    : constantes.buttonBackGroundColor)
  $(this.mask).attr('stroke', (bActivate) ? constantes.buttonActivatedBackGroundColor
    : constantes.buttonBackGroundColor)
  this.isActivated = bActivate
}
Button.prototype.build = function () {
  var self = this
  var w = this.w
  var h = this.h
  var g = kernel.cens('g')
  // g.setAttribute("id",this.tip);
  this.container = g
  var frameRect = kernel.cens('rect', {
    width: w,
    height: h,
    x: '0',
    y: '0',
    stroke: constantes.buttonStroke,
    fill: constantes.buttonFill,
    'fill-opacity': '1'
  })
  if (!this.bframe) $(frameRect).css('visibility', 'hidden')
  this.frameRect = frameRect
  g.appendChild(frameRect)
  var mask = kernel.cens('rect', {
    width: w,
    height: h,
    x: '0',
    y: '0',
    stroke: constantes.buttonStroke,
    fill: 'url(#buttonGrad)',
    'fill-opacity': '0' // Sera passé à un quand la souris passe dessus
  })
  this.mask = mask
  if (!this.bframe) {
    $(mask).css('visibility', 'hidden').css('pointer-events', 'all')
  }
  g.appendChild(mask)
  if (this.isArrow) {
    var points = '3,1 3,' + String(h - 1) + ' ' + String(w - 2) + ',' + String(h / 2)
    var poly = kernel.cens('polygon', {
      points: points,
      fill: 'url(#arrowGrad)',
      'fill-opacity': '0.5',
      style: 'stroke:#A9A9F5;stroke-width:1',
      'pointer-events': 'none'
    })
    this.poly = poly
    g.appendChild(poly)
    g.addEventListener('mouseover', function () {
      self.poly.setAttribute('fill-opacity', '1')
    })
    g.addEventListener('mouseout', function () {
      self.poly.setAttribute('fill-opacity', '0.5')
    })
  } else {
    if (this.fileName.length !== 0) { // fileName est "" pour les OneColorButton
      var gImage = kernel.cens('g', {
        transform: 'translate(1,1)',
        'pointer-events': 'none'// ,
        // opacity: "0.8"
      })
      g.appendChild(gImage)
      this.image = kernel.cens('image', {
        width: String(w - 2),
        height: String(h - 2),
        x: '0',
        y: '0'
      })
      gImage.appendChild(this.image)
      // avec un require, ça plante à la 1re 404
      // this.url = require('../../images/' + this.fileName + '.png')
      // on reste en relatif par rapport au dossier racine, www (où se trouve le html)
      this.url = (this.app.local ? this.app.localPath + 'images/' : this.app.imagePath) + this.fileName + '.png'
      // Chargement en lazy-loading de l'image
      var im = new Image() // eslint-disable-line no-undef
      if (!this.app.local) im.crossOrigin = 'anonymous'
      im.onload = function () {
        self.image.setAttributeNS('http://www.w3.org/1999/xlink', 'href', self.url)
      }
      im.src = this.url
    }
  }
  mask.addEventListener('mouseover', function () {
    if (!self.isActivated) self.mask.setAttribute('fill-opacity', '1')
  })
  mask.addEventListener('mouseout', function () { self.mask.setAttribute('fill-opacity', '0') })
  // Initialement les boutons de la barre de gauche ne sont pas visibles.
  // Ils sont rendus visibles quand on crée les expandableBar
  g.setAttribute('visibility', 'hidden') // Sera changé pour les descendants
  g.addEventListener('mousedown', function (evt) {
    self.devicedown(evt)
  }, false)
  g.addEventListener('mousemove', function (evt) {
    if (!self.tipDisplayed && self.tip !== '') {
      self.deviceMove(evt)
    }
  }, false)
  g.addEventListener('touchstart', function (evt) {
    self.devicedown(evt)
  }, false)
  g.addEventListener('touchmove', function (evt) {
    if (!self.tipDisplayed && self.tip !== '') {
      self.deviceMove(evt)
    }
  }, false)
}

Button.prototype.devicedown = function (evt) {
  // var app = this.app;
  // var doc = app.doc;
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  // doc.type = type;
  this.clicks++
  if (this.clicks >= 2) {
    this.doubleClickAction()
    this.clicks = 0 // Ajout version 6.1.0
  } else {
    var self = this
    setTimeout(function () {
      switch (self.clicks) {
        case 1 :
          self.singleClickAction()
          self.clicks = 0 // Ajout version 6.1.0
          break
        case 2 :
          self.doubleClickAction()
          self.clicks = 0 // Ajout version 6.1.0
      }
    }, 300)
  }
  kernel.preventDefault(evt)
  evt.stopPropagation()
}

Button.prototype.deviceMove = function (evt) {
  // Si le bouton n'a pas de cadre(ToolBarArrow) pas de tip
  if (!this.bframe) return
  var app = this.app
  // var doc = app.doc;
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  // doc.type = type;
  if (!this.tipDisplayed && this.tip !== '') {
    app.cacheTip() // Sans argument pour effacer l'ancien tip quel qu'il soit
    this.tipDisplayed = false
    app.setTip(this)
    var self = this
    setTimeout(function () {
      self.app.cacheTip(self)
    }, 2000)
  }
  kernel.preventDefault(evt)
  evt.stopPropagation()
}
/**
 * Fonction appelée lors d'un simple clic sur la fonction
 * A redéfinir pour les descendants
 */
Button.prototype.singleClickAction = function () {
}
/**
 * Fonction appelée lors d'un double clic sur la fonction
 * A redéfinir pour les descendants si double click implémenté
 */
Button.prototype.doubleClickAction = function () {
  this.singleClickAction()
}
