/*
 * Created by yvesb on 08/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const kernelAdd = require('../kernel/kernelAdd')
const constantes = require('../kernel/constantes')
const $ = require('jquery')

module.exports = Slider

function Slider (app, width, height, min, max, startval, digits, step, tip, transform, append, y, fonc) {
  this.target = 'right' // Pour les tips
  var self = this
  var zf = app.zoomFactor
  width *= zf
  height *= zf
  this.app = app
  this.width = width
  this.height = height
  this.min = min
  this.max = max
  this.val = startval
  this.fonc = fonc
  this.step = step
  this.tip = kernel.getStr(tip)
  this.append = append // L'éventuelle chaîne à rajouter au bout de l'affichage
  this.y = y // Pour la ligne d'affichage du tip
  this.captured = false
  var gap = constantes.sliderGap * zf
  var g = kernel.cens('g', {
    transform: transform
  })
  this.container = g
  // Un rectangle à gauche qui contiendra le curseur et réagira à la souris
  var widthc = width - digits * constantes.sliderDigitsFontSize * 0.6 * zf // Largeur du curseur
  // this.widthc = widthc;
  var rectg = kernel.cens('rect', {
    x: 0,
    y: 0,
    width: widthc,
    height: height,
    stroke: 'none',
    fill: constantes.buttonBackGroundColor
  })
  g.appendChild(rectg)
  var rectd = kernel.cens('rect', {
    x: widthc,
    y: 0,
    width: width - widthc,
    height: height,
    fill: constantes.buttonBackGroundColor
  })
  g.appendChild(rectd)
  var yl = (height - constantes.sliderThickness * zf) / 2
  // On crée le trait horizontal
  this.segmentWidth = widthc - 2 * constantes.sliderRadius * zf - 2 * gap
  this.xc = gap + constantes.sliderRadius * zf // Abscisse de début du segment
  var seg = kernel.cens('line', {
    stroke: constantes.sliderColor,
    'stroke-linecap': 'round',
    x1: this.xc,
    y1: yl,
    x2: gap + this.segmentWidth,
    y2: yl
  })
  g.appendChild(seg)
  /*
  seg.addEventListener("mousedown", function(evt) {
    self.onbuttondevicedown(evt);
  });
  */
  $(seg).css('pointer-events', 'none')
  // On crée le rond qui sera le curseur
  this.xCurseur = gap + this.abscisse(startval)
  this.circ = kernel.cens('ellipse', {
    cx: this.xCurseur,
    cy: yl,
    rx: constantes.sliderRadius * zf,
    ry: (constantes.sliderHeight / 2 - 2) * zf,
    stroke: 'black',
    fill: 'url(#sliderGrad)'
  })
  $(this.circ).css('pointer-events', 'none')
  g.appendChild(this.circ)

  rectg.addEventListener('mouseup', function (ev) {
    self.ondeviceup(ev)
  })
  rectg.addEventListener('touchend', function (ev) {
    self.ondeviceup(ev)
  })
  rectg.addEventListener('mousedown', function (ev) {
    self.ondevicedown(ev, kernelAdd.mousePosition)
  })
  rectg.addEventListener('touchstart', function (ev) {
    self.ondevicedown(ev, kernelAdd.touchPosition)
  })
  rectg.addEventListener('mousemove', function (ev) {
    self.ondevicemove(ev, kernelAdd.mousePosition)
  })
  rectg.addEventListener('touchmove', function (ev) {
    self.ondevicemove(ev, kernelAdd.touchPosition)
  })

  // On crée à droite un affichage de la valeur
  this.txt = kernel.cens('text', {
    x: String(width - 2),
    y: height - 4 * zf,
    'font-size': String(constantes.sliderDigitsFontSize * zf) + 'px',
    style: 'stroke:blue;font-size:' + String(constantes.sliderDigitsFontSize * zf) + 'px;' + 'font-family:serif;text-anchor:end;'
  })
  this.txt.appendChild(document.createTextNode(this.val + this.append))
  g.appendChild(this.txt)
}
Slider.prototype.abscisse = function (val) {
  return (val - this.min) / (this.max - this.min) * this.segmentWidth + constantes.sliderRadius * this.app.zoomFactor
}
/*
Slider.prototype.onbuttondevicedown = function(evt) {
  this.captured = true;
  evt.preventDefault();
  //evt.stopPropagation(evt);
};
*/

Slider.prototype.ondeviceup = function (evt) {
  this.captured = false
  kernel.preventDefault(evt)
  // evt.stopPropagation(evt);
}

Slider.prototype.ondevicedown = function (ev, fonc) {
  var newval
  var zf = this.app.zoomFactor
  // this.captured = false;
  var svg = this.app.svgFigure
  // Ligne suivante modifié version 6.5.1
  var point = fonc(svg, ev, this.app.zoomFactor) // Renvoie les coordonnées de la souris par rapport au svgFigure
  var x = point.x - parseInt(svg.getAttribute('width'))
  if (Math.abs(x - this.xCurseur) < 2 * constantes.sliderRadius * zf) this.captured = true
  else {
    if (x > this.xCurseur) {
      newval = this.val + this.step
      if (newval > this.max) newval = this.val
      this.updateValue(newval)
      this.updatePosition()
    } else {
      if (x < this.xCurseur) {
        newval = this.val - this.step
        if (newval < this.min) newval = this.val
        this.updateValue(newval)
        this.updatePosition()
      }
    }
  }
  kernel.preventDefault(ev)
  // ev.stopPropagation();
}

Slider.prototype.ondevicemove = function (ev, fonc) {
  var self = this
  if (this.captured) {
    var svg = this.app.svgFigure
    // Ligne suivante modifiée version 6.5.1
    var point = fonc(svg, ev, this.app.zoomFactor) // Renvoie les coordonnées de la souris par rapport au svgFigure
    var x = point.x - parseInt(svg.getAttribute('width'))
    var x1 = this.xc + this.segmentWidth
    if (x < this.xc) x = this.xc
    else if (x > x1) x = x1
    var abs = (x - this.xc) / this.segmentWidth
    var newval = Math.floor(this.min + abs * (this.max - this.min) + 0.5)
    this.updateValue(newval)
    this.updatePosition()
    kernel.preventDefault(ev)
    // ev.stopPropagation();
  } else {
    if (!self.tipDisplayed) {
      self.app.cacheTip() // Sans argument pour effacer l'ancien tip quel qu'il soit
      self.tipDisplayed = false // Sera modifié par app
      self.app.setTip(self)
      setTimeout(function () {
        self.app.cacheTip(self)
      }, 2500)
    }
  }
}
Slider.prototype.updatePosition = function () {
  this.xCurseur = this.xc + this.segmentWidth * (this.val - this.min) / (this.max - this.min)
  this.circ.setAttribute('cx', this.xCurseur)
  this.updateDisplay()
}

Slider.prototype.updateDisplay = function () {
  this.txt.removeChild(this.txt.firstChild)
  this.txt.appendChild(document.createTextNode(this.val + this.append))
}

Slider.prototype.updateValue = function (newval) {
  this.val = newval
  this.fonc(newval) // Pour appeler la fonction affectant cette valeur
}

Slider.prototype.update = function (newVal) {
  this.updateValue(newVal)
  this.updatePosition()
}
