/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CFoncComplexe = require('./CFoncComplexe')
const CAppelFonction = require('./CAppelFonction')
module.exports = CAppelFonctionComplexe

/**
 * Objet d'un arbre binaire de calcul représentant un appel de fonction complexe utilisateur.
 * @constructor
 * @extends CAppelFonction
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {type} fonctionAssociee  Peut pointer sur une fonction ou une suite.
 * @param {type} operande  L'opérande dont on cherche l'image.
 * @returns {CAppelFonctionComplexe}
 */
function CAppelFonctionComplexe (listeProprietaire, fonctionAssociee, operande) {
  CAppelFonction.call(this, listeProprietaire, fonctionAssociee, operande)
  // this.z1loc = new mtg32.Complexe(); Version 4.9.7
}
CAppelFonctionComplexe.prototype = new CAppelFonction()
CAppelFonctionComplexe.prototype.constructor = CAppelFonctionComplexe
CAppelFonctionComplexe.prototype.superClass = 'CAppelFonction'
CAppelFonctionComplexe.prototype.className = 'CAppelFonctionComplexe'

CAppelFonctionComplexe.prototype.getClone = function (listeSource, listeCible) {
  var cloneOperande = this.operande.getClone(listeSource, listeCible)
  // à revoir car la fonction a déjà été créée
  var ind1 = listeSource.indexOf(this.fonctionAssociee)
  return new CAppelFonctionComplexe(listeCible, listeCible.get(ind1, 'CCalculAncetre'), cloneOperande)
}

CAppelFonctionComplexe.prototype.getCopie = function () {
  return new CAppelFonctionComplexe(this.listeProprietaire, this.fonctionAssociee, this.operande.getCopie())
}

CAppelFonctionComplexe.prototype.calculAvecValeursRemplacees = function (bfrac) {
  var fon = (this.fonctionAssociee.className === 'CFonc') ? new CFoncComplexe(this.listeProprietaire, null, false, '', '',
    this.fonctionAssociee.chaineVariable, this.fonctionAssociee.calcul.calculAvecValeursRemplacees(bfrac)) : this.fonctionAssociee
  return new CAppelFonction(this.listeProprietaire, fon, this.operande.calculAvecValeursRemplacees(bfrac))
}
