/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CCb = require('./CCb')
const CCbGlob = require('../types/Color')
module.exports = CAppelFonctionInterneNVar
/**
 * @constructor
 * @extends CCb
 * Objet d'un arbre binaire de calcul représentant un appel de fonction interne
 * à n variables.
 * Utilisé dans les calculs de dérivées et dérivées partielles.
 * @param {CCb} fonctionInterne  Pointe sur la fonction à appliquer.
 * @param {CCb} operande  Pointe sur l'opérande
 * @returns {CAppelFonctionInterneNVar}
 */
function CAppelFonctionInterneNVar (fonctionInterne, operande) {
  CCb.call(this, fonctionInterne.listeProprietaire)
  this.fonctionInterne = fonctionInterne
  this.operande = operande
}
CAppelFonctionInterneNVar.prototype = new CCb()
CAppelFonctionInterneNVar.prototype.constructor = CAppelFonctionInterneNVar
CAppelFonctionInterneNVar.prototype.superClass = 'CCb'
CAppelFonctionInterneNVar.prototype.className = 'CAppelFonctionInterneNVar'

CAppelFonctionInterneNVar.prototype.nature = function () {
  return CCbGlob.natAppelFonctionInterneNVar
}

CAppelFonctionInterneNVar.prototype.resultat = function (infoRandom) {
  var nvar = this.operande.length
  var d = new Array(nvar)
  for (var i = 0; i < nvar; i++) d[i] = this.operande[i].resultat(infoRandom)
  return this.fonctionInterne.resultatFonction(infoRandom, d)
}

CAppelFonctionInterneNVar.prototype.resultatFonction = function (infoRandom, valeurParametre) {
  var nvar = this.operande.length
  var d = new Array(nvar)
  for (var i = 0; i < nvar; i++) d[i] = this.operande[i].resultatFonction(infoRandom, valeurParametre)
  return this.fonctionInterne.resultatFonction(infoRandom, d)
}

CAppelFonctionInterneNVar.prototype.getCopie = function () {
  var nvar = this.operande.length
  var tab = new Array(nvar)
  for (var i = 0; i < nvar; i++) tab[i] = this.operande[i].getCopie()
  return new CAppelFonctionInterneNVar(this.fonctionInterne.getCopie(), this.operande)
}

CAppelFonctionInterneNVar.prototype.dependDeVariable = function (ind) {
  var res = false
  var nvar = this.operande.length
  for (var i = 0; i < nvar; i++) {
    res = res || (this.fonctionInterne.dependDeVariable(i) && this.operande[i].dependDeVariable(ind))
  }
  return res
}

CAppelFonctionInterneNVar.prototype.estConstant = function () {
  var res = false
  var nvar = this.operande.length
  for (var i = 0; i < nvar; i++) {
    res = res || this.operande[i].estConstant()
  }
  return res
}

CAppelFonctionInterneNVar.prototype.deriveePossible = function (indiceVariable) {
  var b = true
  for (var i = 0; (i < this.operande.length) && b; i++) {
    if (this.operande[i].dependDeVariable(indiceVariable)) { b = this.fonctionInterne.deriveePossible(i) && this.operande[i].deriveePossible(indiceVariable) }
  }
  return b
}

CAppelFonctionInterneNVar.prototype.calculAvecValeursRemplacees = function (bfrac) {
  var nbvar = this.fonctionInterne.nombreVariables()
  var tabcal = new Array(this.fonctionInterne.nombreVariables())
  for (var i = 0; i < nbvar; i++) {
    tabcal[i] = this.operande[i].calculAvecValeursRemplacees(bfrac)
  }
  return new CAppelFonctionInterneNVar(this.fonctionInterne, tabcal)
}

CAppelFonctionInterneNVar.prototype.membreGauche = function () {
  return this.fonctionInterne.membreGauche()
}

CAppelFonctionInterneNVar.prototype.membreDroit = function () {
  return this.fonctionInterne.membreDroit()
}

// Pour cette classe, isCalcVect renvoie fausse par défaut
CAppelFonctionInterneNVar.prototype.isCalcOK4Vect = function (tabNames) {
  var nbvar = this.fonctionInterne.nombreVariables()
  for (var i = 0; i < nbvar; i++) {
    if (!this.operande[i].isCalcVect(tabNames) && !this.operande[i].isVectNul()) return false
  }
  return true
}
