/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const CCb = require('./CCb')
const CCbGlob = require('../kernel/CCbGlob')
const Ope = require('../types/Ope')
module.exports = CConstante

/**
 * Classe représentant une constante dans un arbre binaire de calcul.
 * @constructor
 * @extends CCb
 * @param {CListeObjets} listeProprietaire  La liste propriétaire de l'objet.
 * @param {Number} valeur  La valeur rebdue par la constante.
 * @returns {CConstante}
 */
function CConstante (listeProprietaire, valeur) {
  CCb.call(this, listeProprietaire)
  if (arguments.length !== 1) this.valeur = valeur
}
CConstante.prototype = new CCb()
CConstante.prototype.constructor = CConstante
CConstante.prototype.superClass = 'CCb'
CConstante.prototype.className = 'CConstante'

CConstante.prototype.nature = function () {
  return CCbGlob.natConstante
}

CConstante.prototype.getClone = function (listeSource, listeCible) {
  return new CConstante(listeCible, this.valeur)
}

CConstante.prototype.getCopie = function () {
  return new CConstante(this.listeProprietaire, this.valeur)
}

CConstante.prototype.dependDeVariable = function (ind) {
  return false
}

CConstante.prototype.resultat = function (infoRandom) {
  return this.valeur
}

CConstante.prototype.resultatComplexe = function (infoRandom, zRes) {
  zRes.x = this.valeur// Par défaut on suppose le calcul réel
  zRes.y = 0
}

CConstante.prototype.chaineCalcul = function (varFor) {
  return kernel.chaineNombre(this.valeur, 12)
}

CConstante.prototype.estConstant = function () {
  return true
}

CConstante.prototype.estConstante0 = function () {
  return this.valeur === 0
}

CConstante.prototype.read = function (inps, list) {
  CCb.prototype.read.call(this, inps, list)
  this.valeur = inps.readDouble()
}

CConstante.prototype.write = function (oups, list) {
  CCb.prototype.write.call(this, oups, list)
  oups.writeDouble(this.valeur)
}

// Déplacé ici version 6.4.7
CConstante.prototype.calculNormalise = function (rempval, sousdivnorm, rempDecParFrac) {
  // Attention : A cause d'un problème webPack les require ci-dessous ne doivent pas être en tête de fichier
  const COp = require('../objets/COperation')
  if (rempDecParFrac) {
    const liste = this.listeProprietaire
    // CConstante co  = (CConstante) this;
    var va = this.valeur
    if (va === Math.floor(va)) return this
    var n = Math.floor(va * Math.pow(10, 9) + 0.5)
    var d = 1000000000
    if (!kernel.zero13(va - n / Math.pow(10, 9))) return this
    var g = kernel.pgcd(n, 1000000000)
    var num = n / g
    var den = d / g
    if (sousdivnorm) {
      if (num === 1) return new COp(liste, new CConstante(liste, 1), new CConstante(liste, d / g), Ope.Divi)
      else {
        return new COp(liste, new CConstante(liste, num), new COp(liste, new CConstante(liste, 1),
          new CConstante(liste, den), Ope.Divi), Ope.Mult)
      }
    } else return new COp(liste, new CConstante(liste, num), new CConstante(liste, den), Ope.Divi)
  } else return this
}
