/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatCal = require('../types/NatCal')
const CFonc = require('./CFonc')
const CalcC = require('../kernel/CalcC')
const CCalcul = require('./CCalcul')
module.exports = CFoncComplexe

/**
 * Classe représentant une fonction complexe utilisateur à une variable.
 * @constructor
 * @extends CFonc
 * @param {CListeObjets} listeProprietaire  La liste propriétaire de l'objet;
 * @param {CImplementationProto} impProto  null ou pointe sur la construction propriétaire de l'objet;
 * @param {boolean} estElementFinal  true si l'objet est un objet inal de construction;
 * @param {String} nomCalcul  Le nom de la fonction.
 * @param {String} chaineCalcul  La formule définissnt la focntion;
 * @param {String} chaineVariable  La varaible formelle utilisée pour la formule.
 * @param {CCb} calcul  Arbre binaire repsentant le calcul.
 * @returns {CFoncComplexe}
 */
function CFoncComplexe (listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul, chaineVariable, calcul) {
  if (arguments.length === 1) CFonc.call(this, listeProprietaire)
  else {
    if (arguments.length === 3) CCalcul.call(this, listeProprietaire, impProto, estElementFinal)
    else {
      this.chaineVariable = chaineVariable
      if (arguments.length <= 6) calcul = null
      if (calcul === null) {
        CFonc.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul,
          chaineVariable, CalcC.ccbComp(chaineCalcul, listeProprietaire, 0, chaineCalcul.length - 1,
            [chaineVariable]))
      } else CCalcul.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul, calcul)
    }
  }
}
CFoncComplexe.prototype = new CFonc()
CFoncComplexe.prototype.constructor = CFoncComplexe
CFoncComplexe.prototype.superClass = 'CFonc'
CFoncComplexe.prototype.className = 'CFoncComplexe'

CFoncComplexe.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.impProto)
  var calculClone = this.calcul.getClone(listeSource, listeCible)
  return new CFoncComplexe(listeCible, listeCible.get(ind1, 'CImplementationProto'),
    this.estElementFinal, this.nomCalcul, this.chaineCalcul, this.chaineVariable, calculClone)
}
CFoncComplexe.prototype.getNatureCalcul = function () {
  return NatCal.NFonctionComplexe
}
