/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatCal = require('../types/NatCal')
const CalcC = require('../kernel/CalcC')
const CFoncNVar = require('./CFoncNVar')
module.exports = CFoncComplexeNVar

/**
 * Fonction complexe de n variables.
 * @constructor
 * @extends CFoncNVar
 * @param {CListeObjets} listeProprietaire  La liste propriétaire de l'objet;
 * @param {CImplementationProto} impProto  null ou pointe sur la construction propriétaire de l'objet;
 * @param {boolean} estElementFinal  true si l'objet est un objet inal de construction;
 * @param {String} nomCalcul  Le nom de la fonction.
 * @param {String} chaineCalcul  La formule définissnt la fonction;
 * @param {Arrray} chaineVariable  Un array de String donnant les variables formelles.
 * @param {CCb} calcul  Un arbre binaire de calcul représentant la fonction.
 * @returns {CFoncComplexeNVar}
 */
// Attention : Ordre des deux derniers paramètres changé par rapport à Java
function CFoncComplexeNVar (listeProprietaire, impProto, estElementFinal, nomCalcul,
  chaineCalcul, chaineVariable, calcul = null) {
  if (arguments.length === 1) CFoncNVar.call(this, listeProprietaire)
  else {
    if (calcul === null) {
      CFoncNVar.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul,
        chaineVariable, CalcC.ccbComp(chaineCalcul, listeProprietaire, 0,
          chaineCalcul.length - 1, chaineVariable))
    } else {
      CFoncNVar.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul,
        chaineVariable, calcul)
    }
  }
}
CFoncComplexeNVar.prototype = new CFoncNVar()
CFoncComplexeNVar.prototype.constructor = CFoncComplexeNVar
CFoncComplexeNVar.prototype.superClass = 'CFoncNVar'
CFoncComplexeNVar.prototype.className = 'CFoncComplexeNVar'

CFoncComplexeNVar.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.impProto)
  var calculClone = this.calcul.getClone(listeSource, listeCible)
  return new CFoncComplexeNVar(listeCible, listeCible.get(ind1, 'CImplementationProto'),
    this.estElementFinal, this.nomCalcul, this.chaineCalcul, this.chaineVariable, calculClone)
}
CFoncComplexeNVar.prototype.getNatureCalcul = function () {
  switch (this.nbVar) {
    case 2 :
      return NatCal.NFonctionComplexe2Var
    case 3 :
      return NatCal.NFonctionComplexe3Var
  }
}
/* Version 4.9.7 : Transféré dans CFoncComplexe
CFoncComplexeNVar.prototype.rendValeurFonctionComplexe = function(infoRandom, parametre, zRes) {
  this.calcul.resultatFonctionComplexe(infoRandom, parametre, zRes);
}
*/
