/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CMacro = require('./CMacro')
const CMacroAvecListe = require('./CMacroAvecListe')
const CSousListeObjets = require('./CSousListeObjets')
module.exports = CMacroSuiteMacros

/**
 * Macro exécutant à la suite les une des autres une liste de macros.
 * @constructor
 * @extends CMacroAvecListe
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CImplementationProto} impProto  null ou la construction propriétaire.
 * @param {boolean} estElementFinal  true si l'objet est un élément final de construction.
 * @param {Color} couleur  La couleur d'éciture de l'éditeur (et du cadre éventuel).
 * @param {number} xNom  L'abscisse d'affichage de la macro
 * @param {number} yNom  L'ordonnée d'affichage de la macro
 * @param {number} decX  Décalage horizontal du nom
 * @param {number} decY  Décalage vertical du nom
 * @param {boolean} masque  true si l'éditeur est masqué
 * @param {CPt} pointLie  null ou pointe sur un point auquel l'affichage est lié.
 * @param {number} taillePolice  Indice de la taiile de police utilisée
 * @param {boolean} effacementFond  true si on efface le fond de l'en-tête.
 * @param {Color} couleurFond  La couleur de fond de l'en-tête.
 * @param {number} alignementHorizontal  0 pour alignement gauche, 1 pour centre, 2 pour droite.
 * @param {number} alignementVertical  0 pour alignement vers le haut, 1 pour centré, 2 pour bas.
 * @param {String} intitule  Le titre de la macro
 * @param {String} commentaireMacro  Eventuel commentaire explicatif.
 * @param {CListeObjets} listeAssociee  Liste contenant les macros à exécuter.
 * @returns {CMacroSuiteMacros}
 */
function CMacroSuiteMacros (listeProprietaire, impProto, estElementFinal, couleur,
  xNom, yNom, decX, decY, masque, pointLie, taillePolice, effacementFond, couleurFond, alignementHorizontal,
  alignementVertical, intitule, commentaireMacro, listeAssociee) {
  if (arguments.length !== 0) {
    if (arguments.length === 1) CMacroAvecListe.call(this, listeProprietaire)
    else {
      CMacroAvecListe.call(this, listeProprietaire, impProto, estElementFinal, couleur,
        xNom, yNom, decX, decY, masque, pointLie, taillePolice, effacementFond, couleurFond,
        alignementHorizontal, alignementVertical, intitule, commentaireMacro, listeAssociee)
    }
  }
}
CMacroSuiteMacros.prototype = new CMacroAvecListe()
CMacroSuiteMacros.prototype.constructor = CMacroSuiteMacros
CMacroSuiteMacros.prototype.superClass = 'CMacroAvecListe'
CMacroSuiteMacros.prototype.className = 'CMacroSuiteMacros'

CMacroSuiteMacros.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.pointLie)
  var ind2 = listeSource.indexOf(this.impProto)
  var listeAssocieeClone = new CSousListeObjets()
  listeAssocieeClone.setImage(this.listeAssociee, listeSource, listeCible)
  return new CMacroSuiteMacros(listeCible, listeCible.get(ind2, 'CImplementationProto'),
    this.estElementFinal, this.couleur, this.xNom, this.yNom, this.decX, this.decY,
    this.masque, listeCible.get(ind1, 'CPt'), this.taillePolice,
    this.effacementFond, this.couleurFond, this.alignementHorizontal, this.alignementVertical,
    this.intitule, this.commentaireMacro, listeAssocieeClone)
}
CMacroSuiteMacros.prototype.macroEnCours = function () {
  if (this.indiceMacroEnCours < this.listeAssociee.longueur()) { return this.listeAssociee.get(this.indiceMacroEnCours).macroEnCours() } else return this
}
/**
 * Ajout version 5.1
 * Il faut mettre indiceMacroEnCours à 0 car une macro peut déjà avoir été utilisée dans une autre macro
 * @param {type} mac
 * @returns {undefined}
 */
CMacroSuiteMacros.prototype.setMacroLanceuse = function (mac) {
  CMacro.prototype.setMacroLanceuse.call(this, mac)
  this.indiceMacroEnCours = 0
}

CMacroSuiteMacros.prototype.initialise = function () {
  var ptMacro = this.listeAssociee.get(0)
  ptMacro.initialise()
  ptMacro.setMacroLanceuse(this)
  this.indiceMacroEnCours = 0
  this.executionEnCours = true
  var mac = this.macroEnCours()
  mac.initialise()
  mac.executionEnCours = true
}
CMacroSuiteMacros.prototype.typeAnimation = function () {
  if (this.indiceMacroEnCours < this.listeAssociee.longueur()) { return this.macroEnCours().typeAnimation() } else return CMacro.sansAnimation
}
/**
 * Fait passer à l'exécution de la macro suivante de la liste.
 * @param {svg} svg  Le svg de la figure.
 * @param {Dimf} dimf  Les dimensions du svg
 * @param {Color} couleurFond  La couleur de fonc de la figure.
 * @param {number} transparence  Le coeficient de transparence de la figure.
 * @returns {undefined}
 */
CMacroSuiteMacros.prototype.passageMacroSuiv = function (svg, dimf, couleurFond, transparence) {
  // Modification java : si la macro en cours est une macro d'apparition d'objets avec clic pour
  // passer à l'objet suivant, on ne passe à la macro suivante qui si le dernier objet a été affiché.
  // Sinon on fait apparaitre l'objet suivant
  /*
  Ajout version 6.4 pour pouvoir décativer un enchainement de macros
   */
  if (this.listeProprietaire.terminerMacros) {
    this.indiceMacroEnCours = 0
    this.executionEnCours = false
    return
  }
  //
  var mac = this.macroEnCours()
  if (mac.className === 'CMacroApparition') {
    if (mac.clicPourObjetSuivant && (mac.indiceDernierObjetTraite < mac.listeAssociee.longueur() - 1)) {
      this.listeProprietaire.update(svg, couleurFond, transparence, true)
      return
    }
  }
  // mac.termineAction(svg, dimf, couleurFond, transparence);
  this.indiceMacroEnCours++
  if (this.indiceMacroEnCours < this.listeAssociee.longueur()) {
    mac = this.listeAssociee.get(this.indiceMacroEnCours) // Puet pointer sur une CMacroSuiteMacros
    mac.setMacroLanceuse(this)
    mac.initialise()
    mac = this.macroEnCours()
    mac.initialise()
    // if (macroEnCours().typeAnimation() == CMacro.sansAnimation) cadre.paneFigure.repaint();
    // executionEnCours = true;
    mac.execute(svg, dimf, couleurFond, transparence, true)
  } else {
    /* Modification version 5.0
    this.listeProprietaire.macroEnCours = null;
    this.executionEnCours = false;
    */
    this.executionEnCours = false
    if (this.macroLanceuse === null) {
      this.listeProprietaire.macroEnCours = null
    } else {
      this.macroLanceuse.passageMacroSuiv(svg, dimf, couleurFond, transparence)
    }
  }
}
CMacroSuiteMacros.prototype.agitSurVariable = function (va) {
  for (var i = 0; i < this.listeAssociee.longueur(); i++) {
    var ptMacro = this.listeAssociee.get(i)
    if (ptMacro.agitSurVariable(va)) return true
  }
  return false
}
CMacroSuiteMacros.prototype.affecteValeurAVariable = function (val) {
  for (var i = 0; i < this.listeAssociee.longueur(); i++) {
    var ptMacro = this.listeAssociee.get(i)
    if (ptMacro.affecteValeurAVariable(val)) return true
  }
  return false
}
CMacroSuiteMacros.prototype.execute = function (svg, dimf, couleurFond, transparence) {
  var mac
  // Modification version 5.1 : On informe chaque macro de la liste de la macro qui la lance
  for (var i = 0; i < this.listeAssociee.longueur(); i++) {
    mac = this.listeAssociee.get(i)
    mac.setMacroLanceuse(this)
  }
  //
  mac = this.listeAssociee.get(0)
  mac.execute(svg, dimf, couleurFond, transparence)
}
CMacroSuiteMacros.prototype.executeSimple = function (dimf) {
  for (var i = 0; i < this.listeAssociee.longueur(); i++) {
    var mac = this.listeAssociee.get(i)
    mac.executeSimple(dimf)
  }
}
// Fonction optimisée version 6.4.1
CMacroSuiteMacros.prototype.executionPossible = function () {
  for (var i = 0; i < this.listeAssociee.longueur(); i++) {
    var ptMacro = this.listeAssociee.get(i)
    if (!ptMacro.executionPossible()) return false
  }
  return true
}
