/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatCal = require('../types/NatCal')
const Vect = require('../types/Vect')
const CMesureX = require('./CMesureX')
module.exports = CMesureY

/**
 *
 * Classe représentant la mesure de l'ordonnnée d'un point dans un repère.
 * @constructor
 * @extends CValDyn
 * @param {CListeObjets} listeProprietaire  la liste propriétaire.
 * @param {CImplementationProto} impProto  null ou la construction propriétaire.
 * @param {boolean} estElementFinal  true si élément final de construction.
 * @param {string} nomCalcul  Le nom du calcul
 * @param {CRepere} repereAssocie  Le repère dans lequel on fait la mesure.
 * @param {CPt} pointMesure  Le point dont on mesure l'abscisse.
 * @returns {CMesureY}
 */
function CMesureY (listeProprietaire, impProto, estElementFinal, nomCalcul, repereAssocie, pointMesure) {
  if (arguments.length === 1) CMesureX.call(this, listeProprietaire)
  else {
    CMesureX.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul, repereAssocie,
      pointMesure)
  }
}
CMesureY.prototype = new CMesureX()
CMesureY.prototype.constructor = CMesureY
CMesureY.prototype.superClass = 'CMesureX'
CMesureY.prototype.className = 'CMesureY'

// Ajout version 5.0
CMesureY.prototype.debutNom = function () {
  return 'yCoord'
}

CMesureY.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.repereAssocie)
  var ind2 = listeSource.indexOf(this.pointMesure)
  var ind3 = listeSource.indexOf(this.impProto)
  // Ajout version 5.0
  var nomCalc = (this.nomCalcul.length === 0) ? listeCible.genereNomPourCalcul(this.debutNom()) : this.nomCalcul
  //
  return new CMesureY(listeCible, listeCible.get(ind3, 'CImplementationProto'),
    this.estElementFinal, nomCalc, listeCible.get(ind1, 'CRepere'), listeCible.get(ind2, 'CPt'))
}
CMesureY.prototype.getNatureCalcul = function () {
  return NatCal.NMesureOrdonneeRepere
}
CMesureY.prototype.positionne = function (infoRandom, dimfen) {
  this.existe = this.repereAssocie.existe && this.pointMesure.existe
  if (!this.existe) return
  // L'orientation sur l'écran est l'inverse de celle du repère
  // choisi.Il faut donc permuter les rôles de u et v}
  var u = new Vect(this.repereAssocie.o, this.repereAssocie.i)
  var v = new Vect(this.repereAssocie.o, this.repereAssocie.j)
  // Calcul des  abscisses et ordonnées de l'origine réelle du repère dans le repère de base
  var absOrigine = this.repereAssocie.valAbscisseOrigine / this.repereAssocie.unitex
  var ordOrigine = this.repereAssocie.valOrdonneeOrigine / this.repereAssocie.unitey
  var xno = this.repereAssocie.o.x - u.x * absOrigine - v.x * ordOrigine
  var yno = this.repereAssocie.o.y - u.y * absOrigine - v.y * ordOrigine
  // Calcul de l'abscisse par résolution d'un système linéaire
  var det = u.x * v.y - v.x * u.y
  this.abscisse = (u.x * (this.pointMesure.y - yno) - u.y * (this.pointMesure.x - xno)) / det * this.repereAssocie.unitey
}
