/*
 * Created by yvesb on 10/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const CAffLiePt = require('../objets/CAffLiePt')
const CElementGraphique = require('../objets/CElementGraphique')
const Color = require('../types/Color')
const TikzColor = require('../types/TikzColor')
const kernel = require('../kernel/kernel')
const kernelAdd = require('../kernel/kernelAdd')
const StyleEncadrement = require('../types/StyleEncadrement')
const Fonte = require('../types/Fonte')

module.exports = CAffLiePt

CAffLiePt.prototype.translateDe = function (decalagex, decalagey) {
  this.xNom += decalagex
  this.yNom += decalagey
}

CAffLiePt.prototype.depDe4Rec = function (p) {
  var dep = CElementGraphique.prototype.depDe4Rec.call(this, p) || this.angText.depDe4Rec(p)
  if (this.pointLie === null) { return dep } else { return dep || this.pointLie.depDe4Rec(p) || this.angText.depDe4Rec(p) }
}

CAffLiePt.prototype.addTikzColors = function (stb, vect) {
  var st = CElementGraphique.prototype.addTikzColors.call(this, stb, vect)
  if (this.effacementFond) {
    var coulFond = this.couleurFond
    if (coulFond !== Color.black) {
      var code = TikzColor.colorString(coulFond)
      if (!TikzColor.containsColor(vect, coulFond)) {
        vect.push(code)
        st += '\\definecolor{' + code + '}{rgb}{' +
          kernel.chaineNombre(coulFond.getRed() / 255, 2) + ',' +
        kernel.chaineNombre(coulFond.getGreen() / 255, 2) + ',' +
        kernel.chaineNombre(coulFond.getBlue() / 255, 2) + '}' + '\n'
      }
    }
  }
  return st
}

CAffLiePt.prototype.tikz = function (dimf, nomaff, coefmult, bu) {
  var list = this.listeProprietaire
  var ang = String(this.angText.rendValeurRadian() * kernel.ConvRadDeg)
  var x0 = this.xNom + this.decX
  var y0 = this.yNom + this.decY
  var style = ''
  switch (this.alignementVertical) {
    case CAffLiePt.alignVerLow :
      style = 'above '
      y0 += 2
      break
    case CAffLiePt.alignVerTop :
      style = 'below '
      y0 -= 2
  }
  switch (this.alignementHorizontal) {
    case CAffLiePt.alignHorLeft :
      style += 'right '
      x0 -= 2
      break
    case CAffLiePt.alignHorRight :
      x0 += 2
      style += 'left '
  }
  var x = list.tikzXcoord(x0, dimf, bu)
  var y = list.tikzYcoord(y0, dimf, bu)

  style += ',' + this.tikzCouleur() + ','
  if (this.encadrement !== StyleEncadrement.Sans) {
    style += 'draw=' + this.tikzCouleur() + ','
    var dinsepx = list.tikzLongueur(2, dimf, bu)
    var dinsepy = list.tikzLongueur(4, dimf, bu)
    var insepx = kernel.chaineNombre(dinsepx, 2)
    var insepy = kernel.chaineNombre(dinsepy, 2)
    style += 'inner xsep =' + insepx + 'cm,inner ysep =' + insepy + 'cm,'
  }
  if (this.effacementFond) style += 'fill=' + TikzColor.colorString(this.couleurFond) + ','

  var ch = Fonte.remplaceUnicodeParLatex(this.chaineAAfficher)
  ch = ch.replace(/\n/g, '\\\\')
  ch = Fonte.traiteCodesDiese(ch)
  ch = kernelAdd.adaptStringTikz(ch)// Pour remplacer les % par \%
  if (ang === '0') {
    return '\\node at ' + '(' + kernel.chaineNombre(x, 3) + ', ' + kernel.chaineNombre(y, 3) +
      ') [align=left,' + style + ',font= \\sf ' + this.tikzFont(dimf, this.taillePolice, coefmult, bu) + '] {' + ch + '};'
  } else {
    return '\\node at ' + '(' + kernel.chaineNombre(x, 3) + ', ' + kernel.chaineNombre(y, 3) + ') [rotate=' + ang + ',align=left,' + style + 'font= \\sf ' +
      this.tikzFont(dimf, this.taillePolice, coefmult, bu) + '] {' + ch + '};'
  }
}

CAffLiePt.prototype.adaptRes = function (coef) {
  CElementGraphique.prototype.adaptRes.call(this, coef)
  this.xNom *= coef
  this.yNom *= coef
  this.taillePolice *= coef
}

CAffLiePt.prototype.estDefiniParObjDs = function (listeOb) {
  var res = this.angText.estDefiniParObjDs(listeOb)
  if (this.pointLie == null) return res
  else return res && this.pointLie.estDefPar(listeOb)
}

CAffLiePt.prototype.infoAngle = function () {
  var chval = this.angText.chaineInfo()
  if (chval === '0') return ''
  else return '\n' + kernel.getStr('Angle') + ' : ' + chval
}
