/*
 * Created by yvesb on 16/12/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const CArcDeCercle = require('../objets/CArcDeCercle')
const AngleArcDlg = require('../dialogs/AngleArcDlg')
const kernel = require('../kernel/kernel')
const CArcDeCercleAncetre = require('../objets/CArcDeCercleAncetre')
var Vect = require('../types/Vect')
module.exports = CArcDeCercle

CArcDeCercle.prototype.modifiableParMenu = function () {
  return !this.estElementFinal && (this.pointFinArc === null)
}

CArcDeCercle.prototype.modifDlg = function (app, callBack1, callBack2) {
  new AngleArcDlg(app, this, true, callBack1, callBack2)
}

CArcDeCercle.prototype.infoHist = function () {
  var ch = this.nom + ' : ' + kernel.getStr('chinfo190')
  ch += ' ' + kernel.getStr('chinfo202') + ' ' + this.o.nom + ' ' + kernel.getStr('chinfo203') + ' ' + this.a.nom
  if (this.pointFinArc !== null) ch += ' ' + kernel.getStr('chinfo204') + ' ' + this.pointFinArc.nom
  else ch += kernel.getStr('chinfo205') + ' ' + this.angleAuCentre.chaineInfo()
  return ch
}

/**
 * Fonction servant à savoir si un point
 * est par définition sur une droite. Renverra true par exemple
 * si la droite est définie par deux points et le point est un de ces deux points
 * ou si le point est lié à la droite
 **/

CArcDeCercle.prototype.contientParDefinition = function (po) {
  return (this.a === po)
}

CArcDeCercle.prototype.distancePointPourSurface = function (xp, yp) {
  if (!this.existe) return -1
  else {
    var appSecteur = this.surArc(xp, yp)
    if (!appSecteur) return -1
    else return Math.abs(kernel.distancePointPoint(this.centreX, this.centreY, xp, yp) - this.rayon)
  }
}

CArcDeCercle.prototype.distancePoint = function (xp, yp, masquage) {
  if (!this.existe || (masquage && this.masque)) return -1
  else return this.distancePointPourSurface(xp, yp, masquage)
}

CArcDeCercle.prototype.depDe4Rec = function (p) {
  if (this.elementTestePourDependDePourRec === p) return this.dependDeElementTestePourRec
  if (this.pointFinArc === null) {
    return this.memDep4Rec(CArcDeCercleAncetre.prototype.depDe4Rec.call(this, p) ||
      this.o.depDe4Rec(p) || this.a.depDe4Rec(p) || this.angleAuCentre.depDe4Rec(p))
  } else {
    return this.memDep4Rec(CArcDeCercleAncetre.prototype.depDe4Rec.call(this, p) ||
    this.o.depDe4Rec(p) || this.a.depDe4Rec(p) || this.pointFinArc.depDe4Rec(p))
  }
}

CArcDeCercle.prototype.tikz = function (dimf, nomaff, coefmult, bu) {
  var list = this.listeProprietaire
  var x1 = list.tikzXcoord(this.a.x, dimf, bu)
  var y1 = list.tikzYcoord(this.a.y, dimf, bu)
  var ray = list.tikzLongueur(this.rayon, dimf, bu)
  var u = new Vect(this.centreX, this.centreY, this.origine_x, this.origine_y)
  var ang1 = u.angleRad()
  var ang2 = ang1 + this.valeurAngleAuCentre
  if (ang2 < 0) ang2 = ang2 + 2 * Math.PI
  if (ang2 >= 2 * Math.PI) ang2 = ang2 - 2 * Math.PI
  var dif = kernel.mesurePrincipale(ang2 - ang1)
  if (dif > 0) {
    if (ang2 < ang1) ang2 = ang2 + 2 * Math.PI
  } else {
    if (ang2 > ang1) ang2 = ang2 - 2 * Math.PI
  }

  ang1 *= kernel.ConvRadDeg
  ang2 *= kernel.ConvRadDeg
  return '\\draw ' + this.tikzLineStyle(dimf, coefmult) + '(' + kernel.chaineNombre(x1, 3) + ',' + kernel.chaineNombre(y1, 3) + ') arc ' +
    '(' + kernel.chaineNombre(ang1, 3) + ' : ' + kernel.chaineNombre(ang2, 3) + ' : ' + kernel.chaineNombre(ray, 3) + ');'
}

CArcDeCercle.prototype.estDefiniParObjDs = function (listeOb) {
  if (this.pointFinArc == null) {
    return this.o.estDefPar(listeOb) &&
    this.a.estDefPar(listeOb) &&
    this.angleAuCentre.estDefPar(listeOb)
  } else {
    return this.o.estDefPar(listeOb) &&
    this.a.estDefPar(listeOb) &&
    this.pointFinArc.estDefPar(listeOb)
  }
}
