/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

// Il a fallu rajouter ce fichier car CListeObjetsAdd était devenu trop gros pour être géré par WebPack

const CListeObjets = require('../objets/CListeObjets')
const NatObj = require('../types/NatObj')
const NatCal = require('../types/NatCal')

module.exports = CListeObjets

/**
 * Fonction renvoyant un objet formé des intitulés des macro auxquelles on peut rajouter ou supprimer des objets
 * et des pointeurs sur ces macros
 * @param bAuMoinsUnElt ! Si true seules les macro ayant plus d'un objet dans la liste associée sont ajoutées.
 * @returns {{pointeurs: Array, noms: Array}}
 */
CListeObjets.prototype.listeMacrosAvecListeModif = function (bAuMoinsUnElt) {
  var pointeurs = []
  var noms = []
  for (var i = 0; i < this.longueur(); i++) {
    var el = this.get(i)
    if (el.estDeNature(NatObj.NMacro) && !el.estElementIntermediaire()) {
      if (el.ajoutOuRetraitObjetsPossible() && (bAuMoinsUnElt ? el.listeAssociee.longueur() > 1 : true)) {
        pointeurs.push(el)
        noms.push(el.intitule)
      }
    }
  }
  return { pointeurs: pointeurs, noms: noms }
}

/**
 * Fonction adaptant tous les objets de la liste à une autre résolution graphique
 * @param coef Le coefficient d'agrandissement-réduction
 */
CListeObjets.prototype.adaptRes = function (coef) {
  for (var i = 0; i < this.longueur(); i++) this.get(i).adaptRes(coef)
}

/**
 * Fonction renvoyant true sir tous les objets de this sont définis exclusivement par des obejst de listeOb
 * @param listeOb
 * @returns {boolean}
 */
CListeObjets.prototype.estDefiniParObjDs = function (listeOb) {
  for (var i = 0; i < this.longueur(); i++) {
    if (!this.get(i).estDefiniParObjDs(listeOb)) return false
  }
  return true
}

/**
 * Fonction renvoyant un objet {pointerus, noms} pointant sur les objets numériques de la figure
 * qui peuvent être choisis comme objets finaux d'une construction
 * @param app
 * @returns {{pointeurs: Array, noms: Array}}
 */
CListeObjets.prototype.listObjFinDisp = function (app) {
  var ind; var i; var elb; var j; var el2
  var pointeurs = []
  var noms = []

  var listeSources = new CListeObjets()
  listeSources.ajouteElementsDe(app.listeSrcG)
  listeSources.ajouteElementsDe(app.listeSrcNG)
  // Les calculs ou fonctions constants et dont dépend au moins un objet dépendant de au moins un objet source
  // sont ajoutés à la liste des objest sources.
  // objets finaux
  // On fait de même avec les variables pour que les variables servant à
  // définir les sommes indicées soient rajoutées

  i = -1
  for (ind = 0; ind < this.longueur(); ind++) {
    i++
    elb = this.get(ind)
    if (elb.estDeNatureCalcul(NatCal.NVariable)) {
      // CValeurDynamique val = (CValeurDynamique) elb;
      if (!listeSources.contains(elb)) {
        for (j = i + 1; j < this.longueur(); j++) {
          el2 = this.get(j)
          if (el2.depDe(elb) && el2.dependDeAuMoinsUn(listeSources)) {
            listeSources.add(elb)
            break
          }
        }
      }
    } else {
      if (elb.estDeNatureCalcul(NatCal.NTtCalcouFoncRouC)) {
        // CCalcul val = (CCalcul) elb;
        if (elb.estConstant()) {
          if (!listeSources.contains(elb)) {
            for (j = i + 1; j < this.longueur(); j++) {
              el2 = this.get(j)
              if (el2.depDe(elb) && el2.dependDeAuMoinsUn(listeSources)) {
                listeSources.add(elb)
                break
              }
            }
          }
        }
      }
    }
  }
  // On rajoute aussi la possibilité de chosir comme objet final une variable qui
  // a été choisie dans les objets sources. Si cette variable est choisie, elle sera retirée des objets sources
  for (ind = 0; ind < this.longueur(); ind++) {
    elb = this.get(ind)
    // On accepte aussi les repères comme objets numériques
    if (elb.estDeNatureCalcul(NatCal.NTtObjNum) && !elb.estElementIntermediaire()) {
      if (elb.estDeNatureCalcul(NatCal.NVariable)) {
        if (listeSources.contains(elb)) {
          pointeurs.push(elb)
          noms.push(elb.getNom())
        }
      } else {
        if ((elb.estDefiniParObjDs(listeSources)) && !listeSources.contains(elb)) {
          pointeurs.push(elb)
          noms.push(elb.getNom())
        }
      }
    }
  }
  return { pointeurs: pointeurs, noms: noms }
}

/**
 * Fonction réordonnant les éléments de this en respectant l'ordre de listeSource
 * @param listeSource
 */
CListeObjets.prototype.reordonne = function (listeSource) {
  var lis = new CListeObjets()
  for (var i = 0; i < listeSource.longueur(); i++) {
    var elb = listeSource.get(i)
    if (this.contains(elb)) lis.add(elb)
  }
  // lis contient maintenant les éléments dans le bon ordre
  this.retireTout()
  this.ajouteElementsDe(lis)
}

/**
 * Fonction retirant de la liste l'objet obj
 * @param obj
 */
CListeObjets.prototype.removeObj = function (obj) {
  var ind = this.indexOf(obj)
  if (ind !== -1) this.remove(this.indexOf(obj))
}

/**
 * Ajout version 6.1.0
 * Renvoie un pointeur sur le premier point de la liste non intermédiaire et non masqué ayant un nom
 * et coincidant avec pt
 * @param pt
 * @returns {Cpt}
 */
CListeObjets.prototype.premierPointNommeProcheDe = function (pt) {
  for (var i = 0; i < this.longueur(); i++) {
    var el = this.get(i)
    if (el.estDeNature(NatObj.NTtPoint) && (el !== pt) && !el.masque && !el.estElementIntermediaire() && (el.nom !== '')) {
      if (el.coincideAvec(pt)) return el
    }
  }
  return null
}

/**
 * Fonction renvoyant un pointeur vers une mesure de longueur d'un segement délimité par les points pt1 et pt2 si
 * elle existe et null sinon.
 * @param pt1
 * @param pt2
 * @returns {*}
 */
CListeObjets.prototype.pointeurMesLong = function (pt1, pt2) {
  for (var i = 0; i < this.longueur(); i++) {
    var el = this.get(i)
    if (el.estDeNatureCalcul(NatCal.NMesureLongueur) && ((el.point1 === pt1 && el.point2 === pt2) || (el.point1 === pt2 && el.point2 === pt1))) return el
  }
  return null
}

// Ajout version 6.3.0
/**
 * Fonction mettant à jour les macros de la liste. Appelé quand on sauve la figure après une action.
 * Par exemple, si une macro incrémente une variable, si on modifie un objet dépendant de cette variable, il doit etre ajouté
 * à la liste des objets sur lesquels agit cette macro.
 */
CListeObjets.prototype.metAJourMacros = function () {
  for (var i = 0; i < this.longueur(); i++) {
    var el = this.get(i)
    if (el.estDeNature(NatObj.NMacro)) el.metAJour()
  }
}
