/*
 * Created by yvesb on 10/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Vect = require('../types/Vect')
const kernel = require('../kernel/kernel')
const CSegment = require('../objets/CSegment')
const CDroite = require('../objets/CDroite')
const NatObj = require('../types/NatObj')
module.exports = CSegment

/**
 * Fonction utilisée par le protocole de la figure et renvoyant soit le nom de la droite (donnée lors de la boîte
 * de dialogue de protocole soit la chaîne de caractères décriavnt la doite ((OA) par exemple).
 * @returns string
 */
CSegment.prototype.getNom = function () {
  return '[' + this.point1.nom + this.point2.nom + ']'
}

/**
 * Fonction utilisée pour générer un nom pour le protocole de la figure
 * @returns {string} Le nom généré
 */
CSegment.prototype.genereNom = function () {
  CSegment.ind++
  return 'seg' + CSegment.ind
}

CSegment.prototype.infoHist = function () {
  return kernel.getStr('Seg') + ' ' + this.getNom()
}

CSegment.prototype.depDe4Rec = function (p) {
  if (this.elementTestePourDependDePourRec === p) return this.dependDeElementTestePourRec
  return this.memDep4Rec(CDroite.prototype.depDe4Rec.call(this, p) ||
    this.point1.depDe4Rec(p) || this.point2.depDe4Rec(p))
}

CSegment.prototype.distancePoint = function (xp, yp, masquage) {
  if (!(this.existe) || (masquage && this.masque)) return -1
  else {
    var vec = new Vect(this.xext1, this.yext1, this.xext2, this.yext2)
    var n = vec.norme()
    if (kernel.zero(n)) return kernel.distancePointPoint(this.xext1, this.yext1, xp, yp)
    else {
      kernel.projetteOrtho(xp, yp, this.xext1, this.yext1, vec, this.pointr)
      if (this.appartientA(this.pointr.x, this.pointr.y)) {
        return kernel.distancePointPoint(xp, yp, this.pointr.x, this.pointr.y)
      } else return -1
    }
  }
}

CSegment.prototype.contientParDefinition = function (po) {
  return (this.point1 === po) || (this.point2 === po)
}

CSegment.prototype.estParalleleParDefinition = function (dte) {
  return (dte === this) || (this.point1.appartientDroiteParDefinition(dte) && this.point2.appartientDroiteParDefinition(dte)) ||
    dte.formentDroiteParallele(this.point1, this.point2)
}

CSegment.prototype.coincideAvec = function (p) {
  if (p.getNature() !== NatObj.NSegment) return false
  return ((this.point1.coincideAvec(p.point1) && (this.point2.coincideAvec(p.point2))) ||
    (this.point1.coincideAvec(p.point2) && (this.point2.coincideAvec(p.point1))))
}

CSegment.prototype.estDefiniParObjDs = function (listeOb) {
  return this.point1.estDefPar(listeOb) &&
  this.point2.estDefPar(listeOb)
}

CSegment.prototype.isIsomTo = function (p) {
  if (p.getNature() !== NatObj.NSegment) return false
  var vect1 = new Vect(this.point1, this.point2)
  var vect2 = new Vect(p.point1, p.point2)
  return (((this.point1.nom === p.point1.nom) && (this.point2.nom === p.point2.nom)) ||
    ((this.point1.nom === p.point2.nom) && (this.point2.nom === p.point1.nom))) &&
    kernel.zero(vect1.norme() - vect2.norme())
}
