/*
 * Created by yvesb on 10/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CSurfaceLieuDroite = require('../objets/CSurfaceLieuDroite')
const CSurface = require('../objets/CSurface')
const kernel = require('../kernel/kernel')
module.exports = CSurfaceLieuDroite

CSurfaceLieuDroite.prototype.infoHist = function () {
  return this.nom + ' : ' + kernel.getStr('chinfo60') + ' ' + this.bord.nom + ' ' + kernel.getStr('et') +
    ' ' + this.droiteAssociee.getNom()
}

CSurfaceLieuDroite.prototype.depDe4Rec = function (p) {
  if (this.elementTestePourDependDePourRec === p) return this.dependDeElementTestePourRec
  return this.memDep4Rec(CSurface.prototype.depDe4Rec.call(this, p) ||
    this.droiteAssociee.depDe4Rec(p))
}

CSurfaceLieuDroite.prototype.metAJourTableaux = function () {
  // + 4 et non +3 car si le lieu est changé en lieu fermé, un point de plus sera rajouté au lieu
  this.abscisses = new Array(this.bord.infoLieu.nombreDePoints + 4)
  this.ordonnees = new Array(this.bord.infoLieu.nombreDePoints + 4)
}

CSurfaceLieuDroite.prototype.aPourBord = function (lieu) {
  return (lieu === this.bord)
}

CSurfaceLieuDroite.prototype.distancePoint = function (xp, yp, masquage, distmin) {
  var dist
  if (!this.existe || (masquage && this.masque)) return -1
  else {
    dist = this.bord.distancePointPourSurface(xp, yp, masquage, distmin)
    if ((dist !== -1) && (dist < distmin)) return dist
    dist = this.segmentDebut.distancePoint(xp, yp, masquage)
    if ((dist !== -1) && (dist < distmin)) return dist
    dist = this.segmentDroite.distancePoint(xp, yp, masquage)
    if ((dist !== -1) && (dist < distmin)) return dist
    dist = this.segmentFin.distancePoint(xp, yp, masquage)
    if ((dist !== -1) && (dist < distmin)) return dist
    return -1
  }
}

CSurfaceLieuDroite.prototype.tikz = function (dimf, nomaff, coefmult, bu) {
  var x1; var y1; var x4; var y4
  var bord = this.bord
  var list = this.listeProprietaire
  var droiteAssociee = this.droiteAssociee
  var point2 = this.point2
  var point3 = this.point3
  // On recopie les coordonnées des points du lieu
  var indicePremierPoint = bord.infoLignes[bord.indicePremiereLigne].indicePremierPoint
  var indiceDernierPoint = bord.infoLignes[bord.indicePremiereLigne].indicePremierPoint +
    this.nbPointsLigne - 1
  x1 = bord.xCoord[indicePremierPoint]
  y1 = bord.yCoord[indicePremierPoint]
  kernel.projetteOrtho(x1, y1, droiteAssociee.point_x, droiteAssociee.point_y, droiteAssociee.vect, point2)
  x4 = bord.xCoord[indiceDernierPoint]
  y4 = bord.yCoord[indiceDernierPoint]
  kernel.projetteOrtho(x4, y4, droiteAssociee.point_x, droiteAssociee.point_y, droiteAssociee.vect, point3)
  return '\\fill' + this.tikzFillStyle() + this.bord.tikzOnePath(dimf, nomaff, coefmult, bu) + ' -- (' +
    kernel.chaineNombre(list.tikzXcoord(point3.x, dimf, bu), 3) + ',' +
    kernel.chaineNombre(list.tikzYcoord(point3.y, dimf, bu), 3) + ')--(' +
    kernel.chaineNombre(list.tikzXcoord(point2.x, dimf, bu), 3) + ',' +
    kernel.chaineNombre(list.tikzYcoord(point2.y, dimf, bu), 3) + ')--cycle;'
}

CSurfaceLieuDroite.prototype.estDefiniParObjDs = function (listeOb) {
  return CSurface.prototype.estDefiniParObjDs.call(this, listeOb) &&
  this.droiteAssociee.estDefPar(listeOb)
}
