/*
 * Created by yvesb on 10/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CSurfaceSecteurCirculaire = require('../objets/CSurfaceSecteurCirculaire')
const kernel = require('../kernel/kernel')
const Vect = require('../types/Vect')
const NatArc = require('../types/NatArc')
module.exports = CSurfaceSecteurCirculaire

CSurfaceSecteurCirculaire.prototype.infoHist = function () {
  return this.nom + ' : ' + kernel.getStr('chinfo60') + ' ' + this.bord.nom
}

CSurfaceSecteurCirculaire.prototype.peutGenererLieuObjet = function () {
  return true
}

CSurfaceSecteurCirculaire.prototype.distancePoint = function (xp, yp, masquage) {
  if (!this.existe || (masquage && this.masque)) return -1
  return this.arc.distancePointPourSurface(xp, yp, masquage)
}

CSurfaceSecteurCirculaire.prototype.tikz = function (dimf, nomaff, coefmult, bu) {
  var ang1; var ang2; var dif
  var list = this.listeProprietaire
  var arc = this.arc
  var x1 = list.tikzXcoord(arc.origine_x, dimf, bu)
  var y1 = list.tikzYcoord(arc.origine_y, dimf, bu)
  var xc = list.tikzXcoord(arc.centreX, dimf, bu)
  var yc = list.tikzYcoord(arc.centreY, dimf, bu)
  var ray = list.tikzLongueur(arc.rayon, dimf, bu)
  var u = new Vect(arc.centreX, arc.centreY, arc.origine_x, arc.origine_y)
  ang1 = u.angleRad()
  ang2 = ang1 + arc.valeurAngleAuCentre
  if (ang2 < 0) ang2 = ang2 + 2 * Math.PI
  if (ang2 >= 2 * Math.PI) ang2 = ang2 - 2 * Math.PI
  switch (this.natureArcBord) {
    case NatArc.PetitArc :
      dif = kernel.mesurePrincipale(ang2 - ang1)
      if (dif > 0) {
        if (ang2 < ang1) ang2 = ang2 + 2 * Math.PI
      } else {
        if (ang2 > ang1) ang2 = ang2 - 2 * Math.PI
      }
      ang1 *= kernel.ConvRadDeg
      ang2 *= kernel.ConvRadDeg
      return '\\fill ' + this.tikzFillStyle() + '(' + kernel.chaineNombre(xc, 3) + ',' + kernel.chaineNombre(yc, 3) + ')--(' +
        '(' + kernel.chaineNombre(x1, 3) + ',' + kernel.chaineNombre(y1, 3) + ') arc ' +
        '(' + kernel.chaineNombre(ang1, 3) + ' : ' + kernel.chaineNombre(ang2, 3) + ' : ' + kernel.chaineNombre(ray, 3) + ')--cycle;'
    case NatArc.GrandArc :
      dif = kernel.mesurePrincipale(ang2 - ang1)
      if (dif > 0) {
        if (ang2 > ang1) ang2 = ang2 - 2 * Math.PI
      } else {
        if (ang2 < ang1) ang2 = ang2 + 2 * Math.PI
      }
      ang1 *= kernel.ConvRadDeg
      ang2 *= kernel.ConvRadDeg
      return '\\fill ' + this.tikzFillStyle() + '(' + kernel.chaineNombre(xc, 3) + ',' + kernel.chaineNombre(yc, 3) + ')--(' +
        kernel.chaineNombre(x1, 3) + ',' + kernel.chaineNombre(y1, 3) + ') arc ' +
        '(' + kernel.chaineNombre(ang1, 3) + ' : ' + kernel.chaineNombre(ang2, 3) + ' : ' + kernel.chaineNombre(ray, 3) + ')--cycle;'
    case NatArc.ArcDirect :
      if (ang2 < ang1) ang2 = ang2 + 2 * Math.PI
      ang1 *= kernel.ConvRadDeg
      ang2 *= kernel.ConvRadDeg
      return '\\fill ' + this.tikzFillStyle() + '(' + kernel.chaineNombre(xc, 3) + ',' + kernel.chaineNombre(yc, 3) + ')--(' +
        '(' + kernel.chaineNombre(x1, 3) + ',' + kernel.chaineNombre(y1, 3) + ') arc ' +
        '(' + kernel.chaineNombre(ang1, 3) + ' : ' + kernel.chaineNombre(ang2, 3) + ' : ' + kernel.chaineNombre(ray, 3) + ')--cycle;'
    case NatArc.ArcIndirect :
      if (ang2 > ang1) ang2 = ang2 - 2 * Math.PI
      ang1 *= kernel.ConvRadDeg
      ang2 *= kernel.ConvRadDeg
      return '\\fill ' + this.tikzFillStyle() + '(' + kernel.chaineNombre(xc, 3) + ',' + kernel.chaineNombre(yc, 3) + ')--(' +
        '(' + kernel.chaineNombre(x1, 3) + ',' + kernel.chaineNombre(y1, 3) + ') arc ' +
        '(' + kernel.chaineNombre(ang1, 3) + ' : ' + kernel.chaineNombre(ang2, 3) + ' : ' + kernel.chaineNombre(ray, 3) + ')--cycle;'
  }
  return ''
}
