/*
 * Created by yvesb on 16/12/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CTransformation = require('../objets/CTransformation')
const ChoixModificationImageDlg = require('../dialogs/ChoixModificationImageDlg')
module.exports = CTransformation

/**
 * Fonction utilisée dans le protocole de la figure et renvoyant true si l'objet peut figurer
 * dans la boîte de dialogue de protocole de la figure.
 * Seulement redéfini pour CBipoint
 * @returns {boolean}
 */
CTransformation.prototype.estVisibleDansHist = function () {
  return true
}

CTransformation.prototype.infoHist = function () {
  return this.complementInfo()
}

CTransformation.prototype.imageModifiableParMenu = function () {
  return false
}

// Modifié version 4.9.9. Renvoie true si uen modification a effectivement eu lieu
// Modifié version 5.1.3.
// Ajout d'un troisième paramètre bEnregistrement.
// Si ce paramètre est true, l'étatt de la figure est enregistré après la modification
// Lorsque cette fonction est appelée par ProtocoloDlg certains points ont été nommés et la figure ne doit pas être enregistée à ce niveau.
CTransformation.prototype.modificationImage = function (app, image, bEnregistement) {
  var liste = app.listePr
  var nbImages = liste.nombreImagesParTransformation(this)
  var self = this
  if (nbImages === 1) {
    // Une seule image pour cette transformation
    // On appelle le dialogue que fournit cette transformation pour se modifier
    this.modifDlg(app, function () { self.callBack0(app, bEnregistement) }, null)
  } else {
    new ChoixModificationImageDlg(app, function () { self.callBack1(app, bEnregistement) }, function () { self.callBack2(app, image) })
  }
}

CTransformation.prototype.modificationImageForProtocol = function (app, image, bEnregistement, callBack) {
  this.modifDlg(app, callBack, null)
}

/**
 * Fonction appelée par le dialogue dans le cas où il n'y a qu'une image par la transformation
 */
CTransformation.prototype.callBack0 = function (app, bEnregistrement, callBack = null) {
  const list = app.listePr
  list.initialiseDependances()
  list.positionneDependantsDe(false, app.dimf, this, true)
  list.updateDependants(this, app.svgFigure, app.doc.couleurFond, app.doc.opacity, true)
  if (bEnregistrement) app.outilActif.saveFig()
  if (callBack !== null) callBack()
}

/**
 * Fonction ajoutant à la liste list des éléments montrant quels sont les objets avec lesquels l'objet a été construit
 * Pour les translations définies par origine et extrémité on rajoute en particulier un vecteur clignotant
 * @param list La liste clignotante à laquelle ajouterr les objets antécédents
 * @param {MtgApp}app L'application propriétaire. Ce paramèètre ne sert que pour les translations
 */
// Attention : Il faut passer le paramètre app à cause du cas des translations qui rajoutent un vecteur virtuel
CTransformation.prototype.ajouteAntecedents = function (list, app) {
}

/**
 * Fonction appelée par le dialogue dans le cas où l'utilisateur a choisi que la modification de la transformation
 * s'applique à tous les objets images.
 */
CTransformation.prototype.callBack1 = function (app, bEnregistrement) {
  var self = this
  this.modifDlg(app, function () {
    self.callBack0(app, bEnregistrement)
  }, null)
}

/**
 * Fonction appelée par le dialogue dans le cas où l'utilisateur a choisi que la modification de la transformation
 * s'applique au seul objet cliqué.
 */
CTransformation.prototype.callBack2 = function (app, image) {
  var self = this
  var list = this.listeProprietaire
  var transclone = this.getClone(list, list)
  app.insereElement(transclone, list.indexOf(this) + 1)
  transclone.modifDlg(app, function () { self.callBack3(app, image, transclone) }, function () { self.callBack4(app, transclone) })
}

CTransformation.prototype.callBack3 = function (app, image, transclone) {
  var list = app.listePr
  list.initialiseDependances()
  list.updateIndexes()
  image.associeATransformation(transclone)
  list.positionneDependantsDe(false, app.dimf, transclone, true)
  list.updateDependants(transclone, app.svgFigure, app.doc.couleurFond, app.doc.opacity, true)
  // A revoir pour eercice de construction
}

CTransformation.prototype.callBack4 = function (app, transclone) {
  app.detruitElement(transclone) // Pour retirer la transformation clone ajoutée
  app.listePr.updateIndexes()
}
