/*
 * Created by yvesb on 16/09/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatObj = require('../types/NatObj')
const CDroiteImage = require('../objets/CDroiteImage')
const CCercleImage = require('../objets/CCercleImage')
const CArcDeCercleImage = require('../objets/CArcDeCercleImage')
const CPointImage = require('../objets/CPointImage')
const CDemiDroiteImage = require('../objets/CDemiDroiteImage')
const CSegment = require('../objets/CSegment')
const CLigneBrisee = require('../objets/CLigneBrisee')
const CPolygone = require('../objets/CPolygone')
const CNoeudPointeurSurPoint = require('../objets/CNoeudPointeurSurPoint')
const StyleTrait = require('../types/StyleTrait')
const CCercleOR = require('../objets/CCercleOR')
const Color = require('../types/Color')
const CValeur = require('../objets/CValeur')
const CDemiDroiteOA = require('../objets/CDemiDroiteOA')
const CIntDroiteCercle = require('../objets/CIntDroiteCercle')
const CPointLieBipoint = require('../objets/CPointLieBipoint')
const MotifPoint = require('../types/MotifPoint')
const CRotation = require('../objets/CRotation')
const CValeurAngle = require('../objets/CValeurAngle')
const CHomothetie = require('../objets/CHomothetie')
const CCommentaire = require('../objets/CCommentaire')
const CAffLiePt = require('../objets/CAffLiePt')
const StyleEncadrement = require('../types/StyleEncadrement')

module.exports = Outil
/**
 * Classe ancêtre de tous les outils pouvant agir sur la figure.
 * @param app Application propriétaire
 * @param {string} toolName Le nom de l'outil
 * @param {int} toolIndex L'index de l'outil tel que dans la version Java
 * @param {boolean} avecClign true si l'outil utilise un clignotement d'objets lors de la création
 * @param {boolean} isSelectable true si l'outil est sélectionnable, false s'il est à action immédiate quand
 * on clique sur le bouton. true par defaut
 * @param {boolean} allways true si l'outil est toujours présent dans les exercices de construction
 * @constructor
 */
function Outil (app, toolName, toolIndex, avecClign = false, isSelectable = true, always = false) {
  this.app = app
  this.toolName = toolName
  this.avecClign = avecClign
  this.toolIndex = toolIndex
  this.timer = null
  this.isSelectable = isSelectable
  this.always = always
  this.cursor = 'crosshair'
}
/**
 * Fonction sélectionnant l'outil
 */

Outil.prototype.select = function () {
  // Le nombre de points créés au clic de la souris si l'option de création des points au clic est activée
  var app = this.app
  // Ajout version 6.4 : Si une macro est en cors d'exécution on l'arrête.
  app.termineMacroEnCours()
  //
  this.nbPtsCrees = 0
  var btn = this.app['button' + this.toolName]
  // Certains outils ne sont pas associés à un bouton (obtenus pas bouton +)
  if (btn) btn.activate(true)
  app.cacheDesignation()
  app.outilActifPrec = app.outilActif // Ajout version 6.3.3
  app.outilActif = this
  // if (app.nameEditor.isVisible) app.nameEditor.montre(false);
}

/**
 * Fonction déselectionant l'outil
 */
Outil.prototype.deselect = function () {
  var app = this.app
  var btn = this.app['button' + this.toolName]
  // Certains outils ne sont pas associés à un bouton (obtenus pas bouton +)
  if (btn) btn.activate(false)
  if (this.avecClign) this.annuleClignotement()
  app.supprimeObjetsVisuels()
  app.listeExclusion.retireTout()
  // La ligne suivante remet à jour la barre d'icônes car une fois qu'un outil est déselectionné pour
  // en sélecionner un autre certains outils peuvent ne plus être activables alors que  d'autres peuvent l'être
  // app.updateToolsToolBar();// Supprimé et déplacé dans Gestionnaire.enregistreFigureEnCours
  // Certains outils d'affichage ont céé un affichage clignotatnt pour désigner l'endroit d'affichage
  // (commentaire libre, latex libre, affichage de valeur libre, macros)
  if (this.comClig) {
    this.comClig.removegElement(app.svgFigure)
    this.comClig = null
  }
  // Lorsque la création de point par défaut est activée on peut avoir créé des points qu'il fait détruire si l'objet
  // n'a pas été créé
  if (this.nbPtsCrees > 0) {
    var list = app.listePourConst
    for (var i = 0; i < this.nbPtsCrees; i++) {
      var pt = list.get(list.longueur() - i - 1)
      if (app.nameEditor.isVisible && app.nameEditor.eltAssocie === pt) app.nameEditor.montre(false)
      pt.removegElement(app.svgFigure)
    }
    app.detruitDerniersElements(this.nbPtsCrees)
    this.nbPtsCrees = 0
  }
  // if (app.nameEditor.isVisible) app.nameEditor.montre(false);
}

Outil.prototype.reselect = function () {
  this.deselect()
  this.select()
}

/**
 * Fonction renvoyant éventuellement une indication supplémentaire à rajouter devant indication()
 * et suivi de :
 * @returns {string}
 */
Outil.prototype.preIndication = function () {
  return ''
}

/**
 * Fonction renvoyant true si l'outil est activable
 * A redéfinir pour les descendants
 * @returns {boolean}
 */
Outil.prototype.activationValide = function () {
  return true
}
/*
 Fonction relançant le clignotement de la liste listeClignotante de l'application this.app
 */
Outil.prototype.resetClignotement = function () {
  if (this.timer !== null) this.annuleClignotement()
  this.app.clignotementPair = true
  var app = this.app
  this.timer = setInterval(function () {
    app.actionClignotement()
  }, 500)
}
/*
 Fonction annulant le clignotement de la liste listeClignotante de l'application this.app
 */
Outil.prototype.annuleClignotement = function () {
  clearInterval(this.timer)
  this.timer = null
  var app = this.app
  if (!this.app.clignotementPair) {
    app.listeClignotante.montreTout(true)
    var doc = app.doc
    app.listeClignotante.update(app.svgFigure, doc.couleurFond, doc.opacity, true)
  }
  this.app.listeClignotante.retireTout()
}
/**
 * Fonction ajoutant el à la liste listeClignotatnte de this.app l'élément el
 * @param el CElementGraphique : l'objet à rajouter à la liste clignotante
 */
Outil.prototype.ajouteClignotementDe = function (el) {
  this.app.listeClignotante.add(el)
}

/**
 * Fonctio enlevant du clignotement actif (l'objet obj
 * @param obj
 */
Outil.prototype.enleveDeClign = function (obj) {
  clearInterval(this.timer)
  this.timer = null
  var app = this.app
  var list = app.listeClignotante
  if (!this.app.clignotementPair) {
    list.montreTout(true)
    var doc = app.doc
    list.update(app.svgFigure, doc.couleurFond, doc.opacity, true)
  }
  for (var i = 0; i < list.longueur(); i++) {
    var el = list.get(i)
    if (el === obj) {
      list.col.splice(i, 1)
    }
  }
  this.app.clignotementPair = true
  this.timer = setInterval(function () {
    app.actionClignotement(app)
  }, 500)
}

/**
 * Fonction ajoutanat à la liste listeObjetsVisuels de l'application this.app
 * les élémengts graphiques servant à visualiser l'action de l'outil.
 * A redéfinir pour les descendants
 */
Outil.prototype.ajouteObjetsVisuels = function () {
}
/**
 * Fonction rajoutant à al liste listeExclusion de l'application this.app l'élément el
 * el ne pourra pas être désigné par poinatge de souris.
 * @param el CElementGraphique : L'objet à rajouter
 */
Outil.prototype.excluDeDesignation = function (el) {
  this.app.listeExclusion.add(el)
}

Outil.prototype.addImage = function (el, bEditionNom = true) {
  this.app.ajouteElement(el, bEditionNom)
}

Outil.prototype.ajouteImageParTransformation = function (objet, trans, nombreObjetsCrees) {
  var app = this.app
  var list = app.listePr
  var coul = app.getCouleur()
  var st = app.getStyleTrait()
  var taille = app.getTaillePoliceNom()
  nombreObjetsCrees.setValue(nombreObjetsCrees.getValue() + 1)
  app.nameEditor.montre(false) // Pour annuler l'édition d'un éventuel objet image précédent
  trans.positionne(false) // Nécessaire car ajouteElement ne positionne que l'image et pas la transformation
  if (objet.estDeNature(NatObj.NTtPoint)) {
    this.addImage(new CPointImage(list, null, false, coul, false, 0, 3, false, '',
      taille, app.getStylePoint(), false, objet, trans))
  } else {
    var nat = objet.getNature()
    switch (nat) {
      case NatObj.NDroite :
        this.addImage(new CDroiteImage(list, null, false, coul, false, 0, 0, false, '', taille, st, 0.9, objet, trans))
        break
      case NatObj.NDemiDroite :
        this.addImage(new CDemiDroiteImage(list, null, false, coul, false, st, objet, trans))
        break
      case NatObj.NCercle :
        this.addImage(new CCercleImage(list, null, false, coul, false, st, objet, trans))
        break
      case NatObj.NArc :
        this.addImage(new CArcDeCercleImage(list, null, false, coul, false, st, objet, trans))
        break
      case NatObj.NSegment :
        var ptim1, ptim2
        ptim1 = list.pointImageDePar(objet.point1, trans)
        if (ptim1 === null) {
          ptim1 = new CPointImage(list, null, false, coul, false, 0, 3, false, '',
            taille, app.getStylePoint(), false, objet.point1, trans)
          this.addImage(ptim1, false)
          nombreObjetsCrees.setValue(nombreObjetsCrees.getValue() + 1)
        }
        ptim2 = list.pointImageDePar(objet.point2, trans)
        if (ptim2 === null) {
          ptim2 = new CPointImage(list, null, false, coul, false, 0, 3, false, '',
            taille, app.getStylePoint(), false, objet.point2, trans)
          this.addImage(ptim2, false)
          nombreObjetsCrees.setValue(nombreObjetsCrees.getValue() + 1)
        }
        this.addImage(new CSegment(list, null, false, coul, false, st, ptim1, ptim2))
        break
      case NatObj.NLigneBrisee :
      case NatObj.NPolygone :
        var col = []
        var lig, ptim
        if (nat === NatObj.NLigneBrisee) { lig = new CLigneBrisee(list, null, false, coul, false, app.getStyleTrait(), col) } else lig = new CPolygone(list, null, false, coul, false, app.getStyleTrait(), col)
        for (var i = 0; i < objet.colPoints.length; i++) {
          var pt = objet.colPoints[i].pointeurSurPoint
          ptim = list.pointImageDePar(pt, trans)
          if (ptim === null) {
            ptim = new CPointImage(list, null, false, coul, false, 0, 3, false, '',
              taille, app.getStylePoint(), false, pt, trans)
            this.addImage(ptim)
            nombreObjetsCrees.setValue(nombreObjetsCrees.getValue() + 1)
          }
          col.push(new CNoeudPointeurSurPoint(list, ptim))
        }
        lig.prepareTableauxCoordonnees()
        this.addImage(lig)
    }
  }
}

/**
 * Fonction qui devra être redéfinie pour les descendants
 * Renverra true si lorsque, sur un périphérique mobile, on relâche le doigt et qu'on traite un objet proche de ce doigt
 * Par exemple, pour un outil de création par deux points, il faudra que this.point1 ne soit pas null,
 * c'est-à-dire que le premier point ait déjà été désigné
 * @returns {boolean}
 */
Outil.prototype.isReadyForTouchEnd = function () {
  return true
}

/**
 * Fonction  redéfinir pour les descendants et servant sur les périphériques mobiles
 * Si cette fonction renvoie true, les événements touch ne sont pas propagés pour éviter
 * par exemple qu'on glisse dans la figure quand on faut glisser le doigt après avoir cliqué sur un premier point
 * @returns {boolean}
 */
Outil.prototype.isWorking = function () {
  return false
}

Outil.prototype.actionApresDlg = function () {
}

/**
 * Fonction qui devra être redéfinie par les outils qui utilisent un clic sur l'icône stop
 * en bas et à droite de la barre d'outils de droite pour finir une action.
 */
Outil.prototype.actionFin = function () {
}

/**
 * Méthode ajoutant un rapporteur à la liste d'objets visuels. Utilisée pour
 * l'outil rapporteur et peut-être plus tard les outils de création d'arc de
 * cercle. Renvoie un pointeur sur un point d'intersection entre le cercle
 * extérieur du rapporteur et une demi-droite joignant le centre de ce cercle
 * avec le point suivant le pointeur souris
 * Modifié version 5.0
 */
Outil.prototype.ajouteRapporteur = function (origine) {
  var app = this.app
  var doc = app.doc
  // On crée d'abord un cercle de centre origine et de rayon fixe
  // qui servira de support extérieur au rapporteur
  var liste = app.listeObjetsVisuels
  var stfc = StyleTrait.stfc(liste)
  var stfp = StyleTrait.stfp(liste)
  var bl = Color.black
  var ptrond = MotifPoint.petitRond
  var taille = app.getTaillePoliceNom()
  // Pour la version js on se sert d'un CCercleOR avec rayon en pixels
  var ptCercleRapExt = new CCercleOR(liste, null, false, bl, false, stfc, origine,
    new CValeur(liste, 80), true)
  app.ajouteObjetVisuel(ptCercleRapExt)
  // Puis un cercle intérieur
  var ptCercleRapInt = new CCercleOR(liste, null, false, bl, false, stfc, origine,
    new CValeur(liste, 40), true)
  app.ajouteObjetVisuel(ptCercleRapInt)
  // On crée un demi-droite de Origine vers PointSouris
  // var ptdemid = new CDemiDroiteOA(liste, null, false, Couleur
  //   .TraduitCouleur(1), false, StyleTrait.traitPointille, origine, frame
  //   .cadreActif().paneFigure.pointSouris);
  var ptdemid = new CDemiDroiteOA(liste, null, false, bl, true, stfp, origine, app.mousePoint)
  app.ajouteObjetVisuel(ptdemid)
  // On crée l'intersection de cette demi-droite avec le cercle extérieur
  // du rapporteur
  // CIntDroiteCercle ptint = new CIntDroiteCercle(liste, null, false, ptdemid,
  //   ptCercleRapExt);
  var ptint = new CIntDroiteCercle(liste, null, false, ptdemid, ptCercleRapExt)
  app.ajouteObjetVisuel(ptint)
  var ptplb = new CPointLieBipoint(liste, null, false, bl, true, 0, 0, false, '', taille, ptrond, false, ptint, 1)
  app.ajouteObjetVisuel(ptplb)
  // On rajoute des graduations au rapporteur
  // On crée d'abord une rotation d'angle 10°
  // CRotation ptrot10 = new CRotation(liste, null, false, origine, 10);
  var ptrot10 = new CRotation(liste, null, false, origine, new CValeurAngle(liste, 10))
  app.ajouteObjetVisuel(ptrot10)
  // On crée une homothétie servant à créer les graduations de 10°
  // CHomothetie hom1 = new CHomothetie(liste, null, false, origine, 0.85);
  var hom1 = new CHomothetie(liste, null, false, origine, new CValeur(liste, 0.85))
  app.ajouteObjetVisuel(hom1)
  // Et une autre pour les graduations de 90°
  var hom2 = new CHomothetie(liste, null, false, origine, new CValeur(liste, 0.5))
  app.ajouteObjetVisuel(hom2)
  // On crée les images par des rotations d'angle 10°
  var ptp1 = ptplb
  for (var i = 1; i <= 36; i++) {
    var ptim = new CPointImage(liste, null, false, Color.green, true, 0, 0, true, '', app.getTaillePoliceNom(),
      ptrond, false, ptp1, ptrot10)
    app.ajouteObjetVisuel(ptim)
    // On crée l'image du point créé par l'homothétie pour créer les
    // graduations
    // Pour les multiples de 90° elles sont rouges, sinon vertes
    // CPointImage ptima = new CPointImage(liste, null, false, Couleur
    //  .TraduitCouleur(3), 13, MotifPoint.Pixel, ptim, hom1);
    var ptima = new CPointImage(liste, null, false, Color.green, true, 0, 0, true, '', taille, ptrond, false, ptim, hom1)
    app.ajouteObjetVisuel(ptima)
    // On crée un segment graduation
    var coul, styleTrait
    if ((i % 9) === 0) {
      coul = 'red'
      styleTrait = new StyleTrait(liste, StyleTrait.styleTraitContinu, 2)
    } else {
      coul = 'blue'
      styleTrait = stfc
    }
    // On crée des graduations joignant le centre aux 90°
    if ((i % 9) === 0) {
      // CPointImage ptim2 = new CPointImage(liste, null, false, Couleur
      //   .TraduitCouleur(1), MotifPoint.Croix, 13, ptim, hom2);
      var ptim2 = new CPointImage(liste, null, false, bl, true, 0, 0, true, '', app.getTaillePoliceNom(), ptrond,
        false, ptim, hom2)
      app.ajouteObjetVisuel(ptim2)
      // CSegment ptseg90 = new CSegment(liste, null, false, origine, ptim2); // Segment
      // en
      // pointillés
      var ptseg90 = new CSegment(liste, null, false, bl, false, stfp, origine, ptim2)
      app.ajouteObjetVisuel(ptseg90)
      // CPointImage ptim3 = new CPointImage(liste, null, false, Couleur
      //  .TraduitCouleur(1), MotifPoint.Croix, 2, ptima, hom1);
      var ptim3 = new CPointImage(liste, null, false, bl, true, 0, 0, true, '', app.getTaillePoliceNom(), ptrond,
        false, ptima, hom1)
      app.ajouteObjetVisuel(ptim3)
      var chai
      switch (i) {
        case 9:
        case 27:
          chai = '90°'
          break
        case 18:
          chai = '180°'
          break
        case 36:
          chai = '0°'
          break
      }
      // CCommentaire ptcom = new CCommentaire(liste, null, false, Couleur
      //  .TraduitCouleur(coul), 0d, 0d,(int)(-8d*k), (int)(-6d*k), false, ptim3, (int)(k*10),
      //  StyleEncadrement.Sans, false, doc.couleurFond, chai,
      //  CAffLiePt.alignHorLeft, CAffLiePt.alignVerTop);
      var ptcom = new CCommentaire(liste, null, false, Color[coul], 0, 0, -8, -6, false, ptim3, 10, StyleEncadrement.Sans,
        false, doc.couleurFond, CAffLiePt.alignHorLeft, CAffLiePt.alignVerTop, chai)
      app.ajouteObjetVisuel(ptcom)
    }
    // CSegment ptgrad = new CSegment(liste, null, false, Couleur
    //  .TraduitCouleur(coul), false, styleTrait, ptima, ptim);
    var ptgrad = new CSegment(liste, null, false, Color[coul], false, styleTrait, ptima, ptim)
    app.ajouteObjetVisuel(ptgrad)
    // Pour 10° et -10° on ajoute un commentaire de graduation
    if ((i === 1) || (i === 35)) {
      // CPointImage ptim4 = new CPointImage(liste, null, false, Couleur
      //   .TraduitCouleur(1), MotifPoint.Croix, 13, ptima, hom1);
      var ptim4 = new CPointImage(liste, null, false, bl, true, 0, 0, true, '', app.getTaillePoliceNom(), ptrond,
        false, ptima, hom1)
      app.ajouteObjetVisuel(ptim4)
      // CCommentaire ptcom = new CCommentaire(liste, null, false, Couleur
      //  .TraduitCouleur(coul), 0d, 0d, (int)(-10d*k), (int)(-6d*k), false, ptim4, (int)(k*10),
      //  StyleEncadrement.Sans, false, doc.couleurFond, "10°",
      //  CAffLiePt.alignHorLeft, CAffLiePt.alignVerTop);

      ptcom = new CCommentaire(liste, null, false, Color[coul], 0, 0, -10, -6, false, ptim4, 10, StyleEncadrement.Sans,
        false, doc.couleurFond, CAffLiePt.alignHorLeft, CAffLiePt.alignVerTop, '10°')
      app.ajouteObjetVisuel(ptcom)
    }
    ptp1 = ptim
  }
  return ptplb
}

Outil.prototype.saveFig = function () {
  this.app.gestionnaire.enregistreFigureEnCours(this.toolName)
}

/**
 * Fonction renvoyant true si l'outil accepte qu'on crée un point par défaut lors d'un clic sur un endroit vide
 */
Outil.prototype.creationPointPossible = function () {
  return false
}

/**
 * Fonction interdisant la désignation comme objets sources graphiques d'objets dépendant des objets sources
 * graphiques ou non graphiques déjà désignés
 */
Outil.prototype.excluDesignationObjDepObjSrc = function () {
  var app = this.app
  var list = app.listePr
  for (var i = 0; i < list.longueur(); i++) {
    var elb = list.get(i)
    if (elb.dependDeAuMoinsUn(app.listeSrcNG) || elb.dependDeAuMoinsUn(app.listeSrcG) ||
      app.listeSrcNG.depDe(elb) || app.listeSrcG.depDe(elb)) {
      app.listeExclusion.add(elb)
    }
  }
}

/**
 * Fonction interdisant la designation d'objets dépendant de l'objet graphique elg
 * @param elg
 */
Outil.prototype.excluDesignationObjDepDe = function (elg) {
  var app = this.app
  var list = app.listePr
  for (var i = 0; i < list.longueur(); i++) {
    var elb = list.get(i)
    if (elb.depDe(elg)) app.listeExclusion.add(elb)
  }
}
