/*
 * Created by yvesb on 15/09/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatObj = require('../types/NatObj')
const Nat = require('../types/Nat')
const Outil = require('./Outil')
module.exports = OutilCapt
/**
 * Outil servant à capturer un point, un affichage, une marque d'angle ...
 * @param app L'application propriétaire
 * @constructor
 */
function OutilCapt (app) {
  Outil.call(this, app, 'Capt', 33023, false, true, true)
}
OutilCapt.prototype = new Outil()

OutilCapt.prototype.select = function () {
  Outil.prototype.select.call(this)
  var app = this.app
  var doc = this.app.doc
  var nature = NatObj.NPointMobile
  // Version 6.4 : Si on est en mode exercice, on ne permet de déplacer que les points mobiles
  if (!app.estExercice) {
    if (!doc.marquesAnglesFigees) nature = Nat.or(nature, NatObj.NMarqueAngle)
    if (!doc.affichagesFiges) nature = Nat.or(nature, NatObj.NAffLieAPoint, NatObj.NLieuObjet)
  }
  // var test = Nat.and(NatObj.NAffLieAPoint, NatObj.NCommentaire);
  app.outilPointageCapture.aDesigner = nature
  // On ajoute à la liste d'exclusion de désignation tous les points mobiles punaisés
  var li = doc.listePr
  for (var i = 0; i < li.longueur(); i++) {
    var elb = li.get(i)
    // Ajout version 6.2.0 pour ne pas pouvoir bouger un affichage de longueur orienté.
    if (elb.fixed) this.app.listeExclusion.add(elb)
    else {
      if (elb.estDeNature(NatObj.NPointMobile)) {
        if (!elb.estLibre()) this.app.listeExclusion.add(elb)
      }
    }
  }
  app.outilPointageActif = this.app.outilPointageCapture
  app.outilPointageActif.reset()
  // Version 6.4.7 : On utilise la pile de MathJax suite à rapport BusNag car sinon on peut vouloir afficher l'indication trop tôt
  // semble-t-il
  MathJax.Hub.Queue(function () {
    app.indication('indCapt')
  })
}

OutilCapt.prototype.isWorking = function () {
  return this.app.elementCapture !== null
}
