/*
 * Created by yvesb on 28/09/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Color = require('../types/Color')
const MotifPoint = require('../types/MotifPoint')
const StyleTrait = require('../types/StyleTrait')
const CImplementationProto = require('../objets/CImplementationProto')
const CSegment = require('../objets/CSegment')
const OutilPar2Pts = require('./OutilPar2Pts')
const CPointBase = require('../objets/CPointBase')
const kernel = require('../kernel/kernel')
module.exports = OutilCarre
/**
 * Outil servant à créer un carré direct en cliquant sur deux points
 * @param app L'application propriétaire
 * @constructor
 */
function OutilCarre (app) {
  OutilPar2Pts.call(this, app, 'Carre', 32990, true)
}

OutilCarre.prototype = new OutilPar2Pts()

OutilCarre.prototype.select = function () {
  this.app.outilPointageActif = this.app.outilPointageCre
  OutilPar2Pts.prototype.select.call(this) // A appeler à la fin
}

OutilCarre.prototype.creeObjet = function () {
  var app = this.app
  var li = app.listePr
  var indiceImpInitial = li.longueur()
  // On charge la construction qui est contenue dans this.app.docConst
  var proto = this.app.docCons.getPrototype('CarreDirect')
  this.annuleClignotement()
  proto.get(0).elementAssocie = this.point1
  proto.get(1).elementAssocie = this.point2
  var impProto = new CImplementationProto(li, proto)
  impProto.implemente(this.app.dimf, proto)
  impProto.nomProto = kernel.getStr('Carre')
  var ind = li.longueur() - 1
  var pt3 = li.get(ind)
  pt3.donneCouleur(app.getCouleur())
  pt3.donneMotif(app.getStylePoint())
  var pt4 = li.get(ind - 2)
  pt4.donneCouleur(app.getCouleur())
  pt4.donneMotif(app.getStylePoint())
  app.addSegment(this.point1, this.point2)
  app.addSegment(this.point2, pt3)
  app.addSegment(pt3, pt4)
  app.addSegment(pt4, this.point1)
  app.addPolygon(this.point1, this.point2, pt3, pt4, this.point1)
  this.saveFig()
  // Corrigé version 5.6.4
  if (app.estExercice) app.listePourConst = app.listePourConstruction()

  this.nbPtsCrees = 0
  li.afficheTout(indiceImpInitial, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
  return true
}

OutilCarre.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var li = app.listeObjetsVisuels
  var b = Color.black
  var stfp = new StyleTrait(li, StyleTrait.styleTraitPointille, 1)
  // var pt1 = this.point1.getClone(app.listePr,li);
  var pt1 = new CPointBase(li, null, false, b, true, 0, 0, true, '', app.getTaillePoliceNom(), MotifPoint.pixel, false, true,
    this.point1.x, this.point1.y)
  // app.ajouteObjetVisuel(pt1);
  // app.ajouteObjetVisuel(app.mousePoint);
  // Pour les edeux lignes suivantes on n'appelle pas ajouteObjetVisuel car les deux objets seront affichés plus tard
  app.listeObjetsVisuels.add(pt1)
  app.listeObjetsVisuels.add(app.mousePoint)
  var proto = app.docCons.getPrototype('CarreDirect')
  proto.get(0).elementAssocie = pt1
  proto.get(1).elementAssocie = app.mousePoint
  var impProto = new CImplementationProto(li, proto)
  impProto.implemente(app.dimf, proto, 0)
  impProto.nomProto = kernel.getStr('Carre')
  var ind = li.longueur() - 1
  // ON écupère les points créés par la construction
  var pt3 = li.get(ind)
  var pt4 = li.get(ind - 2)
  var seg1 = new CSegment(li, null, false, b, false, stfp, pt1, app.mousePoint)
  app.ajouteObjetVisuel(seg1)
  var seg2 = new CSegment(li, null, false, b, false, stfp, app.mousePoint, pt3)
  app.ajouteObjetVisuel(seg2)
  var seg3 = new CSegment(li, null, false, b, false, stfp, pt3, pt4)
  app.ajouteObjetVisuel(seg3)
  var seg4 = new CSegment(li, null, false, b, false, stfp, pt4, pt1)
  app.ajouteObjetVisuel(seg4)
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}

/**
 * Fonction renvoyant true si l'outil accepte qu'on crée un point par défaut lors d'un clic sur un endroit vide
 */
OutilCarre.prototype.creationPointPossible = function () {
  return true
}
