/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Outil = require('./Outil')
const NatObj = require('../types/NatObj')
const NatCal = require('../types/NatCal')
const kernel = require('../kernel/kernel')
const CCalcul = require('../objets/CCalcul')
const CImplementationProto = require('../objets/CImplementationProto')
const CMesureX = require('../objets/CMesureX')
const CourbeAvecTanDlg = require('../dialogs/CourbeAvecTanDlg')
const CourbeAvecTanCoefDlg = require('../dialogs/CourbeAvecTanCoefDlg')

module.exports = OutilCourbeAvecTan

function OutilCourbeAvecTan (app) {
  Outil.call(this, app, 'CourbeAvecTan', 32986, true)
}
OutilCourbeAvecTan.prototype = new Outil()

OutilCourbeAvecTan.prototype.select = function () {
  Outil.prototype.select.call(this)
  var self = this
  new CourbeAvecTanDlg(this.app, function (nbPoints, rep) {
    self.suite(nbPoints, rep)
  }, function () { self.app.activeOutilCapt() })
}

/**
 * Une fois que la boîte de dialogue demandant le repère et le nombre de points a été validé, cette fonction
 * est appelée avec le nombre de points demandés nbPoints et le repère rep en paramètres
 * @param nbPoints
 * @param rep
 */
OutilCourbeAvecTan.prototype.suite = function (nbPoints, rep) {
  this.nbPoints = nbPoints
  this.rep = rep
  var app = this.app
  var list = app.listePr
  // On crée maintenant nbPoints calculs qui contiendront les coefficients directeurs souhaités pour les tangentes
  this.coef = []
  // On crée un tableau qui contiendra des pointeurs sur les points cliqués qui n'étaient pas nommés
  this.ptsSansNom = []
  for (var i = 0; i < nbPoints; i++) {
    this.coef[i] = new CCalcul(list, null, false, list.genereNomPourCalcul('mt', true), '0')
    app.ajouteElement(this.coef[i])
  }
  // Maintenant on va attendre que l'utilisateur ait cliqué sur nnPoints
  this.nbPtsCliques = 0 // Ce nombre augmentera à chaque fois qu'on clique sur un point
  this.points = [] // Ce tableau contiendra des pointeurs sur les points cliqués
  app.outilPointageActif = app.outilPointageCre
  app.outilPointageActif.aDesigner = NatObj.NTtPoint
  app.outilPointageActif.reset()
  app.indication(kernel.getStr('indCourTan1') + '1' + kernel.getStr('indCourTan2'), '', true)
}

OutilCourbeAvecTan.prototype.traiteObjetDesigne = function (elg) {
  var self = this
  var app = this.app
  var list = app.listePr
  var nbPoints = this.nbPoints
  var j; var k; var w
  this.nbPtsCliques++
  this.points.push(elg)
  this.ajouteClignotementDe(elg)
  this.excluDeDesignation(elg)
  if (this.nbPtsCliques === 1) this.resetClignotement()
  // Si tous les points cont été désignés, on implémente la construction correspondant au nombre de points
  // puis on ouvre une boîte de dialogue permetteant de modifier les coefficients
  if (this.nbPtsCliques >= nbPoints) {
    // On réodronne les points par ordre d'abscisse croissante
    // On affecte des noms aux points qui n'étaient pas nommés en les mémorisant de façon à supprimer le nom
    var abs = []
    for (j = 0; j < nbPoints; j++) {
      abs[j] = new CMesureX(list, null, false, '', this.rep, this.points[j])
      abs[j].positionne()
    }
    for (j = 0; j < nbPoints - 1; j++) {
      for (k = j + 1; k < nbPoints; k++) {
        if (abs[k].abscisse < abs[j].abscisse) {
          w = abs[k]
          abs[k] = abs[j]
          abs[j] = w
          w = this.points[k]
          this.points[k] = this.points[j]
          this.points[j] = w
        }
      }
    }
    // si l'utilisateur annule
    for (j = 0; j < nbPoints; j++) {
      if (this.points[j].nom === '') {
        this.ptsSansNom.push(this.points[j])
        this.points[j].nom = list.genereNomPourPoint()
        this.points[j].updateName(app.svg, true)
      }
    }
    var proto = app.docConsAv.getPrototype('CourbeAvecTgte' + nbPoints)
    proto.get(0).elementAssocie = this.rep
    for (j = 0; j < nbPoints; j++) proto.get(j + 1).elementAssocie = this.coef[j]
    for (j = 0; j < nbPoints; j++) proto.get(j + nbPoints + 1).elementAssocie = this.points[j]
    var impProto = new CImplementationProto(list, proto)
    impProto.implemente(app.dimf, proto)
    impProto.nomProto = kernel.getStr('CourbeFonc')
    var lieu = impProto.premierFinal(NatObj.NLieu)
    lieu.infoLieu.nombreDePoints = 500
    lieu.donneCouleur(app.getCouleur())
    lieu.donneStyle(app.getStyleTrait())
    var indImpProto = list.indexOf(impProto)
    list.positionne(false, app.dimf)
    list.setReady4MathJax()
    list.afficheTout(indImpProto, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
    new CourbeAvecTanCoefDlg(this.app, this.coef, this.points, list.longueur() - 1,
      function () {
        self.annuleClignotement()
        app.outilCourbeAvecTan.saveFig()
        // Corrigé version 5.6.4
        if (app.estExercice) app.listePourConst = app.listePourConstruction()
        app.activeOutilCapt()
      }, function () {
        self.annuleClignotement()
        // Il faut enlever les noms des points qui avaient été nommés
        for (j = 0; j < self.ptsSansNom.length; j++) {
          self.ptsSansNom[j].nom = ''
          self.ptsSansNom[j].updateName(app.svgFigure, true)
        }
        // Il faut détruire tout ce qui avait été rajouté
        list.removegElements(app.svgFigure, this.interm, indImpProto)
        app.detruitDerniersElements(list.longueur() - indImpProto + self.nbPoints)
        app.activeOutilCapt()
      })
  } else app.indication(kernel.getStr('indCourTan1') + String(this.nbPtsCliques + 1) + kernel.getStr('indCourTan2'), '', true)
}

OutilCourbeAvecTan.prototype.activationValide = function () {
  var list = this.app.listePr
  return (list.nombreObjetsCalcul(NatCal.NRepere) > 0) && (list.nombreObjetsParNatureVisibles(NatObj.NTtPoint) > 2)
}
