/*
 * Created by yvesb on 27/09/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Color = require('../types/Color')
const StyleMarqueSegment = require('../types/StyleMarqueSegment')
const MotifPoint = require('../types/MotifPoint')
const StyleTrait = require('../types/StyleTrait')
const CDroitePerpendiculaire = require('../objets/CDroitePerpendiculaire')
const CMarqueAngleDroit = require('../objets/CMarqueAngleDroit')
const CMarqueSegment = require('../objets/CMarqueSegment')
const CMediatrice = require('../objets/CMediatrice')
const CMilieu = require('../objets/CMilieu')
const CSegment = require('../objets/CSegment')
const OutilObjetPar2Pts = require('./OutilObjetPar2Pts')
const StyleMarqueAngle = require('../types/StyleMarqueAngle')

module.exports = OutilDtMed

/**
 * Outil servant à créer une médiatrice
 * @param app L'application propriétaire
 * @constructor
 */
function OutilDtMed (app) {
  OutilObjetPar2Pts.call(this, app, 'DtMed', 32790, true)
}

OutilDtMed.prototype = new OutilObjetPar2Pts()

OutilDtMed.prototype.objet = function (pt1, pt2) {
  var app = this.app
  return new CMediatrice(app.listePr, null, false, app.getCouleur(), false, 0, 0, false, '', app.getTaillePoliceNom(),
    app.getStyleTrait(), 0.9, pt1, pt2)
}

OutilDtMed.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var list = app.listeObjetsVisuels
  var b = Color.black
  var stfp = StyleTrait.stfp(list)
  var stfc = StyleTrait.stfc2(list)
  var mil = new CMilieu(list, null, false, b, true, 0, 0, true, '', 13, MotifPoint.petitRond, false, this.point1, app.mousePoint)
  app.ajouteObjetVisuel(mil)
  var seg1 = new CSegment(list, null, false, b, false, stfp, this.point1, mil)
  app.ajouteObjetVisuel(seg1)
  var seg2 = new CSegment(list, null, false, b, false, stfp, mil, app.mousePoint)
  app.ajouteObjetVisuel(seg2)
  var mms = StyleMarqueSegment.marqueSimple
  var ms1 = new CMarqueSegment(list, null, false, b, false, stfc, mms, seg1)
  app.ajouteObjetVisuel(ms1)
  var ms2 = new CMarqueSegment(list, null, false, b, false, stfc, mms, seg2)
  app.ajouteObjetVisuel(ms2)
  // On crée une perpendiculaire plutôt qu'une médiatrice pour pouvoir lui adjoindre
  // une marque d'angle droit
  var dp = new CDroitePerpendiculaire(list, null, false, app.getCouleur(), true, 0, 0, false, '', 13, app.getStyleTrait(), 0.9, mil, seg1)
  app.ajouteObjetVisuel(dp)
  // Création de la marque d'angle droit
  // Modifié version 5?0 pour tenir compte des écrans HD
  var mad = new CMarqueAngleDroit(list, null, false, b, false, stfc, StyleMarqueAngle.marqueSimple, 13, dp)
  app.ajouteObjetVisuel(mad)
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}

/**
 * Fonction renvoyant true si l'outil accepte qu'on crée un point par défaut lors d'un clic sur un endroit vide
 */
OutilDtMed.prototype.creationPointPossible = function () {
  return true
}
