/*
 * Created by yvesb on 06/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatObj = require('../types/NatObj')
const NatCal = require('../types/NatCal')
const Outil = require('../outils/Outil')
const ChoixVariableDlg = require('../dialogs/ChoixVariableDlg')
const AvertDlg = require('../dialogs/AvertDlg')
module.exports = OutilLieuParVariableAncetre

/**
 * Outil ancêtre des outils servant à créer un lieu de points généré par une variable (relié ou discret)
 * Les outils descendants doivent avoir une fonction creeObjet()
 * @param app L'application propriétaire
 * @param id L'id de l'outil
 * @param toolIndex L'index de l'outil
 * @constructor
 */
function OutilLieuParVariableAncetre (app, id, toolIndex) {
  // Outil.call(this, app, "LieuParPtLie", 32819, true);
  Outil.call(this, app, id, toolIndex, true)
}

OutilLieuParVariableAncetre.prototype = new Outil()
OutilLieuParVariableAncetre.prototype.constructor = OutilLieuParVariableAncetre

OutilLieuParVariableAncetre.prototype.select = function () {
  this.pointPourTrace = null
  var self = this
  Outil.prototype.select.call(this)
  new ChoixVariableDlg(this.app, function (va) { self.actionApresDlg(va) }, function () { self.app.activeOutilCapt() })
}

OutilLieuParVariableAncetre.prototype.actionApresDlg = function (va) {
  var app = this.app
  app.indication('indLieu1')
  this.va = va // La variable choisie dans la boîte de dialogue
  app.listeClignotante.ajouteObjetsParNatureDependantDe(app.listePr, NatObj.NTtPoint, va, true)
  app.outilPointageActif = app.outilPointageObjetClignotant
  app.outilPointageActif.aDesigner = NatObj.NTtPoint
  app.outilPointageActif.reset(false, true) // Deuxième paramètre true pour pouvoir désigné un objet clignotant
  this.resetClignotement()
}

OutilLieuParVariableAncetre.prototype.traiteObjetDesigne = function (elg) {
  if (this.pointPourTrace === null) {
    this.pointPourTrace = elg
    this.creeObjet() // Défini dans les outils descendants
  }
}

OutilLieuParVariableAncetre.prototype.actionApresDlgOK = function () {
  var app = this.app
  app.ajouteElement(this.lieu)
  if (app.verifieDernierElement(1)) {
    // Le lieu créé existe bien et n'a pas été déjà créé
    this.lieu.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
    this.annuleClignotement() // Pour que le centre clignotant ne soit pas enregistré comme caché
    this.saveFig()
  } else new AvertDlg(app, 'DejaCree')
  app.activeOutilCapt()
}

OutilLieuParVariableAncetre.prototype.isReadyForTouchEnd = function () {
  return false
}

OutilLieuParVariableAncetre.prototype.activationValide = function () {
  return this.app.listePr.nombreObjetsCalcul(NatCal.NVariable) !== 0
}
