/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Outil = require('./Outil')
const CCommentaire = require('../objets/CCommentaire')
const Color = require('../types/Color')
const StyleEncadrement = require('../types/StyleEncadrement')
const CAffLiePt = require('../objets/CAffLiePt')
const kernel = require('../kernel/kernel')
module.exports = OutilMac

/**
 * Outil ancêtre de tous les outils servant à créer des macros
 * @param app Application propriétaire
 * @param {string} toolName Le nom de l'outil
 * @param {int} toolIndex de l'outil
 * @constructor
 */
function OutilMac (app, toolName, toolIndex) {
  if (arguments.length === 0) return
  Outil.call(this, app, toolName, toolIndex, true)
}

OutilMac.prototype = new Outil()

OutilMac.prototype.select = function () {
  Outil.prototype.select.call(this)
  var app = this.app
  this.mac = null
  this.comClig = null
  app.outilPointageActif = app.outilPointageClic
  app.outilPointageActif.reset()
  app.indication('indEndroitAff')
}

OutilMac.prototype.creeObjet = function () {
  var app = this.app
  this.annuleClignotement()
  app.ajouteElement(this.mac)
  this.mac.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
  this.saveFig()
  app.activeOutilCapt()
}

/**
 * Fonction utilisée par tous les outils de création de macro, ajoutant à this un commentaire
 * clignotant et lançant le clignotement
 */
OutilMac.prototype.addComClign = function (point) {
  var app = this.app
  this.comClig = new CCommentaire(app.listePr, null, false, Color.black, point.x, point.y, 0, 0, false, null, 16, StyleEncadrement.Sans,
    false, app.doc.couleurFond, CAffLiePt.alignHorLeft, CAffLiePt.alignVerTop,
    kernel.getStr('AffIci'))
  this.comClig.positionne(false, app.dimf)
  this.comClig.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
  this.ajouteClignotementDe(this.comClig)
  this.resetClignotement()
}
