/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const OutilMac = require('./OutilMac')
const kernelAdd = require('../kernel/kernelAdd')
const CMacroConstructionIterative = require('../objets/CMacroConstructionIterative')
const MacConsIterDlg = require('../dialogs/MacConsIterDlg')
const ConstImpSrcNumDlg = require('../dialogs/ConstImpSrcNumDlg')
const CAffLiePt = require('../objets/CAffLiePt')
const Color = require('../types/Color')
const StyleTrait = require('../types/StyleTrait')
const CSousListeObjets = require('../objets/CSousListeObjets')
const CPrototype = require('../objets/CPrototype')

module.exports = OutilMacConsIter

/**
 * Outil servant à créer une macro d'implémentation récursive de construction
 * @param app L'application propriétaire
 * @constructor
 */
function OutilMacConsIter (app) {
  if (arguments.length === 0) return
  OutilMac.call(this, app, 'MacConsIter', 33064)
}

OutilMacConsIter.prototype = new OutilMac()

OutilMacConsIter.prototype.select = function () {
  OutilMac.prototype.select.call(this)
  this.mac = null
}

OutilMacConsIter.prototype.traiteObjetDesigne = function (elg, point) {
  var self = this
  var app = this.app
  var doc = app.doc
  var list = app.listePr
  // Par défaut profondeur de récursion de 2 avec la première construction disponible
  if (this.mac === null) {
    this.addComClign(point)
    this.mac = new CMacroConstructionIterative(list, null, false, app.getCouleur(), point.x, point.y, 0, 0, false, null,
      16, false, doc.couleurFond, CAffLiePt.alignHorLeft, CAffLiePt.alignVerTop, '', '', null, doc.tablePrototypes[0],
      200, 0, false, Color.black, 0, StyleTrait.stfc(list), Color.black, false)
    new MacConsIterDlg(app, this.mac, false,
      function () {
        self.actionApresDlg()
      })
  } else {
    var proto = this.mac.proto
    this.excluDeDesignation(elg)
    this.ajouteClignotementDe(elg)
    this.tab.push(elg)
    this.indEnCours++
    if (this.indEnCours < proto.nbObjSources) {
      app.outilPointageCre.aDesigner = kernelAdd.natObjGraphPourProto(proto.get(this.indEnCours).getNature())
      app.indication(proto.chIndSourceGraph(this.indEnCours), 'ClicOn', true)
    } else this.end()
  }
}

/**
 * Fonction appelée une fois qu'on a choisi un prototype à implémenter dans la boîte de dialogue de choix de prototype
 */
// this.tab est un array qui, si on doit choisir des éléments graphiques sources, contient soit un pointeur sur un élément de
// la liste des objets choisi comme source soit une chaîne de caractères qui contient un calcul à créer et à rajouter
// à la figure, on affecte alors comme élément source un pointeur sur ce calcul créé
OutilMacConsIter.prototype.actionApresDlg = function () {
  // Si les éléments sources ne sont pas que des objets graphiques, il faut ouvrir une boîte de dialogue de choix
  // d'objets sources numériques
  var self = this
  var app = this.app
  this.mac.listeAssociee = new CSousListeObjets() // Contiendra les objets sources pour l'implémentation récursive
  var proto = this.mac.proto
  if (proto.natureSources !== CPrototype.graph) {
    new ConstImpSrcNumDlg(app, proto, function (tab) {
      self.tab = tab
      self.suite(tab)
    }, function () {
      app.activeOutilCapt()
    }
    )
  } else {
    this.tab = []
    this.suite()
  }
}

OutilMacConsIter.prototype.suite = function () {
  var app = this.app
  var proto = this.mac.proto
  if (proto.natureSources === CPrototype.calc) this.end()
  else {
    // Il faut maintenant choisir les objets sources graphiques
    this.indEnCours = proto.nbSrcCal() // this.indEnCours contient l'indice de l'objet source (graphique) qu'il faut désigner
    this.app.outilPointageActif = app.outilPointageCre
    app.outilPointageCre.aDesigner = kernelAdd.natObjGraphPourProto(proto.get(this.indEnCours).getNature())
    app.outilPointageCre.reset()
    app.indication(proto.chIndSourceGraph(this.indEnCours), 'ClicOn', true)
  }
}

OutilMacConsIter.prototype.end = function () {
  var tab = this.tab
  var mac = this.mac
  for (var i = 0; i < tab.length; i++) {
    mac.listeAssociee.add(tab[i])
  }
  this.annuleClignotement()
  this.creeObjet()
}

OutilMacConsIter.prototype.activationValide = function () {
  return this.app.doc.tablePrototypes.length > 0
}
