/*
 * Created by yvesb on 03/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const OutilPar3Pts = require('./OutilPar3Pts')
const AvertDlg = require('../dialogs/AvertDlg')
const CImplementationProto = require('../objets/CImplementationProto')
const CMarqueAngleGeometrique = require('../objets/CMarqueAngleGeometrique')
const Vect = require('../types/Vect')
const CValeur = require('../objets/CValeur')
const CPointParAbscisse = require('../objets/CPointParAbscisse')
const MotifPoint = require('../types/MotifPoint')
const CArcDeCercle = require('../objets/CArcDeCercle')
const CBissectrice = require('../objets/CBissectrice')
const CIntDroiteCercle = require('../objets/CIntDroiteCercle')
const CPointLieBipoint = require('../objets/CPointLieBipoint')
const StyleEncadrement = require('../types/StyleEncadrement')
const CAffLiePt = require('../objets/CAffLiePt')
const CCommentaire = require('../objets/CCommentaire')
const CMesureAngleGeometrique = require('../objets/CMesureAngleGeometrique')
const CPointBase = require('../objets/CPointBase')
const StyleTrait = require('../types/StyleTrait')
const Color = require('../types/Color')
module.exports = OutilMesAngNor
/**
 * Outil servant à créer une segment en cliquant sur deux points de la figure
 * @param app L'application propriétaire
 * @constructor
 */
function OutilMesAngNor (app) {
  OutilPar3Pts.call(this, app, 'MesAngNor', 32816, true)
}

OutilMesAngNor.prototype = new OutilPar3Pts()

OutilMesAngNor.prototype.select = function () {
  this.app.outilPointageActif = this.app.outilPointageCre
  OutilPar3Pts.prototype.select.call(this) // A appeler à la fin
}

OutilMesAngNor.prototype.preIndication = function () {
  return 'MesAngNor'
}

OutilMesAngNor.prototype.creeObjet = function () {
  var mes, nbEltsCrees
  var app = this.app
  var li = app.listePr
  var leninit = li.longueur()
  var indiceImpInitial = li.longueur()
  if (li.uniteAngle === kernel.uniteAngle.degre && app.pref_DispMeas) {
    // On charge la construction qui est contenue dans this.app.docConst
    var proto = this.app.docCons.getPrototype('MesureAngle')
    this.annuleClignotement()
    proto.get(0).elementAssocie = this.point1
    proto.get(1).elementAssocie = this.point2
    proto.get(2).elementAssocie = this.point3
    var impProto = new CImplementationProto(li, proto)
    impProto.implemente(this.app.dimf, proto)
    impProto.nomProto = kernel.getStr('MesAngNor')
    var ind = li.longueur() - 1
    var aff = li.get(ind) // L'affichage de la mesure
    aff.donneCouleur(app.getCouleur())
    var marque = li.get(ind - 1)
    marque.donneCouleur(app.getCouleur())
    marque.donneStyle(app.getStyleTrait())
    marque.donneStyleMarque(app.getStyleMarqueAngle())
    mes = li.get(ind - 2) // La longueur mesurée
    nbEltsCrees = ind + 1 - leninit
  } else {
    mes = new CMesureAngleGeometrique(li, null, false, this.point1, this.point2, this.point3)
    app.ajouteElement(mes)
    nbEltsCrees = 1
  }
  // On regarde si la longueur mesurée existe déjà
  var existeDeja = app.existeDeja(mes)
  if (existeDeja) {
    new AvertDlg(app, 'DejaCree')
    app.detruitDerniersElements(nbEltsCrees)
  } else {
    this.saveFig()
    // Corrigé version 5.6.4
    if (app.estExercice) app.listePourConst = app.listePourConstruction()

    li.afficheTout(indiceImpInitial, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
  }
  return !existeDeja
}

OutilMesAngNor.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var li = app.listeObjetsVisuels
  var b = Color.black
  var stfp = StyleTrait.stfp(li)
  var coul = app.getCouleur()
  var coulFond = app.doc.couleurFond
  // Color.BLACK, StyleTrait.traitFin,
  // point1, point2, pane.pointSouris, 2);
  if (app.listePr.uniteAngle === kernel.uniteAngle.degre) {
    var nbObj = li.longueur()
    var p1 = new CPointBase(li, null, false, b, true, 0, 0, true, '', 13, MotifPoint.pixel, false, true, this.point1.x, this.point1.y)
    // Pour les deux lignes suivantes on n'appelle pas ajouteObjetVisuel car les deux objets seront affichés plus tard
    li.add(p1)
    var p2 = new CPointBase(li, null, false, b, true, 0, 0, true, '', 13, MotifPoint.pixel, false, true, this.point2.x, this.point2.y)
    // Pour les deux lignes suivantes on n'appelle pas ajouteObjetVisuel car les deux objets seront affichés plus tard
    li.add(p2)
    li.add(app.mousePoint)
    // li.afficheTout(nbObj, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
    // On charge la construction qui est contenue dans this.app.docConst
    var proto = this.app.docCons.getPrototype('MesureAngle')
    proto.get(0).elementAssocie = p1
    proto.get(1).elementAssocie = p2
    proto.get(2).elementAssocie = app.mousePoint
    var impProto = new CImplementationProto(li, proto)
    impProto.implemente(app.dimf, proto)
    var ind = li.longueur() - 1
    var aff = li.get(ind) // L'affichage de la mesure
    aff.donneCouleur(coul)
    var marque = li.get(ind - 1)
    marque.donneCouleur(coul)
    marque.donneStyle(app.getStyleTrait())
    marque.donneStyleMarque(app.getStyleMarqueAngle())
    app.afficheObjetVisuels(nbObj) // Ajout version 6.4.4
  } else {
    var mar = new CMarqueAngleGeometrique(li, null, false, coul, false, app.getStyleTrait(),
      app.getStyleMarqueAngle(), 35, this.point1, this.point2, app.mousePoint)
    app.ajouteObjetVisuel(mar)
    var vec = new Vect(this.point2, this.point1)
    var d = 17 / vec.norme()
    var ab = new CValeur(li, d)
    var pt1 = new CPointParAbscisse(li, null, false, b, true, 0, 0, true, '', 16, MotifPoint.petitRond, false,
      this.point2, this.point1, ab)
    app.ajouteObjetVisuel(pt1)
    // On crée un petit arc de cercle de centre point2, commençant par point 1
    // et dirigé vers le pointeur souris
    var arc = new CArcDeCercle(li, null, false, b, true, stfp, this.point2, pt1, app.mousePoint, null)
    app.ajouteObjetVisuel(arc)
    // On crée une bissectrice
    var bis = new CBissectrice(li, null, false, b, true, 0, 0, true, '', 16, stfp, 1, this.point1, this.point2,
      app.mousePoint)
    app.ajouteObjetVisuel(bis)
    // On crée l'intersection de l'arc et de la bissectrice
    var inter = new CIntDroiteCercle(li, null, false, bis, arc)
    app.ajouteObjetVisuel(inter)
    var ptl1 = new CPointLieBipoint(li, null, false, b, true, 0, 0, true, '', 16, MotifPoint.petitRond, false, inter, 1)
    app.ajouteObjetVisuel(ptl1)
    var ptl2 = new CPointLieBipoint(li, null, false, b, true, 0, 0, true, '', 16, MotifPoint.petitRond, false, inter, 2)
    app.ajouteObjetVisuel(ptl2)
    var com1 = new CCommentaire(li, null, false, coul, 0, 0, 0, 0, false, ptl1, 17, StyleEncadrement.Sans, false,
      coulFond, CAffLiePt.alignHorCent, CAffLiePt.alignVerCent, '?')
    app.ajouteObjetVisuel(com1)
    var com2 = new CCommentaire(li, null, false, coul, 0, 0, 0, 0, false, ptl2, 17, StyleEncadrement.Sans, false,
      coulFond, CAffLiePt.alignHorCent, CAffLiePt.alignVerCent, '?')
    app.ajouteObjetVisuel(com2)
    app.afficheObjetVisuels(0) // Ajout version 6.4.4
  }
}

OutilMesAngNor.prototype.nomsPointsIndisp = function () {
  return true
}
