/*
 * Created by yvesb on 29/04/2017.
 */
/**
 * Outil servant à créer un point défini par son abscisse relativement à deux autres
 * Si les points ne sont pas nommés, un nom leur sera attribué
 * @param app L'application propriétaire
 * @constructor
 */
const OutilPar2Pts = require('../outils/OutilPar2Pts')
const PolygoneRegDlg = require('../dialogs/PolygoneRegDlg')
const kernel = require('../kernel/kernel')
const NatObj = require('../types/NatObj')
const Pointeur = require('../types/Pointeur')
const CConstante = require('../objets/CConstante')
const COp = require('../objets/COperation')
const Ope = require('../types/Ope')
const CRotation = require('../objets/CRotation')
const CPointImage = require('../objets/CPointImage')
const CValeurAngle = require('../objets/CValeurAngle')
const CNoeudPointeurSurPoint = require('../objets/CNoeudPointeurSurPoint')
const CPolygone = require('../objets/CPolygone')
const CalcR = require('../kernel/CalcR')

module.exports = OutilPolygoneReg

function OutilPolygoneReg (app) {
  OutilPar2Pts.call(this, app, 'PolygoneReg', 32992, true, false)
}

OutilPolygoneReg.prototype = new OutilPar2Pts()

OutilPolygoneReg.prototype.select = function () {
  var app = this.app
  var self = this
  OutilPar2Pts.prototype.select.call(this)
  app.outilPointageActif = app.outilPointageCre
  app.outilPointageActif.aDesigner = NatObj.NTtPoint
  app.outilPointageActif.reset()
  this.nbSommets = new Pointeur(0) // Renverra un pointeur vers le nombre de sommets du polygone
  new PolygoneRegDlg(this.app, this.nbSommets, function () {
    self.app.indication('indCentre')
  }, function () { self.app.activeOutilCapt() })
}

OutilPolygoneReg.prototype.creeObjet = function () {
  var colPoints = []
  var app = this.app
  var coul = app.getCouleur()
  var coulFond = app.doc.couleurFond
  var opacity = app.doc.opacity
  var list = app.listePr
  var degre = list.uniteAngle === kernel.uniteAngle.degre
  var calc = new COp(list, degre ? new CConstante(list, 360) : CalcR.ccb('2*pi', list, 0, 4, null)
    , new CConstante(list, this.nbSommets.getValue()), Ope.Divi)
  var val = new CValeurAngle(list, calc)
  var rot = new CRotation(list, null, false, this.point1, val)
  var pt = this.point2
  app.ajouteElement(rot)
  colPoints.push(new CNoeudPointeurSurPoint(list, pt))
  for (var i = 1; i < this.nbSommets.getValue(); i++) {
    var ptim = new CPointImage(list, null, false, coul, false, 3, 0, false, '', app.getTaillePoliceNom(),
      app.getStylePoint(), false, pt, rot)
    app.ajouteElement(ptim, false)
    ptim.creeAffichage(app.svgFigure, false, coulFond, opacity)
    pt = ptim
    var noeud = new CNoeudPointeurSurPoint(list, pt)
    colPoints.push(noeud)
  }
  // On ferme le polygone
  colPoints.push(new CNoeudPointeurSurPoint(list, this.point2))
  var poly = new CPolygone(list, null, false, app.getCouleur(), false, app.getStyleTrait(), colPoints)
  app.ajouteElement(poly)
  poly.creeAffichage(app.svgFigure, false, coulFond, opacity)
  this.saveFig()
  app.activeOutilCapt()
}

OutilPolygoneReg.prototype.indication = function (ind) {
  switch (ind) {
    case 1 :
      return 'indCentre'
    case 2 :
      return 'indPolReg'
  }
}

OutilPolygoneReg.prototype.ajouteObjetsVisuels = function () {
  var colPoints = []
  var app = this.app
  var coul = app.getCouleur()
  var list = app.listePr
  var degre = list.uniteAngle === kernel.uniteAngle.degre
  var calc = new COp(list, degre ? new CConstante(list, 360) : CalcR.ccb('2*pi', list, 0, 4, null)
    , new CConstante(list, this.nbSommets.getValue()), Ope.Divi)
  var val = new CValeurAngle(list, calc)
  var rot = new CRotation(list, null, false, this.point1, val)
  app.ajouteObjetVisuel(rot)
  var pt = app.mousePoint
  colPoints.push(new CNoeudPointeurSurPoint(list, pt))
  for (var i = 1; i < this.nbSommets.getValue(); i++) {
    var ptim = new CPointImage(list, null, false, coul, false, 3, 0, false, '', 16, app.getStylePoint(), false, pt, rot)
    app.ajouteObjetVisuel(ptim)
    pt = ptim
    var noeud = new CNoeudPointeurSurPoint(list, pt)
    colPoints.push(noeud)
  }
  // On ferme le polygone
  colPoints.push(new CNoeudPointeurSurPoint(list, app.mousePoint))
  var poly = new CPolygone(list, null, false, app.getCouleur(), false, app.getStyleTrait(), colPoints)
  app.ajouteObjetVisuel(poly)
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}
