/*
 * Created by yvesb on 23/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Outil = require('../outils/Outil')
const CPointInterieurCercle = require('../objets/CPointInterieurCercle')
const CPointInterieurPolygone = require('../objets/CPointInterieurPolygone')
const NatObj = require('../types/NatObj')
const Nat = require('../types/Nat')
const Pointeur = require('../types/Pointeur')
const CSurfacePolygone = require('../objets/CSurfacePolygone')
const CSurfaceDisque = require('../objets/CSurfaceDisque')
const Color = require('../types/Color')
const StyleRemplissage = require('../types/StyleRemplissage')
const AvertDlg = require('../dialogs/AvertDlg')
module.exports = OutilPtInterieur

/**
 * Outil servant à créer un point intérieur à un disque ou à un polygone
 * @param app L'application propriétaire
 * @constructor
 */
function OutilPtInterieur (app) {
  Outil.call(this, app, 'PtInterieur', 33009, true)
}

OutilPtInterieur.prototype = new Outil()

OutilPtInterieur.prototype.select = function () {
  Outil.prototype.select.call(this)
  var app = this.app
  this.obj = null // L'objet auquel le point qui sera créé sera intérieur
  app.outilPointageActif = app.outilPointageObjetClignotant
  var nature = Nat.or(NatObj.NCercle, NatObj.NPolygone)
  app.listeClignotante.ajouteObjetsParNatureNonMasques(app.listePr, nature)
  app.outilPointageActif.aDesigner = nature
  app.outilPointageActif.reset(false, true)
  this.resetClignotement()
  app.indication('indPtInt1')
}

OutilPtInterieur.prototype.traiteObjetDesigne = function (elg, point) {
  var app = this.app
  var liste = app.listePr
  if (this.obj === null) {
    this.obj = elg
    this.annuleClignotement()
    this.ajouteClignotementDe(elg)
    app.outilPointageActif = app.outilPointageClic
    app.outilPointageActif.reset()
    // frame.indication(getStr("pointInterieur2"));
    this.ajouteObjetsVisuels()
    this.resetClignotement()
    app.indication('indPtInt2')
  } else {
    var self = this
    var x = point.x
    var y = point.y
    var bpointInt = false
    if (this.obj.estDeNature(NatObj.NPolygone)) {
      // On regarde si le point cliqué est bien intérieur au objgone
      if (this.obj.pointInterieur(x, y)) {
        var estSurCote = new Pointeur(false)
        var indicePremierSommet = new Pointeur(0)
        var alpha = new Pointeur(0)
        var beta = new Pointeur(0)
        var gamma = new Pointeur(0)
        bpointInt = this.obj.determinePositionInterieure(x, y, indicePremierSommet, estSurCote, alpha, beta, gamma)
        if (bpointInt) {
          var pointInterieurPolygone = new CPointInterieurPolygone(liste, null, false, app.getCouleur(), false, 0, 3,
            false, '', app.getTaillePoliceNom(), app.getStylePoint(), false, true, this.obj, estSurCote.getValue(),
            indicePremierSommet.getValue(), alpha.getValue(), beta.getValue(), gamma.getValue())
          pointInterieurPolygone.positionne(false, app.dimf)
          if (pointInterieurPolygone.existe) this.creeObjet(pointInterieurPolygone)
          else bpointInt = false
        }
      }
      if (!bpointInt) new AvertDlg(app, 'PtIntDlg1', function () { self.app.activeOutilCapt() })
      app.activeOutilCapt()
    } else {
      // On regarde si le point cliqué est bien intérieur au cercle
      bpointInt = this.obj.pointInterieur(x, y)
      if (bpointInt) {
        /*
        this.pointInterieurCercle = new CPointInterieurCercle(liste, null, false, frame.couleurActive(),
          false, 0d, 0d, false, "", frame.pref_indiceTaillePoliceNom, frame.stylePointActif,
          true, cercle, 0, 0, false);
          */
        var pointInterieurCercle = new CPointInterieurCercle(liste, null, false, app.getCouleur(), false, 0, 3, false,
          '', app.getTaillePoliceNom(), app.getStylePoint(), false, true, this.obj, 0, 0)
        pointInterieurCercle.placeEn(x, y)
        pointInterieurCercle.positionne(false, app.dimf) // Détermine l'angle et le rapport
        if (pointInterieurCercle.existe) this.creeObjet(pointInterieurCercle)
        else bpointInt = false
      }
      if (!bpointInt) new AvertDlg(app, 'PtIntDlg1', function () { self.app.activeOutilCapt() })
      app.activeOutilCapt()
    }
  }
}

OutilPtInterieur.prototype.creeObjet = function (ptint) {
  var app = this.app
  this.annuleClignotement()
  app.ajouteElement(ptint)
  ptint.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
  this.saveFig()
  app.activeOutilCapt() // Après création on revient à l'outil de capture
}

OutilPtInterieur.prototype.ajouteObjetsVisuels = function () {
  const app = this.app
  var liste = app.listePr
  if (this.obj.estDeNature(NatObj.NPolygone)) {
    this.app.ajouteObjetVisuel(new CSurfacePolygone(liste, null, false, Color.yellow, false, StyleRemplissage.quadrillagePleinTransp, this.obj))
  } else {
    this.app.ajouteObjetVisuel(new CSurfaceDisque(liste, null, false, Color.yellow, false, StyleRemplissage.quadrillagePleinTransp, this.obj))
  }
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}

OutilPtInterieur.prototype.activationValide = function () {
  return this.app.listePr.nombreObjetsParNatureVisibles(Nat.or(NatObj.NPolygone, NatObj.NCercle)) !== 0
}
