/*
 * Created by yvesb on 03/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatObj = require('../types/NatObjAdd')
const CConstante = require('../objets/CConstante')
const CValeurAngle = require('../objets/CValeurAngle')
const CRotation = require('../objets/CRotation')
const RotationDlg = require('../dialogs/RotationDlg')
const OutilTransfor = require('./OutilTransfor')
const Color = require('../types/Color')
const StyleTrait = require('../types/StyleTrait')
const CSegment = require('../objets/CSegment')
const CPointImage = require('../objets/CPointImage')
const CMarqueAngleOriente = require('../objets/CMarqueAngleOriente')
const MotifPoint = require('../types/MotifPoint')
const StyleMarqueAngle = require('../types/StyleMarqueAngle')
const StyleFleche = require('../types/StyleFleche')

module.exports = OutilRot

/**
 * Outil servant à créer l'image d'un objet par une rotation
 * @param app L'application propriétaire
 * @constructor
 */
function OutilRot (app) {
  OutilTransfor.call(this, app, 'Rot', 32803)
}

OutilRot.prototype = new OutilTransfor()

OutilRot.prototype.select = function () {
  OutilTransfor.prototype.select.call(this)
  var app = this.app
  this.point = null
  this.objet = null
  this.trans = null
  this.imageDejaCree = false
  app.outilPointageActif = app.outilPointageCre
  app.outilPointageCre.aDesigner = NatObj.NTtPoint
  app.outilPointageCre.reset()
  app.indication('indCentRot')
}

OutilRot.prototype.traiteObjetDesigne = function (elg) {
  var app = this.app
  // if (app.lastDlgId() === "rotDlg") return;
  var li = app.listePr
  if (this.point === null) {
    this.point = elg
    var ang = new CValeurAngle(li, new CConstante(li, 0))
    this.trans = new CRotation(li, null, false, elg, ang)
    this.excluDeDesignation(this.point)
    this.ajouteClignotementDe(this.point)
    this.resetClignotement() // Sous Chrome, si on fait clignoter le centre le curseur de la boîte de dialogue ne clignote plus ...
    // this.annuleClignotement();
    var self = this
    new RotationDlg(this.app, this.trans, false, function () { self.actionApresDlg() }, function () { self.reselect() })
    // Version JavaScript : Ne peut pas être ici. Est lancé par le dialogue quand on le valide
    // this.ajouteObjetsVisuels();
    // app.outilPointageCre.aDesigner = NatObj.NTtObj;
  } else {
    this.objet = elg
    this.creeObjet()
  }
}

OutilRot.prototype.actionApresDlg = function () {
  var app = this.app
  this.ajouteObjetsVisuels()
  this.app.indication('indImTrans')
  this.app.outilPointageCre.aDesigner = app.natPourImages()
  this.trans.positionne(false, this.app.dimf)
}

OutilRot.prototype.clignCentreOuAxe = function () {
  this.ajouteClignotementDe(this.point)
  this.resetClignotement()
}

OutilRot.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var list = app.listeObjetsVisuels
  var b = Color.black
  var stfp = StyleTrait.stfp(list)
  var stfc = StyleTrait.stfc(list)
  var seg1 = new CSegment(list, null, false, b, false, stfp, this.point, app.mousePoint)
  app.ajouteObjetVisuel(seg1)
  var ptim = new CPointImage(list, null, false, b, true, 0, 0, false, '', 13, MotifPoint.petitRond, false, app.mousePoint, this.trans)
  app.ajouteObjetVisuel(ptim)
  var seg2 = new CSegment(list, null, false, b, false, stfp, this.point, ptim)
  app.ajouteObjetVisuel(seg2)
  var marque = new CMarqueAngleOriente(list, null, false, b, false, stfc, StyleMarqueAngle.marqueSimple, 18,
    app.mousePoint, this.point, ptim, StyleFleche.FlecheLonguePleine)
  app.ajouteObjetVisuel(marque)
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}

OutilRot.prototype.isWorking = function () {
  return this.point !== null
}
