/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Outil = require('./Outil')
const NatObj = require('../types/NatObj')
const CListeObjets = require('../objets/CListeObjets')
const CSousListeObjets = require('../objets/CSousListeObjets')
const AddSupObjMacDlg = require('../dialogs/AddSupObjMacDlg')
const AvertDlg = require('../dialogs/AvertDlg')
module.exports = OutilSupObjMac

/**
 * Outil servant à majouter des objets à une macro gérant une liste d'objets
 * @param app L'application propriétaire
 * @constructor
 */
function OutilSupObjMac (app) {
  Outil.call(this, app, 'SupObjMac', 32994, true)
}

OutilSupObjMac.prototype = new Outil()

OutilSupObjMac.prototype.select = function () {
  var app = this.app
  var self = this
  Outil.prototype.select.call(this)
  this.buttonStop = false
  this.listAdd = new CListeObjets() // Contiendra la liste des objets à retirer
  this.list = null
  new AddSupObjMacDlg(app, true, 'AddObjMac', function (mac) {
    self.macro = mac // La macro à modifier
    app.outilPointageActif = app.outilPointageObjetClignotant
    app.outilPointageActif.aDesigner = NatObj.NTtObj
    // On crée une liste formée des objets de la liste de la macro et qui sont masqués de façon
    // à pouvoir les démasquer et les faire clignoter
    self.list = new CListeObjets()
    var listAssociee = mac.listeAssociee
    for (var i = 0; i < listAssociee.longueur(); i++) {
      var el = listAssociee.get(i)
      self.ajouteClignotementDe(el)
      if (el.masque) {
        self.list.add(el)
        el.montre()
      }
    }
    self.resetClignotement()
    app.outilPointageActif.reset()
    app.indication('indSupObj')
  })
}

OutilSupObjMac.prototype.traiteObjetDesigne = function (elg) {
  var app = this.app
  if (this.macro.listeAssociee.longueur() - this.listAdd.longueur() === 1) {
    new AvertDlg(this.app, 'SupImp')
    return
  }
  if (!this.buttonStop) {
    this.buttonStop = true
    app.showStopButton(true)
  }
  this.listAdd.add(elg)
  this.excluDeDesignation(elg)
  this.enleveDeClign(elg)
}

OutilSupObjMac.prototype.deselect = function () {
  Outil.prototype.deselect.call(this)
  var app = this.app
  app.showStopButton(false)
  if (this.list !== null) {
    this.list.montreTout(false) // On cache à nouveau les objets de la macro qui  étaient cachés
    this.list.retireTout()
    app.listePr.update(app.svgFigure, app.doc.couleurFond, app.doc.opacity, true) // true pour que les objets masqués soient masqués
  }
}

OutilSupObjMac.prototype.actionFin = function () {
  var listeAssociee = this.macro.listeAssociee
  var list = new CSousListeObjets()
  for (var i = 0; i < listeAssociee.longueur(); i++) {
    var el = listeAssociee.get(i)
    if (!this.listAdd.contains(el)) list.add(el)
  }
  this.macro.listeAssociee = list
  this.app.activeOutilCapt()
}

OutilSupObjMac.prototype.activationValide = function () {
  return this.app.listePr.listeMacrosAvecListeModif(true).noms.length > 0
}
