/*
 * Created by yvesb on 05/12/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatObj = require('../types/NatObjAdd')
const CSymetrieAxiale = require('../objets/CSymetrieAxiale')
const OutilTransfor = require('./OutilTransfor')
const Color = require('../types/Color')
const StyleTrait = require('../types/StyleTrait')
const CMilieu = require('../objets/CMilieu')
const CSegment = require('../objets/CSegment')
const CPointImage = require('../objets/CPointImage')
const CMarqueAngleDroit = require('../objets/CMarqueAngleDroit')
const MotifPoint = require('../types/MotifPoint')
const StyleMarqueAngle = require('../types/StyleMarqueAngle')
const CMarqueSegment = require('../objets/CMarqueSegment')
const StyleMarqueSegment = require('../types/StyleMarqueSegment')
const CDroitePerpendiculaire = require('../objets/CDroitePerpendiculaire')

module.exports = OutilSymAxiale

/**
 * Outil servant à créer l'image d'un objet dans une symétrie axiale
 * @param app L'application propriétaire
 * @constructor
 */
function OutilSymAxiale (app) {
  OutilTransfor.call(this, app, 'SymAxiale', 32799)
}

OutilSymAxiale.prototype = new OutilTransfor()

OutilSymAxiale.prototype.select = function () {
  OutilTransfor.prototype.select.call(this)
  var app = this.app
  this.dte = null
  this.objet = null
  this.trans = null // La symétrie utilisée
  this.app.outilPointageActif = app.outilPointageCre
  app.outilPointageCre.aDesigner = NatObj.NTteDroite
  app.outilPointageCre.reset()
  app.indication('indAxeSym')
}

/* Supprimé car inutile
OutilSymAxiale.prototype.deselect = function() {
  // Si aucune image n'a été créée et qu'on avait créée un symétrie il faut la détruire
  var liste = this.app.listePr;
  if (this.trans === liste.get(liste.longueur() - 1)) {
    this.app.detruitDernierElement();
  }
  OutilTransfor.prototype.deselect.call(this);
}
*/

OutilSymAxiale.prototype.traiteObjetDesigne = function (elg) {
  if (this.dte === null) {
    // frame.indication(getStr("symAxiale2"));
    this.dte = elg
    this.trans = new CSymetrieAxiale(this.app.listePr, null, false, this.dte)
    this.excluDeDesignation(elg)
    this.ajouteClignotementDe(elg)
    this.ajouteObjetsVisuels()
    var app = this.app
    app.outilPointageCre.aDesigner = app.natPourImages()
    this.resetClignotement()
    app.indication('indImTrans')
  } else {
    this.objet = elg
    this.creeObjet()
  }
}

OutilSymAxiale.prototype.clignCentreOuAxe = function () {
  this.ajouteClignotementDe(this.dte)
  this.resetClignotement()
}

OutilSymAxiale.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var list = app.listeObjetsVisuels
  var b = Color.black
  var stfp = StyleTrait.stfp(list)
  var stfc1 = StyleTrait.stfc(list)
  var stfc2 = StyleTrait.stfc2(list)
  var trans = new CSymetrieAxiale(list, null, false, this.dte)
  app.ajouteObjetVisuel(trans)
  var ptim = new CPointImage(list, null, false, b, true, 0, 0, false, '', 0, MotifPoint.petitRond, false, app.mousePoint, trans)
  app.ajouteObjetVisuel(ptim)
  var mil = new CMilieu(list, null, false, b, true, 0, 0, false, '', 0, MotifPoint.petitRond, false, app.mousePoint, ptim)
  app.ajouteObjetVisuel(mil)
  var seg1 = new CSegment(list, null, false, b, false, stfp, app.mousePoint, mil)
  app.ajouteObjetVisuel(seg1)
  var seg2 = new CSegment(list, null, false, b, false, stfp, mil, ptim)
  app.ajouteObjetVisuel(seg2)
  var ms1 = new CMarqueSegment(list, null, false, b, false, stfc2, StyleMarqueSegment.marqueSimple, seg1)
  app.ajouteObjetVisuel(ms1)
  var ms2 = new CMarqueSegment(list, null, false, b, false, stfc2, StyleMarqueSegment.marqueSimple, seg2)
  app.ajouteObjetVisuel(ms2)
  var perp = new CDroitePerpendiculaire(list, null, false, b, true, 0, 0, true, '', 0, stfc2, 0, app.mousePoint, this.dte)
  app.ajouteObjetVisuel(perp)
  var mad = new CMarqueAngleDroit(list, null, false, b, false, stfc1, StyleMarqueAngle.marqueSimple, 12, perp)
  app.ajouteObjetVisuel(mad)
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}

/*
OutilSymAxiale.prototype.isReadyForTouchEnd = function() {
  return this.dte !== null;
}
*/

OutilSymAxiale.prototype.isReadyForTouchEnd = function () {
  return false
}

OutilSymAxiale.prototype.isWorking = function () {
  return this.dte !== null
}
