/*
 * Created by yvesb on 13/12/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const NatObj = require('../types/NatObjAdd')
const CSymetrieCentrale = require('../objets/CSymetrieCentrale')
const OutilTransfor = require('./OutilTransfor')
const Color = require('../types/Color')
const StyleTrait = require('../types/StyleTrait')
const CMilieu = require('../objets/CMilieu')
const CSegment = require('../objets/CSegment')
const CPointImage = require('../objets/CPointImage')
const MotifPoint = require('../types/MotifPoint')
const CMarqueSegment = require('../objets/CMarqueSegment')
const StyleMarqueSegment = require('../types/StyleMarqueSegment')

module.exports = OutilSymCentrale

/**
 * Outil servant à créer l'image d'un objet dans une symétrie axiale
 * @param app L'application propriétaire
 * @constructor
 */
function OutilSymCentrale (app) {
  OutilTransfor.call(this, app, 'SymCentrale', 32802)
}

OutilSymCentrale.prototype = new OutilTransfor()

OutilSymCentrale.prototype.select = function () {
  OutilTransfor.prototype.select.call(this)
  var app = this.app
  this.centre = null
  this.objet = null
  this.trans = null // La symétrie utilisée
  this.app.outilPointageActif = app.outilPointageCre
  app.outilPointageCre.aDesigner = NatObj.NTtPoint
  app.outilPointageCre.reset()
  app.indication('indCentSym')
}

OutilSymCentrale.prototype.traiteObjetDesigne = function (elg) {
  if (this.centre === null) {
    // frame.indication(getStr("symCentrale2"));
    this.centre = elg
    this.trans = new CSymetrieCentrale(this.app.listePr, null, false, elg)
    this.excluDeDesignation(elg)
    this.ajouteClignotementDe(elg)
    this.ajouteObjetsVisuels()
    var app = this.app
    app.outilPointageCre.aDesigner = app.natPourImages()
    this.resetClignotement()
    app.indication('indImTrans')
  } else {
    this.objet = elg
    this.creeObjet()
  }
}

OutilSymCentrale.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var list = app.listeObjetsVisuels
  var b = Color.black
  var stfp = StyleTrait.stfp(list)
  var stfc2 = StyleTrait.stfc2(list)
  var sym = new CSymetrieCentrale(list, null, false, this.centre)
  app.ajouteObjetVisuel(sym)
  var ptim = new CPointImage(list, null, false, b, true, 0, 0, false, '', 0, MotifPoint.petitRond, false, app.mousePoint, sym)
  app.ajouteObjetVisuel(ptim)
  var mil = new CMilieu(list, null, false, b, true, 0, 0, true, '', 0, MotifPoint.petitRond, false, app.mousePoint, ptim)
  app.ajouteObjetVisuel(mil)
  var seg1 = new CSegment(list, null, false, b, false, stfp, app.mousePoint, mil)
  app.ajouteObjetVisuel(seg1)
  var seg2 = new CSegment(list, null, false, b, false, stfp, mil, ptim)
  app.ajouteObjetVisuel(seg2)
  var ms1 = new CMarqueSegment(list, null, false, b, false, stfc2, StyleMarqueSegment.marqueSimple, seg1)
  app.ajouteObjetVisuel(ms1)
  var ms2 = new CMarqueSegment(list, null, false, b, false, stfc2, StyleMarqueSegment.marqueSimple, seg2)
  app.ajouteObjetVisuel(ms2)
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}

/*
OutilSymCentrale.prototype.isReadyForTouchEnd = function() {
  return this.centre !== null;
}
*/

OutilSymCentrale.prototype.isReadyForTouchEnd = function () {
  return false
}

OutilSymCentrale.prototype.isWorking = function () {
  return this.centre !== null
}
