/*
 * Created by yvesb on 10/02/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const NatCal = require('../types/NatCal')
const CVecteur = require('../objets/CVecteur')
const CTranslationParCoord = require('../objets/CTranslationParCoord')
const OutilTransfor = require('./OutilTransfor')
const Color = require('../types/Color')
const StyleTrait = require('../types/StyleTrait')
const StyleFleche = require('../types/StyleFleche')
const CPointImage = require('../objets/CPointImage')
const MotifPoint = require('../types/MotifPoint')
const CValeur = require('../objets/CValeur')
const TransParCoordDlg = require('../dialogs/TransParCoordDlg')

module.exports = OutilTransParCoord

/**
 * Outil servant à créer l'image d'un objet dans une translation
 * @param app L'application propriétaire
 * @constructor
 */
function OutilTransParCoord (app) {
  OutilTransfor.call(this, app, 'TransParCoord', 33035, true)
}

OutilTransParCoord.prototype = new OutilTransfor()

OutilTransParCoord.prototype.select = function () {
  var app = this.app
  var list = app.listePr
  var self = this
  OutilTransfor.prototype.select.call(this)
  this.trans = new CTranslationParCoord(list, null, false, new CValeur(list, 0, new CValeur(list, 0)))
  new TransParCoordDlg(this.app, this.trans, false, function () { self.actionApresDlg() }, function () { self.app.activeOutilCapt() })
}

OutilTransParCoord.prototype.actionApresDlg = function () {
  var app = this.app
  // app.ajouteElement(this.trans);
  app.outilPointageActif = app.outilPointageCre
  app.outilPointageCre.aDesigner = app.natPourImages()
  app.outilPointageCre.reset()
  this.ajouteObjetsVisuels()
  app.indication('indImTrans')
}

OutilTransParCoord.prototype.traiteObjetDesigne = function (elg) {
  this.objet = elg
  this.creeObjet()
}

OutilTransParCoord.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var list = app.listeObjetsVisuels
  var b = Color.black
  var stfp = new StyleTrait(list, StyleTrait.styleTraitPointille, 1)
  // CTranslationParCoord transx = new CTranslationParCoord(liste, null, false, trans.rep, trans.x, new CValeurAssocieeAVariable(liste, 0));
  var transx = new CTranslationParCoord(list, null, false, this.trans.x, new CValeur(list, 0), this.trans.rep)
  // CPointImage ptimx = new CPointImage(liste, null, false, frame.couleurActive(),
  //  MotifPoint.PetitRond, 13, pane.pointSouris, transx);
  var ptimx = new CPointImage(list, null, false, b, true, 0, 0, true, '', 16, MotifPoint.petitRond,
    false, app.mousePoint, transx)
  app.ajouteObjetVisuel(transx)
  app.ajouteObjetVisuel(ptimx)
  var vecx = new CVecteur(list, null, false, Color.blue, false, stfp, app.mousePoint, ptimx, StyleFleche.FlecheCourtePleine)
  app.ajouteObjetVisuel(vecx)
  var transy = new CTranslationParCoord(list, null, false, new CValeur(list, 0), this.trans.y, this.trans.rep)
  app.ajouteObjetVisuel(transy)
  var ptimy = new CPointImage(list, null, false, b, true, 0, 0, true, '', 16, MotifPoint.petitRond,
    false, ptimx, transy)
  app.ajouteObjetVisuel(ptimy)
  app.ajouteObjetVisuel(this.trans)
  var ptim = new CPointImage(list, null, false, b, true, 0, 0, false, '', 16, MotifPoint.petitRond, false, app.mousePoint, this.trans)
  app.ajouteObjetVisuel(ptim)
  var vecy = new CVecteur(list, null, false, Color.green, false, stfp, ptimx, ptim, StyleFleche.FlecheCourtePleine)
  app.ajouteObjetVisuel(vecy)
  var vec = new CVecteur(list, null, false, b, false, stfp, app.mousePoint, ptim, StyleFleche.FlecheCourtePleine)
  app.ajouteObjetVisuel(vec)
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}

OutilTransParCoord.prototype.activationValide = function () {
  return this.app.listePr.nombreObjetsCalcul(NatCal.NRepere) > 0
}
