/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const OutilSave = require('../outils/OutilSave')
const SaveDlg = require('../dialogs/SaveDlg')
const AvertDlg = require('../dialogs/AvertDlg')
const kernel = require('../kernel/kernel')
module.exports = OutilSave

OutilSave.prototype.select = function () {
  const app = this.app
  // app.functionOnSave n'est pas null et pointe sur la fonction externe à appeler quand l'élève estime avoir fini et enregistre sa figure.
  // Si on est en mode aperçu de la ressource, app.preview vaut true et app.functionOnSave est null
  if (app.functionOnSave || app.preview) {
    // Si on est en train de faire ou de tester un exercice de construction et si des objets à construire devant être nommés
    // n'ont pas été nommés, on affiche un message donnant leur nom et on n'enregistre pas la figure
    if (app.estExercice) {
      // Pour un exercice de construction, s'il manque des objest ayant le nom d'objets à construire, on refuse d'enregistrer
      if (app.objectConstructed()) {
        let mes2 = ''
        let mes1 = ''
        // Si on est dans un composant MathGraph32 de la bibli ou de LaboMep et si l'élève est en train de compléter une figure
        const missingTypes = app.getMissingTypes()
        const bMissingTypes = missingTypes.length !== 0
        if (bMissingTypes) {
          mes1 = kernel.getStr('aconst') + ' '
          for (let k = 0; k < missingTypes.length; k++) {
            mes1 += missingTypes[k]
            if (k !== missingTypes.length - 1) mes1 += ', '
          }
        }
        const missingNames = app.getMissingNames()
        const bMissingNames = missingNames.length !== 0
        if (bMissingNames) {
          mes2 = kernel.getStr('anom') + ' '
          for (let k = 0; k < missingNames.length; k++) {
            mes2 += missingNames[k]
            if (k !== missingNames.length - 1) mes2 += ', '
          }
        }
        // S'il manque des objets de type demandé ou si des objets n'ont pas été nommés, on refuse d'enregistrer
        // et on affiche un message d'avertissement
        if (bMissingTypes || bMissingNames) {
          let mes = bMissingTypes ? mes1 : ''
          if (bMissingNames) {
            if (missingTypes) mes += '</br>'
            mes += mes2
          }
          new AvertDlg(app, mes)
          return
        }
      } else {
        new AvertDlg(app, 'none')
        return
      }
    }
    try {
      const result = app.getResult()
      const mes = kernel.getStr(app.estExercice ? (result.score === 1 ? 'res' : 'notres') : 'saved')
      let cbHasBeenCalled = false
      const sendResult = function () {
        if (!cbHasBeenCalled) {
          cbHasBeenCalled = true
          app.functionOnSave(result)
        }
      }
      new AvertDlg(app, mes, app.functionOnSave ? sendResult : null)
      if (app.functionOnSave) setTimeout(sendResult, 4000)
    } catch (error) {
      console.error(error)
      // @todo afficher un truc à l'utilisateur pour lui dire que ça a planté
    }
  } else {
    new SaveDlg(this.app)
  }
}
