/*
 * Created by yvesb on 31/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatObj = require('../types/NatObj')
const Point = require('../types/Point')
const kernel = require('../kernel/kernel')
const kernelAdd = require('../kernel/kernelAdd')
const OutilPointage = require('./OutilPointage')
module.exports = OutilPointageCaptureNom

/**
 * Outil de pountage utilisé par l'outil de capture d'objet mobile
 * Hérite de OutilPointage
 * @param app L'application propriétaire
 * @constructor
 */
function OutilPointageCaptureNom (app) {
  OutilPointage.call(this, app)
}
OutilPointageCaptureNom.prototype = new OutilPointage()

OutilPointageCaptureNom.prototype.reset = function () {
  OutilPointage.prototype.reset.call(this)
  this.app.elementCapture = null
}

OutilPointageCaptureNom.prototype.mousemove = function (evt) {
  this.devicemove(evt, 'mouse', kernelAdd.mousePosition)
}
OutilPointageCaptureNom.prototype.touchmove = function (evt) {
  this.devicemove(evt, 'touch', kernelAdd.touchPosition)
}

OutilPointageCaptureNom.prototype.devicemove = function (evt, type, fonc) {
  var tab, xn, yn, eltNomProche, nouvelleAbs, x1, y1, proj, decxn, decyn
  var app = this.app
  var svg = app.svgFigure
  var doc = app.doc
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  if ((type === 'mouse') && (doc.type === 'touch')) return
  doc.type = type
  var elc = app.elementCapture
  tab = fonc(svg, evt, app.zoomFactor)
  xn = tab.x
  yn = tab.y
  if (elc === null) {
    eltNomProche = app.listePr.premierObjetNomProcheDe(this.aDesigner, xn, yn, type, app.listeExclusion)
    // document.body.style.cursor = (eltNomProche !== null)? "pointer" : "default";
    app.outilActif.cursor = (eltNomProche !== null) ? 'pointer' : 'default'
  } else {
    if (elc.estDeNature(NatObj.NTtPoint)) {
      decxn = xn - this.xinit + elc.decX
      decyn = yn - this.yinit + elc.decY
      if (decyn < -180) decyn = -180
      else {
        if (decyn > 90) decyn = 90
      }
      if (decxn < -180) decxn = -180
      else {
        if (decxn > 90) decxn = 90
      }
      elc.decaleNom(decxn, decyn)
    } else {
      if (elc.estDeNature(NatObj.NDroite)) {
        x1 = elc.xNom + elc.decX + xn - this.xinit
        y1 = elc.yNom + elc.decY + yn - this.yinit
        proj = new Point()
        kernel.projetteOrtho(xn, yn, elc.point_x, elc.point_y, elc.vect, proj)
        if (elc.xext1 !== elc.xext2) nouvelleAbs = (proj.x - elc.xext1) / (elc.xext2 - elc.xext1)
        else nouvelleAbs = (proj.y - elc.yext1) / (elc.yext2 - elc.yext1)
        decxn = x1 - proj.x
        decyn = y1 - proj.y
        if (decyn < -90) decyn = -90
        else {
          if (decyn > 90) decyn = 90
        }
        if (decxn < -90) decxn = -90
        else {
          if (decxn > 90) decxn = 90
        }
        elc.decaleNom(decxn, decyn)
        elc.abscisseNom = nouvelleAbs
      }
    }
    this.xinit = xn
    this.yinit = yn
    elc.positionne(false, app.dimf)
    elc.updateNamePosition()
    // Il faut aussi mettre à jour les éventuels objets dupliqués de l'objet en cours d'édition
    var list = app.listePr
    for (var i = 0; i < list.longueur(); i++) {
      var el = list.get(i)
      if (el.estDeNature(NatObj.NObjetDuplique) && el.elementDuplique === elc) el.update(svg, doc.couleurFond, doc.opacity)
    }
  }
}

OutilPointageCaptureNom.prototype.mousedown = function (evt) {
  this.devicedown(evt, 'mouse', kernelAdd.mousePosition)
}

OutilPointageCaptureNom.prototype.touchstart = function (evt) {
  this.devicedown(evt, 'touch', kernelAdd.touchPosition)
}

OutilPointageCaptureNom.prototype.devicedown = function (evt, type, fonc) {
  var app = this.app
  var svg = app.svgFigure
  var doc = app.doc
  if (!doc.isActive) return
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  if ((type === 'mouse') && (doc.type === 'touch')) return
  doc.type = type
  var point = fonc(svg, evt, app.zoomFactor)
  app.elementCapture = app.listePr.premierObjetNomProcheDe(this.aDesigner, point.x, point.y, type, app.listeExclusion)
  if (app.elementCapture !== null) {
    this.xinit = point.x
    this.yinit = point.y
  }
}

OutilPointageCaptureNom.prototype.deviceup = function () {
  var app = this.app
  app.doc.type = '' // Pour autoriser à nouveau tous les événements souris ou touch
  if (app.elementCapture) app.outilCaptNom.saveFig()
  app.elementCapture = null
  document.body.style.cursor = 'default'
}

OutilPointageCaptureNom.prototype.touchend = function () {
  this.deviceup()
}

OutilPointageCaptureNom.prototype.touchcancel = function () {
  this.deviceup()
}

OutilPointageCaptureNom.prototype.mouseup = function () {
  this.deviceup()
}
