/*
 * Created by yvesb on 10/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const OutilPointageCre = require('./OutilPointageCre')
const PremierDernierDlg = require('../dialogs/PremierDernierDlg')
const CPointBase = require('../objets/CPointBase')
const Color = require('../types/Color')
const ChoixTypeDlg = require('../dialogs/ChoixTypeDlg')
const kernel = require('../kernel/kernel')

module.exports = OutilPointageClicOuPt

/**
 * Outil servant à créer des objets soit en cliquant sur un point déjà créé
 * soit en cliquant à un endroit libre de la figure auquel cas un point libre
 * est créé à l'endroit du clic et c'est ce point qui est utilisé
 * Si on crée un point lors du clic, this.bptCree est lis à true sinon false
 * Hérite de OutilPointageCre
 * @param app L'application propriétaire
 * @constructor
 */
function OutilPointageClicOuPt (app) {
  OutilPointageCre.call(this, app)
}

OutilPointageClicOuPt.prototype = new OutilPointageCre()

OutilPointageClicOuPt.prototype.devicedown = function (evt, type, fonc) {
  var app = this.app
  var svg = app.svgFigure
  var doc = app.doc
  var liste = this.listeUtilisee()
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  if ((type === 'mouse') && (doc.type === 'touch')) return
  doc.type = type
  var point = fonc(svg, evt, app.zoomFactor)
  // Modification version 6.4.8 : On utilise un clone de l'objet global app.infoProx car,
  // par exemple quand on ouvre une boîte de dialogue de choix par type, il se peut qu'un événement devicemove
  // ait modifié entre temps cet InfoProx
  // var info = app.infoProx
  var info = app.infoProx.getClone()
  // Sur un périphérique mobile on ne peut pas savoir si avant le touch on était proche d'un objet
  app.mousePoint.placeEn(point.x, point.y)
  app.updateObjetsVisuels()
  if (type === 'touch') {
    liste.procheDe(this.aDesigner, point, info, app.listeExclusion, true, type,
      this.bModifiableParMenu, this.bMemeMasque)
  }
  var nombreObjetsProches = info.nombreObjetsProches
  // Les deux lignes suivantes pour que, si l'outil ouvre une boîte de dialogue, le clic éventuel à l'endroit d'un bouton
  // de cette boîte ne soit pas traité
  kernel.preventDefault(evt)
  evt.stopPropagation()

  var self = this
  if (nombreObjetsProches > 0) {
    this.bptCree = false
    if (info.nombreTypesProches > 1) {
      new ChoixTypeDlg(app, info, info.typesProches, this,
        'objetDesigne', function () { self.callBackAfterDeviceEnd(point) })
    } else {
      if (nombreObjetsProches === 1) {
        this.objetDesigne = info.premierVoisin()
        this.callBackAfterDeviceEnd(point)
      } else {
        new PremierDernierDlg(app, this, 'objetDesigne', info.premierVoisin(),
          info.dernierVoisin(), function () {
            self.callBackAfterDeviceEnd(point)
          })
      }
    }
    app.updateObjetsVisuels()
  } else {
    this.bptCree = true
    // On crée un point libre à l'endroit où a eu lieu le clic.
    var pt = new CPointBase(liste, null, false, Color.black, false, 3, 0, false, '', app.getTaillePoliceNom(),
      app.getStylePoint(), false, true, point.x, point.y)
    app.ajouteElement(pt, false)
    pt.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
    this.objetDesigne = pt
    this.callBackAfterDeviceEnd(point)
  }
}
