/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
*/
const NatObj = require('../types/NatObj')
const kernel = require('../kernel/kernel')
const kernelAdd = require('../kernel/kernelAdd')
const OutilPointage = require('./OutilPointage')
const PremierDernierDlg = require('../dialogs/PremierDernierDlg')
const ChoixTypeDlg = require('../dialogs/ChoixTypeDlg')

module.exports = OutilPointageCre

/**
 * Outil servant à créer des objets en cliqnant sur des objets de la figure
 * Hérite de OutilPointage
 * @param app L'application propriétaire
 * @constructor
 */
function OutilPointageCre (app) {
  OutilPointage.call(this, app)
  this.objetDesigne = null // Sera l'objet transmis à l'outil après d'éventuelles boîtes de dialogue de choix
}
OutilPointageCre.prototype = new OutilPointage()
OutilPointageCre.constructor = OutilPointageCre // Outil PointageClicOuPt hérite de cet outil

OutilPointageCre.prototype.reset = function (bModifiablesParMenu = false, bMemeMasque = false) {
  OutilPointage.prototype.reset.call(this)
  this.bModifiableParMenu = bModifiablesParMenu
  this.bMemeMasque = bMemeMasque
}

OutilPointageCre.prototype.mousemove = function (evt) {
  this.devicemove(evt, 'mouse', kernelAdd.mousePosition)
}
OutilPointageCre.prototype.touchmove = function (evt) {
  this.devicemove(evt, 'touch', kernelAdd.touchPosition)
}
OutilPointageCre.prototype.devicemove = function (evt, type, fonc) {
  var point, x, y
  var app = this.app
  var doc = app.doc
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  if ((type === 'mouse') && (doc.type === 'touch')) return
  doc.type = type
  var svg = app.svgFigure
  // var elc = app.elementCapture; A revoir mtgApp
  /*
  var tab = kernel.getMousePosition(evt);
  var par = svg.parentNode.parentNode; // Il faut que le parent soit le div contenant l'appli
  var posdiv = kernel.getPosition2(par);
  var x = tab[0] - posdiv.x - constantes.svgPanelWidth;
  var y = tab[1] - posdiv.y - constantes.topIconSize;
  */
  point = fonc(svg, evt, app.zoomFactor)
  x = point.x
  y = point.y
  /* Devenu inutile depuis que les événements souris sont interceptés par un rectangle
  if (x<0 || x>app.dimf.x || y<0 || y>app.dimf.y) {
    document.body.style.cursor = "default";
    return;
  }
  */
  // document.body.style.cursor = "crosshair";
  // app.outilActif.cursor = "crosshair";
  this.app.mousePoint.placeEn(x, y)
  var liste = this.listeUtilisee()
  var info = app.infoProx
  // var point = new Point(x, y);
  var nbObjetsProches = liste.procheDe(this.aDesigner, point, info, app.listeExclusion, true, type,
    this.bModifiableParMenu, this.bMemeMasque)
  var nbTypesProches = info.nombreTypesProches
  // var designationDejaAffichee = (app.commentaireDesignation.chaineCommentaire.length != 0);
  // true si une désignation est déjà affichée près du pointeur
  if (nbObjetsProches > 0) {
    var ch = ''
    // Si lors d'un dialogue de choix d'objet proche on a déjà choisi un objet
    // et si celui-ci est proche du pointeur souris, c'est celui-là qui sera sélectionnné
    if (info.premierPointVoisin !== null) {
      ch = 'desPoint'
    } else {
      if (nbTypesProches === 1) {
        for (var i = 0; i < NatObj.nombreTypesDesignables; i++) {
          if (info.infoParType[i].nombreVoisins > 0) {
            ch = info.infoParType[i].premierVoisin.chaineDesignation()
            break
          }
        }
      } else {
        ch = 'desAPreciser'
      }
    }
    app.setDesignation(x, y, ch)
  } else app.cacheDesignation()
  app.updateObjetsVisuels()
}

OutilPointageCre.prototype.mousedown = function (evt) {
  this.devicedown(evt, 'mouse', kernelAdd.mousePosition)
}

OutilPointageCre.prototype.touchstart = function (evt) {
  this.devicedown(evt, 'touch', kernelAdd.touchPosition)
}

OutilPointageCre.prototype.devicedown = function (evt, type, fonc) {
  var app = this.app
  var svg = app.svgFigure
  var doc = app.doc
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  if ((type === 'mouse') && (doc.type === 'touch')) return
  doc.type = type
  var point = fonc(svg, evt, app.zoomFactor)
  // Modification version 6.4.8 : On utilise un clone de l'objet global app.infoProx car,
  // par exemple quand on ouvre une boîte de dialogue de choix par type, il se peut qu'un événement devicemove
  // ait modifié entre temps cet InfoProx
  // var info = app.infoProx
  var info = app.infoProx.getClone()
  // Sur un périphérique mobile on ne peut pas savoir si avant le touch on était proche d'un objet
  app.mousePoint.placeEn(point.x, point.y)
  app.updateObjetsVisuels()
  if (type === 'touch') {
    var liste = this.listeUtilisee()
    liste.procheDe(this.aDesigner, point, info, app.listeExclusion, true, type,
      this.bModifiableParMenu, this.bMemeMasque)
  }
  var nombreObjetsProches = info.nombreObjetsProches
  var self = this
  if (nombreObjetsProches > 0) {
    // Les deux lignes suivantes pour que, si l'outil ouvre une boîte de dialogue, le clic éventuel à l'endroit d'un bouton
    // de cette boîte ne soit pas traité
    kernel.preventDefault(evt)
    evt.stopPropagation()
    if (info.nombreTypesProches > 1) {
      new ChoixTypeDlg(app, info, info.typesProches, this,
        'objetDesigne', function () { self.callBackAfterDeviceEnd(point) })
    } else {
      if (nombreObjetsProches === 1) {
        this.objetDesigne = info.premierVoisin()
        this.callBackAfterDeviceEnd(point)
      } else {
        new PremierDernierDlg(app, this, 'objetDesigne', info.premierVoisin(),
          info.dernierVoisin(), function () {
            self.callBackAfterDeviceEnd(point)
          })
      }
    }
    app.updateObjetsVisuels()
  } else {
    if (app.pref_PointsAuto && app.outilActif.creationPointPossible()) {
      var pt = app.ajoutePoint(point.x, point.y)
      app.outilActif.nbPtsCrees++
      app.outilActif.traiteObjetDesigne(pt)
    }
  }
}
OutilPointageCre.prototype.mouseup = function () {
  this.app.doc.type = '' // Pour autoriser à nouveau tout événement mouse ou touch
}

// Quand sur un périphérique on relache le doigt et qu'il était proche d'un objet servant à la désiganntion,
// On fait comme si l'utilisateur avait fait un clic souris dessus
/**
 * Fonction appelée quand on relache le doigt et qu'il était proche d'un objet servant à la désignation.
 * Si la fonction isReadyForTouchEnd de l'outil renvoie true, on fait comme si l'utilisateur avait fait un clic
 * droit sur l'objet.
 */
OutilPointageCre.prototype.touchend = function () {
  // Si un dialogue est ouvert, on ne traite pas l'événement
  if (this.app.dlg.length === 0) {
    var app = this.app
    if (app.outilActif.isReadyForTouchEnd()) {
      // Version 4.6.8 : On utilise un clone de infoProx car un événement devicemove pourrait passer par làa
      // avant la création de l'objet (suite rapport busNag)
      const info = app.infoProx.getClone()
      var nombreObjetsProches = info.nombreObjetsProches
      var self = this
      if (nombreObjetsProches > 0) {
        if (info.nombreTypesProches > 1) {
          new ChoixTypeDlg(app, info, info.typesProches, this,
            'objetDesigne', function () {
              self.callBackAfterTouchEnd()
            })
        } else {
          if (nombreObjetsProches === 1) {
            this.objetDesigne = info.premierVoisin()
            this.callBackAfterTouchEnd()
          } else {
            new PremierDernierDlg(app, this, 'objetDesigne', info.premierVoisin(),
              info.dernierVoisin(), function () {
                self.callBackAfterTouchEnd()
              })
          }
        }
      }
    }
  }
}

OutilPointageCre.prototype.callBackAfterDeviceEnd = function (point) {
  this.app.outilActif.traiteObjetDesigne(this.objetDesigne, point)
  this.objetDesigne = null
}

OutilPointageCre.prototype.callBackAfterTouchEnd = function () {
  const app = this.app
  // Try catch rajouté suite àa rapport BusNag
  try {
    app.outilActif.traiteObjetDesigne(this.objetDesigne, app.mousePoint)
  } catch (e) {}
  this.objetDesigne = null
}
