/*
 * Created by yvesb on 11/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const Point = require('../types/Point')
const Pointeur = require('../types/Pointeur')
const OutilPointage = require('../pointage/OutilPointage')
const kernelAdd = require('../kernel/kernelAdd')
module.exports = OutilPointageRapporteur

/**
 * Outil servant à finir d'utiliser les outils utilisant un rapporteur virtuel
 * Hérite de OutilPointage
 * @param app L'application propriétaire
 * @constructor
 */
function OutilPointageRapporteur (app) {
  OutilPointage.call(this, app)
  this.objetDesigne = null // Sera l'objet transmis à l'outil après d'éventuelles boîtes de dialogue de choix
}
OutilPointageRapporteur.prototype = new OutilPointage()

OutilPointageRapporteur.prototype.reset = function () {
  this.ptPointLieCercle = null
}

OutilPointageRapporteur.prototype.associePointLie = function (ptInit) {
  this.ptPointLieCercle = ptInit
}

OutilPointageRapporteur.prototype.mousemove = function (evt) {
  this.devicemove(evt, 'mouse', kernelAdd.mousePosition)
}
OutilPointageRapporteur.prototype.touchmove = function (evt) {
  this.devicemove(evt, 'touch', kernelAdd.touchPosition)
}

OutilPointageRapporteur.prototype.devicemove = function (evt, type, fonc) {
  var point, x, y
  var app = this.app
  var doc = app.doc
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  if ((type === 'mouse') && (doc.type === 'touch')) return
  doc.type = type
  var svg = app.svgFigure
  point = fonc(svg, evt, app.zoomFactor)
  x = point.x
  y = point.y
  // var poss = this.ptPointLieCercle.testDeplacement(cadre.dimf,mouseEvent.getX(),
  //   mouseEvent.getY(), pointres, pointabs);
  var pointres = new Point()
  var abs = new Pointeur()
  var poss = this.ptPointLieCercle.testDeplacement(app.dimf, x, y, pointres, abs)
  if (poss) {
    this.ptPointLieCercle.donneAbscisse(abs.getValue())
    app.listeObjetsVisuels.positionne(false, app.dimf)
    app.listeObjetsVisuels.update(svg, app.doc.couleurFond, app.doc.opacity, true)
  }
}

OutilPointageRapporteur.prototype.mousedown = function (evt) {
  this.devicedown(evt, 'mouse', kernelAdd.mousePosition)
}

OutilPointageRapporteur.prototype.touchstart = function (evt) {
  this.devicedown(evt, 'touch', kernelAdd.touchPosition)
}

OutilPointageRapporteur.prototype.devicedown = function (evt, type, fonc) {
  this.app.outilActif.creeObjet()
}

OutilPointageRapporteur.prototype.touchcancel = function (evt) {
  this.devicedown(evt, 'touch', kernelAdd.touchPosition)
}

OutilPointageRapporteur.prototype.touchend = function (evt) {
  this.devicedown(evt, 'touch', kernelAdd.touchPosition)
}
