/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

/**
 * Contient les textes utilisés dans le logiciel en langue anglaise
 * @type {Textes}
 * @private
 */
const textesEnAdd = {
  infoabs: 'abs(x) : Absolute value of x',
  infointgr: 'int(x) : Integer part of x',
  infosqrt: 'sqrt(x) : Root value of x',
  inforac: 'sqrt(x) : Root value of x',
  infosin: 'sin(x) : Sine of x',
  infocos: 'cos(x) : Cosine of x',
  infotan: 'tan(x) : Tangent of x',
  infoln: 'ln(x): Natural logarithm of x (base e)',
  infoexp: 'exp(x) : Exponential of x',
  infoarctan: 'atan(x) : Arc tangent of x',
  infoarccos: 'acos(x) : Arc cosine of x',
  infoarcsin: 'asin(x) : Arc sine of x',
  infoch: 'cosh(x) : Hyperbolic cosine of x',
  infosh: 'sinh(x) : Hyperbolic sine of x',
  infoth: 'tanh(x) : Hyperbolic tangent of x',
  infoargch: 'acosh(x) : Hyperbolic arc cosine of x',
  infoargsh: 'asinh(x) : Hyperbolic arc sine of x',
  infoargth: 'atanh(x) : Hyperbolic arc tangent of x',
  inforand: 'rand(x) : random number in [0;1[',
  infoconj: 'conj(z) : Conjugate complex of z',
  infoarg: 'angle(z) : Principal argument of complex z',
  infore: 're(z) : Real part of complex z',
  infoim: 'im(z) : Imaginary part of complex z',
  infomaxi: 'max(x,y) : Maximum value of x and y',
  infomini: 'min(x,y) : Minimum value of x and y',
  infogcd: 'gcd(x,y) : Greatest common divisor of x and y',
  infolcm: 'lcm(x,y) : Least common multiple of x and y',
  infoncr: 'nCr(n,p) : Number of sets of p elements in a set of n elements',
  infonpr: 'nPr(n,p) : Number of permutations of p elements in a set of n elements',
  infomod: 'mod(a,b) : Rest of euclidian division of a through b',
  infosi: 'if(test,value1,value2) : Returns value1 if test equals 1 and value2 otherwise',
  infointegrale: 'integral(expr,var,a,b) : Approximated value of integral of expr from a to b\nvar is the integration variable',
  infoprimitive: 'primitive(f,var,a,b) : Returns f(b) - f(a) where f est a function formula\nvar is the variable of the function',
  infofact: 'fact(n) : Factorial of n (if n integer value positive)',
  infosomme: 'sum(Expr, j, start, end, step)  Returns sum of Expr \nfor j integer getting start value to end with step increment',
  infoproduit: 'prod(Expr, j, start, end, step)  Returns product of Expr \nfor j integer getting start value to end with step increment',
  infoLeft: 'left(a) : left member of a if a is a test',
  infoRight: 'right(a) : right member of a if a is a test',
  infoTransp: 'transp(M) : returns the transpose of matrix M. \nIf M is a number returns the number',
  infoDeter: 'deter(M) : returns the determinant of matrix M. \nIf M is a number returns the number',
  OK: 'OK',
  Cancel: 'Cancel',
  Oui: 'Yes',
  Non: 'No',
  Dte: 'Line',
  Ddte: 'Ray',
  Seg: 'Segment',
  Vect: 'Vector',
  Cerc: 'Circle',
  Point: 'Point',
  PointLibre: 'Free point',
  PointCons: 'Constructed point',
  PointLie: 'Linked point',
  PointRep: 'Point through coordinates',
  PointLibreEnt: 'Free point with integer coordinates',
  PointLiePoint: 'Point linked to a point',
  Macro: 'Macro',
  PointParAff: 'Point through affix',
  Arc: 'Circle arc',
  Polygone: 'Polygon',
  Surf: 'Surface',
  LieuPt: 'Point locus',
  LieuPtDis: 'Disconnected point locus',
  LigneBrisee: 'Broken line',
  LieuObj: 'Object locus',
  Image: 'Image',
  GrapheSuiteR: 'Recurrent sequence graph',
  GrapheSuiteComp: 'Complex recurrent sequence grap',
  Comment: 'Text display',
  ObjetDup: 'Duplicated object',
  DemiPlan: 'Half-plan',
  PtIntPoly: 'Point inside polygon',
  PtIntCerc: 'Point inside circle',
  EditeurFor: 'Formula editor',
  PtClone: 'Point clone',
  DteClone: 'Line clone',
  DdteClone: 'Ray clone',
  SegClone: 'Segment clone',
  CercClone: 'Circle clone',
  Texte: 'Text',
  desPoint: 'this point',
  desDroite: 'this line',
  desSegment: 'this segment',
  desVecteur: 'this vector',
  desDemiDroite: 'this ray',
  desCercle: 'this circle',
  desArc: 'this arc',
  desPolygone: 'this polygon',
  desLigneBrisee: 'this line',
  desSurface: 'this surface',
  desDemiPlan: 'this half-plan',
  desCommentaire: 'this text',
  desAffichageValeur: 'this display value',
  desMacro: 'this macro',
  desMarqueAngle: 'thisangle mark',
  desMarqueSegment: 'this segment mark',
  desGrapheSuiteRec: 'this sequence graph',
  desGrapheSuiteRecComplexe: 'this complex sequence graph',
  desLieuPoint: 'this point locus',
  desLieuObjet: 'this object locus',
  desImage: 'this image',
  desObjetDuplique: 'this duplicated object',
  desAPreciser: 'to be specified',
  desIntersection: 'this intersection',
  desLatex: 'this LaTeX display',
  desEditeur: 'this formula editor',
  Annuler: 'Cancel',
  Refaire: 'Redo',
  Creer: 'Create ',
  Capt: 'Capture',
  CerOA: 'Circle through center and point',
  CerOR: 'Circle through center and radius',
  ArcPetit: 'Little arc',
  ArcGrand: 'Long arc',
  ArcDirect: 'Direct arc',
  ArcIndirect: 'Indirect arc',
  ArcPetitParAng: 'Little arc through angle',
  ArcGrandParAng: 'Long arc throuh angle',
  ArcDirectParAng: 'Direct arc through angle',
  ArcIndirectParAng: 'Indirect arc through angle',
  Rapporteur: 'Protractor',
  ArcParRapporteur: 'Little arc through protractor',
  ArcGrandParRapporteur: 'Long arc through protractor',
  LieuParPtLie: 'Point locus through linked point',
  LieuParVariable: 'Point locus through variable',
  LieuDiscretParPtLie: 'Disconnected point locus through linked point',
  LieuDiscretParVariable: 'Disconnected point locus through variable',
  LieuObjetParPtLie: 'Object locus through linked point',
  LieuObjetParVariable: 'Object locus through variable',
  MesAngNor: 'Non-oriented angle measure',
  Fonc: 'Real function',
  SolutionEq: 'Approximated root of equation',
  Derivee: 'Derivative function',
  DemiDt: 'Ray',
  DtAB: 'Line through two points',
  DtPar: 'Parallel line',
  DtPer: 'Perpendicular line',
  DtBis: 'Angle bisector',
  DtMed: 'Perpendicular bisector',
  DtHor: 'Horizontal line',
  DtVer: 'Vertical line',
  PtLib: 'Free point',
  CentreGrav: 'Gravity center',
  Milieu: 'midpoint',
  MarqueSeg: 'Segment mark',
  MarqueAng: 'Angle mark',
  MarqueAngOr: 'Oriented angle mark',
  MarqueAngOrSD: 'Direct oriented angle mark',
  MarqueAngOrSI: 'Indirect oriented angle mark',
  Carre: 'Direct square',
  TriangleEq: 'Direct equilatéral triangle',
  Parallelog: 'Parallelogram',
  PtParCoord: 'Point through coordinates',
  PtParAff: 'Point through affix',
  MarquerPt: 'Mark a point for trace',
  DemarquerPt: 'Unmark a point for trace',
  Punaiser: 'Pin up free point',
  Depunaiser: 'Free pinned point',
  Rideau: 'Unmask object',
  Gomme: 'Mask objet',
  Recalculer: 'Recalculate and display',
  TranslationFigure: 'Figure translation',
  ExecutionMacro: 'Macro execution',
  Surface: 'Surface through circle, polygon, arc or point locus',
  Surface2Lieux: 'Surface through two point locuses',
  SurfaceLieuDroite: 'Surface through point locus and line',
  SurfaceLieu2Pts: 'Surface through point locus and two points',
  New: 'New figure',
  PtBaseEnt: 'Free point with integer coordinates',
  PtInterieur: 'Point inside circle or polygon',
  DtParEq: 'Line defined by equation',
  Latex: 'LaTeX display',
  LatexLiePt: 'LaTeX display linked to point',
  Commentaire: 'Text display',
  CommentaireLiePt: 'Text display linked to point',
  AffichageValeur: 'Value display',
  AffichageValeurLiePt: 'Value display linked to point',
  CreationLiaison: 'Link creation between point and object',
  SuppressionLiaison: 'Point-object ling deletion',
  Rot: 'Rotation',
  Hom: 'Dilation',
  Sim: 'Similitude',
  Sup: 'Graphical object deletion',
  Int: 'Intersection',
  Proj: 'Perpendicular projected point',
  Angle: 'Angle',
  Rapport: 'Ratio',
  Rayon: 'Radius',
  SymAxiale: 'Axial symmetry',
  SymCentrale: 'Cental symmetry',
  Trans: 'Translation',
  ChoixCouleur: 'Predefined color',
  ChoixStyleTrait: 'Line style',
  ChoixEpaisseur: 'Line thickness',
  ChoixOpacite: 'Opacity for surfaces',
  ChoixStylePoint: 'Point style',
  ChoixMarqueSeg: 'Segment mark style',
  ChoixStyleFleche: 'Arrow style',
  ChoixMarqueAng: 'Angle mark style',
  ChoixStyleRemp: 'Filling style for surfaces',
  CouleurPerso: 'Owner defined color',
  Fermer: 'Close',
  AngleRot: 'Rotation angle',
  RapportHom: 'Dilation ratio',
  RayCer: 'Circle radius',
  AngArc: 'arc angle',
  Avert: 'Warning',
  ErrSyntaxe: 'Syntax error',
  ErrCalculNonExist: "Warning : the result doesn't exist",
  DejaCree: 'Same object was already been created',
  ChoixEntre: 'Choose between',
  PremierObj: 'The first object',
  DernierObj: 'The last object',
  ChoixAPrec: 'Choice to be detailed',
  rotDlg: 'Rotation angle',
  ChoixTypeDlg1: 'Several objects are<br/> near the clicked area<br/>Choose between : ',
  ChoixValeur: 'Value choice',
  ChoixValeurDlg1: 'Choose a value',
  ChoixValeurDlg2: 'in the right panel',
  ChoixValeurDlg3: 'Information on the selected value :',
  ChoixVariableDlg1: 'Choose a variable',
  NewRepDlg: 'New figure with frame',
  NewRepDlg1: 'Orthonormal',
  NewRepDlg2: 'Orthogonal',
  NewRepDlg3: 'Oblique',
  NewRepDlg4: 'Oblique with equal length unities',
  NewRepDlg5: 'Vertical grid',
  NewRepDlg6: 'Horizontal grid',
  NewRepDlg7: 'dotted',
  NewRepDlg8: 'Create axis if necessary',
  NewRepDlg12: 'without grid',
  NewRepDlg13: 'Simple scaling',
  NewRepDlg14: 'Tigonometry scaling',
  NewRepDlg15: 'With vectors',
  testEquivalenceDlg4: 'Existence condition :',
  vardlg1: 'Mini value must be inferior to maxi value',
  vardlg2: 'Step value is not valid',
  vardlg3: 'Current value is not valid',
  lieuObjetDlg1: 'Number of positions<br/>to generate the locus : <br/>(from 2 to 100000)',
  PtIntDlg1: 'The clicked place is not inside',
  DerDlg1: 'Name of the derivative :',
  DerDlg2: 'Derivative of :',
  InfoFonc: 'Function information :',
  TgteDlg1: 'Tangent to the curve of',
  TgteDlg2: 'At point of abscissa',
  TgteDlg3: 'Name of the slope measure',
  SolEqDlg1: 'Equation solving (dichotomy) on interval [a,b]',
  SolEqDlg2: 'Name of solution : ',
  SolEqDlg3: 'Name of the unknown to solve',
  SolEqDlg4: 'Equation to solve',
  SolEqDlg5: 'a Value :',
  SolEqDlg6: 'b Value :',
  SolEqDlg7: 'Uncertainty :',
  MaxDlg1: 'Abscissa of a function maximum on [a;b]',
  MaxDlg2: 'Name of maximum :',
  MinDlg1: 'Abscissa of a function minimum on [a;b]',
  MinDlg2: 'Name of minimum :',
  courbeDlg1: 'Curve to draw on :',
  courbeDlg2: ']-&infin;;+&infin;[',
  courbeDlg3: '][a;b][',
  courbeDlg4: '][a;+&infin;[',
  courbeDlg5: ']-&infin;;a][',
  courbeDlg7: 'Generating linked point hidden',
  courbeDlg8: 'Point generating curve hidden',
  courbeDlg9: 'Automatic management of discontinuity',
  courbeDlg10: 'Number of points of the curve :',
  suiteRecDlg1: 'Name of the sequence :',
  suiteRecDlg2: 'Associated function',
  graphesrDlg1: 'Lines joining abscissa',
  graphesrDlg2: 'Points connected with segments',
  graphesrDlg3: 'Associated sequence',
  saveDlg1: 'Enter the name<br/>of the file (without suffix) : ',
  NomCalcul: 'Name of the calculation',
  NomFonction: 'Name of the function',
  Formule: 'Formula',
  Calcul: 'Calculation',
  CalculComp: 'Complex calculation',
  CalculReel: 'Real calculation',
  Variable: 'Variable',
  AffVal: 'value display',
  NomVar: 'Name of the variable',
  ValMin: 'Mini Value :',
  ValMax: 'Maxi Value :',
  ValPas: 'Step Value :',
  ValAct: 'Actual value',
  DlgAss: 'Associated dialog :',
  Fonction: 'Function',
  FoncComp: 'Complex function',
  TypeAPrec: 'To be specified',
  NomInvalide: 'The name is not valid',
  ExisteMemeNom: 'This name has already been given',
  ch6: 'The intersection point(s) already exist',
  ch18: "Doesn't exist",
  ch34: 'Unity length of the figure',
  ch87: 'defined by',
  ch109: 'Non-secant objects',
  ch112: 'Derivative of function',
  ch185: 'The equation must depend on the unknown',
  ch186: 'Equivalence test of two calculations',
  ch187: 'With',
  ch188: 'Without',
  ch189: 'Replacement of values',
  ch190: 'Partial derivative of',
  ch191: 'through variable',
  ch192: 'Variable dependence test of function',
  ch196: 'Operator nature test',
  ch200: 'Factorisation test of',
  chinfo17: 'Length measure',
  chinfo29: 'Non-oriented angle measure',
  chinfo30: 'Oriented angle measure',
  chinfo31: 'Abscissa measure of',
  chinfo32: 'relative to frame',
  chinfo71: 'Area',
  chinfo72: 'of polygone',
  chinfo73: 'Length',
  chinfo74: 'of line',
  chinfo77: 'Mini value :',
  chinfo78: 'Maxi value :',
  chinfo79: 'Valeur du pas :',
  chinfo80: 'Step value :',
  chinfo81: 'Ordinate of',
  chinfo84: 'associated to function',
  chinfo86: 'Terms number :',
  chinfo93: 'Approximated value of integral',
  chinfo94: 'of function',
  chinfo95: 'Lower bound :',
  chinfo96: 'Upper bound :',
  chinfo97: 'Current value : ',
  chinfo98: 'Approximated root',
  chinfo99: 'of equation',
  chinfo100: 'ranging between',
  et: 'and',
  chinfo102: 'Precision :',
  chinfo107: 'of complex',
  chinfo108: 'Complex affix',
  chinfo109: 'of point',
  chinfo110: 'measured in frame',
  chinfo112: 'Modulus',
  chinfo115: 'of the value of',
  chinfo116: 'Indexed through values of',
  chinfo117: 'Mini index :',
  chinfo118: 'Maxi index :',
  chinfo153: 'Line slope measure of',
  chinfo157: 'Scalar product of vector',
  chinfo158: 'and vector',
  chinfo160: 'Indexed product',
  chinfo164: 'Point of affix',
  chinfo176: 'Maximum abscissa',
  chinfo177: 'Minimum abscissa',
  chinfo178: 'Origin x-coordinate',
  chinfo179: 'Origin y-coordinate',
  chinfo186: 'Second terme :',
  chinfo180: 'x-unity',
  chinfo181: 'y-unity',
  Valeurs: 'Values',
  Fonctions: 'Functions',
  avRapNul: 'Ratio must not be null',
  avRay: 'Radius must be stricty positive',
  ch37: 'One or several dependant objects will also be deleted. Do you confirm deletion ?',
  ch38: 'Deleting this object will also delete the intersection of two objects. Do you confirm deletion ?',
  ModifObjGraph: 'Graphical object modification',
  ChoixTrans1: 'Several objects are images through the same geometric transformation<br/>as this object.<br/>Do you want to apply this transformation to :',
  ChoixTrans2: 'all the images',
  ChoixTrans3: 'this image only',
  RayPix: 'Radius in pixels',
  ModifObj: 'Modify object',
  ModifObjNum: 'Modify numerical object',
  Palette: 'Change color or style of object',
  Nommer: 'Name a point or a line',
  CaptNom: 'Move a name',
  NouvelleFigAvecRep: 'New Figure with frame',
  AvertDirty: 'Current figure has not been saved<br/>Do you confirm ?',
  newFig1: 'Figure with frame',
  newFig3: 'Figure without frame and with unity length',
  newFig4: 'Figure without frame and without unity length',
  newFig5: 'Basic figure',
  newFig6: 'Figure with alterable frame without vectors',
  newFig7: 'Figure with alterable frame with vectors',
  newFig8: 'figure with graph of functions or sequences',
  newFig9: 'Figure of probability',
  newFig10: 'Basic 3D figure',
  newFig11: 'Figure through Base64 code',
  Abscisse: 'Abscissa',
  Ordonnee: 'Ordinate',
  Affixe: 'Affixe',
  Repere: 'Frame',
  PtLie: 'Linked point',
  MesLong: 'Length measure',
  MesAngOr: 'Oriented angle measure',
  MesAbs: 'Abscissa measure',
  dans: 'in',
  de: 'of',
  Modifier: 'Modify',
  Supprimer: 'Delete',
  ObjNum: 'Numerical objects',
  TracerCourbe: 'Create curve',
  nbPtsCourbe: 'Number of points of the curve',
  nbPtsCourbeMsg: 'Points number must range between 4 and 5000',
  NomMesure: 'Measure name',
  OrAbs: 'x-Coordinate origin : ',
  OrOrd: 'y-Coordinate origin : ',
  Unitex: 'x-unity : ',
  Unitey: 'y-unity : ',
  PtLibreEnt: 'Free point with integer coordinates',
  AbsLibres: 'x-coordinates free',
  OrdLibres: 'y-coordinate free',
  AbsMin: 'x-coordinate<br/> mini',
  AbsMax: 'x-coordinate<br/> maxi',
  OrdMin: 'y-coordinate<br/> mini',
  OrdMax: 'y-coordinate<br/> maxi',
  MesAbsRep: 'x-coordinate measure in frame',
  MesOrdRep: 'y-coordinate measure in frame',
  MesAffRep: 'Affix measure',
  MesCoefDir: 'Slope measure',
  CoefDir: 'Slope',
  DtParCoef: 'Line through slope',
  NewFig: 'New figure',
  MesLongLigne: 'Line length measure',
  MesAire: 'Polygon area measure',
  EqDroite: 'Line equation',
  ch41: 'A polygon needs three vertex at least',
  ch42: 'A broken line needs three vertex at least',
  UniteAngFig: '<b>Angle unity of the figure</b>',
  Degre: 'Degree',
  Radian: 'Radian',
  Nom: 'Name',
  NomMasque: 'Name masked',
  EntreeNom: 'Name entry',
  ZoomPlus: 'Zoom in',
  ZoomMoins: 'Zoom out',
  ModeTrace: 'Trace mode on or off',
  Valeura: 'a Value',
  Valeurb: 'b Value',
  TransParCoord: 'Translation of coordinates (a,b)',
  PtPunaise: 'Point has benn pinned down',
  PtDepunaise: 'Point has been unpinned',
  PtMarque: 'Point has been marked for trace',
  PtDemarque: 'Point has been unmarked for trace',
  NbPts: 'Number of points',
  LieuFerme: 'Closed locus',
  Epaisseur: 'Thickness',
  Align: 'Align',
  Taille: 'Size',
  EffFond: 'Background erase',
  EncSans: 'Without frame',
  EncSimple: 'Simple frame',
  Enc3D: '3D frame',
  CoulFond: 'Background<br/>color',
  ModeAff: 'Display mode',
  Valeur: 'Value',
  EnTete: 'Starting string :',
  SuiviDe: 'Ending string :',
  Decimales: 'Digits',
  AffIci: 'Display here',
  AffLatex: 'LaTeX display',
  CodeLatex: 'LaTeX code',
  Apercu: 'Preview',
  InsVal: 'Value insertion',
  TexteAAff: 'Text to display',
  Vide: 'Text is empty',
  OpImp: 'Operation impossible',
  SupLiaisonImp1: 'The link cannot be suppressed because this object is a final object of construction',
  SupLiaisonImp2: 'The link cannot be suppressed because positions of this point are used by a point or object locus or by a macro',
  PtParAbs: 'Point through abscissa on line',
  SuppressionLiaisonAff: 'Suppression of link display-point',
  CreationLiaisonAff: 'Creation of link display-point',
  indCapt: 'Capture a free point, an angle mark or a display',
  indPtLib: 'Click to create a free point',
  ind2Pt1: 'Click on the first point',
  ind2Pt2: 'Click on the second point',
  ind3Pt3: 'Click on the third point',
  indInt: 'Click at the intersection of two objects or on two objects one after another',
  indPtBaseEnt: 'Click to cerate a free point with integer coordinates',
  indProj1: 'Click on the line, segment or array on which the point must be projected',
  indProj2: 'Click on the point to be projected',
  indPtInt1: 'Click on a polygon or a circle',
  indPtInt2: 'Click inside the yellow area',
  indCaptNom: 'Capture a name of point or line',
  intPtParAbs1: 'Click on the point of abscissa 0',
  intPtParAbs2: 'Click on the point of abscissa 1',
  indCrLiaison1: 'Click on a free point to get it linked to an object',
  indCrLiaison2: 'Click on the object you want the point linked to',
  indSupLiaison: 'Click on a linked point to get it turned in a free point',
  indPun: 'Click on a point to get it pinned-up',
  indDepun: 'Click on a poit to get it unpinned',
  indMarq: 'Click on a point to get it marked for trace',
  indDemarq: 'Click on a point to get it unmarked for trace',
  indParDtePt1: 'Click on a line, a segment\nor a ray (or a point)',
  indPt: 'Click on a point',
  indDte: 'Click on a line',
  indSeg: 'Click on a segment',
  indPoly: 'Click on a polygon',
  indPolOuLig: 'Click on a polygon or a broken line',
  indSurface: 'Click on a circle, a polygon, an arc or a point locus',
  indTrans1: 'Click on the origin vertex of the translation or on a vector',
  indTrans2: 'Click on the extremity vertex of the translation',
  indImTrans: " Click on an object to get the object's image created",
  indCrPoly: 'Click on the vertexes of the polygon. Click on STOP\nbutton or on the first vertex to close the polygon',
  indCrLigne: 'Click on the vertexes of the polygon. Click on STOP button to create the line',
  indCentre: 'Click on the center',
  indPtCer: 'Click on a point of the circle',
  indLieu1: 'Click on the point traces of which will generate the locus',
  indLieu2: 'Click on the linked point positions of which will generate the locus',
  indPtLie: 'Clcik on an object to create a point linked to it',
  indCentSym: 'Click on the symmetry center',
  indAxeSym: 'Click on the symmetry axis',
  indCentRot: 'Click on the rotation center',
  indCentHom: 'Click on the dilation center',
  indCentSim: 'Click on the similitude center',
  indLieuObj1: 'Click on the object traces of which will generate the locus',
  indEndroitAff: 'Click on the place you want the display to stand',
  indCrLiaisonAff1: 'Click on a display to get it linked to a point',
  indCrLiaisonAff2: 'Click on the point the display is to be linked to',
  indSupLiaisonAff: 'Click on a linked display to get it freed',
  indSurf2Lieux1: 'Click on the fisrt locus',
  indSurf2Lieux2: 'Click on the second locus',
  indSurfLieuDte1: 'Click on the first edge (point locus)',
  indSurfLieuDte2: 'Click on the second edge (droite)',
  indSurfLieu2pts2: 'Click on the first point',
  indSurfLieu2pts3: 'Click on the second point',
  indDemiPl1: 'Click on the edge of the half-plan (line)',
  indDemiPl2: 'Click on a point of the half-plan',
  solutionEq: 'Approximated root of equation',
  Der: 'Derivative function',
  Max: 'Maximum of function',
  Min: 'Minimum of function',
  InfoSuit: 'Sequence information :',
  CourbeFonc: 'Function curve',
  SuiteRec: 'Real recurrent sequence u(n+1)=f[u(n)]',
  SuiteRecComplexe: 'Complex recurrent sequence z(n+1)=f[z(n)]',
  Lieuhf: "The locus doesn't seem to be visible<br/>Do you confimr creation ?",
  Curseur: 'Cursor',
  CurHor: 'Horizontal cursor',
  CurVer: 'Vertical cursor',
  CurSmall: 'Little size',
  CurBig: 'Big size',
  CurEnt: 'Cursor with step',
  CurNom: 'Name of the calculation representing cursor :',
  CurPun: 'Pin-down extremities',
  indCurseur1: 'Click on a point or at a place of the figure',
  ErrMinMax: 'Mini value must be inferior to maxi value',
  Tangente: 'Tangent to function curve',
  Add: 'More tools',
  ChoixOutil: 'Choose a tool',
  Fonc2Var: 'Real function of two variables',
  Fonc3Var: 'Real function of three variables',
  FoncComp2Var: 'Complex function of two variables',
  FoncComp3Var: 'Complex function of three variables',
  PremTerm: 'First term :',
  NbTerm: 'Number of terms<br>(from 2 to 100000) :',
  GrapheSuiteRec: 'u(n+1)=f[u(n)] sequence graph',
  GrapheSuiteRecComp: 'z(n+1)=f[z(n)] complex sequence graph',
  OptionsFig: 'Figure options',
  CodeBase64: 'Get figure Base64 code',
  Base64: 'Base64 code of the figure :',
  FigParCode: 'Paste here the Base64 code of the figure',
  Base64Err: 'Base64 code is not valid',
  Open: 'Open file',
  Save: 'Save file',
  NomFichierErr: 'File name is not valid',
  FichierErr: 'No valid file chosen',
  Parcourir: 'Browse',
  /// ////////////////////////////////////////////////////
  PartieReelle: 'Real part',
  PartieImaginaire: 'Imaginary part',
  Module: 'Module',
  Argument: 'Principal argument',
  TestExistence: 'Existence test',
  chinfo92: 'of value',
  Coef: 'Coefficient :',
  Barycentre: 'Barycenter',
  indBar1: 'Click on a point to get it ponderated,, clic on Stop to create the barycenter',
  ErrBar: 'Creation impossible. The sum of coefficients est null',
  HorsFen: 'Created object is outside the window.<br/>Do you confirm creation ?',
  unAng: 'Angle unity : ',
  niv0: 'Elementry level',
  niv1: 'Mid-level',
  niv2: 'Advanced level without complex numbers',
  niv3: 'Advanced level with complex numbers',
  SegmentParLong: 'Segment of given length',
  LongSeg: 'Segment length :',
  long: 'length',
  PolygoneReg: 'Regular polygon',
  NbSommets: 'Number of vertexes : <br/> (from 3 to 100)',
  indPolReg: 'Click on the first vertex',
  /// /////////////
  Animation: 'Linked point animation',
  indAnim: 'Click on the blinking point to get animated',
  AnimCycl: 'Cyclic animation',
  Freq: 'Frequency (in s)',
  NbPtsPourAnim: 'Number of points<br/>(from 2 to 10000): ',
  InvSens: 'Inverted direction',
  OptionsAnim: 'Animation options',
  /// //
  PtParMultVec: 'Point M such as Vect(AM) = kVect(u)',
  PtParMultVecDlg2: 'Value of k',
  indVect: 'Click on a vector',
  indVect1: 'Click on the first vector',
  indVect2: 'Click on the second vectorr',
  PtParSommeVec: 'Point M suc as Vect(AM) = Vect(u) + Vect(v)',
  DeriveePart: 'Partial derivative',
  DerPartDlg1: 'Name of the derivative :',
  DerPartDlg2: 'Derivative relative to :',
  /// //
  indImage: 'Click to create an image display',
  ChoixImFond: 'Choice of bacground image',
  DelImFond: 'Background image deletion',
  AvertNonAnnul: 'This action cannot be cancelled.<br/>Do you confirm ?',
  EditeurFormule: 'Formula editor',
  EditForDlg1: 'Modifying formula of :',
  EditForDlg2: '(empty for default starting string)',
  EditForDlg3: 'Multiplication signs implicit<br/>(variables of one character)',
  EditForDlg4: 'LaTeX display of the formula',
  EditForDlg5: 'LaTeX starting code :',
  EditForDlg6: 'Initial Display of the formula',
  EditForDlg7: 'Real time display of the formula',
  Colonnes: 'Columns',
  RenCalcDlg: 'Enter the new name then<br/>press button Enter',
  RenommerCalcul: 'Rename a calculation',
  Renommer: 'Rename',
  AvertRecDeb: 'The object has been reclassed towards beginning of list',
  AvertRecFin: 'Object has been reclassed towards end of list',
  AvertRecImp: "object could'nt be reclassed",
  indRecDeb: 'Click on an object to get it reclassed towards beginning',
  indRecFin: 'Click on an object to get it reclassed towards end',
  ReclassDebObjGra: 'Reclass towards beginning',
  ReclassFinObjGra: 'Reclass towards end',
  NbGradx: 'Number of scaling marks<br/>on x-axis :',
  NbGrady: 'Number of scaling marks<br/>on y-axis :',
  GraduationAxes: 'Add scales to a frame',
  /// / Ajouts pour historique
  chinfo202: 'of center',
  chinfo203: 'and origin',
  chinfo204: 'ending in',
  chinfo205: ', angle',
  chinfo191: 'Direct circle arc',
  chinfo192: 'Indirect circle arc',
  chinfo37: 'Circle arc image of',
  chinfo193: 'Intersection point of',
  chinfo194: 'with',
  chinfo227: 'else than',
  chinfo150: 'Center of',
  chinfo35: 'Circle image of',
  chinfo225: 'of axis',
  chinfo195: 'Circle of center',
  par: 'through',
  chinfo36: 'Ray image of',
  bord: 'border',
  chinfo196: 'going through',
  chinfo50: 'Horizontal line',
  chinfo51: 'Vertical Line',
  chinfo34: 'Line image of',
  chinfo152: 'Line of slope',
  chinfo175: 'Line of equation',
  chinfo6: 'Line perpendicular to',
  chinfo88: 'Graph of real recurrent sequence',
  chinfo90: 'Graph of complex recurrent sequence',
  chinfo167: 'LaTeX display',
  chinfo207: 'generated by',
  chinfo126: 'Macro sending linked point',
  chinfo230: 'to position of',
  chinfo127: 'Macro giving value of',
  chinfo128: 'to variable',
  chinfo129: 'Macro updating the figure',
  chinfo124: 'Macro animating linked point',
  chinfo119: 'Macro Unmasking ',
  chinfo130: 'Loop macro with animation',
  chinfo133: 'through values of variable',
  chinfo132: 'Macro blinking',
  MacConsItDlg1: 'Iterative construction macro',
  MacConsRecDlg1: 'Recursive construction macro',
  chinfo122: 'Macro decrementing value',
  chinfo122Bis: 'of variable',
  chinfo120: 'Macro masking ',
  chinfo121: 'Macro incrementing value',
  chinfo123: 'Macro playing sound',
  chinfo170: 'Macro setting mode trace on',
  chinfo171: 'Macro setting mode trace off',
  chinfo134: 'Pause macro, duration of',
  chinfo135: 'Macro executing macros',
  chinfo136: 'Macro of automatic trace generated through linked point',
  chinfo137: 'Macro of automatic trace generated through values of variable',
  chinfo165: 'Right angle mark',
  chinfo66: 'Non-oriented angle mark',
  chinfo67: 'Oriented angle mark',
  chinfo68: 'Direct oriented angle mark',
  chinfo69: 'Indirect oriented angle mark',
  chinfo233: 'Duplicated of',
  chinfo139: 'Free point with integer coordinates',
  chinfo18: 'Image point of',
  chinfo169: 'Point inside',
  chinfo2: 'Point linked to',
  chinfo16: 'Point with abscissa of',
  chinfo217: 'Point such as',
  chinfo218: 'vect',
  chinfo60: 'Surface delimited by ',
  ProtoFig: 'Figure protocol',
  Const: 'Construction',
  chinfo138: 'Implement start of construction',
  rabsor: 'absor',
  rordor: 'ordor',
  ruxrep: 'uxfr',
  ruyrep: 'uyfr',
  rarc: 'arc',
  rcerc: 'circ',
  rcom: 'txt',
  rdd: 'ray',
  rdplan: 'hplan',
  rpt: 'Pt',
  rdt: 'lin',
  redit: 'edit',
  rgsr: 'recsg',
  rgsc: 'crecsg',
  rim: 'im',
  rlat: 'lat',
  rlieu: 'loc',
  rlieuOb: 'objloc',
  rmac: 'mac',
  rma: 'ma',
  rmseg: 'segm',
  raffval: 'valdisp',
  ObjInterm: 'Intermediary objets',
  liea: 'linked to',
  chinfo197: 'and radius',
  chinfo140: 'Point with coordinates of',
  chinfo28: ', ratio',
  ObjMas: 'Hidden object',
  ObjMasVis: 'Hidden objects visible',
  Protocole: 'Protocol',
  ModifBar: 'Barycenter modification',
  /// //////////////////////////////////
  RapportInv: 'Inversion ration',
  ImageInv: 'Image of point by inversion',
  indCentInv: 'Click on the inversion center',
  indCer: 'Click on a circle',
  CentreCercle: 'Circle center creation',
  Divers: 'Miscellaneaous tools',
  ObjetDuplique: 'Creation of the duplicated of an object',
  indDup: "Click on a object to create it's duplicated",
  ObjetClone: 'Create a clone of an objet',
  indClone: 'Click sur un point, a segment, a line, a ray or a circle',
  NoObjNum: 'No numerical objetc has been created',
  MesProSca: 'Scalar produc measure',
  indPtOuDte: 'Click on a point or a line',
  Rap1: 'Click on a point to center the protractor',
  Rap2: 'Click on a point to align the zero of the protractor',
  Rap3: 'Move the mouse and click when the desired angle is displayed',
  Gom: 'Click on an object to get it masked',
  Rid: 'Click on a blinking object to get it unmasked',
  indSup: 'click on an object to get it deleted',
  /// ////////////////////
  LongUnit: 'Unity length creation',
  AvertLu1: 'Length ',
  AvertLu2: ' is now unity of the figure',
  DispMeas: 'Automatic display of lengths and angles measures',
  /// ////////////////////
  Intitule: 'Title : ',
  ClicSuiv: 'Click for newt object : ',
  MacApp: 'Unmasking objects macro',
  indMacApp: 'Click on the objects the macro will unmask. STOP to end',
  indMacDisp: 'Click on the objects the macro will mask. STOP to end',
  MacDisp: 'Masking objects macro',
  MacAnim: 'Animation macro',
  MacAnimTr: 'Animation macro with trace',
  UnCyc: 'Only one cycle',
  RetFinCyc: 'Point return when animation ends',
  EffFinCyc: 'Traces erasing at the end of each cycle',
  StopClic: 'Animation stopped by mouse click',
  DureeDon: 'Duration of animation (en 1/10s, 0 à 3000) : ',
  Info: 'Information',
  indAnimTr: 'Click on the objects that will generate a trace then click STOP',
  indTrAuto: 'Click on the linked point positions of which will generate the locus',
  MacTraceAuto: 'Auto trace through linked point macro',
  MacTraceAutoVa: 'Auto trace through variable macro',
  MacAffValVar: 'Macro affecting value to variable',
  AffectA: 'Give value to',
  MacClign: 'Blinking macro',
  indMacClign: 'Click on the objects to get blinking. Stop to end.',
  MacIncVar: 'Incrementing variable macro',
  MacDecVar: 'Decrementing variable macro',
  MacModifVar: 'Modifying variable macro',
  ChoixMac: 'Choose the type of macro to be created :',
  NewMac: 'New Macro',
  /// //////////////////////////////
  MacAffPtLie: 'Moving linked point',
  ErrMAfPtLie: 'Creation impossible : no other point linked to the same object is visible',
  indMacAffPtLie1: 'Click on the linked point that is to be moved',
  indMacAffPtLie2: 'Click on the linked point you want the first point to be moved to',
  MacAffPtLieDlg: 'Macro updating the figure',
  MacReaff: 'Macro updating the figure',
  MacActTr: 'Macro setting mode trace on',
  MacDesactTr: 'Macro setting mode trace off',
  MacPause: 'Macro of pause',
  Jusqclic: 'until click',
  Durs: 'Duration in seconds',
  MacDispo: 'Available macros :',
  MacAExec: 'Macros to be executed :',
  Ins: 'Insert',
  AddDeb: 'Insert to start position',
  AddFin: 'Add to end position',
  Ret: 'Withdraw',
  RetAll: 'Withdraw all',
  ErrNoMac: 'No macro chosen',
  MacSuiteMac: 'Macro sequence execution',
  MacJouantSon: 'Macro playing sound',
  Path: 'Sound file path :',
  AddMac: 'Macros creation',
  EdFor: 'Formula editor',
  AnimVis: 'Animation visible',
  MacBoucAnim: 'Macro of loop with animation',
  MacAvBou: 'Macro to be executed before the first loop :',
  MacFinBou: 'Macro to be executed at the end of each loop :',
  MacBoucTr: 'Macro of loop with trace',
  ChoixOutils: 'Tools choice',
  Droites: 'Lines',
  Points: 'Points',
  Segments: 'Segments',
  Cercles: 'Circles',
  Polys: 'Polygons',
  Marques: 'Marks',
  Lieux: 'Locuses',
  Transf: 'Transformations',
  Mes: 'Measures',
  Disp: 'Displays',
  ImageLibre: 'Image free',
  ImageLiee: 'Image linked to point',
  Calculs: 'Calculations',
  Surfaces: 'Surfaces',
  OutilsPer: 'Personalize available tools',
  /// //////////////////////////////////////////////
  /// //////////////////////////////////////////////
  ChoixOutDlg1: 'Selected items refer to :',
  ChoixOutDlg2: 'deactivated tools',
  ChoixOutDlg3: 'activated tools',
  /// //////////////////////////////////////////////
  AvecGrad: 'With graduations',
  PasCur: 'Step of cursor :',
  PasGrad: 'Step of graduation :',
  step: 'step',
  stepGrid: 'gradstep',
  ErrPas: 'Step must divide maxi value - mini value',
  /// /////////////////////////////////////////////
  TestEq: 'Equivalence test',
  AComp: 'Choose the calculations to be compared :',
  RempVal: 'Replacement of values<br>before comparison',
  EqDecFr: 'Equivalence decimal - irreductible fraction',
  CondEx: 'Existence condition',
  TestFact: 'Factorisation test',
  FacComp: 'Factorisation test of left choice by right choice :',
  TestDepVar: 'Function-Variable dependence test',
  DepRap: 'Dependence relative to :',
  TestEqNatOp: 'Operator nature test',
  SomInd: 'Indexed sum',
  ProdInd: 'Indexed product',
  ValSom: 'Value to be summed :',
  ValProd: 'Value to be multiplied :',
  VarInd: 'Index variable :',
  AffCad: 'Use a frame of given dimensions',
  Larg: 'Width',
  Haut: 'Height',
  dep: 'from',
  a: 'to',
  SaveDim: 'Save with the frame dimensions',
  ToolsBarHor: 'Horizontal Toolbar tools',
  SuiteRec2: 'Real u(n+1)=f[n,u(n)]recurrent sequence ',
  SuiteRec3: 'Real u(n+2)=f[n,u(n+1),u(n)] recurrent sequence',
  SuiteRecComplexe2: 'Complex u(n+1)=f[n,u(n)]recurrent sequence',
  SuiteRecComplexe3: 'Complex u(n+2)=f[n,u(n+1),u(n)]recurrent sequence',
  Integ: 'Integral approximated value',
  IntegDlg1: 'Calculation of an integral of f on [a; b] approximated value<br/>through Simpson method (2n intervals)',
  IntegDlg2: 'Function f',
  IntegDlg3: 'n Value : (200 max)',
  FusionImpConst: 'Merge constructions implemented in figure',
  ErrNomInd: 'This point must be named because it is used by a measure or a frame',
  grec: 'greec',
  math: 'math',
  arrows: 'arrows',
  CodeOK: 'Base64 code of the figure has been copied in clipboard',
  CopyOK: 'Figure has been copied in clipboard',
  SavePNG: 'Export in PNG file',
  SaveJPG: 'Export in JPEG file',
  SaveSVG: 'Export in SVG file',
  Export: 'Export figure',
  ToolsDem: 'Use this choice for next start',
  CodeTikz: 'Get figure LaTeX Tikz code',
  TikzDlg1: 'Use length unity of figure',
  TikzDlg2: 'Choose dimensions of figure',
  TikzDlg3: 'Length in cm pour the length unity (0.1 to 10) : ',
  TikzDlg4: 'Figure width in cm (1 to 40) : ',
  TikzDlg5: ' Commented code',
  Tikz: 'Tikz code of the figure',
  About: 'About MathGraph32',
  NivDem: 'Use chosen level for next start',
  Copy: 'Copy figure in clipboard',
  fig0: 'Triangle',
  fig1: 'Right angled triangle',
  fig2: 'Isosceles triangle',
  fig3: 'Isosceles right angled triangle',
  fig4: 'Equilateral triangle',
  fig5: 'Square',
  fig6: 'Rectangle',
  fig7: 'Parallelogram',
  fig8: 'Rhombus',
  fig9: 'Trapezium',
  fig10: 'Right angled trapezium',
  fig11: 'Isosceles trapezium',
  fig12: 'Regular hexagon',
  fig13: 'Regular pentagon',
  NewFigWith: 'New figure with',
  reporthon: 'Orthonormal 3D frame',
  reporthog: 'Orthogonal 3D frame',
  cube: 'Cube',
  boite: 'Parallelepiped rectangle',
  cyl: 'Cylinder',
  coneSimp: 'Simple cone',
  coneDb: 'Double cone',
  tetra: 'Tetrahedron',
  tetrareg: 'Regular tetrahedron',
  pyrcarre: 'Pyramid with square base',
  pyrregbasecar: 'Regular pyramid with square base',
  pyrrect: 'Pyramid with rectangular base',
  prismtriang: 'Prism with triangular base',
  prismtrap: 'Prism with trapeze base',
  surf: 'Surface of equation z=f(x,y)',
  cour: 'Functions graph',
  courParam: 'Parametric curve',
  courPol: 'Polar Curve',
  graSuiteunfn: 'un = f(n) sequence graph',
  graSuitRecPrPt: 'u(n+1)=f[u(n)] sequence graph with first term moveable',
  graSuitRecPrTer: 'u(n+1)=f[u(n)] sequence graph with first term editable',
  loiBin: 'Binomial distribution',
  loiBinIntConf: 'Binomial distribution : Confidence interval',
  loiBinApp: 'Binomial distribution : Approximation through Normal distribution',
  loiExp: 'Exponential distribution',
  loiNorm: 'Normal distribution',
  loiNormeualpha: 'Normal distribution : Calculation of u_alpha',
  loiNormInv: 'Normal distribution inverted',
  ChoixLang: 'Language choice',
  fr: 'French',
  en: 'English',
  es: 'Spanish',
  Restart: 'MathGraph32 must be restarted',
  Zoom: 'Zoom',
  Web: 'MathGraph32 Web Site',
  CouFond: 'Figure background color',
  Help: 'Help',
  ToggleToolsAdd: 'More tools',
  CopierStyle: 'Select object style',
  CopSty1: "Click on an object to select it's style",
  marAngFigees: 'Angle marks unmovable',
  affFiges: 'Displays unmoveable',
  EltsFiges: 'Fixed objects',
  ExportHTML: 'Export in HTML',
  Title: 'Page title : ',
  Aut: 'Author : ',
  TextHtm1: 'Text preceeding figure',
  TextHtm2: 'Text following figure',
  HTML: 'Dynamic MathGraph32 worksheet',
  CrWith: 'Created with',
  SavePNGWithUnity: 'Exporter in PNG file with unity',
  Unit: 'Length of unity in cm : ',
  DimFig: 'Figure dimensions : ',
  CopyWithUnity: 'Copy figure in clipboard with unity length',
  TaillePlus: 'Increase font sizes',
  TailleMoins: 'Decrease font sizes',
  ImFondVis: 'Background image visible',
  AddObjMac: 'Add objects to macro',
  SupObjMac: 'Remove objects from macro',
  AdImp: 'Adding this object to the macro is impossible : The object was defined after the macro.</br>Use object reclassification',
  indAdObj: 'Click on the objects to be added to the macro',
  indSupObj: 'Click on blinking objects to withhdraw them from the macro',
  SupImp: 'One object at least must remain',
  coefMult: 'Scale factor for export of images (0.25 to 4) : ',
  ModePointsAuto: 'Toggle auto point creation mode',
  DysDem: 'Use "dys" mode for next start',
  start: 'Starting figure :',
  start0: 'Frame with grid',
  start1: 'Frame dotted',
  start2: 'No frame',
  AngUs: 'Usual angles',
  MacAppParAut: 'Unmasking macro from another macro',
  MacDispParAut: 'Masking macro from another macro',
  MacAppDispDlg1: 'Create from objects belonging to :',
  ConstFig: 'Available constructions of the figure :',
  Implem: 'Implement',
  NomConst: 'Construction name : ',
  ModifConst: 'Modify the information of a construction',
  GestionConst: 'Manage constructions',
  SupConst: 'Delete a construction of the figure',
  SrcNumDlg1: 'Choice of numerical source objects',
  SrcNumDlg2: 'Click on each number on left list and bind it to an objec',
  SrcNumDlg3: 'Source n°',
  SrcNumDlg4: 'Associated element',
  SrcNumDlg5: 'Object required :',
  SrcNumDlg6: 'Choose a value in the left panel',
  SrcNumDlg7: 'or enter a formula :',
  SrcNumDlg8: 'Information on selected element :',
  SrcNumDlg9: 'Informations on the construction :',
  CopyCode: 'Copy code',
  CodeNotOK: 'Code could not be copied in clipboard',
  ClicOn: 'Click on ',
  ErrLong: "This construction cannot be implemented because the figure doesn't maintain a length unity",
  ImpConstFig: 'Implement a construction of the figure',
  ValR: 'A real value',
  ValC: 'A complex value',
  ImpImposs: 'Implementing this construction is impossible.</br>Following objects are missing :</br>',
  AddConst: 'Incorporate construction from file',
  SaveConst: 'Save a construction of the figure',
  ChoixSrcNumConst: 'Numerical sources objects choice',
  SrcDlg2: 'Source objects available',
  SrcDlg3: 'Chosen source objects',
  ChoixSrcGraphConst: 'Graphical sources objects choice',
  ChoixFinGraphConst: 'Graphical final objects choice',
  indSrcGr: 'Click on the sources graphical objects of the construction to be created',
  indFinGr: 'Click on the final graphical objects of the construction to be created',
  CreationConst: 'Construction creation',
  ChoixFinNumConst: 'Numerical final objects choice',
  FinDlg1: 'Final objects available :',
  ErrConst: 'Construction rejected : un of the source objects is not used',
  AvConst1: 'The sources variables chosen as final objects</br>are no longer sources objects',
  FinirConst: 'Finish current construction',
  courbeCrDlg6: 'Brackets radius in pixels',
  CourbeFoncCr: 'Function curve with bracket(s)',
  Couronne: 'Annulus',
  ReInitConst: 'Reset current construction',
  CourbeAvecTan: 'Curve through points and tangente',
  CourbeAvecTanDlg1: 'Choose the frame and the number of points<br/> the curve will pass through.',
  CourbeAvecTanDlg2: 'Next you will have to click on the points <br/>then choose the slope of the tangents.',
  CourbeAvecTanDlg3: 'Slopes of the tangeats ',
  CourbeAvecTanDlg4: 'At point :',
  indCourTan1: 'Click on point n° ',
  indCourTan2: ' of the curve',
  Term: 'Terminer',
  CourbePoly: 'Polynom curve through points',
  AffichageEq: 'Equation display of line or circle',
  indEq: 'Click on a line or a circle',
  AffichageEqLie: 'Equation display of line or circle linked to point',
  nbdec: 'nbdig',
  indAffLie: 'Click on the point the display will be linked to',
  infConst: 'Construction infos',
  NoMac: 'No macro',
  ChoixMacDem: 'Choice of a starting macro',
  MacRecDlg1: 'Recursion depth : ',
  MacRecDlg2: 'Recursion level : ',
  MacRecDlg3: 'Number of iterations by level : ',
  MacRecDlg4: 'Iteration step (>=1) : ',
  MacRecDlg5: 'Number of sources objects common for all iterations : ',
  MacRecDlg6: 'Create only last generation objects',
  MacRecDlg7: 'Impose a color for created objects ',
  MacRecDlg8: 'Polygone filled',
  MacRecDlg9: "Don't join final points",
  MacRecDlg10: 'Join final points with a polygon',
  MacRecDlg11: 'Join final points with a broken line',
  errRec: 'Invalid recursion parameters',
  errImpMax: 'This construction is not allowed : more than 200000 objets created',
  MacConsRec: 'Recursive construction macro',
  ObjSrc: 'Sources objects : ',
  MacItDlg1: 'Number of iterations (>=2) : ',
  errIter: 'Invalid iteration parameters',
  MacConsIter: 'Iterative construction macro',
  AffichageCoord: 'Coordinates display',
  AffichageCoordLie: 'Coordinates display linked to point',
  indCoord: 'Click on the point coordinates of which are to be displayed',
  ErrLongUn: "This construction cannot be implemented because the figure doesn't maintain a length unity",
  PapMil: 'Millimetered paper',
  newFig2: 'Figure with millimetered frame',
  ObjFin: 'Objet final de construction',
  AbsOrRep: 'x-coordinate of frame origin',
  OrdOrRep: 'y-coordinate of frame origin',
  UnitexRep: 'x-unity of frame',
  UniteyRep: 'y-unity of frame',
  ObjInter: 'Construction intermediary object',
  ProtoNbMax: 'Protocole restricted to the 2000 first objects',
  ch150: 'Do you want this free point source to become a final object ?',
  errInvFile: 'Invalid file',
  TextFig: 'Text of the figure',
  // Ce qui suit pour les exercices fais via la bibliothèque
  res: 'The exercise has been solved and is saved',
  notres: 'The exercise has not been solved and is saved',
  saved: 'The figure has been saved',
  none: 'No object has been constructed',
  anom: 'The following objects must be named : ',
  InsDynDlg1: 'Dynamical value insertion',
  InsDynDlg2: 'Sign + imposed',
  InsDynDlg3: 'Code to be inserted :',
  InsForDlg1: 'Formula insertion',
  InsForDlg2: 'Formula of :',
  InsForDlg3: 'Simplified formula',
  aconst: 'Object are missing of type : ',
  indRay1: 'Click on the first point setting the radius',
  indRay2: 'Click on the second point setting the radius',
  CerOAB: 'Compass',
  VersionErr: 'This file was created by a newer version of MathGraph32</n>You must update MathGraph32 to get it opened',
  MesLongOr: 'Length measure display oriented',
  proj: 'Orthogonal projectd point',
  sur: 'on',
  debArc: 'Click on the starting point of the arc',
  finArc: 'Click on a point giving the final point direction',
  chinfo5: 'Line parallel to',
  cloneCree: 'The object clone has been created',
  tag: 'Object tag',
  changeTag: 'Change tag',
  tagErr: 'This tag is already used',
  Matrice: 'Matrix',
  CalculMat: 'Matrix calculation',
  NbLig: 'Number of lines',
  NbCol: 'Number of columns',
  NomMat: 'Name of the matrix',
  infodotmult: 'dotmult(A,B) : returns the term by term product of two matrix A and B',
  infoInv: 'inv(A) : Inverse term by term of matrix A',
  MatriceAleat: 'Matrix with random integer values',
  MatAleatComp: 'Matrix with distinc values laying between ',
  MatriceParForm: 'Matrix by formula',
  formMat: 'if(i=j,1,0)',
  Det: 'Déterminant',
  FracRat: 'Approximation by rational fraction',
  infofrac: 'frac(x) : Returns a one row matrix first element of which is the numerator<br>' +
    'of the approxiamted rational fraction of x and second element is th denominator.<br>' +
    'Can also be applied to a row or line matrix'
}

module.exports = textesEnAdd
