/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

/* eslint-disable camelcase */

/**
 * Contient les textes utilisés dans le logiciel en langue Française
 * @type {Textes}
 * @private
 */
const textesFrAdd = {
  infoabs: 'abs(x) : Valeur absolue de x',
  infointgr: 'int(x): Partie entière de x',
  infosqrt: 'sqrt(x) : Racine carrée de x',
  inforac: 'rac(x) : Racine carrée de x',
  infosin: 'sin(x) : Sinus de x',
  infocos: 'cos(x) : Cosinus de x',
  infotan: 'tan(x) : Tangente de x',
  infoln: 'ln(x) : Logarithme népérien de x',
  infoexp: 'exp(x) : Exponentielle de x',
  infoarctan: 'arctan(x) : Arc tangente de x',
  infoarccos: 'arccos(x) : Arc cosinus de x',
  infoarcsin: 'arcsin(x) : Arc sinus de x',
  infoch: 'ch(x) : Cosinus hyperbolique de x',
  infosh: 'sh(x) : Sinus hyperbolique de x',
  infoth: 'th(x) : Tangente hyperbolique de x',
  infoargch: '=argch(x) : Arc cosinus hyperbolique de x',
  infoargsh: 'argsh(x) : Arc sinus hyperbolique de x',
  infoargth: 'argth(x) : Arc tangente hyperbolique de x',
  inforand: 'rand(x) : Nombre aléatoire de [0;1[',
  infoconj: 'conj(z) : Nombre conjugué de z',
  infoarg: 'arg(z) : Argument principal du complexe z',
  infore: 're(z) : Partie réelle du complexe z',
  infoim: 'im(z) : Partie imaginaire de z',
  infomaxi: 'max(x,y) : Plus grand des deux nombres x et y',
  infomini: 'min(x,y) : Plus petit des deux nombres x et y',
  infogcd: 'pgcd(x,y) : pgcd de x et y',
  infolcm: 'ppcm(x,y) : ppcm de x et y',
  infoncr: "nCr(n,p) : Nombre de combinaisons à p éléments d'un ensemble à n éléments",
  infonpr: "nPr(n,p) : Nombre de permutations à p éléments d'un ensemble à n éléments",
  infomod: 'mod(a,b) : Reste de la division euclidienne de a par b',
  infosi: 'si(test,valeur1,valeur2) : Renvoie valeur1 si test est égal à 1 et sinon valeur2',
  infointegrale: "integrale(expr,var,a,b) : Valeur approchée de l'intégrale de expr de a à b\nvar étant la variable d'intégration",
  infoprimitive: 'primitive(f,var,a,b) : Renvoie f(b) - f(a) où f est une formule de fonction \nvar étant la variable de la fonction',
  infofact: 'fact(n) : Factorielle de n (si n entier positif)',
  infosomme: 'somme(Expr, j, deb, fin, pas) : Renvoie la somme de Expr \n pour j entier variant de deb à fin par incrément de pas',
  infoproduit: 'produit(Expr, j, deb, fin, pas) : Renvoie le produit de Expr \n pour j entier variant de deb à fin par incrément de pas',
  infoLeft: 'gauche(a) : Membre gauche de a si a est un test',
  infoRight: 'droit(a) : Membre droite de a si a est un test',
  infoTransp: 'transp(M) : Renvoie la transposée de la matrice M. \nSi M est un nombre, renvoie le nombre',
  infoDeter: 'deter(M) : Renvoie le déterminant de la matrice M. \nSi M est un nombre, renvoie le nombre',
  OK: 'OK',
  Cancel: 'Annuler',
  Oui: 'Oui',
  Non: 'Non',
  Dte: 'Droite',
  Ddte: 'Demi-droite',
  Seg: 'Segment',
  Vect: 'Vecteur',
  Cerc: 'Cercle',
  Point: 'Point',
  PointLibre: 'Point libre',
  PointCons: 'Point construit',
  PointLie: 'Point lié',
  // Bary: "Barycentre", A supprimer
  PointRep: 'Point par coordonnées',
  PointLibreEnt: 'Point libre à coordonnées entières',
  PointLiePoint: 'Point lié à point',
  Macro: 'Macro',
  PointParAff: 'Point par affixe',
  Arc: 'Arc de cercle',
  Polygone: 'Polygone',
  Surf: 'Surface',
  LieuPt: 'Lieu de Points',
  LieuPtDis: 'Lieu de points discret',
  LigneBrisee: 'Ligne brisée',
  LieuObj: "Lieu d'objets",
  Image: 'Image',
  GrapheSuiteR: 'Graphe de suite réelle',
  GrapheSuiteComp: 'Graphe de suite complexe',
  Comment: 'Affichage de texte',
  ObjetDup: 'Objet dupliqué',
  DemiPlan: 'Demi-plan',
  PtIntPoly: 'Point intérieur à polygone',
  PtIntCerc: 'Point intérieur à cercle',
  EditeurFor: 'Editeur de formule',
  PtClone: 'Point clone',
  DteClone: 'Droite clone',
  DdteClone: 'Demi-droite clone',
  SegClone: 'Segment clone',
  CercClone: 'Cercle clone',
  Texte: 'Texte',
  desPoint: 'ce point',
  desDroite: 'cette droite',
  desSegment: 'ce segment',
  desVecteur: 'ce vecteur',
  desDemiDroite: 'cette demi-droite',
  desCercle: 'ce cercle',
  desArc: 'cet arc',
  desPolygone: 'ce polygone',
  desLigneBrisee: 'cette ligne',
  desSurface: 'cette surface',
  desDemiPlan: 'ce demi-plan',
  desCommentaire: 'ce commentaire',
  desAffichageValeur: 'cet affichage de valeur',
  desMacro: 'cette macro',
  desMarqueAngle: "cette marque d'angle",
  desMarqueSegment: 'cette marque de segment',
  desGrapheSuiteRec: 'ce graphe de suite récurrente',
  desGrapheSuiteRecComplexe: 'ce graphe de suite récurrente complexe',
  desLieuPoint: 'ce lieu de points',
  desLieuObjet: "ce lieu d'objets",
  desImage: 'cette image',
  desObjetDuplique: 'cet objet dupliqué',
  desAPreciser: 'à préciser',
  desIntersection: 'cette intersection',
  desLatex: 'Cet affichage LaTeX',
  desEditeur: 'Cet éditeur de formule',
  Annuler: 'Annuler',
  Refaire: 'Refaire',
  Creer: 'Créer ',
  Capt: 'Capturer',
  CerOA: 'Cercle par centre et point',
  CerOR: 'Cercle par centre et rayon',
  ArcPetit: 'Petit arc de cercle',
  ArcGrand: 'Grand arc de cercle',
  ArcDirect: 'Arc de cercle direct',
  ArcIndirect: 'Arc de cercle indirect',
  ArcPetitParAng: 'Petit arc de cercle par angle',
  ArcGrandParAng: 'Grand arc de cercle par angle',
  ArcDirectParAng: 'Arc de cercle direct par angle',
  ArcIndirectParAng: 'Arc de cercle indirect par angle',
  Rapporteur: 'Rapporteur',
  ArcParRapporteur: 'Petit arc par rapporteur',
  ArcGrandParRapporteur: 'Grand arc par rapporteur',
  LieuParPtLie: 'Lieu de points reliés par point lié',
  LieuParVariable: 'Lieu de points reliés par variable',
  LieuDiscretParPtLie: 'Lieu de points non reliés par point lié',
  LieuDiscretParVariable: 'Lieu de points non reliés par variable',
  LieuObjetParPtLie: "Lieu d'objet par point lié",
  LieuObjetParVariable: "Lieu d'objet par variable",
  MesAngNor: "Mesure d'angle non orienté",
  Fonc: 'Fonction réelle',
  SolutionEq: "Solution d'équation",
  Derivee: 'Calcul de dérivée',
  DemiDt: 'Demi-droite',
  DtAB: 'Droite par deux points',
  DtPar: 'Parallèle',
  DtPer: 'Perpendiculaire',
  DtBis: 'Bissectrice',
  DtMed: 'Médiatrice',
  DtHor: 'Droite horizontale',
  DtVer: 'Droite verticale',
  PtLib: 'Point libre',
  CentreGrav: 'Centre de gravité',
  Milieu: 'Milieu',
  MarqueSeg: 'Marque de segment',
  MarqueAng: "Marque d'angle",
  MarqueAngOr: "Marque d'angle orienté",
  MarqueAngOrSD: "Marque d'angle orienté de sens direct",
  MarqueAngOrSI: "Marque d'angle orienté de sens indirect",
  Carre: 'Carré direct',
  TriangleEq: 'Triangle équilatéral direct',
  Parallelog: 'Parallelogramme',
  PtParCoord: 'Point par coordonnées',
  PtParAff: 'Point par affixe',
  MarquerPt: 'Marquer point pour trace',
  DemarquerPt: 'Démarquer point pour trace',
  Punaiser: 'Punaiser un point mobile',
  Depunaiser: 'Dépunaiser un point punaisé',
  Rideau: 'Démasquer un objet masqué',
  Gomme: 'Masquer un objet',
  Recalculer: 'Recalculer la figure',
  TranslationFigure: 'Translater la figure',
  ExecutionMacro: 'Exécuter une macro',
  Surface: 'Surface délimitée par cercle, polygone, arc ou lieu de points',
  Surface2Lieux: 'Surface délimitée par deux lieux de points',
  SurfaceLieuDroite: 'Surface délimitée par lieu et droite',
  SurfaceLieu2Pts: 'Surface délimitée par lieu et deux points',
  New: 'Nouvelle figure',
  PtBaseEnt: 'Point libre à coordonnées entières',
  PtInterieur: 'Point intérieur à cercle ou polygone',
  DtParEq: 'Droite par équation',
  Latex: 'Affichage LaTeX',
  LatexLiePt: 'Affichage Latex lié à point',
  Commentaire: 'Affichage de texte',
  CommentaireLiePt: 'Affichage de texte lié à point',
  AffichageValeur: 'Affichage de valeur',
  AffichageValeurLiePt: 'Affichage de valeur lié à point',
  CreationLiaison: "Création d'une liaison entre un point libre et un objet",
  SuppressionLiaison: "Suppression d'une liaison point-objet",
  Rot: 'Rotation',
  Hom: 'Homothétie',
  Sim: 'Similitude',
  Sup: "Suppression d'objet graphique",
  Int: 'Intersection',
  Proj: 'Projeté orthogonal',
  Angle: 'Angle',
  Rapport: 'Rapport',
  Rayon: 'Rayon',
  SymAxiale: 'Symétrie axiale',
  SymCentrale: 'Symétrie centrale',
  Inv: 'Inversion',
  Trans: 'Translation',
  ChoixCouleur: 'Couleur prédéfinie',
  ChoixStyleTrait: 'Style de trait',
  ChoixEpaisseur: 'Epaisseur de trait',
  ChoixOpacite: 'Transparence pour les surfaces',
  ChoixStylePoint: 'Style de point',
  ChoixMarqueSeg: 'Style de marque de segment',
  ChoixStyleFleche: 'Style de flèche',
  ChoixMarqueAng: 'Style de marque d\'angle',
  ChoixStyleRemp: 'Style de remplissage pour les surfaces',
  CouleurPerso: 'Couleur personnalisée',
  Fermer: 'Fermer',
  AngleRot: 'Angle de la rotation',
  RapportHom: "Rapport de l'homothetie",
  RayCer: 'Rayon du cercle',
  AngArc: "angle de l'arc",
  Avert: 'Avertissement',
  ErrSyntaxe: 'Erreur de syntaxe',
  ErrCalculNonExist: "Attention : le résultat n'existe pas",
  DejaCree: 'Un même objet a déjà été créé',
  ChoixEntre: 'Choisissez entre',
  PremierObj: 'Le premier objet',
  DernierObj: 'Le dernier objet',
  ChoixAPrec: 'Choix à préciser',
  rotDlg: 'Angle de rotation',
  ChoixTypeDlg1: "Plusieurs types d'objets sont<br/>proches de l'endroit cliqué<br/>Choisissez entre : ",
  ChoixValeur: "Choix d'une valeur",
  ChoixValeurDlg1: 'Choisissez une valeur',
  ChoixValeurDlg2: 'ci-contre',
  ChoixValeurDlg3: 'Information sur la valeur sélectionnée :',
  ChoixVariableDlg1: 'Choisissez une variable',
  NewRepDlg: 'Nouvelle figure avec repère',
  NewRepDlg1: 'Orthonormal',
  NewRepDlg2: 'Orthogonal',
  NewRepDlg3: 'Oblique',
  NewRepDlg4: 'Oblique normé',
  NewRepDlg5: 'Quadrillage vertical',
  NewRepDlg6: 'Quadrillage horizontal',
  NewRepDlg7: 'Avec pointillés',
  NewRepDlg8: 'Créer les axes si nécessaire',
  NewRepDlg12: 'Sans graduation',
  NewRepDlg13: 'Graduation simple',
  NewRepDlg14: 'Graduation pour trigonométrie',
  NewRepDlg15: 'Avec vecteurs',
  testEquivalenceDlg4: "Condition d'existence :",
  vardlg1: 'La valeur mini doit être inférieure à la valeur maxi',
  vardlg2: 'Le pas est incorrect',
  vardlg3: 'La valeur actuelle est incorrecte',
  lieuObjetDlg1: 'Nombre de positions<br/>Pour générer le lieu : <br/>(de 2 à 100000)',
  PtIntDlg1: "L'endroit cliqué n'est pas intérieur",
  DerDlg1: 'Nom de la dérivée :',
  DerDlg2: 'Dérivée de :',
  InfoFonc: 'Information sur la fonction :',
  TgteDlg1: 'Tangente à la courbe de',
  TgteDlg2: "Au point d'abscisse :",
  TgteDlg3: 'Nom du coefficient directeur',
  SolEqDlg1: "Solution approchée d'une équation sur l'intervalle [a,b]",
  SolEqDlg2: 'Nom de la solution : ',
  SolEqDlg3: "Nom de l'iconnue",
  SolEqDlg4: 'Equation à résoudre',
  SolEqDlg5: 'Valeur de a :',
  SolEqDlg6: 'Valeur de b :',
  SolEqDlg7: 'Incertitude :',
  MaxDlg1: 'Abscisse de maximum de fonction sur [a;b]',
  MaxDlg2: 'Nom du maximum :',
  MinDlg1: 'Abscisse de minimum de fonction sur [a;b]',
  MinDlg2: 'Nom du minimum :',
  courbeDlg1: 'Courbe à tracer sur :',
  courbeDlg2: ']-&infin;;+&infin;[',
  courbeDlg3: '][a;b][',
  courbeDlg4: '][a;+&infin;[',
  courbeDlg5: ']-&infin;;a][',
  courbeDlg7: 'Point lié générateur caché',
  courbeDlg8: 'Point générateur de la courbe caché',
  courbeDlg9: 'Gestion auto des discontinuités',
  courbeDlg10: 'Nombre de points de la courbe :',
  suiteRecDlg1: 'Nom de la suite :',
  suiteRecDlg2: 'Fonction associée',
  graphesrDlg1: 'Traits de rappels en abscisse',
  graphesrDlg2: 'Points joints par des segments',
  graphesrDlg3: 'Suite associée',
  saveDlg1: 'Entrez le nom<br/>du fichier (sans suffixe) : ',
  NomCalcul: 'Nom du calcul',
  NomFonction: 'Nom de la fonction',
  Formule: 'Formule',
  Calcul: 'Calcul',
  CalculComp: 'Calcul complexe',
  CalculReel: 'Calcul réel',
  Variable: 'Variable',
  AffVal: 'Affichage de valeur',
  NomVar: 'Nom de la variable',
  ValMin: 'Valeur Mini :',
  ValMax: 'Valeur maxi :',
  ValPas: 'Valeur du pas :',
  ValAct: 'Valeur actuelle :',
  DlgAss: 'Dialogue associé :',
  Fonction: 'Fonction',
  FoncComp: 'Fonction complexe',
  TypeAPrec: 'Type à préciser',
  NomInvalide: "Le nom n'est pas valide",
  ExisteMemeNom: 'Ce nom a déjà été attribué',
  ch6: "Le ou les points d'intersection existent déjà",
  ch18: "N'existe pas",
  ch34: 'Longueur unité de la figure',
  ch87: 'définie par',
  ch109: 'Objets non sécants',
  ch112: 'Dérivée de la fonction',
  ch185: "L'équation doit dépendre de l'inconnue",
  ch186: "Test d'équivalence des calculs",
  ch187: 'Avec',
  ch188: 'Sans',
  ch189: 'Remplacement des valeurs',
  ch190: 'dérivee partielle de',
  ch191: 'par rapport à la variable',
  ch192: 'Test de dépendance de la fonction',
  ch196: "Test d'équivalence de nature d'opérateur",
  ch200: 'Test de factorisation de',
  chinfo17: 'Mesure de la longueur',
  chinfo29: "Mesure de l'angle géométrique",
  chinfo30: "Mesure de l'angle orienté",
  chinfo31: 'Abscisse de',
  chinfo32: 'dans le repère',
  chinfo71: 'Aire',
  chinfo72: 'du polygone',
  chinfo73: 'Longueur',
  chinfo74: 'de la ligne',
  chinfo77: 'Valeur mini :',
  chinfo78: 'Valeur maxi :',
  chinfo79: 'Valeur du pas :',
  chinfo80: 'Valeur actuelle :',
  chinfo81: 'Ordonnée de',
  chinfo84: 'associée à la fonction',
  chinfo86: 'Nombre de termes :',
  chinfo93: "Valeur approchée d'intégrale",
  chinfo94: 'de la fonction',
  chinfo95: 'Borne inférieure :',
  chinfo96: 'Borne supérieure :',
  chinfo97: 'Valeur actuelle : ',
  chinfo98: 'Solution approchée',
  chinfo99: "de l'équation",
  chinfo100: 'comprise entre',
  et: 'et',
  chinfo102: 'Précision :',
  chinfo107: 'du complexe',
  chinfo108: 'Affixe complexe',
  chinfo109: 'du point',
  chinfo110: 'mesurée dans le repère',
  chinfo112: 'Module',
  chinfo115: 'de la valeur de',
  chinfo116: 'indicée par les valeurs de',
  chinfo117: 'indice mini :',
  chinfo118: 'indice maxi :',
  chinfo153: 'Mesure du coefficient directeur de',
  chinfo157: 'Produit scalaire du vecteur',
  chinfo158: 'et du vecteur',
  chinfo160: 'Produit indicé',
  chinfo164: "Point d'affixe",
  chinfo176: 'Abscisse de maximum',
  chinfo177: 'Abscisse de minimum',
  chinfo178: "Abscisse à l'origine",
  chinfo179: "Ordonnée à l'origine",
  chinfo186: 'Deuxième terme :',
  chinfo180: "Unité sur l'axe des abscisses",
  chinfo181: "Unité sur l'axe des ordonnées",
  Valeurs: 'Valeurs',
  Fonctions: 'Fonctions',
  avRapNul: 'Le rapport doit être non nul',
  avRay: 'Le rayon doit être strictement positif',
  ch37: 'Un ou plusieurs objet dépendant de cet objet seront aussi supprimés. Confirmez-vous la suppression ?',
  ch38: "Supprimer cet objet détruira l'intersection de deux objets. Confirmez-vous la suppression ?",
  ModifObjGraph: "Modification d'objet graphique",
  ChoixTrans1: "Plusieurs objets sont images par la même transformation<br/>que cet objet.<br/>Voulez-vous que la modification s'applique",
  ChoixTrans2: 'à tous les autres objets images',
  ChoixTrans3: 'à cet objet seul',
  RayPix: 'Rayon en pixels',
  ModifObj: 'Modifier objet',
  ModifObjNum: 'Modifier objet numérique',
  Palette: "Changer la couleur ou le style d'un objet",
  Nommer: 'Nommer un point ou une droite',
  CaptNom: 'Déplacer un nom',
  NouvelleFigAvecRep: 'Nouvelle figure avec repère',
  AvertDirty: "La figure actuelle n'a pas été sauvegardée<br/>Voulez-vous continuer ?",
  newFig1: 'Figure avec repère',
  newFig3: 'Figure sans repère avec longueur unité',
  newFig4: 'Figure sans repère et sans longueur unité',
  newFig5: 'Figure de base',
  newFig6: 'Figure avec repère modifiable sans vecteurs',
  newFig7: 'Figure avec repère modifiable avec vecteurs',
  newFig8: 'Figure avec graphes de fonctions ou suites',
  newFig9: 'Figure de probabilités',
  newFig10: 'Figure 3D de base',
  newFig11: 'Figure par code Base64',
  Abscisse: 'Abscisse',
  Ordonnee: 'Ordonnee',
  Affixe: 'Affixe',
  Repere: 'Repère',
  PtLie: 'Point lié',
  MesLong: 'Mesure de longueur',
  MesAngOr: "Mesure d'angle orienté",
  MesAbs: "Mesure d'abscisse",
  dans: 'dans',
  de: 'de',
  Modifier: 'Modifier',
  Supprimer: 'Supprimer',
  ObjNum: 'Objets numériques',
  TracerCourbe: 'Tracer courbe',
  nbPtsCourbe: 'Nombre de points de la courbe',
  nbPtsCourbeMsg: 'Le nombre de points doit être compris entre 4 et 5000',
  NomMesure: 'Nom de la mesure',
  OrAbs: 'Origine des abscisses : ',
  OrOrd: 'Origine des ordonnées : ',
  Unitex: 'Unité en abscisses : ',
  Unitey: 'Unité en ordonnées : ',
  PtLibreEnt: 'Point libre à coordonnées entières',
  AbsLibres: 'Abscisses libres',
  OrdLibres: 'Ordonnées libres',
  AbsMin: 'Abscisse<br/> mini',
  AbsMax: 'Abscisse<br/> maxi',
  OrdMin: 'Ordonnée<br/> mini',
  OrdMax: 'Ordonnée<br/> maxi',
  MesAbsRep: "Mesure d'abscisse dans repère",
  MesOrdRep: "Mesure d'ordonnée dans repère",
  MesAffRep: "Mesure d'affixe",
  MesCoefDir: 'Mesure de coefficient directeur',
  CoefDir: 'Coefficient directeur',
  DtParCoef: 'Droite par coefficient directeur',
  NewFig: 'Nouvelle figure',
  MesLongLigne: "Mesure de la longueur d'une ligne",
  MesAire: "Mesure d'aire de polygone",
  EqDroite: 'Equation de la droite',
  ch41: 'Un polygone doit avoir au moins 3 sommets',
  ch42: 'Une ligne brisée doit avoir au moins 3 sommets',
  UniteAngFig: "<b>Unité d'angle de la figure</b>",
  Degre: 'Degré',
  Radian: 'Radian',
  Nom: 'Nom',
  NomMasque: 'Nom masqué',
  EntreeNom: 'Entrée du nom',
  ZoomPlus: 'Zoomer',
  ZoomMoins: 'Dézoomer',
  ModeTrace: 'Basculer le mode trace',
  Valeura: 'Valeur de a',
  Valeurb: 'Valeur de b',
  TransParCoord: 'Translation de coordonnées (a,b)',
  PtPunaise: 'Le point a été punaisé',
  PtDepunaise: 'Le point a été dépunaisé',
  PtMarque: 'Le point a été marqué pour la trace',
  PtDemarque: 'Le point a été démarqué pour la trace',
  NbPts: 'Nombre de points',
  LieuFerme: 'Lieu fermé',
  Epaisseur: 'Epaisseur',
  Align: 'Alignement',
  Taille: 'Taille',
  EffFond: 'Effacement du fond',
  EncSans: 'Sans encadrement',
  EncSimple: 'Encadrement simple',
  Enc3D: 'Encadrement 3D',
  CoulFond: 'Couleur<br/>de fond',
  ModeAff: "Mode d'affichage",
  Valeur: 'Valeur',
  EnTete: 'En-tête :',
  SuiviDe: 'Suivi de :',
  Decimales: 'Décimales',
  AffIci: 'Affichage Ici',
  AffLatex: 'Affichage LaTeX',
  CodeLatex: 'Code LaTeX',
  Apercu: 'Aperçu',
  InsVal: 'Insertion de valeur',
  TexteAAff: 'Texte à afficher',
  Vide: 'Le texte est vide',
  OpImp: 'Opération impossible',
  SupLiaisonImp1: 'La liaison ne peut pas être supprimée car ce point lié est un objet final de construction',
  SupLiaisonImp2: "La liaison ne peut pas être supprimée car les positions de ce point lié sont utilisées pas un lieu de point, un lieu d'objet ou une macro",
  PtParAbs: 'Point par abscisse relative à deux points',
  SuppressionLiaisonAff: 'Suppression de laison affichage-point',
  CreationLiaisonAff: 'Creation de liaison affichage-point',
  indCapt: "Capturez un point mobile, une marque d'angle ou un affichage",
  indPtLib: 'Cliquez pour créer un point libre',
  ind2Pt1: 'Cliquez sur le premier point',
  ind2Pt2: 'Cliquez sur le deuxième point',
  ind3Pt3: 'Cliquez sur le troisième point',
  indInt: "Cliquez à l'intersection de deux objets ou sur deux objets l'un après l'autre",
  indPtBaseEnt: 'Cliquez pour créer un point libre à coordonnées entières',
  indProj1: 'Cliquez sur la droite sur laquelle projeter (ou segment ou demi-droite)',
  indProj2: 'Cliquez sur le point à projeter',
  indPtInt1: 'Cliquez sur un polygone ou un cercle',
  indPtInt2: "Cliquez à l'intérieur de la surface jaune",
  indCaptNom: 'Faites glisser un nom de point ou de droite',
  intPtParAbs1: "Cliquez sur le point d'abscisse 0",
  intPtParAbs2: "Cliquez sur le point d'abscisse 1",
  indCrLiaison1: 'Cliquez sur un point libre pour le lier à un objet',
  indCrLiaison2: "Cliquez sur l'objet auquel lier le point",
  indSupLiaison: 'Cliquez sur un point lié pour le transformer en point libre',
  indPun: 'Cliquez sur un point pour le punaiser',
  indDepun: 'Cliquez sur un point pour le dépunaiser',
  indMarq: "Cliquer sur un point pour qu'il laisse une trace",
  indDemarq: "Cliquer sur un point pour qu'il ne laisse plus de trace",
  indParDtePt1: 'Cliquez sur une droite, un segment\nou une demi-droite (ou un point)',
  indPt: 'Cliquez sur un point',
  indDte: 'Cliquez sur une droite',
  indSeg: 'Cliquez sur un segment',
  indPoly: 'Cliquez sur un polygone',
  indPolOuLig: 'Cliquez sur un polygone ou une ligne brisée',
  indSurface: 'Cliquez sur un cercle, un polygone, un arc de cercle ou un lieu de points',
  indTrans1: "Cliquez sur l'origine du vecteur ou sur le vecteur de la translation",
  indTrans2: "Cliquez sur l'extrémité du vecteur de la translation",
  indImTrans: 'Cliquez sur un objet pour obtenir son image',
  indCrPoly: 'Cliquez sur les sommets du polygone puis cliquez sur le bouton\nSTOP à droite ou sur le premier point pour fermer le polygone',
  indCrLigne: 'Cliquez sur les sommets de la ligne puis cliquez sur le bouton STOP à droite',
  indCentre: 'Cliquez sur le centre',
  indPtCer: 'Cliquez sur un point du cercle',
  indLieu1: 'Cliquez sur le point dont la trace génère le lieu',
  indLieu2: 'Cliquez sur le point lié dont les positions génèrent le lieu',
  indPtLie: 'Cliquez sur un objet pour créer un point lié à ce objet',
  indCentSym: 'Cliquez sur le centre de la symétrie',
  indAxeSym: "Cliquez sur l'axe de la symétrie",
  indCentRot: 'Cliquez sur le centre de la rotation',
  indCentHom: "Cliquez sur le centre de l'homothétie",
  indCentSim: 'Cliquez sur le centre de la similitude',
  indLieuObj1: "Cliquez sur l'objet dont la trace génère le lieu",
  indEndroitAff: "Cliquer à l'endroit d'affichage",
  indCrLiaisonAff1: 'Cliquez sur un affichage pour le lier à un point',
  indCrLiaisonAff2: "Cliquez sur le point auquel lier l'affichage",
  indSupLiaisonAff: 'Cliquez sur un affichage lié pour le transformer en affichage libre',
  indSurf2Lieux1: 'Cliquez sur le premier lieu de points',
  indSurf2Lieux2: 'Cliquez sur le deuxième lieu de points',
  indSurfLieuDte1: 'Cliquez sur le premier bord (lieu de points)',
  indSurfLieuDte2: 'Cliquez sur le deuxième bord (droite)',
  indSurfLieu2pts2: 'Cliquez sur le premier point',
  indSurfLieu2pts3: 'Cliquez sur le deuxième point',
  indDemiPl1: 'Cliquez sur le bord du demi plan (droite)',
  indDemiPl2: 'Cliquez sur un point du demi plan',
  solutionEq: "Solution approchée d'équation",
  Der: 'Fonction dérivée',
  Max: 'Maximum de fonction',
  Min: 'Minimum de fonction',
  InfoSuit: 'Information sur la suite :',
  CourbeFonc: 'Courbe de fonction',
  SuiteRec: 'Suite récurrente réelle u(n+1)=f[u(n)]',
  SuiteRecComplexe: 'Suite récurrente complexe z(n+1)=f[z(n)]',
  Lieuhf: 'Le lieu ne semble pas visible<br/>Confirmez-vous sa création ?',
  Curseur: 'Curseur',
  CurHor: 'Curseur horizontal',
  CurVer: 'Curseur vertical',
  CurSmall: 'Petite taille',
  CurBig: 'Grande taille',
  CurEnt: 'Curseur avec pas',
  CurNom: 'Nom du calcul représentant le curseur :',
  CurPun: 'Punaiser les extrémités',
  indCurseur1: 'Cliquez sur un point ou à un endroit de la figure',
  ErrMinMax: 'La valeur mini doit être inférieure à la valeur maxi',
  Tangente: 'Tangente à courbe de fonction',
  Add: 'Outils supplémentaires',
  ChoixOutil: 'Choisissez un outil',
  Fonc2Var: 'Fonction réelle de deux variables',
  Fonc3Var: 'Fonction réelle de trois variables',
  FoncComp2Var: 'Fonction complexe de deux variables',
  FoncComp3Var: 'Fonction complexe de trois variables',
  PremTerm: 'Premier terme :',
  NbTerm: 'Nombre de termes<br>(de 2 à 100000) :',
  GrapheSuiteRec: 'Graphe de suite réelle u(n+1)=f[u(n)]',
  GrapheSuiteRecComp: 'Graphe de suite complexe z(n+1)=f[z(n)]',
  OptionsFig: 'Options',
  CodeBase64: 'Obtenir le code base64 de la figure',
  Base64: 'Code Base64 de la figure :',
  FigParCode: 'Collez ici le code Base64 de la figure',
  Base64Err: "Le code Base64 n'est pas valide",
  Open: 'Ouvrir fichier',
  Save: 'Enregistrer',
  NomFichierErr: "Le nom du fichier n'est pas correct",
  FichierErr: 'Pas de fichier valide choisi',
  Parcourir: 'Parcourir',
  /// ////////////////////////////////////////////////////
  PartieReelle: 'Partie réelle',
  PartieImaginaire: 'Partie imaginaire',
  Module: 'Module',
  Argument: 'Argument principal',
  TestExistence: "Test d'existence",
  chinfo92: 'de la valeur',
  Coef: 'Coefficient :',
  Barycentre: 'Barycentre',
  indBar1: 'Cliquez sur un point pour le pondérer, clic sur Stop pour créer le barycentre',
  ErrBar: 'Création impossible. La somme des coefficients est nulle',
  HorsFen: "L'objet créé est en-dehors de la fenêtre.<br/>Confirmez-vous sa création ?",
  unAng: "Unité d'angle : ",
  niv0: 'Niveau élémentaire',
  niv1: 'Niveau collège',
  niv2: 'Niveau avancé sans nombres complexes',
  niv3: 'Niveau avancé avec nombres complexes',
  SegmentParLong: 'Segment de longueur donnée',
  LongSeg: 'Longueur du segment :',
  long: 'long',
  PolygoneReg: 'Polygone régulier',
  NbSommets: 'Nombre de sommets : <br/> (de 3 à 100)',
  indPolReg: 'Cliquez sur le premier sommet',
  /// ///
  Animation: "Animation d'un point lié",
  indAnim: "Cliquez sur le point clignotant à l'animer",
  AnimCycl: 'Animation cyclique',
  Freq: 'Fréquence (en s)',
  NbPtsPourAnim: 'Nombre de points<br/>(de 2 à 10000): ',
  InvSens: 'Inverser le sens',
  OptionsAnim: "Options d'animation",
  /// //
  PtParMultVec: 'Point M tel que Vect(AM) = kVect(u)',
  PtParMultVecDlg2: 'Valeur de k',
  indVect: 'Cliquez sur un vecteur',
  indVect1: 'Cliquez sur le premier vecteur',
  indVect2: 'Cliquez sur le deuxième vecteur',
  PtParSommeVec: 'Point M tel que Vect(AM) = Vect(u) + Vect(v)',
  DeriveePart: 'Dérivée partielle',
  DerPartDlg1: 'Nom de la dérivée :',
  DerPartDlg2: 'Dérivée partielle par rapport à :',
  /// //
  indImage: 'Cliquez pour créer une image',
  ChoixImFond: "Choix de l'image de fond",
  DelImFond: "Suppression de l'image de fond",
  AvertNonAnnul: 'Cette action ne peut pas être annulée.<br/>Confirmez-vous ?',
  EditeurFormule: 'Editeur de formule',
  EditForDlg1: 'Modifie la formule de :',
  EditForDlg2: '(vide pour en-tête par défaut)',
  EditForDlg3: 'Signes de multiplication implicite<br/>(variables à un caractère)',
  EditForDlg4: 'Affichage de la formule en LaTeX',
  EditForDlg5: 'Pré-code LaTeX',
  EditForDlg6: 'Affichage initial de la formule',
  EditForDlg7: 'Affichage temps réel de la formule',
  Colonnes: 'Colonnes',
  RenCalcDlg: 'Entrez ci-dessous le nouveau nom<br/>puis cliquez sur le bouton Renommer',
  RenommerCalcul: 'Renommer un calcul',
  Renommer: 'Renommer',
  AvertRecDeb: "L'objet a été reclassé vers le début",
  AvertRecFin: "L'objet a été reclassé vers la fin",
  AvertRecImp: "L'objet n'a pas pu être reclassé",
  indRecDeb: 'Cliquez sur un objet pour le reclasser vers le début',
  indRecFin: 'Cliquez sur un objet pour le reclasser vers la fin',
  ReclassDebObjGra: 'Reclassement vers le début',
  ReclassFinObjGra: 'Reclassement vers la fin',
  /// /
  NbGradx: "Nombre de graduation<br/> sur l'axe (xx') :",
  NbGrady: "Nombre de graduation<br/> sur l'axe (yy') :",
  GraduationAxes: "Graduation des axes d'un repère",
  /// / Ajouts pour historique
  chinfo202: 'de centre',
  chinfo203: 'et origine',
  chinfo204: 'finissant en',
  chinfo205: ', angle',
  chinfo191: 'Arc de cercle direct',
  chinfo192: 'Arc de cercle indirect',
  chinfo37: 'Arc image de',
  chinfo193: "Point d'intersection de",
  chinfo194: 'avec',
  chinfo227: 'autre que',
  chinfo150: 'Centre de',
  chinfo35: 'Cercle image de',
  chinfo225: "d'axe",
  chinfo195: 'Cercle de centre',
  par: 'par',
  chinfo36: 'Demi-droite image de',
  bord: 'bord',
  chinfo196: 'passant par',
  chinfo50: 'Droite horizontale',
  chinfo51: 'Droite verticale',
  chinfo34: 'Droite image de',
  chinfo152: 'Droite de coefficient directeur',
  chinfo175: "Droite d'équation",
  chinfo6: 'Droite perpendiculaire à',
  chinfo88: 'Graphe de la suite récurrente réelle',
  chinfo90: 'Graphe de la suite récurrente complexe',
  chinfo167: 'Affichage latex',
  chinfo207: 'généré par',
  chinfo126: 'Macro envoyant le point lié',
  chinfo230: 'à la position de',
  chinfo127: 'Macro affectant la valeur de',
  chinfo128: 'à la variable',
  chinfo129: 'Macro réaffichant toute la figure',
  chinfo124: "Macro d'animation de",
  chinfo119: "Macro d'apparition de ",
  chinfo130: 'Macro de boucle avec animation',
  chinfo133: 'à partir des valeurs de la variable',
  chinfo132: 'Macro de clignotement',
  MacConsItDlg1: 'Macro de construction iterative',
  MacConsRecDlg1: 'Macro de construction récursive',
  chinfo122: 'Macro de décrémentation',
  chinfo122Bis: 'de la variable',
  chinfo120: 'Macro masquant ',
  chinfo121: "Macro d'incrémentation",
  chinfo123: 'Macro jouant le fichier sonore',
  chinfo170: 'Macro activant le mode trace',
  chinfo171: 'Macro désactivant le mode trace',
  chinfo134: "Macro de pause d'une durée de",
  chinfo135: 'Macro exécutant les macros',
  chinfo136: "Macro de trace automatique d'objets par le point lié",
  chinfo137: "Macro de trace automatique d'objets par les valeurs de la variable",
  chinfo165: "Marque d'angle droit",
  chinfo66: "Marque d'angle non orientée",
  chinfo67: "Marque d'angle orientée",
  chinfo68: "Marque d'angle orientée dans le sens direct",
  chinfo69: "Marque d'angle orientée dans le sens indirect",
  chinfo233: 'Dupliqué de',
  chinfo139: 'Point libre à coordonnées entières',
  chinfo18: 'Point image de',
  chinfo169: 'Point intérieur à',
  chinfo2: 'Point lié à',
  chinfo16: "Point d'abscisse",
  chinfo217: 'Point tel que',
  chinfo218: 'vect',
  chinfo60: 'Surface délimitée par ',
  ProtoFig: 'Protocole de la figure',
  Const: 'Construction',
  chinfo138: "Début d'implémentation de la construction",
  rabsor: 'absor',
  rordor: 'ordor',
  ruxrep: 'uxrep',
  ruyrep: 'uyrep',
  rarc: 'arc',
  rcerc: 'cerc',
  rcom: 'com',
  rdd: 'dd',
  rdplan: 'dplan',
  rpt: 'Pt',
  rdt: 'dt',
  redit: 'edit',
  rgsr: 'gsr',
  rgsc: 'gsc',
  rim: 'im',
  rlat: 'lat',
  rlieu: 'lieu',
  rlieuOb: 'lieuOb',
  rmac: 'mac',
  rma: 'ma',
  rmseg: 'mseg',
  raffval: 'affval',
  ObjInterm: 'Objets intermédiaires',
  liea: 'lié à',
  chinfo197: 'et rayon',
  chinfo140: 'Point de coordonnées',
  chinfo28: ', rapport',
  ObjMas: 'Objet masqué',
  ObjMasVis: 'Objets masqués visibles',
  Protocole: 'Protocole',
  ModifBar: 'Modification de barycentre',
  /// ////////////////
  RapportInv: "Rapport de l'inversion",
  ImageInv: "Image d'un point par une inversion",
  indCentInv: "Cliquez sur le centre de l'inversion",
  indCer: 'Cliquez sur un cercle',
  CentreCercle: "Créer le centre d'un cercle",
  Divers: 'Outils divers',
  ObjetDuplique: "Créer le dupliqué d'un objet",
  indDup: 'Cliquer sur un objet pour le dupliquer',
  ObjetClone: "Créer un clone d'un objet",
  indClone: 'Cliquez sur un point, un segment, une droite, une demi-droite ou un cercle',
  NoObjNum: "Aucun objet numérique n'a été créé",
  MesProSca: 'Mesure de produit scalaire',
  indPtOuDte: 'Cliquez sur un point ou une droite',
  Rap1: 'Cliquez sur un point pour centrer le rapporteur',
  Rap2: 'Cliquez sur un point pour aligner le zéro du rapporteur',
  Rap3: "Bougez la souris puis cliquez quand l'angle est le bon",
  Gom: 'Cliquez sur un objet pour le masquer',
  Rid: 'Cliquez sur un objet clignotant pour le démasquer',
  indSup: 'cliquez sur un objet pour le supprimer',
  /// ////////////////////////////
  LongUnit: 'Créer longueur unité',
  AvertLu1: 'La longueur ',
  AvertLu2: ' est maintenant la longueur unité de la figure',
  DispMeas: 'Affichage automatique des mesures de longueur et angle',
  /// ////////////////////////////
  Intitule: 'Intitulé : ',
  ClicSuiv: 'Clic pour objet suivant : ',
  MacApp: "Macro d'apparition",
  indMacApp: 'Cliquez sur les objets que la macro doit faire apparaître. STOP pour finir',
  indMacDisp: 'Cliquez sur les objets que la macro doit faire disparaître. STOP pour finir',
  MacDisp: 'Macro de masquage',
  MacAnim: "Macro d'animation",
  MacAnimTr: "Macro d'animation avec trace",
  UnCyc: 'Un seul cycle',
  RetFinCyc: "Retour du point en fin d'animation",
  EffFinCyc: 'Effacement des traces en fin de cycle',
  StopClic: 'Animation arrêtée par un clic de souris',
  DureeDon: 'Animation de durée (en 1/10s, 0 à 3000) : ',
  Info: 'Information',
  indAnimTr: 'Cliquez sur les objets qui laisseront une trace puis STOP',
  indTrAuto: 'Cliquez sur le point clignotant dont les positions généreront la trace',
  MacTraceAuto: 'Macro de trace auto par point lié',
  MacTraceAutoVa: 'Macro de trace auto par variable',
  MacAffValVar: "Macro d'affectation de valeur à variable",
  AffectA: 'Affecter à',
  MacClign: 'Macro de clignotement',
  indMacClign: 'Cliquez sur les objets à faire clignoter, puis STOP',
  MacIncVar: "Macro d'incrémentation de variable",
  MacDecVar: 'Macro de décrémentation de variable',
  MacModifVar: 'Macro de modification de variable',
  ChoixMac: 'Choisissez le type de macro à créer :',
  NewMac: 'Nouvelle macro',
  /// //////////////////////////////
  MacAffPtLie: 'Déplacement de point lié',
  ErrMAfPtLie: "Creation impossible : Aucun autre point lié au même objet n'est visible",
  indMacAffPtLie1: 'Cliquez sur le point lié à déplacer',
  indMacAffPtLie2: 'Cliquez sur le point lié vers lequel le point doit être déplacé',
  MacReaff: 'Macro redessinant la figure',
  MacActTr: 'Macro activant le mode trace',
  MacDesactTr: 'Macro désactivant le mode trace',
  MacPause: 'Macro de pause',
  Jusqclic: 'Jusque clic',
  Durs: 'Durée en secondes',
  MacDispo: 'Macros disponibles :',
  MacAExec: 'Macros à exécuter :',
  Ins: 'Insérer',
  AddDeb: 'Insérer au début',
  AddFin: 'Ajouter à la fin',
  Ret: 'Retirer',
  RetAll: 'Retirer tout',
  ErrNoMac: 'Aucune macro choisie',
  MacSuiteMac: 'Macro exécutant une suite de macros',
  MacJouantSon: 'Macro jouant un fichier sonore',
  Path: "Chemin d'accès du fichier sonore :",
  AddMac: 'Création de macros',
  EdFor: 'Editeur de formule',
  AnimVis: 'Animation visible',
  MacBoucAnim: 'Macro de boucle avec animation',
  MacAvBou: 'Macro à exécuter avant la première boucle :',
  MacFinBou: 'Macro à exécuter à la fin de chaque boucle :',
  MacBoucTr: 'Macro de boucle avec traces',
  ChoixOutils: 'Choix des outils',
  Points: 'Points',
  Droites: 'Droites',
  Segments: 'Segments',
  Cercles: 'Cercles',
  Polys: 'Polygones',
  Marques: 'Marques',
  Lieux: 'Lieux',
  Transf: 'Transformations',
  Mes: 'Mesures',
  Disp: 'Affichages',
  ImageLibre: 'Image libre',
  ImageLiee: 'Image liée à point',
  Calculs: 'Calculs',
  Surfaces: 'Surfaces',
  OutilsPer: 'Personnaliser les outils disponibles',
  /// //////////////////////////////////////////////
  ChoixOutDlg1: 'Le items cochés correspondent :',
  ChoixOutDlg2: 'Aux outils permis',
  ChoixOutDlg3: 'Aux outils interdits',
  /// /////////////////////////////////////////////
  AvecGrad: 'Avec graduations',
  PasCur: 'Pas du curseur :',
  PasGrad: 'Pas de graduation :',
  step: 'pas',
  stepGrid: 'pasgrad',
  ErrPas: 'Le pas doit diviser la différence valeur maxi - valeur mini',
  ErrPasGrad: 'Le pas de graduation doit diviser la différence valeur maxi- valeur mini',
  /// /////////////////////////////////////////////
  TestEq: "Test d'équivalence",
  AComp: 'Choisissez les calculs à comparer :',
  RempVal: 'Remplacement des valeurs<br>avant la comparaison',
  EqDecFr: 'Equivalence décimal - fraction irréductible',
  CondEx: "Condition d'existence",
  TestFact: 'Test de factorisation',
  FacComp: 'Teste la factorisation du choix de gauche par celui de droite :',
  TestDepVar: 'Test de dépendance fonction - variable',
  DepRap: 'Dépendance par rapport à :',
  TestEqNatOp: "Test d'équivalence de nature d'opérateur",
  SomInd: 'Somme indicée',
  ProdInd: 'Produit indicé',
  ValSom: 'Valeur à sommer',
  ValProd: 'Valeur à multiplier',
  VarInd: 'Variable indice',
  AffCad: 'Utiliser un cadre de dimensions données',
  Larg: 'Largeur',
  Haut: 'Hauteur',
  dep: 'de',
  a: 'à',
  SaveDim: 'Enregister avec les dimensions du cadre',
  ToolsBarHor: 'Outils de la barre horizontale',
  SuiteRec2: 'Suite récurrente réelle u(n+1)=f[n,u(n)]',
  SuiteRec3: 'Suite récurrente réelle u(n+2)=f[n,u(n+1),u(n)]',
  SuiteRecComplexe2: 'Suite récurrente complexe u(n+1)=f[n,u(n)]',
  SuiteRecComplexe3: 'Suite récurrente complexe u(n+2)=f[n,u(n+1),u(n)]',
  Integ: "Calcul approché d'intégrale",
  IntegDlg1: "Calcul d'une valeur approchée d'intégrale de f sur [a; b]<br/>par la méthode de Simpson (2n intervalles)",
  IntegDlg2: 'Fonction f',
  IntegDlg3: 'Valeur de n : (200 max)',
  FusionImpConst: 'Fusionner les constructions implémentées de la figure',
  ErrNomInd: 'Ce point doit être nommé car il est utilisé pour une mesure ou un repère',
  grec: 'grec',
  math: 'math',
  arrows: 'flèches',
  CodeOK: 'Le code de la figure a été copié dans le presse-papier',
  CopyOK: 'La figure a été copiée dans le presse-papier',
  SavePNG: 'Exporter dans un fichier PNG',
  SaveJPG: 'Exporter dans un fichier JPEG',
  SaveSVG: 'Exporter dans un fichier SVG',
  Export: 'Exporter la figure',
  ToolsDem: "Utiliser ce choix d'outil pour le prochain démarrage du logiciel",
  CodeTikz: 'Obtenir le code LaTeX Tikz de la figure',
  TikzDlg1: 'Utiliser la longueur unité de la figure',
  TikzDlg2: 'Imposer les dimensions de la figure',
  TikzDlg3: 'Longueur en cm pour la longueur unité (0.1 à 10) : ',
  TikzDlg4: 'Largeur en cm de la figure (1 à 40) : ',
  TikzDlg5: 'Commenter le code',
  Tikz: 'Code Tikz de la figure',
  About: 'A propos de MathGraph32',
  NivDem: 'Utiliser le niveau choisi pour le prochain démarrage du logiciel',
  Copy: 'Copier la figure dans le presse-papier',
  fig0: 'Triangle quelconque',
  fig1: 'Triangle rectangle',
  fig2: 'Triangle isocèle',
  fig3: 'Triangle rectangle isocèle',
  fig4: 'Triangle équilatéral',
  fig5: 'Carré',
  fig6: 'Rectangle',
  fig7: 'Parallélogramme',
  fig8: 'Losange',
  fig9: 'Trapèze',
  fig10: 'Trapèze rectangle',
  fig11: 'Trapèze isocèle',
  fig12: 'Hexagone régulier',
  fig13: 'Pentagone régulier',
  NewFigWith: 'Nouvelle figure avec',
  reporthon: 'Repère 3D Orthonormal',
  reporthog: 'Repère 3D Orthogonal',
  cube: 'Cube',
  boite: 'Parallélépipède rectangle',
  cyl: 'Cylindre droit',
  coneSimp: 'Cône simple',
  coneDb: 'Cône double',
  tetra: 'Tétraèdre',
  tetrareg: 'Tétraèdre régulier',
  pyrcarre: 'Pyramide à base carrée',
  pyrregbasecar: 'Pyramide régulière à base carrée',
  pyrrect: 'Pyramide à base rectangulaire',
  prismtriang: 'Prisme à base triangulaire',
  prismtrap: 'Prisme à base trapézoïdale',
  surf: "Surface d'équation z=f(x,y)",
  cour: 'Courbes de fonctions',
  courParam: 'Courbe paramétrique',
  courPol: 'Courbe polaire',
  graSuiteunfn: 'Graphe de suite un = f(n)',
  graSuitRecPrPt: 'Graphe de suite u(n+1)=f[u(n)] avec premier terme capturable',
  graSuitRecPrTer: 'Graphe de suite u(n+1)=f[u(n)] avec édition du premier terme',
  loiBin: 'Loi binomiale',
  loiBinIntConf: 'Loi binomiale : Intervalle de confiance au seuil de 95%',
  loiBinApp: 'Loi binomiale : Approximation par loi normale',
  loiExp: 'Loi exponentielle',
  loiNorm: 'Loi normale',
  loiNormeualpha: 'Loi normale : Calcul de u_alpha',
  loiNormInv: 'Loi normale inverse',
  ChoixLang: 'Choix de la langue',
  fr: 'Français',
  en: 'Anglais',
  es: 'Espagnol',
  Restart: 'MathGraph32 doit être redémarré',
  Zoom: 'Zoom',
  Web: 'Le site de MathGraph32',
  CouFond: 'Couleur de fond de la figure',
  Help: 'Aide',
  ToggleToolsAdd: 'Autres outils',
  CopierStyle: "Sélectionner le style d'un objet",
  CopSty1: 'Cliquer sur un objet pour sélectionner son style',
  marAngFigees: "Marques d'angles figées",
  affFiges: 'Affichages figés',
  EltsFiges: 'Elements figés',
  ExportHTML: 'Exporter en HTML',
  Title: 'Titre de la page :',
  Aut: 'Auteur : ',
  TextHtm1: 'Texte précédent la figure',
  TextHtm2: 'Texte suivant la figure',
  HTML: 'Feuille de travail dynamique MathGraph32',
  CrWith: 'Cr&#233&#233 avec',
  SavePNGWithUnity: 'Exporter dans un fichier PNG avec unité',
  Unit: "Longueur de l'unité en cm : ",
  DimFig: 'Dimensions de la figure : ',
  CopyWithUnity: 'Copier la figure dans le presse-papier avec longueur unité',
  TaillePlus: 'Augmenter taille des polices',
  TailleMoins: 'Diminuer taille des polices',
  ImFondVis: 'Image de fond visible',
  AddObjMac: "Ajout d'objets à une macro",
  SupObjMac: "Retrait d'objets d'une macro",
  AdImp: "Ajout à la macro impossible : l'objet cliqué a été défini après la macro.</br>Utilisez le reclassement d'objets",
  indAdObj: 'Cliquez sur des objets pour les rajouter à la macro',
  indSupObj: 'Cliquez sur des objets clignotants pour les retirer de la macro',
  SupImp: 'Il doit rester au moins un objet',
  coefMult: "Coefficient mutiplicateur pour l'export des images (0.25 à 4) : ",
  ModePointsAuto: 'Basculer le mode de création de points auto',
  DysDem: 'Utiliser le mode "dys" au prochain démarrage',
  start: 'Figure de démarrage :',
  start0: 'Repère quadrillé',
  start1: 'Repère pointillé',
  start2: 'Sans repère',
  AngUs: 'Angles usuels',
  MacAppParAut: "Macro d'apparition à partir d'une autre macro",
  MacDispParAut: "Macro de masquage à partir d'une autre macro",
  MacAppDispDlg1: 'Créer à partir des objets de :',
  ConstFig: 'Constructions de la figure disponibles :',
  Implem: 'Implémenter',
  NomConst: 'Nom de la construction : ',
  ModifConst: "Modification des infos d'une construction",
  GestionConst: 'Gérer les constructions',
  SupConst: 'Supprimer une construction de la figure',
  SrcNumDlg1: 'Choix des objets sources numeriques',
  SrcNumDlg2: 'Cliquez sur chaque numéro de gauche et associez-lui un objet',
  SrcNumDlg3: 'Source n°',
  SrcNumDlg4: 'Elément associé',
  SrcNumDlg5: 'Objet requis :',
  SrcNumDlg6: 'Choisir une valeur ci-contre',
  SrcNumDlg7: 'ou entrer ci-dessous une formule',
  SrcNumDlg8: "Information sur l'élément sélectionné :",
  SrcNumDlg9: 'Informations sur la construction :',
  CopyCode: 'Copier le code',
  CodeNotOK: "Le code n'a pas pu être copié dans le presse-papier",
  ClicOn: 'Cliquez sur ',
  ErrLong: 'Cette construction ne peut pas être implémentée car la figure ne contient pas de longueur unité',
  ImpConstFig: 'Implémenter une construction de la figure',
  ValR: 'Une valeur réelle',
  ValC: 'Une valeur complexe',
  ImpImposs: "L'implémentationde cette construction est impossible.</br>Il manque les objets suivants :</br>",
  AddConst: 'Incorporer une construction depuis un fichier',
  SaveConst: 'Enregistrer une construction de la figure',
  ChoixSrcNumConst: 'Choix des objets sources numériques',
  SrcDlg2: 'Eléments sources disponibles',
  SrcDlg3: 'Eléments sources choisis',
  ChoixSrcGraphConst: 'Choix des éléments sources graphiques',
  ChoixFinGraphConst: 'Choix des éléments finaux graphiques',
  indSrcGr: 'Cliquez sur les objets sources graphiques de la construction à créer',
  indFinGr: 'Cliquez sur les objets finaux graphiques de la construction à créer',
  CreationConst: "Création d'une construction",
  ChoixFinNumConst: 'Choix des objets finaux numériques',
  FinDlg1: 'Eléments finaux disponibles :',
  ErrConst: "Construction refusée : un des objets sources n'est pas utilisé",
  AvConst1: 'Les variables sources choisies comme objet final</br>ont été retirées des objets sources',
  FinirConst: 'Finir la construction en cours',
  courbeCrDlg6: 'Rayon des crochets en pixels',
  CourbeFoncCr: 'Courbe de fonctions avec crochet(s)',
  indCour: 'Cliquez sur un cercle, un polygone ou un lieu de points fermé',
  Couronne: 'Couronne',
  ReInitConst: 'Réinitialiser la construction en cours',
  CourbeAvecTan: 'Courbe par points et tangentes',
  CourbeAvecTanDlg1: 'Choisissez le repère et le nombre de points<br/> par lesquels la courbe doit passer.',
  CourbeAvecTanDlg2: 'Vous devrez ensuite cliquer sur les points puis choisir<br/>les coefficients directeurs des tangentes.',
  CourbeAvecTanDlg3: 'Coefficients directeurs des tangentes',
  CourbeAvecTanDlg4: 'Au point :',
  indCourTan1: 'Cliquez sur le point n° ',
  indCourTan2: ' de la courbe',
  Term: 'Terminer',
  CourbePoly: 'Courbe de fonction polynôme par points',
  AffichageEq: "Affichage d'équation de droite ou cercle",
  indEq: 'Cliquez sur une droite ou un cercle',
  AffichageEqLie: "Affichage d'équation de droite ou cercle lié à un point",
  nbdec: 'nbdec',
  indAffLie: "Cliquez sur le point auquel l'affichage sera lié",
  infConst: 'Informations sur la construction :',
  NoMac: 'Aucune macro',
  ChoixMacDem: "Choix d'une macro pour le démarrage",
  MacRecDlg1: 'Profondeur de récursion : ',
  MacRecDlg2: 'Niveau de récursion : ',
  MacRecDlg3: "Nombre d'itérations par niveau : ",
  MacRecDlg4: "Pas d'itération (>=1) :",
  MacRecDlg5: "Nombre d'objets sources à réutiliser à chaque itération : ",
  MacRecDlg6: 'Ne créer que les objets de dernière génération',
  MacRecDlg7: 'Imposer une couleur pour les objet créés ',
  MacRecDlg8: 'Polygone rempli',
  MacRecDlg9: 'Ne pas joindre les points finaux',
  MacRecDlg10: 'Joindre les points finaux par un polygone',
  MacRecDlg11: 'Joindre les points finaux par une ligne brisée',
  errRec: 'Paramètres de récursion invalides',
  errImpMax: 'Cette construction est refusée : plus de 200000 objets créés',
  MacConsRec: 'Macro de construction récursive',
  ObjSrc: 'Objets sources : ',
  MacItDlg1: "Nombre d'itérations (>=2) ",
  errIter: "Paramètre d'itération invalides",
  MacConsIter: 'Macro de construction itérative',
  AffichageCoord: 'Affichage de coordonnées',
  AffichageCoordLie: 'Affichage de coordonnées lié à un point',
  indCoord: 'Cliquez sur le point dont vous voulez afficher les coordonnées',
  ErrLongUn: 'Cette construction ne peut pas être implémentée car la figure ne contient pas de longueur unité',
  PapMil: 'Papier millimétré',
  newFig2: 'Figure avec repère millimétré',
  ObjFin: 'Objet final de construction',
  AbsOrRep: "Abscisse à l'origine de repère",
  OrdOrRep: "Ordonnée à l'origine de repère",
  UnitexRep: 'Unité x de repère',
  UniteyRep: 'Unité y de repère',
  ObjInter: 'Objet intermédiaire de construction',
  ProtoNbMax: 'Protocole restreint aux 2000 premiers objets',
  ch150: 'Souhaitez-vous que ce point libre source devienne un objet final ?',
  errInvFile: 'Fichier non valide',
  TextFig: 'Texte de la figure',
  // Ce qui suit pour les exercices fais via la bibliothèque
  res: "L'exercice a été résolu et est enregistré",
  notres: "L'exercice n'a pas été résolu et est enregistré",
  saved: 'La figure a été enregistrée',
  none: "Aucun objet n'a été construit",
  anom: 'Les objets suivants doivent être nommés : ',
  InsDynDlg1: 'Insertion de valeur dynamique',
  InsDynDlg2: 'Signe + imposé',
  InsDynDlg3: 'Code à insérer :',
  InsForDlg1: 'Insertion de formule',
  InsForDlg2: 'Formule de :',
  InsForDlg3: 'Formule Simplifiée',
  aconst: 'Il manque un ou des éléments de type : ',
  indRay1: 'Cliquez sur le premier point donnant le rayon',
  indRay2: 'Cliquez sur le second point donnant le rayon',
  CerOAB: 'Compas',
  VersionErr: "Cette figure a été créée par une nouvelle version de MathGraph32</br>Vous devez mettre le logiciel à jour pour l'ouvrir",
  MesLongOr: 'Affichage de longueur orienté',
  proj: 'Projeté orthogonal',
  sur: 'sur',
  debArc: "Cliquer sur le point de début de l'arc",
  finArc: "Cliquez sur un point donnant la direction du point final de l'arc",
  chinfo5: 'Droite parallèle à',
  cloneCree: "Le clone de l'objet a été créé",
  tag: "Tag de l'objet",
  changeTag: 'Changer le tag',
  tagErr: 'Ce tag a déjà été attribué',
  Matrice: 'Matrice',
  CalculMat: 'Calcul matriciel',
  NbLig: 'Nombre de lignes',
  NbCol: 'Nombre de colonnes',
  NomMat: 'Nom de la matrice',
  infodotmult: 'dotmult(A,B) : Renvoie le produit terme à terme des deux matrices A et B',
  infoInv: 'inv(A) : Inverse terme à terme de la matrice A',
  MatriceAleat: 'Matrice à valeurs entières aléatoires',
  MatAleatComp: 'Matrice à valeurs distinctes comprises entre ',
  MatriceParForm: 'Matrice par formule',
  formMat: 'si(i=j,1,0)',
  Det: 'Déterminant',
  FracRat: 'Approximation par fraction rationnelle',
  infofrac: 'frac(x) : Renvoie une matrice ligne dont le premier élément est le numérateur<br>' +
    'de la fraction rationnelle approchée de x et le deuxième le dénominateur.<br>' +
    'Peut-être aussi appliqué à une matrice ligne ou colonne'
}

module.exports = textesFrAdd
